@@ -1,4 +1,3 @@
-# -*-python-*-
 # GemRB - Infinity Engine Emulator
 # Copyright (C) 2003-2004 The GemRB Project
 #
@@ -31,11 +30,12 @@ from ie_slots import SLOT_QUIVER
 from ie_restype import RES_2DA
 from ie_sounds import CHAN_HITS
 from GameCheck import MAX_PARTY_SIZE
-from Clock import UpdateClock
+from Clock import UpdateClock, CreateClockButton
 import GameCheck
 import GUICommon
 import CommonTables
 import CommonWindow
+import Container
 import LUCommon
 import InventoryCommon
 if not GameCheck.IsPST():
@@ -58,6 +58,10 @@ else:
 	MageSpellsKey = 'Wizard_Spells'
 	CharacterStatsKey = 'Character_Record'
 
+StatesFont = "STATES2"
+if GameCheck.IsIWD1() or GameCheck.IsIWD2():
+	StatesFont = "STATES"
+
 #The following tables deal with the different control indexes and string refs of each game
 #so that actual interface code can be game neutral
 #the dictionary keys match entries in keymap.2da
@@ -135,7 +139,7 @@ def SetupMenuWindowControls (Window, Gears=None, CloseWindowCallback=None):
 	#store these instead of doing 50 calls...
 	
 	EscButton = Window.CreateButton (99, 0, 0, 0, 0);
-	EscButton.SetEvent(IE_GUI_BUTTON_ON_PRESS, CloseTopWindow)
+	EscButton.OnPress (CloseTopWindow)
 	EscButton.MakeEscape()
 
 	if iwd2: # IWD2 has one spellbook to rule them all
@@ -145,19 +149,19 @@ def SetupMenuWindowControls (Window, Gears=None, CloseWindowCallback=None):
 
 		# AI
 		Button = InitOptionButton(Window, 'Toggle_AI')
-		Button.SetEvent (IE_GUI_BUTTON_ON_PRESS, AIPress)
+		Button.OnPress (AIPress)
 		AIPress (0) #this initialises the state and tooltip
 
 		# Select All
 		Button = InitOptionButton(Window, 'SelectAll', False)
-		Button.SetEvent(IE_GUI_BUTTON_ON_PRESS, GUICommon.SelectAllOnPress)
+		Button.OnPress (GUICommon.SelectAllOnPress)
 	elif pst: #pst has these three controls here instead of portrait pane
 		# (Un)Lock view on character
 		Button = InitOptionButton(Window, 'Follow', False)  # or 41648 Unlock ...
-		Button.SetEvent(IE_GUI_BUTTON_ON_PRESS, OnLockViewPress)
+		Button.OnPress (OnLockViewPress)
 		# AI
 		Button = InitOptionButton(Window, 'Toggle_AI')
-		Button.SetEvent (IE_GUI_BUTTON_ON_PRESS, AIPress)
+		Button.OnPress (AIPress)
 		AIPress(0) #this initialises the state and tooltip
 
 		# Message popup FIXME disable on non game screen...
@@ -177,7 +181,7 @@ def SetupMenuWindowControls (Window, Gears=None, CloseWindowCallback=None):
 	# Party managment / character arbitration. Distinct form reform party window.
 	if not pst:
 		Button = Window.GetControl (OptionControl['Party'])
-		Button.SetEvent (IE_GUI_BUTTON_ON_PRESS, None) #TODO: OpenPartyWindow
+		Button.OnPress (None) #TODO: OpenPartyWindow
 		if bg1 or bg2:
 			Button.SetState (IE_GUI_BUTTON_DISABLED)
 			Button.SetFlags (IE_GUI_BUTTON_NO_IMAGE, OP_OR)
@@ -233,35 +237,21 @@ def SetupMenuWindowControls (Window, Gears=None, CloseWindowCallback=None):
 		Button.SetSprites ("GUILSOP", 0,12,13,26,12)
 	if iwd1:
 		Button.SetSprites ("GUILSOP", 0,12,13,26,26)
-
+	ButtonOptionFrame = Button.GetFrame()
 
 	# pause button
 	if Gears:
-		# Pendulum, gears, sun/moon dial (time)
-		# FIXME: display all animations: CPEN, CGEAR, CDIAL
 		if how: # how doesn't have this in the right place
 			pos = Window.GetFrame()["h"] - 71
 			Window.CreateButton (OptionControl['Time'], 0, pos, 64, 71)
+		CreateClockButton(Window.GetControl(OptionControl['Time']))
 
-		Button = Window.GetControl (OptionControl['Time'])
-		if bg2:
-			Label = Button.CreateLabel (0x10000009, "NORMAL", "", IE_FONT_SINGLE_LINE)
-			Label.SetAnimation ("CPEN")
-
-		Button.SetAnimation ("CGEAR")
-		Button.SetState (IE_GUI_BUTTON_ENABLED)
-		Button.SetFlags (IE_GUI_BUTTON_PICTURE|IE_GUI_BUTTON_ANIMATED|IE_GUI_BUTTON_NORMAL, OP_SET)
-		Button.SetEvent(IE_GUI_BUTTON_ON_PRESS, GUICommon.GearsClicked)
 		if iwd2:
-			Button.SetState (IE_GUI_BUTTON_LOCKED) #no button depression, timer is an inset stone planet
 			rb = OptionControl['Rest']
 		else:
 			rb = 11
-		UpdateClock ()
 	else:
 		rb = OptionControl['Rest']
-		if iwd2:
-			UpdateClock ()
 
 	# BG1 doesn't have a rest button on the main window, so this creates one
 	# from what would be the multiplayer arbitration control
@@ -278,8 +268,14 @@ def SetupMenuWindowControls (Window, Gears=None, CloseWindowCallback=None):
 	Button = Window.GetControl (rb)
 	if Button:
 		Button.SetTooltip (OptionTip['Rest'])
-		Button.SetEvent (IE_GUI_BUTTON_ON_PRESS, RestPress)
-
+		Button.OnPress (RestPress)
+	elif rb == 11 and bg2:
+		#The 15 gap came from the guiw8.chu that is the network button
+		pos = ButtonOptionFrame["y"] + ButtonOptionFrame["h"] + 15
+		Button = Window.CreateButton (rb, ButtonOptionFrame["x"], pos, ButtonOptionFrame["w"], ButtonOptionFrame["h"])
+		Button.SetSprites ("GUIRSBUT", 0,0,1,0,0)
+		Button.SetTooltip (OptionTip['Rest'])
+		Button.OnPress (RestPress)
 	return
 
 def OnLockViewPress ():
@@ -360,9 +356,14 @@ def EmptyControls ():
 		Button.SetVisible (False)
 	return
 
+def SelectFormation (btn):
+	GemRB.GameSetFormation (btn.Value)
+	return
+
 def SelectFormationPreset ():
 	"""Choose the default formation."""
-	GemRB.GameSetFormation (GemRB.GetVar ("Value"), GemRB.GetVar ("Formation") )
+	GemRB.GameSetFormation (GemRB.GetVar ("Value"), GemRB.GetVar ("Formation")) # save
+	GemRB.GameSetFormation (GemRB.GetVar ("Formation")) # set
 	GroupControls ()
 	return
 
@@ -374,12 +375,13 @@ def SetupFormation ():
 		Button.SetSprites ("GUIBTBUT",0,0,1,2,3)
 		Button.SetBAM ("FORM%x"%i,0,0,-1)
 		Button.SetVarAssoc ("Value", i)
-		Button.SetEvent (IE_GUI_BUTTON_ON_PRESS, SelectFormationPreset)
+		Button.OnPress (SelectFormationPreset)
 		Button.SetState (IE_GUI_BUTTON_UNPRESSED)
 	return
 
 def GroupControls ():
 	"""Sections that control group actions."""
+
 	GemRB.SetVar ("ActionLevel", UAW_STANDARD)
 	Button = CurrentWindow.GetControl (ActionBarControlOffset)
 	if GameCheck.IsBG2():
@@ -398,38 +400,35 @@ def GroupControls ():
 	Button.SetActionIcon (globals(), -1, 6)
 	Button = CurrentWindow.GetControl (6+ActionBarControlOffset)
 	Button.SetActionIcon (globals(), -1, 7)
-	GemRB.SetVar ("Formation", GemRB.GameGetFormation ())
+
+	formation = GemRB.GameGetFormation () # formation index
+	GemRB.SetVar ("Formation", formation)
 	for i in range (5):
 		Button = CurrentWindow.GetControl (7+ActionBarControlOffset+i)
-		Button.SetState (IE_GUI_BUTTON_ENABLED)
 		idx = GemRB.GameGetFormation (i)
 		Button.SetFlags (IE_GUI_BUTTON_RADIOBUTTON|IE_GUI_BUTTON_NORMAL, OP_SET)
 		# kill the previous sprites or they show through
 		Button.SetSprites ("GUIBTBUT",0,0,1,2,3)
 		Button.SetBAM ("FORM%x"%idx,0,0,-1)
 		Button.SetVarAssoc ("Formation", i)
-		Button.SetEvent (IE_GUI_BUTTON_ON_PRESS, GUICommon.SelectFormation)
-		Button.SetEvent (IE_GUI_BUTTON_ON_RIGHT_PRESS, SetupFormation)
+		Button.OnPress (SelectFormation)
+		Button.OnRightPress (SetupFormation)
 		Button.SetTooltip (4935)
 		# 0x90 = F1 key
 		Button.SetHotKey (chr(7+i+0x90), 0, True)
+
+	# work around radiobutton preselection issue
+	for i in range (5):
+		Button = CurrentWindow.GetControl (7 + ActionBarControlOffset + i)
+		if i == formation:
+			Button.SetState (IE_GUI_BUTTON_SELECTED)
+		else:
+			Button.SetState (IE_GUI_BUTTON_ENABLED)
+
 	return
 
 def OpenActionsWindowControls (Window):
-	# 1280 and higher don't have this control
-	if not Window.GetControl (62):
-		UpdateActionsWindow ()
-		return
-	# Gears (time) when options pane is down
-	Button = Window.GetControl (62)
-	Label = Button.CreateLabel (0x1000003e, "NORMAL", "", IE_FONT_SINGLE_LINE)
-
-	# FIXME: display all animations
-	Label.SetAnimation ("CPEN")
-	Button.SetAnimation ("CGEAR")
-	Button.SetState (IE_GUI_BUTTON_ENABLED)
-	Button.SetFlags (IE_GUI_BUTTON_PICTURE|IE_GUI_BUTTON_ANIMATED|IE_GUI_BUTTON_NORMAL, OP_SET)
-	Button.SetEvent(IE_GUI_BUTTON_ON_PRESS, GUICommon.GearsClicked)
+	CreateClockButton(Window.GetControl (62))
 	UpdateActionsWindow ()
 	return
 
@@ -497,8 +496,8 @@ def SetupItemAbilities(pc, slot, only):
 					Button.SetSprites ("GUIBTBUT", 0, 0,1,3,5)
 					Button.SetItemIcon (ammoslot['ItemResRef'])
 					Button.SetText (str(ammoslot["Usages0"]))
-					Button.SetEvent (IE_GUI_BUTTON_ON_PRESS, SelectQuiverSlot)
-					Button.SetEvent (IE_GUI_BUTTON_ON_RIGHT_PRESS, SelectQuiverSlot)
+					Button.OnPress (SelectQuiverSlot)
+					Button.OnRightPress (SelectQuiverSlot)
 					Button.SetVarAssoc ("Slot", ammoslots[i])
 					if Tips[0] != -1:
 						Button.SetTooltip (Tips[0])
@@ -555,8 +554,8 @@ def SetupItemAbilities(pc, slot, only):
 			Button.SetSprites ("GUIBTBUT", 0, 0,1,2,5)
 			Button.SetItemIcon (slot_item['ItemResRef'], i+6)
 			Button.SetText ("")
-			Button.SetEvent (IE_GUI_BUTTON_ON_PRESS, SelectItemAbility)
-			Button.SetEvent (IE_GUI_BUTTON_ON_RIGHT_PRESS, SelectItemAbility)
+			Button.OnPress (SelectItemAbility)
+			Button.OnRightPress (SelectItemAbility)
 			Button.SetVarAssoc ("Ability", i)
 			Button.SetState (IE_GUI_BUTTON_ENABLED)
 			if Tips[i] != -1:
@@ -612,7 +611,13 @@ def UpdateActionsWindow ():
 	"""Redraws the actions section of the window."""
 	global CurrentWindow
 	global level, TopIndex
-	
+
+	# don't do anything if the GameControl is temporarily unavailable
+	try:
+		GUICommon.GameControl.GetFrame()
+	except RuntimeError:
+		return
+
 	PortraitWindow = GemRB.GetView("PORTWIN")
 	ActionsWindow = GemRB.GetView("ACTWIN")
 
@@ -704,6 +709,8 @@ def UpdateActionsWindow ():
 		Spellbook.SetupSpellIcons(CurrentWindow, spelltype, TopIndex, ActionBarControlOffset)
 	elif level == UAW_QSONGS: # songs selection
 		spelltype = 1<<IE_IWD2_SPELL_SONG
+		if GameCheck.IsIWD1():
+			spelltype = 1 << IE_SPELL_TYPE_SONG
 		GemRB.SetVar ("Type", spelltype)
 		Spellbook.SetupSpellIcons(CurrentWindow, spelltype, TopIndex, ActionBarControlOffset)
 	elif level == UAW_BOOK: # spellbook selection
@@ -884,10 +891,13 @@ def ActionShapeChangePressed ():
 	return
 
 def SelectBardSong (which):
+	songType = IE_IWD2_SPELL_SONG
+	if GameCheck.IsIWD1 ():
+		songType = IE_SPELL_TYPE_SONG
 	pc = GemRB.GameGetFirstSelectedActor ()
-	songs = Spellbook.GetKnownSpells (pc, IE_IWD2_SPELL_SONG)
+	songs = Spellbook.GetKnownSpells (pc, songType)
 	# "which" is a mashup of the spell index with it's type
-	idx = which % ((1<<IE_IWD2_SPELL_SHAPE)*100)
+	idx = which % ((1 << songType) * 100)
 	qsong = songs[idx]['SpellResRef']
 	# the effect needs to be set each tick, so we use FX_DURATION_INSTANT_PERMANENT==1 timing mode
 	# GemRB.SetModalState can also set the spell, but it wouldn't persist
@@ -1081,46 +1091,49 @@ def SpellShiftPressed ():
 	pc = GemRB.GameGetFirstSelectedActor ()
 	ClassRowName = GUICommon.GetClassRowName (pc)
 	SponCastTableName = CommonTables.ClassSkills.GetValue (ClassRowName, "SPONCAST")
-	if SponCastTableName != "*":
-		SponCastTable = GemRB.LoadTable (SponCastTableName, 1)
-		if not SponCastTable:
-			print("SpellShiftPressed: skipping, non-existent spontaneous casting table used! ResRef:", SponCastTableName)
-			SpellPressed ()
-			return
+	if SponCastTableName == "*":
+		# proceed as if nothing happened
+		SpellPressed ()
+		return
 
-		# determine the column number (spell variety) depending on alignment
-		CureOrHarm = GemRB.GetPlayerStat (pc, IE_ALIGNMENT)
-		if CureOrHarm % 16 == 3: # evil
-			CureOrHarm = 1
-		else:
-			CureOrHarm = 0
+	SponCastTable = GemRB.LoadTable (SponCastTableName, True, True)
+	if not SponCastTable:
+		print("SpellShiftPressed: skipping, non-existent spontaneous casting table used! ResRef:", SponCastTableName)
+		SpellPressed ()
+		return
 
-		# get the unshifted booktype
-		BaseType = 0
-		tmp = Type
-		while tmp > 1:
-			tmp = tmp>>1
-			BaseType += 1
+	# determine the column number (spell variety) depending on alignment
+	CureOrHarm = GemRB.GetPlayerStat (pc, IE_ALIGNMENT)
+	if CureOrHarm % 16 == 3: # evil
+		CureOrHarm = 1
+	else:
+		CureOrHarm = 0
 
-		# figure out the spell's details
-		# TODO: find a simpler way
-		Spell = None
-		MemorisedSpells = Spellbook.GetSpellinfoSpells (pc, BaseType)
-		for spell in MemorisedSpells:
-			if spell['SpellIndex']%(255000) == SpellIndex: # 255 is the engine value of Type
-				Spell = spell
-				break
+	# get the unshifted booktype
+	BaseType = 0
+	tmp = Type
+	while tmp > 1:
+		tmp = tmp >> 1
+		BaseType += 1
 
-		# rownames==level; col1: good+neutral; col2: evil resref
-		Level = Spell['SpellLevel']
-		ReplacementSpell = SponCastTable.GetValue (Level-1, CureOrHarm).upper()
-		if ReplacementSpell != Spell['SpellResRef'].upper():
-			SpellIndex = GemRB.PrepareSpontaneousCast (pc, Spell['SpellResRef'], Spell['BookType'], Level, ReplacementSpell)
-			GemRB.SetVar ("Spell", SpellIndex+1000*Type)
-			if GameCheck.IsIWD2():
-				GemRB.DisplayString (39742, ColorWhite, pc) # Spontaneous Casting
+	# figure out the spell's details
+	# TODO: find a simpler way
+	Spell = None
+	MemorisedSpells = Spellbook.GetSpellinfoSpells (pc, BaseType)
+	for spell in MemorisedSpells:
+		if spell['SpellIndex'] % (255000) == SpellIndex: # 255 is the engine value of Type
+			Spell = spell
+			break
+
+	# rownames==level; col1: good+neutral; col2: evil resref
+	Level = Spell['SpellLevel']
+	ReplacementSpell = SponCastTable.GetValue (Level - 1, CureOrHarm).upper()
+	if ReplacementSpell != Spell['SpellResRef'].upper():
+		SpellIndex = GemRB.PrepareSpontaneousCast (pc, Spell['SpellResRef'], Spell['BookType'], Level, ReplacementSpell)
+		GemRB.SetVar ("Spell", SpellIndex + 1000 * Type)
+		if GameCheck.IsIWD2 ():
+			GemRB.DisplayString (39742, ColorWhite, pc) # Spontaneous Casting
 
-	# proceed as if nothing happened
 	SpellPressed ()
 
 # This is the endpoint for spellcasting, finally calling SpellCast. This always happens at least
@@ -1136,7 +1149,7 @@ def SpellPressed ():
 	Spell = GemRB.GetVar ("Spell")
 	Type = GemRB.GetVar ("Type")
 
-	if Type == 1<<IE_IWD2_SPELL_SONG:
+	if Type == 1 << IE_IWD2_SPELL_SONG or Type == 1 << IE_SPELL_TYPE_SONG:
 		SelectBardSong (Spell)
 		ActionBardSongPressed()
 		return
@@ -1365,7 +1378,7 @@ else:
 	DefaultWinPos = WINDOW_CENTER
 
 def CreateTopWinLoader(id, pack, loader, initer = None, selectionHandler = None, pos = DefaultWinPos, pause = False):
-	def ret (btn = None, val = None):
+	def ret (btn = None):
 		topwin = GemRB.GetView("WIN_TOP")
 		if topwin and topwin.HasFocus == False:
 			return None # we cannot close the current WIN_TOP unless it has focus
@@ -1402,7 +1415,7 @@ def CreateTopWinLoader(id, pack, loader, initer = None, selectionHandler = None,
 				rtgbtn.SetState(IE_GUI_BUTTON_UNPRESSED)
 			if btn:
 				btn.SetState(IE_GUI_BUTTON_SELECTED)
-				GemRB.SetVar("OPTBTN", val) # cant use btn.ID because it is "too large to convert to C long"
+				GemRB.SetVar ("OPTBTN", btn.ID)
 			
 			GameWin = GemRB.GetView("GAMEWIN")
 			GameWin.SetDisabled(True)
@@ -1493,7 +1506,7 @@ def GetPortraitButtonPairs (Window, ExtraSlots=0, Mode="vertical"):
 
 	# GUIWORLD doesn't have a separate portraits window, so we need to skip
 	# all this magic when reforming an overflowing party
-	if GemRB.GetPartySize () > MAX_PARTY_SIZE:
+	if PartySize > MAX_PARTY_SIZE:
 		return pairs
 
 	# generate new buttons by copying from existing ones
@@ -1517,18 +1530,20 @@ def GetPortraitButtonPairs (Window, ExtraSlots=0, Mode="vertical"):
 	else:
 		# reduce it by existing slots + 0 slots in framed views (eg. inventory) and
 		# 1 in main game control (for spacing and any other controls below (ai/select all in bg2))
-		maxHeight = windowHeight - buttonHeight*6 - buttonHeight // 2
+		maxHeight = windowHeight - buttonHeight * PartySize - buttonHeight // 2
 		if windowHeight != ScreenHeight:
 			maxHeight += buttonHeight // 2
 		limit = maxHeight
 		# for framed views, limited to 6, we downscale the buttons to fit, clipping their portraits
 		if maxHeight < buttonHeight:
-			unused = 20 # remaining unused space below the portraits
+			unused = -40 if GameCheck.IsBG1() else 20 # remaining unused space below the portraits
 			scale = 1
 			portraitGap = buttonHeight
-			buttonHeight = (buttonHeight * 6 + unused) // PartySize
+			buttonHeight = (windowHeight + unused) // PartySize
+			if portraitGap == buttonHeight:
+				scale = 0 # ensure idempotence, eg. when this gets called on click
 			portraitGap = portraitGap - buttonHeight - 2 # 2 for a quasi border
-			limit = windowHeight - buttonHeight*6 + unused
+			limit = windowHeight - buttonHeight * 6 + unused
 		limitStep = buttonHeight
 
 	for i in range(len(pairs), PartySize):
@@ -1547,40 +1562,51 @@ def GetPortraitButtonPairs (Window, ExtraSlots=0, Mode="vertical"):
 
 		button = Window.GetControl (nextID)
 		button.SetSprites ("GUIRSPOR", 0, 0, 1, 0, 0)
+		button.SetVarAssoc ("portrait", i + 1)
+		SetupButtonBorders (Window, button, i)
+		button.SetFont (StatesFont)
+
+		button.OnRightPress (OpenInventoryWindowClick)
+		button.OnPress (PortraitButtonOnPress)
+		button.OnShiftPress (PortraitButtonOnShiftPress)
+		button.SetAction (PortraitButtonOnShiftPress, IE_ACT_MOUSE_PRESS, GEM_MB_ACTION, GEM_MOD_CTRL, 1)
+		button.SetAction (ButtonDragSourceHandler, IE_ACT_DRAG_DROP_SRC)
+		button.SetAction (ButtonDragDestHandler, IE_ACT_DRAG_DROP_DST)
 
 		pairs[i] = button
 		limit -= limitStep
 
 	# move the buttons back up, to combine the freed space
 	if scale:
-		for i in range(oldSlotCount):
+		for i in range(PartySize - 1):
 			button = pairs[i]
 			button.SetSize (buttonWidth, buttonHeight)
 			if i == 0:
 				continue # don't move the first portrait
-			rect = button.GetRect ()
-			x = rect["X"]
-			y = rect["Y"]
+			rect = button.GetFrame ()
+			x = rect["x"]
+			y = rect["y"]
 			button.SetPos (x, y-portraitGap*i)
 
 	return pairs
 
-def OpenInventoryWindowClick (btn, pcID):
+def OpenInventoryWindowClick (btn):
 	import GUIINV
-	GemRB.GameSelectPC (pcID, True, SELECT_REPLACE)
+	GemRB.GameSelectPC (btn.Value, True, SELECT_REPLACE)
 	GUIINV.OpenInventoryWindow ()
 	return
 
 DragButton = None
-def ButtonDragSourceHandler(btn):
+def ButtonDragSourceHandler (btn):
 	global DragButton
 	DragButton = btn;
 	
-def ButtonDragDestHandler(btn, pcID):
+def ButtonDragDestHandler (btn):
 	global DragButton
 	
+	pcID = btn.Value
 	# So far this is only used for portrait buttons
-	if DragButton: # DragButton will be none for item drags instantiated by clicking (not sragging) an inventory item
+	if DragButton: # DragButton will be none for item drags instantiated by clicking (not dragging) an inventory item
 		if DragButton.VarName == "portrait" and btn.VarName == "portrait":
 			GemRB.GameSwapPCs (DragButton.Value, pcID)
 			
@@ -1591,6 +1617,49 @@ def ButtonDragDestHandler(btn, pcID):
 		
 	DragButton = None
 
+def AddStatusFlagLabel (Window, Button, i):
+	label = Window.GetControl (200 + i)
+	if label:
+		return label
+
+	# label for status flags (dialog, store, level up)
+	align = IE_FONT_ALIGN_TOP | IE_FONT_ALIGN_RIGHT | IE_FONT_SINGLE_LINE
+	if GameCheck.IsBG2 ():
+		align = IE_FONT_ALIGN_TOP | IE_FONT_ALIGN_CENTER | IE_FONT_SINGLE_LINE
+	label = Button.CreateLabel (200 + i, StatesFont, "", align) #level up icon is on the right
+	label.SetFrame (Button.GetInsetFrame (4))
+	return label
+
+# overlay a label, so we can display the hp with the correct font. Regular button label
+#   is used by effect icons
+def AddHPLabel (Window, Button, i):
+	label = Window.GetControl (100 + i)
+	if label:
+		return label
+
+	label = Button.CreateLabel (100 + i, "NUMFONT", "", IE_FONT_ALIGN_TOP | IE_FONT_ALIGN_LEFT | IE_FONT_SINGLE_LINE)
+	return label
+
+def SetupButtonBorders (Window, Button, i):
+	# unlike other buttons, this one lacks extra frames for a selection effect
+	# so we create it and shift it to cover the grooves of the image
+	# except iwd2's second frame already has it incorporated (but we miscolor it)
+	yellow = {'r' : 255, 'g' : 255, 'b' : 0, 'a' : 255}
+	green = {'r' : 0, 'g' : 255, 'b' : 0, 'a' : 255}
+	if GameCheck.IsIWD2 ():
+		Button.SetBorder (FRAME_PC_SELECTED, green)
+		Button.SetBorder (FRAME_PC_TARGET, yellow, 0, 0, Button.GetInsetFrame (2, 2, 3, 3))
+	elif GameCheck.IsPST ():
+		Button.SetBorder (FRAME_PC_SELECTED, green, 0, 0, Button.GetInsetFrame (1, 1, 2, 2))
+		Button.SetBorder (FRAME_PC_TARGET, yellow, 0, 0, Button.GetInsetFrame (3, 3, 4, 4))
+		Button.SetBAM ("PPPANN", 0, 0, -1) # NOTE: just a dummy, won't be visible
+		ButtonHP = Window.GetControl (6 + i)
+		ButtonHP.OnPress (PortraitButtonHPOnPress)
+		ButtonHP.SetVarAssoc ("pid", i + 1) # storing so the callback knows which button it's operating on
+	else:
+		Button.SetBorder (FRAME_PC_SELECTED, green, 0, 0, Button.GetInsetFrame (4, 3, 4, 3))
+		Button.SetBorder (FRAME_PC_TARGET, yellow, 0, 0, Button.GetInsetFrame (2, 2, 3, 3))
+
 def OpenPortraitWindow (needcontrols=0, pos=WINDOW_RIGHT|WINDOW_VCENTER):
 	#take care, this window is different in how/iwd
 	if GameCheck.HasHOW() and needcontrols:
@@ -1608,9 +1677,9 @@ def OpenPortraitWindow (needcontrols=0, pos=WINDOW_RIGHT|WINDOW_VCENTER):
 			if GameCheck.IsIWD():
 				# Rest (iwd)
 				Button.SetTooltip (11942)
-				Button.SetEvent (IE_GUI_BUTTON_ON_PRESS, RestPress)
+				Button.OnPress (RestPress)
 			else:
-				Button.SetEvent (IE_GUI_BUTTON_ON_PRESS, MinimizePortraits)
+				Button.OnPress (MinimizePortraits)
 		else:
 			if GameCheck.HasHOW():
 				# Rest (how)
@@ -1618,7 +1687,7 @@ def OpenPortraitWindow (needcontrols=0, pos=WINDOW_RIGHT|WINDOW_VCENTER):
 				Button = Window.CreateButton (8, 6, pos, 55, 37)
 				Button.SetSprites ("GUIRSBUT", 0,0,1,0,0)
 				Button.SetTooltip (11942)
-				Button.SetEvent (IE_GUI_BUTTON_ON_PRESS, RestPress)
+				Button.OnPress (RestPress)
 
 				pos = pos - 37
 				Window.CreateButton (6, 6, pos, 27, 36)
@@ -1637,14 +1706,14 @@ def OpenPortraitWindow (needcontrols=0, pos=WINDOW_RIGHT|WINDOW_VCENTER):
 		else:
 			Button = Window.GetControl (7)
 		Button.SetTooltip (10485)
-		Button.SetEvent (IE_GUI_BUTTON_ON_PRESS, GUICommon.SelectAllOnPress)
+		Button.OnPress (GUICommon.SelectAllOnPress)
 	else:
 		# Rest
 		if not GameCheck.IsIWD2():
 			Button = Window.GetControl (6)
 			if Button:
 				Button.SetTooltip (11942)
-				Button.SetEvent (IE_GUI_BUTTON_ON_PRESS, RestPress)
+				Button.OnPress (RestPress)
 
 	PortraitButtons = GetPortraitButtonPairs (Window)
 	for i, Button in PortraitButtons.items():
@@ -1653,49 +1722,23 @@ def OpenPortraitWindow (needcontrols=0, pos=WINDOW_RIGHT|WINDOW_VCENTER):
 		Button.SetVarAssoc("portrait", pcID)
 		
 		if not GameCheck.IsPST():
-			fontref = "STATES2"
-			if GameCheck.IsIWD1() or GameCheck.IsIWD2():
-				fontref = "STATES"
-			
-			Button.SetFont (fontref)
-			# label for status flags (dialog, store, level up)
-			align = IE_FONT_ALIGN_TOP | IE_FONT_ALIGN_CENTER | IE_FONT_SINGLE_LINE
-			label = Button.CreateLabel(200 + i, fontref, "", align) #level up icon is on the right
-			label.SetFrame(Button.GetInsetFrame(4))
+			Button.SetFont (StatesFont)
+
+			AddStatusFlagLabel (Window, Button, i)
 
 		if needcontrols or GameCheck.IsIWD2():
-			Button.SetEvent (IE_GUI_BUTTON_ON_RIGHT_PRESS, OpenInventoryWindowClick)
+			Button.OnRightPress (OpenInventoryWindowClick)
 		else:
-			Button.SetEvent (IE_GUI_BUTTON_ON_RIGHT_PRESS, PortraitButtonOnPress)
+			Button.OnRightPress (PortraitButtonOnPress)
 
-		Button.SetEvent (IE_GUI_BUTTON_ON_PRESS, PortraitButtonOnPress)
-		Button.SetEvent (IE_GUI_BUTTON_ON_SHIFT_PRESS, PortraitButtonOnShiftPress)
-		Button.SetAction(ButtonDragSourceHandler, IE_ACT_DRAG_DROP_SRC)
-		Button.SetAction(ButtonDragDestHandler, IE_ACT_DRAG_DROP_DST)
+		Button.OnPress (PortraitButtonOnPress)
+		Button.OnShiftPress (PortraitButtonOnShiftPress)
+		Button.SetAction (PortraitButtonOnShiftPress, IE_ACT_MOUSE_PRESS, GEM_MB_ACTION, GEM_MOD_CTRL, 1)
+		Button.SetAction (ButtonDragSourceHandler, IE_ACT_DRAG_DROP_SRC)
+		Button.SetAction (ButtonDragDestHandler, IE_ACT_DRAG_DROP_DST)
 
-		if GameCheck.IsIWD1() or GameCheck.IsIWD2():
-			# overlay a label, so we can display the hp with the correct font. Regular button label
-			#   is used by effect icons
-			Button.CreateLabel(100+i, "NUMFONT", "", IE_FONT_ALIGN_TOP|IE_FONT_ALIGN_LEFT|IE_FONT_SINGLE_LINE)
-
-		# unlike other buttons, this one lacks extra frames for a selection effect
-		# so we create it and shift it to cover the grooves of the image
-		# except iwd2's second frame already has it incorporated (but we miscolor it)
-		yellow = {'r' : 255, 'g' : 255, 'b' : 0, 'a' : 255}
-		green = {'r' : 0, 'g' : 255, 'b' : 0, 'a' : 255}
-		if GameCheck.IsIWD2():
-			Button.SetBorder (FRAME_PC_SELECTED, green)
-			Button.SetBorder (FRAME_PC_TARGET, yellow, 0, 0, Button.GetInsetFrame(2,2,3,3))
-		elif GameCheck.IsPST():
-			Button.SetBorder (FRAME_PC_SELECTED, green, 0, 0, Button.GetInsetFrame(1,1,2,2))
-			Button.SetBorder (FRAME_PC_TARGET, yellow, 0, 0, Button.GetInsetFrame(3,3,4,4))
-			Button.SetBAM ("PPPANN", 0, 0, -1) # NOTE: just a dummy, won't be visible
-			ButtonHP = Window.GetControl (6 + i)
-			ButtonHP.SetEvent (IE_GUI_BUTTON_ON_PRESS, PortraitButtonHPOnPress)
-			ButtonHP.SetVarAssoc ("pid", i + 1) # storing so the callback knows which button it's operating on
-		else:
-			Button.SetBorder (FRAME_PC_SELECTED, green, 0, 0, Button.GetInsetFrame(4,3,4,3))
-			Button.SetBorder (FRAME_PC_TARGET, yellow, 0, 0, Button.GetInsetFrame(2,2,3,3))
+		AddHPLabel (Window, Button, i)
+		SetupButtonBorders (Window, Button, i)
 
 	UpdatePortraitWindow ()
 	SelectionChanged ()
@@ -1718,7 +1761,7 @@ def UpdatePortraitWindow ():
 		if indialog:
 			Button.SetHotKey(None)
 		if (pcID <= GemRB.GetPartySize()):
-			Button.SetAction(lambda btn, val, pc=pcID: GemRB.GameControlLocateActor(pc), IE_ACT_MOUSE_ENTER);
+			Button.SetAction(lambda btn, pc=pcID: GemRB.GameControlLocateActor(pc), IE_ACT_MOUSE_ENTER);
 			Button.SetAction(lambda: GemRB.GameControlLocateActor(-1), IE_ACT_MOUSE_LEAVE);
 			if (i < 6 and not indialog):
 				Button.SetHotKey(chr(ord('1') + i), 0, True)
@@ -1747,38 +1790,48 @@ def UpdatePortraitWindow ():
 			continue
 
 		portraitFlags = IE_GUI_BUTTON_PORTRAIT | IE_GUI_BUTTON_HORIZONTAL | IE_GUI_BUTTON_ALIGN_LEFT | IE_GUI_BUTTON_ALIGN_BOTTOM
-
-		if GameCheck.IsIWD2():
-			Button.SetEvent (IE_GUI_BUTTON_ON_RIGHT_PRESS, OpenInventoryWindowClick)
-			Button.SetEvent (IE_GUI_BUTTON_ON_PRESS, PortraitButtonOnPress)
-
 		Button.SetFlags (portraitFlags, OP_SET)
 
 		Button.SetState (IE_GUI_BUTTON_LOCKED)
+		if pic == None:
+			pic = ""
 		Button.SetPicture (pic, "NOPORTSM")
 		ratio_str, color = GUICommon.SetupDamageInfo (pcID, Button, Window)
 
-		# character - 1 == bam cycle
-		talk = store = flag = blank = ' '
+		# character - 1 == bam cycle, sometimes
+		# only frames have all the glyphs
+		# only bg2 and iwds have a proper blank glyph
+		# so avoid using blanks except in bg2
+		flag = blank = bytearray([33])
 		if GameCheck.IsBG2():
-			# as far as I can tell only BG2 has icons for talk or store
-			flag = blank = chr(238)
-			talk = 154 # dialog icon
-			store = 155 # shopping icon
+			# only BG2 has icons for talk or store
+			flag = blank = bytearray([238])
+			talk = bytearray([154]) # dialog icon
+			store = bytearray([155]) # shopping icon
 
 			if pc == pcID and GemRB.GetStore() != None:
-				flag = chr(store)
-
+				flag = store
 			# talk icon
-			if GemRB.GameGetSelectedPCSingle(1) == pcID:
-				flag = chr(talk)
+			elif GemRB.GameGetSelectedPCSingle(1) == pcID:
+				flag = talk
 
 		if LUCommon.CanLevelUp (pcID):
-			flag = flag + blank + chr(255)
-		elif GameCheck.IsIWD1() or GameCheck.IsIWD2():
-			HPLabel = Window.GetControl (100+i)
-			HPLabel.SetText (ratio_str)
-			HPLabel.SetTextColor (color)
+			if GameCheck.IsBG2():
+				flag = flag + blank + bytearray([255])
+			else:
+				flag = bytearray([255])
+		else:
+			if GameCheck.IsBG2():
+				flag = flag + blank + blank
+			else:
+				flag = ""
+			if GameCheck.IsIWD1() or GameCheck.IsIWD2():
+				HPLabel = AddHPLabel (Window, Button, i) # missing if new pc joined since the window was opened
+				HPLabel.SetText (ratio_str)
+				HPLabel.SetColor (color)
+			
+		FlagLabel = AddStatusFlagLabel (Window, Button, i) # missing if new pc joined since the window was opened
+		FlagLabel.SetText(flag)
 
 		#add effects on the portrait
 		effects = GemRB.GetPlayerStates (pcID)
@@ -1786,23 +1839,15 @@ def UpdatePortraitWindow ():
 		numCols = 4 if GameCheck.IsIWD2() else 3
 		numEffects = len(effects)
 
-		states = b""
 		# calculate the partial row
 		idx = numEffects % numCols
-		states = effects[0:idx]
+		states = bytearray(effects[0:idx])
 		# now do any rows that are full
 		for x in range(idx, numEffects):
 			if (x - idx) % numCols == 0:
-				states = states + b"\n"
-			states = states + bytes(effects[x])
+				states.append(ord('\n'))
+			states.append(effects[x])
 
-		# FIXME: hack, check shouldn't be needed
-		FlagLabel = Window.GetControl (200 + i)
-		if FlagLabel:
-			if flag != blank:
-				FlagLabel.SetText (flag.ljust(3, blank))
-			else:
-				FlagLabel.SetText ("")
 		Button.SetText(states)
 	return
 
@@ -1814,7 +1859,7 @@ def UpdateAnimatedPortrait (Window,i):
 	pic = GemRB.GetPlayerPortrait (i+1, 0)["ResRef"]
 	if not pic:
 		Button.SetFlags (IE_GUI_BUTTON_NO_IMAGE, OP_SET)
-		Button.SetAnimation ("")
+		Button.SetAnimation (None)
 		ButtonHP.SetFlags (IE_GUI_BUTTON_NO_IMAGE, OP_SET)
 		ButtonHP.SetText ("")
 		ButtonHP.SetBAM ("", 0, 0)
@@ -1838,7 +1883,7 @@ def UpdateAnimatedPortrait (Window,i):
 	else:
 		cycle = 0
 
-	Button.SetFlags (IE_GUI_BUTTON_PICTURE | IE_GUI_BUTTON_ANIMATED, OP_SET)
+	Button.SetFlags (IE_GUI_BUTTON_PICTURE, OP_SET)
 	if cycle<6:
 		Button.SetFlags (IE_GUI_BUTTON_PLAYRANDOM, OP_OR)
 
@@ -1855,7 +1900,7 @@ def UpdateAnimatedPortrait (Window,i):
 	g = int (255 * ratio)
 
 	ButtonHP.SetText ("%s / %d" %(hp, hp_max))
-	ButtonHP.SetTextColor ({'r' : r, 'g' : g, 'b' : 0})
+	ButtonHP.SetColor ({'r' : r, 'g' : g, 'b' : 0})
 	ButtonHP.SetBAM ('FILLBAR', 0, 0, -1)
 	ButtonHP.SetPictureClipping (ratio)
 
@@ -1867,8 +1912,9 @@ def UpdateAnimatedPortrait (Window,i):
 
 	return
 
-def PortraitButtonOnPress (btn, pcID):
+def PortraitButtonOnPress (btn):
 	"""Selects the portrait individually."""
+	pcID = btn.Value
 
 	if GemRB.GameControlGetTargetMode() != TARGET_MODE_NONE:
 		GemRB.ActOnPC (pcID)
@@ -1883,9 +1929,10 @@ def PortraitButtonOnPress (btn, pcID):
 		SelectionChanged ()
 	return
 
-def PortraitButtonOnShiftPress (btn, pcID):
+def PortraitButtonOnShiftPress (btn):
 	"""Handles selecting multiple portaits with shift."""
-
+	
+	pcID = btn.Value
 	if (not SelectionChangeHandler):
 		sel = GemRB.GameIsPCSelected (pcID)
 		sel = not sel
@@ -1921,19 +1968,13 @@ def SelectionChanged ():
 			SelectionChangeMultiHandler ()
 	else:
 		sel = GemRB.GameGetSelectedPCSingle ()
-
-		#update mage school
-		GemRB.SetVar ("MAGESCHOOL", 0)
-		Kit = GUICommon.GetKitIndex (sel)
-		if Kit and CommonTables.KitList.GetValue (Kit, 7) == 1:
-			MageTable = GemRB.LoadTable ("magesch")
-			GemRB.SetVar ("MAGESCHOOL", MageTable.FindValue (3, CommonTables.KitList.GetValue (Kit, 6) ) )
+		GUICommon.UpdateMageSchool (sel)
 
 		PortraitButtons = GetPortraitButtonPairs (PortraitWindow)
 		for i, Button in PortraitButtons.items():
 			Button.EnableBorder (FRAME_PC_SELECTED, i + 1 == sel)
 
-	CommonWindow.CloseContainerWindow()
+	Container.CloseContainerWindow()
 	if SelectionChangeHandler:
 		SelectionChangeHandler ()
 	return
@@ -1962,8 +2003,6 @@ def SetItemButton (Window, Button, Slot, PressHandler, RightPressHandler): #rela
 	if Slot != None:
 		Item = GemRB.GetItem (Slot['ItemResRef'])
 		identified = Slot['Flags'] & IE_INV_ITEM_IDENTIFIED
-		#Button.SetVarAssoc ("LeftIndex", LeftTopIndex+i)
-		#Button.SetSprites ('IVSLOT', 0,  0, 0, 0, 0)
 		Button.SetItemIcon (Slot['ItemResRef'],0)
 
 		if Item['MaxStackAmount'] > 1:
@@ -1977,31 +2016,23 @@ def SetItemButton (Window, Button, Slot, PressHandler, RightPressHandler): #rela
 		else:
 			Button.SetTooltip (Item['ItemNameIdentified'])
 
-		#Button.SetFlags (IE_GUI_BUTTON_PICTURE, OP_OR)
-		#Button.SetFlags (IE_GUI_BUTTON_NO_IMAGE, OP_NAND)
-
-		Button.SetEvent (IE_GUI_BUTTON_ON_PRESS, PressHandler)
-		Button.SetEvent (IE_GUI_BUTTON_ON_RIGHT_PRESS, RightPressHandler)
-		#Button.SetEvent (IE_GUI_BUTTON_ON_SHIFT_PRESS, ShiftPressHandler)
-		#Button.SetEvent (IE_GUI_BUTTON_ON_DRAG_DROP, DragDropHandler)
+		Button.OnPress (PressHandler)
+		Button.OnRightPress (RightPressHandler)
 
 	else:
-		#Button.SetVarAssoc ("LeftIndex", -1)
 		Button.SetItemIcon ('')
 		Button.SetTooltip (4273)  # Ground Item
 		Button.SetText ('')
 		Button.SetFlags (IE_GUI_BUTTON_PICTURE, OP_NAND)
 
-		Button.SetEvent (IE_GUI_BUTTON_ON_PRESS, None)
-		Button.SetEvent (IE_GUI_BUTTON_ON_RIGHT_PRESS, None)
-		#Button.SetEvent (IE_GUI_BUTTON_ON_SHIFT_PRESS, None)
-		#Button.SetEvent (IE_GUI_BUTTON_ON_DRAG_DROP, None)
+		Button.OnPress (None)
+		Button.OnRightPress (None)
 
 def OpenWaitForDiscWindow ():
 	global DiscWindow
 
 	if DiscWindow:
-		DiscWindow.Unload ()
+		DiscWindow.Close ()
 		DiscWindow = None
 		return
 
@@ -2023,6 +2054,8 @@ def OpenWaitForDiscWindow ():
 
 
 def CheckLevelUp(pc):
+	if GameCheck.IsGemRBDemo ():
+		return
 	GemRB.SetVar ("CheckLevelUp"+str(pc), LUCommon.CanLevelUp (pc))
 
 def ToggleAlwaysRun():
@@ -2049,7 +2082,7 @@ def RealRestPress ():
 			Label.SetText (info["ErrorMsg"])
 			Button = Window.GetControl (1)
 			Button.SetText (1403)
-			Button.SetEvent (IE_GUI_BUTTON_ON_PRESS, lambda: Window.Close ())
+			Button.OnPress (Window.Close)
 			Window.ShowModal (MODAL_SHADOW_GRAY)
 		else:
 			GemRB.DisplayString (info["ErrorMsg"], ColorRed)
@@ -2072,5 +2105,5 @@ def OpenPSTDeathWindow ():
 	Label.SetText (48155)
 	Button = Window.GetControl (1)
 	Button.SetText (1403)
-	Button.SetEvent (IE_GUI_BUTTON_ON_PRESS, ShowCredits)
+	Button.OnPress (ShowCredits)
 	Window.ShowModal (MODAL_SHADOW_GRAY)
