@@ -1,4 +1,3 @@
-# -*-python-*-
 # GemRB - Infinity Engine Emulator
 # Copyright (C) 2012 The GemRB Project
 #
@@ -29,7 +28,7 @@
 # 9 - Autopause options window
 
 ###################################################
-import CommonWindow
+import Container
 import GameCheck
 import GemRB
 import GUICommonWindows
@@ -43,8 +42,6 @@ HelpTextArea = None
 
 LoadMsgWindow = None
 QuitMsgWindow = None
-SubOptionsWindow = None
-SubSubOptionsWindow = None
 
 if GameCheck.IsBG1():
 	HelpTextArea2 = None
@@ -56,43 +53,43 @@ else:
 def InitOptionsWindow (Window):
 	"""Open main options window"""
 
-	CommonWindow.CloseContainerWindow ()
+	Container.CloseContainerWindow ()
 
 	# Return to Game
 	Button = Window.GetControl (11)
 	Button.SetText (10308)
-	Button.SetEvent (IE_GUI_BUTTON_ON_PRESS, lambda: Window.Close())
+	Button.OnPress (Window.Close)
 	Button.MakeEscape()
 
 	# Quit Game
 	Button = Window.GetControl (10)
 	Button.SetText (13731)
-	Button.SetEvent (IE_GUI_BUTTON_ON_PRESS, OpenQuitMsgWindow)
+	Button.OnPress (OpenQuitMsgWindow)
 
 	# Load Game
 	Button = Window.GetControl (5)
 	Button.SetText (13729)
-	Button.SetEvent (IE_GUI_BUTTON_ON_PRESS, LoadGamePress)
+	Button.OnPress (LoadGamePress)
 
 	# Save Game
 	Button = Window.GetControl (6)
 	Button.SetText (13730)
-	Button.SetEvent (IE_GUI_BUTTON_ON_PRESS, OpenSaveMsgWindow)
+	Button.OnPress (OpenSaveMsgWindow)
 
 	# Video Options
 	Button = Window.GetControl (7)
 	Button.SetText (17162)
-	Button.SetEvent (IE_GUI_BUTTON_ON_PRESS, OpenVideoOptionsWindow)
+	Button.OnPress (OpenVideoOptionsWindow)
 
 	# Audio Options
 	Button = Window.GetControl (8)
 	Button.SetText (17164)
-	Button.SetEvent (IE_GUI_BUTTON_ON_PRESS, OpenAudioOptionsWindow)
+	Button.OnPress (OpenAudioOptionsWindow)
 
 	# Gameplay Options
 	Button = Window.GetControl (9)
 	Button.SetText (17165)
-	Button.SetEvent (IE_GUI_BUTTON_ON_PRESS, OpenGameplayOptionsWindow)
+	Button.OnPress (OpenGameplayOptionsWindow)
 
 	# game version, e.g. v1.1.0000
 	VersionLabel = Window.GetControl (0x1000000b)
@@ -102,17 +99,17 @@ def InitOptionsWindow (Window):
 		# Keyboard shortcuts
 		KeyboardButton = Window.GetControl (13)
 		KeyboardButton.SetText (33468)
-		KeyboardButton.SetEvent (IE_GUI_BUTTON_ON_PRESS, OpenHotkeyOptionsWindow)
+		KeyboardButton.OnPress (OpenHotkeyOptionsWindow)
 
 		# Movies
 		MoviesButton = Window.GetControl (14)
 		MoviesButton.SetText (15415)
-		MoviesButton.SetEvent (IE_GUI_BUTTON_ON_PRESS, OpenMovieWindow)
+		MoviesButton.OnPress (OpenMovieWindow)
 
 	return
 
-ToggleOptionsWindow = GUICommonWindows.CreateTopWinLoader(2, "GUIOPT", GUICommonWindows.ToggleWindow, InitOptionsWindow)
-OpenOptionsWindow = GUICommonWindows.CreateTopWinLoader(2, "GUIOPT", GUICommonWindows.OpenWindowOnce, InitOptionsWindow)
+ToggleOptionsWindow = GUICommonWindows.CreateTopWinLoader(2, "GUIOPT", GUICommonWindows.ToggleWindow, InitOptionsWindow, None, GUICommonWindows.DefaultWinPos, True)
+OpenOptionsWindow = GUICommonWindows.CreateTopWinLoader(2, "GUIOPT", GUICommonWindows.OpenWindowOnce, InitOptionsWindow, None, GUICommonWindows.DefaultWinPos, True)
 
 def TrySavingConfiguration():
 	if not GemRB.SaveConfig():
@@ -121,16 +118,15 @@ def TrySavingConfiguration():
 ###################################################
 
 def CloseVideoOptionsWindow ():
-	CloseSubOptionsWindow ()
+	GemRB.GetView("SUB_WIN", 0).Close()
 	TrySavingConfiguration()
 
 def OpenVideoOptionsWindow ():
 	"""Open video options window"""
-	global SubOptionsWindow, HelpTextArea
-
-	CloseSubOptionsWindow ()
+	global HelpTextArea
 
 	Window = GemRB.LoadWindow (6, "GUIOPT")
+	Window.AddAlias("SUB_WIN", 0)
 	Window.SetFlags (WF_BORDERLESS, OP_OR)
 
 	HelpTextArea = GUIOPTControls.OptHelpText ('VideoOptions', Window, 33, 18038)
@@ -148,11 +144,14 @@ def OpenVideoOptionsWindow ():
 	GUIOPTControls.OptCheckbox (18038, 18000, HelpTextArea, Window, 9, 38, 17131, 'Full Screen', DisplayHelpFullScreen)
 
 	GUIOPTControls.OptCheckbox (18038, 20620, HelpTextArea, Window, 51, 50, 20617, 'Translucent Shadows')
-	GUIOPTControls.OptCheckbox (18038, 18004, HelpTextArea, Window, 40, 44, 17134, 'SoftMirrorBlt')
+	GUIOPTControls.OptCheckbox (18038, 15135, HelpTextArea, Window, 40, 44, 17134, 'SoftMirrorBlt')
 	GUIOPTControls.OptCheckbox (18038, 18006, HelpTextArea, Window, 41, 46, 17136, 'SoftSrcKeyBlt') # software standard blit
 	GUIOPTControls.OptCheckbox (18038, 18007, HelpTextArea, Window, 42, 48, 17135, 'SoftBltFast') # software transparent blit
 
-	SubOptionsWindow = Window
+	# maybe not present in original iwd, but definitely in how
+	if GameCheck.IsIWD1 () or GameCheck.IsIWD2 ():
+		GUIOPTControls.OptCheckbox (18038, 15141, HelpTextArea, Window, 56, 52, 14447, 'TranslucentBlt')
+		GUIOPTControls.OptCheckbox (18038, 18004, HelpTextArea, Window, 57, 54, 14578, 'StaticAnims')
 
 	Window.ShowModal (MODAL_SHADOW_GRAY)
 	return
@@ -186,16 +185,15 @@ def SetGfxCorrection ():
 ###################################################
 
 def CloseAudioOptionsWindow ():
-	CloseSubOptionsWindow ()
+	GemRB.GetView("SUB_WIN", 0).Close()
 	TrySavingConfiguration()
 
 def OpenAudioOptionsWindow ():
 	"""Open audio options window"""
-	global SubOptionsWindow, HelpTextArea
-
-	CloseSubOptionsWindow ()
+	global HelpTextArea
 
 	Window = GemRB.LoadWindow (7, "GUIOPT")
+	Window.AddAlias("SUB_WIN", 0)
 	Window.SetFlags (WF_BORDERLESS, OP_OR)
 	HelpTextArea = GUIOPTControls.OptHelpText ('AudioOptions', Window, 14, 18040)
 
@@ -211,8 +209,6 @@ def OpenAudioOptionsWindow ():
 
 	GUIOPTControls.OptCheckbox (18040, 18022, HelpTextArea, Window, 26, 28, 20689, 'Environmental Audio')
 
-	SubOptionsWindow = Window
-
 	Window.ShowModal (MODAL_SHADOW_GRAY)
 	return
 
@@ -226,37 +222,41 @@ def DisplayHelpMusicVolume ():
 
 ###################################################
 
-def CloseCharacterSoundsWindow ():
-	CloseSubSubOptionsWindow ()
-
 def OpenCharacterSoundsWindow ():
 	"""Open character sounds window"""
 
-	global SubSubOptionsWindow, HelpTextArea2
-
-	CloseSubSubOptionsWindow ()
+	global HelpTextArea2
 
 	Window = GemRB.LoadWindow (12, "GUIOPT")
+	Window.AddAlias("SUB_WIN", 1)
 	Window.SetFlags (WF_BORDERLESS, OP_OR)
 	HelpTextArea2 = GUIOPTControls.OptHelpText ('CharacterSounds', Window, 16, 18041)
 
-	GUIOPTControls.OptDone (CloseCharacterSoundsWindow, Window, 24)
-	GUIOPTControls.OptCancel (CloseCharacterSoundsWindow, Window, 25)
+	GUIOPTControls.OptDone (Window.Close, Window, 24)
+	GUIOPTControls.OptCancel (Window.Close, Window, 25)
 
 	GUIOPTControls.OptCheckbox (18041, 18015, HelpTextArea2, Window, 5, 20, 17138, 'Subtitles')
 	GUIOPTControls.OptCheckbox (18041, 18013, HelpTextArea2, Window, 6, 18, 17139, 'Attack Sounds')
 	GUIOPTControls.OptCheckbox (18041, 18014, HelpTextArea2, Window, 7, 19, 17140, 'Footsteps')
+
+	frequency = GemRB.GetVar ('Command Sounds Frequency')
+	frequency2 = GemRB.GetVar ('Selection Sounds Frequency')
 	GUIOPTControls.OptRadio (DisplayHelpCommandSounds, Window, 8, 21, 'Command Sounds Frequency', 3)
 	GUIOPTControls.OptRadio (DisplayHelpCommandSounds, Window, 9, 21, 'Command Sounds Frequency', 2)
 	GUIOPTControls.OptRadio (DisplayHelpCommandSounds, Window, 10, 21, 'Command Sounds Frequency', 1)
 	GUIOPTControls.OptRadio (DisplayHelpSelectionSounds, Window, 58, 57, 'Selection Sounds Frequency', 3)
 	GUIOPTControls.OptRadio (DisplayHelpSelectionSounds, Window, 59, 57, 'Selection Sounds Frequency', 2)
 	GUIOPTControls.OptRadio (DisplayHelpSelectionSounds, Window, 60, 57, 'Selection Sounds Frequency', 1)
+	# work around radiobutton preselection issues
+	for i in [8, 9, 10, 58, 59, 60]:
+		Button = Window.GetControl (i)
+		if (i <= 10 and (10 - i + 1) == frequency) or (i > 10 and (60 - i + 1) == frequency2):
+			Button.SetState (IE_GUI_BUTTON_SELECTED)
+		else:
+			Button.SetState (IE_GUI_BUTTON_ENABLED)
 
 	Window.ShowModal (MODAL_SHADOW_GRAY)
 
-	SubSubOptionsWindow = Window
-
 def DisplayHelpCommandSounds ():
 	# same as HelpTextArea if not BG1
 	HelpTextArea2.SetText (18016)
@@ -270,17 +270,16 @@ def DisplayHelpSelectionSounds ():
 def CloseGameplayOptionsWindow ():
 	# FIXME: don't need to do anything, since we save stuff immediately
 	# ergo canceling does not work
-	CloseSubOptionsWindow ()
+	GemRB.GetView("SUB_WIN", 0).Close()
 	TrySavingConfiguration()
 
 def OpenGameplayOptionsWindow ():
 	"""Open gameplay options window"""
-	global SubOptionsWindow, HelpTextArea
-
-	CloseSubOptionsWindow ()
+	global HelpTextArea
 
 	#gameplayoptions
 	Window = GemRB.LoadWindow (8, "GUIOPT")
+	Window.AddAlias("SUB_WIN", 0)
 	Window.SetFlags (WF_BORDERLESS, OP_OR)
 
 	HelpTextArea = GUIOPTControls.OptHelpText ('GameplayOptions', Window, 40, 18042)
@@ -292,7 +291,7 @@ def OpenGameplayOptionsWindow ():
 	GUIOPTControls.OptSlider (18042, 18018, HelpTextArea, Window, 2, 22, 17144, 'Mouse Scroll Speed', DisplayHelpMouseScrollingSpeed, 5)
 	GUIOPTControls.OptSlider (18042, 18019, HelpTextArea, Window, 3, 23, 17145, 'Keyboard Scroll Speed', None, 5)
 
-	GUIOPTControls.OptSlider (18042, 18020, HelpTextArea, Window, 12, 24, 13911, 'Difficulty Level', None, 0)
+	GUIOPTControls.OptSlider (18042, 18020, HelpTextArea, Window, 12, 24, 13911, 'Difficulty Level', None)
 	if GemRB.GetVar ("Nightmare Mode") == 1:
 		# lock the slider
 		Slider = Window.GetControl (12)
@@ -312,13 +311,16 @@ def OpenGameplayOptionsWindow ():
 	if GameCheck.IsBG2():
 		GUIOPTControls.OptButton (OpenHotkeyOptionsWindow, Window, 51, 816)
 
-	SubOptionsWindow = Window
 	Window.ShowModal (MODAL_SHADOW_GRAY)
 	return
 
 def DisplayHelpTooltipDelay ():
 	HelpTextArea.SetText (18017)
-	GemRB.SetTooltipDelay (GemRB.GetVar ("Tooltips") * TOOLTIP_DELAY_FACTOR/10)
+	delay_var = GemRB.GetVar ("Tooltips")
+	# BG2 disables tooltips on max setting, we just set it to an extremely high value for simplicity
+	if delay_var == 100:
+		delay_var = 10000000
+	GemRB.SetTooltipDelay (delay_var * TOOLTIP_DELAY_FACTOR//10)
 
 def DisplayHelpMouseScrollingSpeed ():
 	HelpTextArea.SetText (18018)
@@ -326,19 +328,13 @@ def DisplayHelpMouseScrollingSpeed ():
 
 ###################################################
 
-def CloseFeedbackOptionsWindow ():
-	# FIXME: don't need to do anything, since we save stuff immediately
-	# ergo canceling does not work
-	CloseSubSubOptionsWindow ()
-
 def OpenFeedbackOptionsWindow ():
 	"""Open feedback options window"""
 
-	global SubSubOptionsWindow, HelpTextArea2
-
-	CloseSubSubOptionsWindow ()
+	global HelpTextArea2
 
 	Window = GemRB.LoadWindow (9, "GUIOPT")
+	Window.AddAlias("SUB_WIN", 1)
 	Window.SetFlags (WF_BORDERLESS, OP_OR)
 
 	# same as HelpTextArea if not BG1
@@ -346,8 +342,8 @@ def OpenFeedbackOptionsWindow ():
 
 	GemRB.SetVar ("Circle Feedback", GemRB.GetVar ("GUI Feedback Level") - 1)
 
-	GUIOPTControls.OptDone (CloseFeedbackOptionsWindow, Window, 26)
-	GUIOPTControls.OptCancel (CloseFeedbackOptionsWindow, Window, 27)
+	GUIOPTControls.OptDone (Window.Close, Window, 26)
+	GUIOPTControls.OptCancel (Window.Close, Window, 27)
 
 	GUIOPTControls.OptSlider (18043, 18024, HelpTextArea2, Window, 8, 30, 13688, 'Circle Feedback', DisplayHelpMarkerFeedback)
 	GUIOPTControls.OptSlider (18043, 18025, HelpTextArea2, Window, 9, 31, 17769, 'Locator Feedback Level')
@@ -364,8 +360,6 @@ def OpenFeedbackOptionsWindow ():
 
 	Window.ShowModal (MODAL_SHADOW_GRAY)
 
-	SubSubOptionsWindow = Window
-
 	return
 
 def DisplayHelpMarkerFeedback ():
@@ -375,25 +369,19 @@ def DisplayHelpMarkerFeedback ():
 
 ###################################################
 
-def CloseAutopauseOptionsWindow ():
-	# FIXME: don't need to do anything, since we save stuff immediately
-	# ergo canceling does not work
-	CloseSubSubOptionsWindow ()
-
 def OpenAutopauseOptionsWindow ():
 	"""Open autopause options window"""
 
-	global SubSubOptionsWindow, HelpTextArea2
-
-	CloseSubSubOptionsWindow ()
+	global HelpTextArea2
 
 	Window = GemRB.LoadWindow (10, "GUIOPT")
+	Window.AddAlias("SUB_WIN", 1)
 	Window.SetFlags (WF_BORDERLESS, OP_OR)
 
 	HelpTextArea2 = GUIOPTControls.OptHelpText ('AutopauseOptions', Window, 15, 18044)
 
-	GUIOPTControls.OptDone (CloseAutopauseOptionsWindow, Window, 11)
-	GUIOPTControls.OptCancel (CloseAutopauseOptionsWindow, Window, 14)
+	GUIOPTControls.OptDone (Window.Close, Window, 11)
+	GUIOPTControls.OptCancel (Window.Close, Window, 14)
 
 	# checkboxes OR the values if they associate to the same variable
 	GUIOPTControls.OptCheckbox (18044, 18032, HelpTextArea2, Window, 1, 17, 17155, 'Auto Pause State', None, 4) # hit
@@ -403,7 +391,7 @@ def OpenAutopauseOptionsWindow ():
 	GUIOPTControls.OptCheckbox (18044, 18036, HelpTextArea2, Window, 5, 21, 17159, 'Auto Pause State', None, 1) # weapon unusable
 	GUIOPTControls.OptCheckbox (18044, 18037, HelpTextArea2, Window, 13, 22, 17160, 'Auto Pause State', None, 32) # target gone
 	GUIOPTControls.OptCheckbox (18044, 10640, HelpTextArea2, Window, 25, 24, 10639, 'Auto Pause State', None, 64) # end of round
-	if GameCheck.IsIWD2():
+	if GameCheck.IsIWD2() or GameCheck.IsIWD1():
 		GUIOPTControls.OptCheckbox (18044, 23514, HelpTextArea2, Window, 30, 31, 23516, 'Auto Pause State', None, 128) # enemy sighted
 		GUIOPTControls.OptCheckbox (18044, 18560, HelpTextArea2, Window, 26, 28, 16519, 'Auto Pause State', None, 256) # trap found
 		GUIOPTControls.OptCheckbox (18044, 26311, HelpTextArea2, Window, 36, 37, 26310, 'Auto Pause State', None, 512) # spell cast
@@ -415,8 +403,6 @@ def OpenAutopauseOptionsWindow ():
 		GUIOPTControls.OptCheckbox (18044, 31872, HelpTextArea2, Window, 34, 33, 57354, 'Auto Pause State', None, 256) # trap found
 		GUIOPTControls.OptCheckbox (18044, 10571, HelpTextArea2, Window, 37, 36, 10574, 'Auto Pause Center', None, 1)
 
-	SubSubOptionsWindow = Window
-
 	Window.ShowModal (MODAL_SHADOW_GRAY)
 	return
 
@@ -429,9 +415,10 @@ def MoviePlayPress():
 # for iwd2 only, the rest use GUIMOVIE
 # TODO: investigate a merger, so it can get removed from GUIOPT
 def OpenMovieWindow ():
-	global SubOptionsWindow, TextAreaControl, MoviesTable
+	global TextAreaControl, MoviesTable
 
-	SubOptionsWindow = Window = GemRB.LoadWindow(2, "GUIMOVIE")
+	Window = GemRB.LoadWindow(2, "GUIMOVIE")
+	Window.AddAlias("SUB_WIN", 0)
 	Window.SetFlags (WF_BORDERLESS, OP_OR)
 	TextAreaControl = Window.GetControl(0)
 	PlayButton = Window.GetControl(2)
@@ -439,40 +426,19 @@ def OpenMovieWindow ():
 	DoneButton = Window.GetControl(4)
 	MoviesTable = GemRB.LoadTable("MOVIDESC")
 	opts = [MoviesTable.GetValue (i, 0) for i in range(MoviesTable.GetRowCount ()) if GemRB.GetVar(MoviesTable.GetRowName (i)) == 1]
+	TextAreaControl.SetColor (ColorWhitish, TA_COLOR_OPTIONS)
 	TextAreaControl.SetOptions (opts, "MovieIndex", 0)
 	PlayButton.SetText(17318)
 	CreditsButton.SetText(15591)
 	DoneButton.SetText(11973)
-	PlayButton.SetEvent(IE_GUI_BUTTON_ON_PRESS, MoviePlayPress)
-	CreditsButton.SetEvent(IE_GUI_BUTTON_ON_PRESS, lambda: GemRB.PlayMovie("CREDITS"))
-	DoneButton.SetEvent(IE_GUI_BUTTON_ON_PRESS, CloseSubOptionsWindow)
+	PlayButton.OnPress (MoviePlayPress)
+	CreditsButton.OnPress (lambda: GemRB.PlayMovie("CREDITS"))
+	DoneButton.OnPress (Window.Close)
 	Window.ShowModal (MODAL_SHADOW_GRAY)
 	return
 
 ###################################################
 
-def CloseSubOptionsWindow ():
-	global SubOptionsWindow, GameOptionsWindow
-
-	if SubOptionsWindow:
-		SubOptionsWindow.Unload ()
-		SubOptionsWindow = None
-	if GameOptionsWindow:
-		GameOptionsWindow.ShowModal (MODAL_SHADOW_GRAY)
-	return
-
-def CloseSubSubOptionsWindow ():
-	global SubSubOptionsWindow, SubOptionsWindow
-
-	if SubSubOptionsWindow:
-		SubSubOptionsWindow.Unload ()
-		SubSubOptionsWindow = None
-	if SubOptionsWindow:
-		SubOptionsWindow.ShowModal (MODAL_SHADOW_GRAY)
-	return
-
-###################################################
-
 def OpenSaveMsgWindow ():
 	GemRB.SetVar("QuitAfterSave",0)
 	GUISAVE.OpenSaveWindow ()
@@ -498,9 +464,9 @@ def QuitGamePress ():
 		GemRB.Quit()
 		return
 
-	GemRB.SetVar ("SelectedWindow", 0)
 	CloseQuitMsgWindow()
 
+	GUICommonWindows.CloseTopWindow ()
 	GemRB.QuitGame ()
 	GemRB.SetNextScript ("Start")
 	return
@@ -519,7 +485,7 @@ def OpenQuitMsgWindow ():
 	# Save
 	Button = Window.GetControl (0)
 	Button.SetText (15589)
-	Button.SetEvent (IE_GUI_BUTTON_ON_PRESS, SaveGamePress)
+	Button.OnPress (SaveGamePress)
 	if GemRB.GetView("GC") is not None:
 		Button.SetState (IE_GUI_BUTTON_ENABLED)
 	else:
@@ -528,13 +494,13 @@ def OpenQuitMsgWindow ():
 	# Quit Game
 	Button = Window.GetControl (1)
 	Button.SetText (15417)
-	Button.SetEvent (IE_GUI_BUTTON_ON_PRESS, QuitGamePress)
+	Button.OnPress (QuitGamePress)
 	Button.MakeDefault()
 
 	# Cancel
 	Button = Window.GetControl (2)
 	Button.SetText (GUIOPTControls.STR_OPT_CANCEL)
-	Button.SetEvent (IE_GUI_BUTTON_ON_PRESS, CloseQuitMsgWindow)
+	Button.OnPress (CloseQuitMsgWindow)
 	Button.MakeEscape()
 
 	# Do you wish to save the game ....
@@ -548,7 +514,7 @@ def CloseQuitMsgWindow ():
 	global QuitMsgWindow
 
 	if QuitMsgWindow:
-		QuitMsgWindow.Unload ()
+		QuitMsgWindow.Close ()
 	QuitMsgWindow = None
 
 	GemRB.SetVar("AskAndExit", 0)
