@@ -1,4 +1,3 @@
-# -*-python-*-
 # GemRB - Infinity Engine Emulator
 # Copyright (C) 2003-2009 The GemRB Project
 #
@@ -27,9 +26,9 @@ import Spellbook
 import CommonTables
 import LUCommon
 import LevelUp
-import GUIWORLD
 import DualClass
 import GUIRECCommon
+import PartyReform
 from GUIDefines import *
 from GUICommonWindows import CreateTopWinLoader, ToggleWindow, OpenWindowOnce, DefaultWinPos
 from ie_stats import *
@@ -42,6 +41,8 @@ ColorIndex = None
 ScriptTextArea = None
 SelectedTextArea = None
 
+SkillsTable = GemRB.LoadTable ("skills", False, True)
+
 ###################################################
 
 def InitRecordsWindow (Window):
@@ -50,38 +51,38 @@ def InitRecordsWindow (Window):
 	# dual class
 	Button = Window.GetControl (0)
 	Button.SetText (7174)
-	Button.SetEvent (IE_GUI_BUTTON_ON_PRESS, DualClass.DualClassWindow)
+	Button.OnPress (DualClass.DualClassWindow)
 
 	# levelup
 	Button = Window.GetControl (37)
 	Button.SetText (7175)
-	Button.SetEvent (IE_GUI_BUTTON_ON_PRESS, LevelUp.OpenLevelUpWindow)
+	Button.OnPress (LevelUp.OpenLevelUpWindow)
 
 	# information
 	Button = Window.GetControl (1)
 	Button.SetText (11946)
-	Button.SetEvent (IE_GUI_BUTTON_ON_PRESS, OpenInformationWindow)
+	Button.OnPress (OpenInformationWindow)
 
 	# reform party
 	Button = Window.GetControl (51)
 	Button.SetText (16559)
-	Button.SetEvent (IE_GUI_BUTTON_ON_PRESS, OpenRecReformPartyWindow)
+	Button.OnPress (OpenRecReformPartyWindow)
 
 	# customize
 	Button = Window.GetControl (50)
 	Button.SetText (10645)
-	Button.SetEvent (IE_GUI_BUTTON_ON_PRESS, GUIRECCommon.OpenCustomizeWindow)
+	Button.OnPress (GUIRECCommon.OpenCustomizeWindow)
 
 	# export
 	Button = Window.GetControl (36)
 	Button.SetText (13956)
-	Button.SetEvent (IE_GUI_BUTTON_ON_PRESS, GUIRECCommon.OpenExportWindow)
+	Button.OnPress (GUIRECCommon.OpenExportWindow)
 
 	# kit info
 	if GameCheck.IsBG2():
 		Button = Window.GetControl (52)
 		Button.SetText (61265)
-		Button.SetEvent (IE_GUI_BUTTON_ON_PRESS, OpenKitInfoWindow)
+		Button.OnPress (OpenKitInfoWindow)
 
 	return
 
@@ -89,13 +90,7 @@ def UpdateRecordsWindow (Window):
 	global alignment_help
 
 	pc = GemRB.GameGetSelectedPCSingle ()
-
-	#update mage school
-	GemRB.SetVar ("MAGESCHOOL", 0)
-	Kit = GUICommon.GetKitIndex (pc)
-	if Kit and CommonTables.KitList.GetValue (Kit, 7) == 1:
-		MageTable = GemRB.LoadTable ("magesch")
-		GemRB.SetVar ("MAGESCHOOL", MageTable.FindValue (3, CommonTables.KitList.GetValue (Kit, 6) ) )
+	GUICommon.UpdateMageSchool (pc)
 
 	# exportable
 	Button = Window.GetControl (36)
@@ -127,6 +122,8 @@ def UpdateRecordsWindow (Window):
 	Button.SetFlags (IE_GUI_BUTTON_NO_IMAGE | IE_GUI_BUTTON_PICTURE, OP_SET)
 	Button.SetState (IE_GUI_BUTTON_LOCKED)
 	pic = GemRB.GetPlayerPortrait (pc, 0)["Sprite"]
+	if not pic:
+		pic = ""
 	if GameCheck.IsBG2() and not GameCheck.IsBG2Demo():
 		Button.SetPicture (pic, "NOPORTMD")
 	else:
@@ -167,27 +164,27 @@ def UpdateRecordsWindow (Window):
 
 	Label = Window.GetControl (0x1000002f)
 	Label.SetText (sstr)
-	Label.SetTextColor (cstr)
+	Label.SetColor (cstr)
 
 	Label = Window.GetControl (0x10000009)
 	Label.SetText (sdex)
-	Label.SetTextColor (cdex)
+	Label.SetColor (cdex)
 
 	Label = Window.GetControl (0x1000000a)
 	Label.SetText (scon)
-	Label.SetTextColor (ccon)
+	Label.SetColor (ccon)
 
 	Label = Window.GetControl (0x1000000b)
 	Label.SetText (sint)
-	Label.SetTextColor (cint)
+	Label.SetColor (cint)
 
 	Label = Window.GetControl (0x1000000c)
 	Label.SetText (swis)
-	Label.SetTextColor (cwis)
+	Label.SetColor (cwis)
 
 	Label = Window.GetControl (0x1000000d)
 	Label.SetText (schr)
-	Label.SetTextColor (cchr)
+	Label.SetColor (cchr)
 
 	# class
 	ClassTitle = GUICommon.GetActorClassTitle (pc)
@@ -227,7 +224,7 @@ OpenRecordsWindow = CreateTopWinLoader(2, "GUIREC", OpenWindowOnce, InitRecordsW
 #original returns to game before continuing...
 def OpenRecReformPartyWindow ():
 	ToggleRecordsWindow()
-	GUIWORLD.OpenReformPartyWindow()
+	PartyReform.OpenReformPartyWindow()
 	return
 
 #don't allow exporting polymorphed or dead characters
@@ -246,14 +243,27 @@ def GetStatColor (pc, stat):
 		return {'r' : 255, 'g' : 255, 'b' : 0}
 	return {'r' : 0, 'g' : 255, 'b' : 0}
 
-# GemRB.GetPlayerStat wrapper that only returns nonnegative values
-def GSNN (pc, stat):
+# GemRB.GetPlayerStat wrapper that crosschecks skill availability
+SkillStatNames = { IE_PICKPOCKET : "PICK_POCKETS", IE_LOCKPICKING : "OPEN_LOCKS", IE_TRAPS : "FIND_TRAPS", IE_STEALTH : "STEALTH", IE_HIDEINSHADOWS : "HIDE_IN_SHADOWS", IE_DETECTILLUSIONS : "DETECT_ILLUSION", IE_SETTRAPS : "SET_TRAPS" }
+if GameCheck.IsBG2 ():
+	SkillStatNames[IE_STEALTH] = "MOVE_SILENTLY"
+def GetValidSkill (pc, className, stat):
 	val = GemRB.GetPlayerStat (pc, stat)
-	if val >= 0:
-		return val
-	else:
+	if val < 0:
 		return 0
 
+	if className == "BARD":
+		if stat != IE_PICKPOCKET:
+			return 0
+	elif className == "RANGER":
+		if stat != IE_STEALTH:
+			return 0
+	else:
+		if SkillsTable.GetValue (SkillStatNames[stat], className) == -1:
+			return 0
+
+	return val
+
 # shorthand wrappers for Modified/Base stat and ability bonus
 def GS (pc, stat):
 	return GemRB.GetPlayerStat (pc, stat)
@@ -424,16 +434,10 @@ def GetEffectIcons(pc,LevelDiff):
 	# but don't display them in levelup stat view
 	if sum (LevelDiff) == 0:
 		effects = GemRB.GetPlayerStates (pc)
-		print(222, effects)
 		if len (effects):
 			for c in effects:
-				c2 = c
-				if isinstance(c, int):
-					c2 = "".join(chr(x) for x in bytes([c]))
-				else:
-					c = ord(c)
 				tmp = StateTable.GetValue (str(c - 66), "DESCRIPTION")
-				stats.append ((tmp, c2, 'a'))
+				stats.append ((tmp, c, 'a'))
 	return TypeSetStats (stats, pc)
 
 ########################################################################
@@ -459,10 +463,12 @@ def GetProficiencies(pc, cdet):
 		stats.append ( (9457, str(tohit["Base"])+" ("+str(tohit["Total"])+")", '0') )
 
 	tmp = cdet["APR"]
+	tmp2 = str(tmp // 2)
 	if (tmp&1):
-		tmp2 = str (tmp // 2) + chr (189) #must use one higher than the frame count
+		tmp2 += "[+int=" + str(189) + "]" #must use one higher than the frame count
 	else:
-		tmp2 = str (tmp // 2)
+		tmp2 += " " # add some trailing padding to ensure display in bg2
+
 	stats.append ( (9458, tmp2, '') )
 	return TypeSetStats (stats, pc)
 
@@ -470,7 +476,10 @@ def GetProficiencies(pc, cdet):
 
 def GetLore(pc):
 	stats = []
-	stats.append ( (9459, GSNN (pc, IE_LORE), '0') )
+	lore = GemRB.GetPlayerStat (pc, IE_LORE)
+	if lore < 0:
+		lore = 0
+	stats.append ( (9459, lore, '0') )
 	return TypeSetStats (stats, pc)
 
 ########################################################################
@@ -495,23 +504,24 @@ def GetPartyReputation(pc):
 
 def GetSkills(pc):
 	stats = []
-	stats.append ( (9460, GSNN (pc, IE_LOCKPICKING), '') )
-	stats.append ( (9462, GSNN (pc, IE_TRAPS), '') )
-	stats.append ( (9463, GSNN (pc, IE_PICKPOCKET), '') )
-	stats.append ( (9461, GSNN (pc, IE_STEALTH), '') )
+	className = GUICommon.GetClassRowName (pc)
+	stats.append ( (9460, GetValidSkill (pc, className, IE_LOCKPICKING), '') )
+	stats.append ( (9462, GetValidSkill (pc, className, IE_TRAPS), '') )
+	stats.append ( (9463, GetValidSkill (pc, className, IE_PICKPOCKET), '') )
+	stats.append ( (9461, GetValidSkill (pc, className, IE_STEALTH), '') )
 	HatedRace = GS (pc, IE_HATEDRACE)
 	if HatedRace:
 		HateTable = GemRB.LoadTable ("haterace")
 		Racist = HateTable.FindValue (1, HatedRace)
-		if Racist != -1:
+		if Racist is not None:
 			HatedRace = HateTable.GetValue (Racist, 0, GTV_REF)
 			stats.append ( (15982, HatedRace, '') )
 
 	# these skills were new in bg2
 	if GameCheck.IsBG2() or GameCheck.IsIWD1():
-		stats.append ( (34120, GSNN (pc, IE_HIDEINSHADOWS), '') )
-		stats.append ( (34121, GSNN (pc, IE_DETECTILLUSIONS), '') )
-		stats.append ( (34122, GSNN (pc, IE_SETTRAPS), '') )
+		stats.append ( (34120, GetValidSkill (pc, className, IE_HIDEINSHADOWS), '') )
+		stats.append ( (34121, GetValidSkill (pc, className, IE_DETECTILLUSIONS), '') )
+		stats.append ( (34122, GetValidSkill (pc, className, IE_SETTRAPS), '') )
 	stats.append ( (12128, GS (pc, IE_BACKSTABDAMAGEMULTIPLIER), 'x') )
 	stats.append ( (12126, GS (pc, IE_TURNUNDEADLEVEL), '') )
 
@@ -725,8 +735,9 @@ def GetWeaponStyleBonuses(pc, cdet):
 			bonusrefs = { "THAC0BONUSRIGHT":56911, "THAC0BONUSLEFT":56910, "DAMAGEBONUS":10336, "CRITICALHITBONUS":32140, "PHYSICALSPEED":32141, "AC":10339, "ACVSMISSLE":10340 }
 			WStyleTable = GemRB.LoadTable (wstyletables[wstyle % 1000])
 			for col in range(WStyleTable.GetColumnCount()):
-				value = WStyleTable.GetValue (profcount, col)
-				stats.append ((bonusrefs[WStyleTable.GetColumnName(col)], value, ''))
+				colName = WStyleTable.GetColumnName (col);
+				value = WStyleTable.GetValue (str(profcount), colName, GTV_INT)
+				stats.append ((bonusrefs[colName], value, ''))
 		stats.append ("\n")
 	return TypeSetStats (stats, pc)
 
@@ -774,7 +785,7 @@ def TypeSetStats(stats, pc=0):
 				res.append (GemRB.GetString (strref) +': x' + str (val) )
 			elif stattype == 'a': #value (portrait icon) + string
 				# '%' is the separator glyph in the states font
-				res.append ("[cap]" + val + "%[/cap][p]" + GemRB.GetString (strref) + "[/p]")
+				res.append ("[cap][int=" + str(val) + "]%[/cap][p]" + GemRB.GetString (strref) + "[/p]")
 				noP = True
 			elif stattype == 'b': #strref is an already resolved string
 				res.append (strref+": "+str (val))
@@ -818,12 +829,12 @@ def OpenInformationWindow ():
 	# Biography
 	Button = Window.GetControl (26)
 	Button.SetText (18003)
-	Button.SetEvent (IE_GUI_BUTTON_ON_PRESS, GUIRECCommon.OpenBiographyWindow)
+	Button.OnPress (GUIRECCommon.OpenBiographyWindow)
 
 	# Done
 	Button = Window.GetControl (24)
 	Button.SetText (11973)
-	Button.SetEvent (IE_GUI_BUTTON_ON_PRESS, lambda: InformationWindow.Close())
+	Button.OnPress (InformationWindow.Close)
 	Button.MakeEscape()
 
 	TotalPartyExp = 0
@@ -923,7 +934,7 @@ def OpenKitInfoWindow ():
 	#back button (setting first, to be less error prone)
 	DoneButton = KitInfoWindow.GetControl (2)
 	DoneButton.SetText (11973)
-	DoneButton.SetEvent (IE_GUI_BUTTON_ON_PRESS, KitDonePress)
+	DoneButton.OnPress (KitDonePress)
 
 	#kit or class description
 	TextArea = KitInfoWindow.GetControl (0)
@@ -945,7 +956,7 @@ def OpenKitInfoWindow ():
 			else:
 				text = CommonTables.Classes.GetValue (GUICommon.GetClassRowName(Dual[1], "index"), "DESC_REF", GTV_REF)
 	
-			text += "\n"
+			text += "\n\n"
 			if Dual[0] == 3:
 				text += CommonTables.KitList.GetValue (Dual[2], 3, GTV_REF)
 			else:
@@ -964,11 +975,10 @@ def OpenKitInfoWindow ():
 
 def KitDonePress():
 	if KitInfoWindow:
-		KitInfoWindow.Unload()
+		KitInfoWindow.Close ()
 	return
 
 def OpenColorWindow ():
-	global PortraitWindow
 	global PaperdollButton
 	global HairButton, SkinButton, MajorButton, MinorButton
 	global HairColor, SkinColor, MajorColor, MinorColor
@@ -978,34 +988,35 @@ def OpenColorWindow ():
 	MajorColor = GemRB.GetPlayerStat (pc, IE_MAJOR_COLOR)
 	SkinColor = GemRB.GetPlayerStat (pc, IE_SKIN_COLOR)
 	HairColor = GemRB.GetPlayerStat (pc, IE_HAIR_COLOR)
-	GUIRECCommon.SubCustomizeWindow = GemRB.LoadWindow (21)
+	Window = GemRB.LoadWindow (21)
+	Window.AddAlias("SUB_WIN", 0)
 
-	PaperdollButton = GUIRECCommon.SubCustomizeWindow.GetControl (0)
+	PaperdollButton = Window.GetControl (0)
 	PaperdollButton.SetFlags (IE_GUI_BUTTON_PICTURE|IE_GUI_BUTTON_NO_IMAGE,OP_SET)
 	PaperdollButton.SetState (IE_GUI_BUTTON_LOCKED)
 
-	HairButton = GUIRECCommon.SubCustomizeWindow.GetControl (3)
-	SkinButton = GUIRECCommon.SubCustomizeWindow.GetControl (4)
-	MajorButton = GUIRECCommon.SubCustomizeWindow.GetControl (5)
-	MinorButton = GUIRECCommon.SubCustomizeWindow.GetControl (6)
+	HairButton = Window.GetControl (3)
+	SkinButton = Window.GetControl (4)
+	MajorButton = Window.GetControl (5)
+	MinorButton = Window.GetControl (6)
 
-	DoneButton = GUIRECCommon.SubCustomizeWindow.GetControl (12)
+	DoneButton = Window.GetControl (12)
 	DoneButton.SetText (11973)
 	DoneButton.MakeDefault()
 
-	CancelButton = GUIRECCommon.SubCustomizeWindow.GetControl (13)
+	CancelButton = Window.GetControl (13)
 	CancelButton.SetText (13727)
 	CancelButton.MakeEscape()
 
-	HairButton.SetEvent (IE_GUI_BUTTON_ON_PRESS, SetHairColor)
-	SkinButton.SetEvent (IE_GUI_BUTTON_ON_PRESS, SetSkinColor)
-	MajorButton.SetEvent (IE_GUI_BUTTON_ON_PRESS, SetMajorColor)
-	MinorButton.SetEvent (IE_GUI_BUTTON_ON_PRESS, SetMinorColor)
-	DoneButton.SetEvent (IE_GUI_BUTTON_ON_PRESS, DoneColorWindow)
-	CancelButton.SetEvent (IE_GUI_BUTTON_ON_PRESS, GUIRECCommon.CloseSubCustomizeWindow)
+	HairButton.OnPress (SetHairColor)
+	SkinButton.OnPress (SetSkinColor)
+	MajorButton.OnPress (SetMajorColor)
+	MinorButton.OnPress (SetMinorColor)
+	DoneButton.OnPress (DoneColorWindow)
+	CancelButton.OnPress (Window.Close)
 	UpdatePaperDoll ()
 
-	GUIRECCommon.SubCustomizeWindow.ShowModal (MODAL_SHADOW_GRAY)
+	Window.ShowModal (MODAL_SHADOW_GRAY)
 	return
 
 def DoneColorWindow ():
@@ -1014,7 +1025,7 @@ def DoneColorWindow ():
 	GemRB.SetPlayerStat (pc, IE_MAJOR_COLOR, MajorColor)
 	GemRB.SetPlayerStat (pc, IE_SKIN_COLOR, SkinColor)
 	GemRB.SetPlayerStat (pc, IE_HAIR_COLOR, HairColor)
-	GUIRECCommon.CloseSubCustomizeWindow ()
+	GemRB.GetView("SUB_WIN", 0).Close()
 	return
 
 def UpdatePaperDoll ():
@@ -1063,11 +1074,12 @@ def SetMajorColor ():
 	return
 
 def OpenColorPicker ():
-	GUIRECCommon.SubSubCustomizeWindow = GemRB.LoadWindow (22)
+	Window = GemRB.LoadWindow (22)
+	Window.AddAlias("SUB_WIN", 1)
 
 	GemRB.SetVar ("Selected",-1)
 	for i in range (1,35):
-		Button = GUIRECCommon.SubSubCustomizeWindow.GetControl (i)
+		Button = Window.GetControl (i)
 		Button.SetState (IE_GUI_BUTTON_LOCKED)
 		Button.SetFlags (IE_GUI_BUTTON_PICTURE,OP_OR)
 
@@ -1075,22 +1087,22 @@ def OpenColorPicker ():
 		MyColor = ColorTable.GetValue (ColorIndex, i)
 		if MyColor == "*":
 			break
-		Button = GUIRECCommon.SubSubCustomizeWindow.GetControl (i+1)
+		Button = Window.GetControl (i+1)
 		Button.SetBAM("COLGRAD", 2, 0, MyColor)
 		if PickedColor == MyColor:
 			GemRB.SetVar ("Selected",i)
 		Button.SetState (IE_GUI_BUTTON_ENABLED)
 		Button.SetVarAssoc("Selected",i)
-		Button.SetEvent (IE_GUI_BUTTON_ON_PRESS, DonePress)
+		Button.OnPress (DonePress)
 
-	GUIRECCommon.SubSubCustomizeWindow.ShowModal (MODAL_SHADOW_GRAY)
+	Window.ShowModal (MODAL_SHADOW_GRAY)
 	return
 
 def DonePress():
 	global HairColor, SkinColor, MajorColor, MinorColor
 	global PickedColor
 
-	GUIRECCommon.CloseSubSubCustomizeWindow ()
+	GemRB.GetView("SUB_WIN", 1).Close()
 	PickedColor=ColorTable.GetValue (ColorIndex, GemRB.GetVar ("Selected"))
 	if ColorIndex==0:
 		HairColor=PickedColor
