@@ -1,4 +1,3 @@
-# -*-python-*-
 # GemRB - Infinity Engine Emulator
 # Copyright (C) 2011 The GemRB Project
 #
@@ -26,10 +25,7 @@ from GUIDefines import *
 from ie_stats import IE_SEX, IE_RACE, IE_MC_FLAGS, MC_EXPORTABLE
 from ie_restype import RES_WAV
 
-BiographyWindow = None
 CustomizeWindow = None
-SubCustomizeWindow = None
-SubSubCustomizeWindow = None
 ExportWindow = None
 NameField = ExportDoneButton = None
 ScriptsTable = None
@@ -92,7 +88,7 @@ def OpenCustomizeWindow ():
 	if not GameCheck.IsIWD2():
 		ColorButton = CustomizeWindow.GetControl (2)
 		ColorButton.SetText (10646)
-		ColorButton.SetEvent (IE_GUI_BUTTON_ON_PRESS, GUIREC.OpenColorWindow)
+		ColorButton.OnPress (GUIREC.OpenColorWindow)
 		if not Exportable:
 			ColorButton.SetState (IE_GUI_BUTTON_DISABLED)
 
@@ -103,7 +99,7 @@ def OpenCustomizeWindow ():
 	if not (GameCheck.IsBG1() or GameCheck.IsPST()):
 		BiographyButton = CustomizeWindow.GetControl (9)
 		BiographyButton.SetText (18003)
-		BiographyButton.SetEvent (IE_GUI_BUTTON_ON_PRESS, OpenBiographyEditWindow)
+		BiographyButton.OnPress (OpenBiographyEditWindow)
 		if not Exportable:
 			BiographyButton.SetState (IE_GUI_BUTTON_DISABLED)
 
@@ -118,19 +114,20 @@ def OpenCustomizeWindow ():
 	CancelButton.SetText (13727)
 	CancelButton.MakeEscape()
 
-	PortraitSelectButton.SetEvent (IE_GUI_BUTTON_ON_PRESS, OpenPortraitSelectWindow)
-	SoundButton.SetEvent (IE_GUI_BUTTON_ON_PRESS, OpenSoundWindow)
-	ScriptButton.SetEvent (IE_GUI_BUTTON_ON_PRESS, OpenScriptWindow)
-	CustomizeDoneButton.SetEvent (IE_GUI_BUTTON_ON_PRESS, lambda: CustomizeWindow.Close())
-	CancelButton.SetEvent (IE_GUI_BUTTON_ON_PRESS,  lambda: CustomizeWindow.Close())
+	PortraitSelectButton.OnPress (OpenPortraitSelectWindow)
+	SoundButton.OnPress (OpenSoundWindow)
+	ScriptButton.OnPress (OpenScriptWindow)
+	CustomizeDoneButton.OnPress (CustomizeWindow.Close)
+	CancelButton.OnPress (CustomizeWindow.Close) # FIXME: this should revert changes I assume
 
 	CustomizeWindow.ShowModal (MODAL_SHADOW_GRAY)
 	return
 
 def OpenPortraitSelectWindow ():
-	global PortraitPictureButton, SubCustomizeWindow
+	global PortraitPictureButton
 
 	SubCustomizeWindow = GemRB.LoadWindow (18)
+	SubCustomizeWindow.AddAlias("SUB_WIN", 0)
 
 	PortraitPictureButton = SubCustomizeWindow.GetControl (0)
 	PortraitPictureButton.SetFlags (IE_GUI_BUTTON_PICTURE|IE_GUI_BUTTON_NO_IMAGE,OP_SET)
@@ -138,27 +135,27 @@ def OpenPortraitSelectWindow ():
 
 	PortraitLeftButton = SubCustomizeWindow.GetControl (1)
 	PortraitLeftButton.SetState (IE_GUI_BUTTON_ENABLED)
-	PortraitLeftButton.SetEvent (IE_GUI_BUTTON_ON_PRESS, PortraitLeftPress)
+	PortraitLeftButton.OnPress (PortraitLeftPress)
 
 	PortraitRightButton = SubCustomizeWindow.GetControl (2)
 	PortraitRightButton.SetState (IE_GUI_BUTTON_ENABLED)
-	PortraitRightButton.SetEvent (IE_GUI_BUTTON_ON_PRESS, PortraitRightPress)
+	PortraitRightButton.OnPress (PortraitRightPress)
 
 	PortraitDoneButton = SubCustomizeWindow.GetControl (3)
 	PortraitDoneButton.SetState (IE_GUI_BUTTON_ENABLED)
-	PortraitDoneButton.SetEvent (IE_GUI_BUTTON_ON_PRESS, PortraitDonePress)
+	PortraitDoneButton.OnPress (PortraitDonePress)
 	PortraitDoneButton.SetText (11973)
 	PortraitDoneButton.MakeDefault()
 
 	PortraitCancelButton = SubCustomizeWindow.GetControl (4)
 	PortraitCancelButton.SetState (IE_GUI_BUTTON_ENABLED)
-	PortraitCancelButton.SetEvent (IE_GUI_BUTTON_ON_PRESS, CloseSubCustomizeWindow)
+	PortraitCancelButton.OnPress (SubCustomizeWindow.Close)
 	PortraitCancelButton.SetText (13727)
 	PortraitCancelButton.MakeEscape()
 
 	PortraitCustomButton = SubCustomizeWindow.GetControl (5)
 	PortraitCustomButton.SetState (IE_GUI_BUTTON_ENABLED)
-	PortraitCustomButton.SetEvent (IE_GUI_BUTTON_ON_PRESS, OpenCustomPortraitWindow)
+	PortraitCustomButton.OnPress (OpenCustomPortraitWindow)
 	PortraitCustomButton.SetText (17545)
 
 	# get players gender and portrait
@@ -180,7 +177,7 @@ def PortraitDonePress ():
 		GemRB.FillPlayerInfo (pc, Portrait.Name () + "M", Portrait.Name () + "S")
 	else:
 		GemRB.FillPlayerInfo (pc, Portrait.Name () + "L", Portrait.Name () + "S")
-	CloseSubCustomizeWindow ()
+	GemRB.GetView("SUB_WIN", 0).Close()
 	return
 
 def PortraitLeftPress ():
@@ -194,59 +191,62 @@ def PortraitRightPress ():
 	PortraitPictureButton.SetPicture (Portrait.Next () + PortraitNameSuffix, "NOPORTLG")
 
 def OpenCustomPortraitWindow ():
-	global SubSubCustomizeWindow
 	global PortraitList1, PortraitList2
 	global RowCount1, RowCount2
 
-	SubSubCustomizeWindow = GemRB.LoadWindow (19)
+	Window = GemRB.LoadWindow (19)
+	Window.AddAlias("SUB_WIN", 1)
 
-	CustomPortraitDoneButton = SubSubCustomizeWindow.GetControl (10)
+	CustomPortraitDoneButton = Window.GetControl (10)
 	CustomPortraitDoneButton.SetState (IE_GUI_BUTTON_DISABLED)
-	CustomPortraitDoneButton.SetEvent (IE_GUI_BUTTON_ON_PRESS, CustomPortraitDonePress)
+	CustomPortraitDoneButton.OnPress (CustomPortraitDonePress)
 	CustomPortraitDoneButton.SetText (11973)
 	CustomPortraitDoneButton.MakeDefault()
 
-	CustomPortraitCancelButton = SubSubCustomizeWindow.GetControl (11)
+	CustomPortraitCancelButton = Window.GetControl (11)
 	CustomPortraitCancelButton.SetState (IE_GUI_BUTTON_ENABLED)
-	CustomPortraitCancelButton.SetEvent (IE_GUI_BUTTON_ON_PRESS, CloseSubSubCustomizeWindow)
+	CustomPortraitCancelButton.OnPress (Window.Close)
 	CustomPortraitCancelButton.SetText (13727)
 	CustomPortraitCancelButton.MakeEscape()
 
 	if not GameCheck.IsIWD1():
-		SmallPortraitButton = SubSubCustomizeWindow.GetControl (1)
+		SmallPortraitButton = Window.GetControl (1)
 		SmallPortraitButton.SetFlags (IE_GUI_BUTTON_PICTURE|IE_GUI_BUTTON_NO_IMAGE,OP_SET)
-		LargePortraitButton = SubSubCustomizeWindow.GetControl (0)
+		LargePortraitButton = Window.GetControl (0)
 		LargePortraitButton.SetFlags (IE_GUI_BUTTON_PICTURE|IE_GUI_BUTTON_NO_IMAGE,OP_SET)
 
 	# Portrait List Large
-	PortraitList1 = SubSubCustomizeWindow.GetControl (2)
+	PortraitList1 = Window.GetControl (2)
 	if GameCheck.IsIWD2():
 		RowCount1 = len(PortraitList1.ListResources (CHR_PORTRAITS, 2))
 	else:
 		RowCount1 = len(PortraitList1.ListResources (CHR_PORTRAITS, 1))
-	PortraitList1.SetEvent (IE_GUI_TEXTAREA_ON_SELECT, LargeCustomPortrait)
+	PortraitList1.OnSelect (LargeCustomPortrait)
 	PortraitList1.SetVarAssoc ("Row1",RowCount1)
 
 	# Portrait List Small
-	PortraitList2 = SubSubCustomizeWindow.GetControl (3)
+	PortraitList2 = Window.GetControl (3)
 	RowCount2 = len(PortraitList2.ListResources (CHR_PORTRAITS, 0))
-	PortraitList2.SetEvent (IE_GUI_TEXTAREA_ON_SELECT, SmallCustomPortrait)
+	PortraitList2.OnSelect (SmallCustomPortrait)
 	PortraitList2.SetVarAssoc ("Row2",RowCount2)
 
-	SubSubCustomizeWindow.ShowModal (MODAL_SHADOW_GRAY)
+	Window.ShowModal (MODAL_SHADOW_GRAY)
 	return
 
 def CustomPortraitDonePress ():
 	pc = GemRB.GameGetSelectedPCSingle ()
 	GemRB.FillPlayerInfo (pc, PortraitList1.QueryText () , PortraitList2.QueryText ())
-
-	CloseSubSubCustomizeWindow ()
+	
 	#closing the generic portraits, because we just set a custom one
-	CloseSubCustomizeWindow ()
+	subwin = GemRB.GetView("SUB_WIN", 0)
+	if subwin:
+		subwin.Close()
+		
+	GemRB.GetView("SUB_WIN", 1).Close()
 	return
 
 def LargeCustomPortrait ():
-	Window = SubSubCustomizeWindow
+	Window = GemRB.GetView("SUB_WIN", 1)
 
 	Portrait = PortraitList1.QueryText ()
 	#small hack
@@ -269,7 +269,7 @@ def LargeCustomPortrait ():
 	return
 
 def SmallCustomPortrait ():
-	Window = SubSubCustomizeWindow
+	Window = GemRB.GetView("SUB_WIN", 1)
 
 	Portrait = PortraitList2.QueryText ()
 	#small hack
@@ -293,11 +293,11 @@ def SmallCustomPortrait ():
 
 def OpenSoundWindow ():
 	global VoiceList, OldVoiceSet
-	global SubCustomizeWindow
 
 	pc = GemRB.GameGetSelectedPCSingle ()
 	OldVoiceSet = GemRB.GetPlayerSound (pc)
 	SubCustomizeWindow = GemRB.LoadWindow (20)
+	SubCustomizeWindow.AddAlias("SUB_WIN", 0)
 
 	VoiceList = SubCustomizeWindow.GetControl (5)
 	Voices = VoiceList.ListResources (CHR_SOUNDS)
@@ -328,9 +328,9 @@ def OpenSoundWindow ():
 	CancelButton.SetText (13727)
 	CancelButton.MakeEscape()
 
-	PlayButton.SetEvent (IE_GUI_BUTTON_ON_PRESS, PlaySoundPressed)
-	DoneButton.SetEvent (IE_GUI_BUTTON_ON_PRESS, DoneSoundWindow)
-	CancelButton.SetEvent (IE_GUI_BUTTON_ON_PRESS, CloseSoundWindow)
+	PlayButton.OnPress (PlaySoundPressed)
+	DoneButton.OnPress (DoneSoundWindow)
+	CancelButton.OnPress (CloseSoundWindow)
 
 	SubCustomizeWindow.ShowModal (MODAL_SHADOW_GRAY)
 	return
@@ -338,7 +338,7 @@ def OpenSoundWindow ():
 def CloseSoundWindow ():
 	pc = GemRB.GameGetSelectedPCSingle ()
 	GemRB.SetPlayerSound (pc, OldVoiceSet)
-	CloseSubCustomizeWindow ()
+	GemRB.GetView("SUB_WIN", 0).Close()
 	return
 
 def DoneSoundWindow ():
@@ -347,7 +347,7 @@ def DoneSoundWindow ():
 	CharSound = GUICommon.OverrideDefaultVoiceSet (Gender, CharSound)
 	GemRB.SetPlayerSound (pc, CharSound)
 
-	CloseSubCustomizeWindow ()
+	GemRB.GetView("SUB_WIN", 0).Close()
 	return
 
 def PlaySoundPressed():
@@ -392,11 +392,11 @@ def FindScriptFile(selected):
 
 options = {}
 def OpenScriptWindow ():
-	global SubCustomizeWindow
 	global ScriptTextArea, SelectedTextArea
 	global options
 
 	SubCustomizeWindow = GemRB.LoadWindow (11)
+	SubCustomizeWindow.AddAlias("SUB_WIN", 0)
 
 	ScriptTextArea = SubCustomizeWindow.GetControl (2)
 	scripts = ScriptTextArea.ListResources (CHR_SCRIPTS)
@@ -421,7 +421,7 @@ def OpenScriptWindow ():
 		if scriptindex == -1:
 			# custom
 			GemRB.SetToken ("script", script)
-			options[(script, idx)] = GemRB.GetString (17167)
+			options[(script, idx)] = GemRB.GetString (17167) + "\n"
 		else:
 			title = ScriptsTable.GetValue (scriptindex, 0, GTV_REF)
 			if title:
@@ -429,7 +429,7 @@ def OpenScriptWindow ():
 				txt = title
 				if desc:
 					txt += " " + desc.rstrip()
-				options[(script, idx)] = txt + "\n"*1
+				options[(script, idx)] = txt + "\n"*2
 			else:
 				# none
 				idx = idx - 1
@@ -441,16 +441,20 @@ def OpenScriptWindow ():
 	if script == None:
 		script = "None"
 
-	scriptindex = ScriptsTable.GetRowIndex (script)
+	scriptindex = 0
+	for (filename, idx) in options:
+		if filename == script:
+			scriptindex = idx
+			break
 
 	SelectedTextArea = SubCustomizeWindow.GetControl (4)
 
-	def UpdateScriptSelection(ta, val):
-		name = FindScriptFile (val)
-		SelectedTextArea.SetText (options[(name, val)])
+	def UpdateScriptSelection(ta):
+		name = FindScriptFile (ta.Value)
+		SelectedTextArea.SetText (options[(name, ta.Value)])
 		return
 	
-	ScriptTextArea.SetEvent (IE_GUI_TEXTAREA_ON_SELECT, UpdateScriptSelection)
+	ScriptTextArea.OnSelect (UpdateScriptSelection)
 	ScriptTextArea.SetVarAssoc("Selected", scriptindex)
 
 	DoneButton = SubCustomizeWindow.GetControl (5)
@@ -461,8 +465,8 @@ def OpenScriptWindow ():
 	CancelButton.SetText (13727)
 	CancelButton.MakeEscape()
 
-	DoneButton.SetEvent (IE_GUI_BUTTON_ON_PRESS, DoneScriptWindow)
-	CancelButton.SetEvent (IE_GUI_BUTTON_ON_PRESS, CloseSubCustomizeWindow)
+	DoneButton.OnPress (DoneScriptWindow)
+	CancelButton.OnPress (SubCustomizeWindow.Close)
 
 	SubCustomizeWindow.ShowModal (MODAL_SHADOW_GRAY)
 	return
@@ -472,7 +476,7 @@ def DoneScriptWindow ():
 	selected = GemRB.GetVar ("Selected")
 	script = FindScriptFile (selected)
 	GemRB.SetPlayerScript (pc, script)
-	CloseSubCustomizeWindow ()
+	GemRB.GetView("SUB_WIN", 0).Close()
 	return
 
 def RevertBiography(ta):
@@ -491,7 +495,6 @@ def RevertBiography(ta):
 	return
 
 def OpenBiographyEditWindow ():
-	global SubCustomizeWindow
 	global RevertButton
 	global BioStrRef
 	global TextArea
@@ -507,6 +510,8 @@ def OpenBiographyEditWindow ():
 		SubCustomizeWindow = GemRB.LoadWindow (51)
 	else:
 		SubCustomizeWindow = GemRB.LoadWindow (23)
+		
+	SubCustomizeWindow.AddAlias("SUB_WIN", 0)
 
 	ClearButton = SubCustomizeWindow.GetControl (5)
 	if GameCheck.IsBG2():
@@ -535,10 +540,10 @@ def OpenBiographyEditWindow ():
 	TextArea.SetText (BioStrRef)
 	TextArea.Focus ()
 
-	ClearButton.SetEvent (IE_GUI_BUTTON_ON_PRESS, lambda: ClearBiography(TextArea))
-	DoneButton.SetEvent (IE_GUI_BUTTON_ON_PRESS, lambda: DoneBiographyWindow(TextArea))
-	RevertButton.SetEvent (IE_GUI_BUTTON_ON_PRESS, lambda: RevertBiography(TextArea))
-	CancelButton.SetEvent (IE_GUI_BUTTON_ON_PRESS, CloseSubCustomizeWindow)
+	ClearButton.OnPress (lambda: ClearBiography(TextArea))
+	DoneButton.OnPress (lambda: DoneBiographyWindow(TextArea))
+	RevertButton.OnPress (lambda: RevertBiography(TextArea))
+	CancelButton.OnPress (SubCustomizeWindow.Close)
 
 	SubCustomizeWindow.ShowModal (MODAL_SHADOW_GRAY)
 	return
@@ -562,15 +567,13 @@ def DoneBiographyWindow (ta):
 	if BioStrRef != 33347:
 		GemRB.CreateString (BioStrRef, ta.QueryText())
 	GemRB.SetPlayerString (pc, BioStrRefSlot, BioStrRef)
-	CloseSubCustomizeWindow ()
+	GemRB.GetView("SUB_WIN", 0).Close()
 	return
 
 def OpenBiographyWindow ():
-	global BiographyWindow
+	BiographyWindow = GemRB.LoadWindow (12)
 
-	BiographyWindow = Window = GemRB.LoadWindow (12)
-
-	TextArea = Window.GetControl (0)
+	TextArea = BiographyWindow.GetControl (0)
 	pc = GemRB.GameGetSelectedPCSingle ()
 	if GameCheck.IsBG1 () and GemRB.GetPlayerName (pc, 2) == 'none':
 		TextArea.SetText (GetProtagonistBiography (pc))
@@ -578,26 +581,11 @@ def OpenBiographyWindow ():
 		TextArea.SetText (GemRB.GetPlayerString (pc, BioStrRefSlot))
 
 	# Done
-	Button = Window.GetControl (2)
+	Button = BiographyWindow.GetControl (2)
 	Button.SetText (11973)
-	Button.SetEvent (IE_GUI_BUTTON_ON_PRESS, CloseBiographyWindow)
+	Button.OnPress (BiographyWindow.Close)
 
-	Window.ShowModal (MODAL_SHADOW_GRAY)
-	return
-
-def CloseBiographyWindow ():
-	import GUIREC
-	global BiographyWindow
-
-	if BiographyWindow:
-		BiographyWindow.Unload ()
-		BiographyWindow = None
-	# TODO: check if bg1 wouldn't benefit from modality too
-	if GUIREC.InformationWindow:
-		if GameCheck.IsBG2() or GameCheck.IsIWD1():
-			GUIREC.InformationWindow.ShowModal (MODAL_SHADOW_GRAY)
-		else:
-			GUIREC.InformationWindow.Focus()
+	BiographyWindow.ShowModal (MODAL_SHADOW_GRAY)
 	return
 
 def GetProtagonistBiography (pc):
@@ -632,9 +620,9 @@ def OpenExportWindow ():
 
 	NameField = ExportWindow.GetControl (6)
 
-	ExportDoneButton.SetEvent (IE_GUI_BUTTON_ON_PRESS, ExportDonePress)
-	CancelButton.SetEvent (IE_GUI_BUTTON_ON_PRESS, ExportCancelPress)
-	NameField.SetEvent (IE_GUI_EDIT_ON_CHANGE, ExportEditChanged)
+	ExportDoneButton.OnPress (ExportDonePress)
+	CancelButton.OnPress (ExportCancelPress)
+	NameField.OnChange (ExportEditChanged)
 	ExportWindow.ShowModal (MODAL_SHADOW_GRAY)
 	NameField.Focus()
 	return
@@ -644,13 +632,13 @@ def ExportDonePress():
 	pc = GemRB.GameGetSelectedPCSingle ()
 	GemRB.SaveCharacter (pc, NameField.QueryText ())
 	if ExportWindow:
-		ExportWindow.Unload()
+		ExportWindow.Close ()
 	return
 
 def ExportCancelPress():
 	global ExportWindow
 	if ExportWindow:
-		ExportWindow.Unload()
+		ExportWindow.Close ()
 		ExportWindow = None
 	return
 
@@ -661,23 +649,3 @@ def ExportEditChanged():
 	else:
 		ExportDoneButton.SetState (IE_GUI_BUTTON_ENABLED)
 	return
-
-def CloseSubCustomizeWindow ():
-	global SubCustomizeWindow, CustomizeWindow
-
-	if SubCustomizeWindow:
-		SubCustomizeWindow.Unload ()
-		SubCustomizeWindow = None
-	if CustomizeWindow:
-		CustomizeWindow.ShowModal (MODAL_SHADOW_GRAY)
-	return
-
-def CloseSubSubCustomizeWindow ():
-	global SubSubCustomizeWindow, SubCustomizeWindow
-
-	if SubSubCustomizeWindow:
-		SubSubCustomizeWindow.Unload ()
-		SubSubCustomizeWindow = None
-	if SubCustomizeWindow:
-		SubCustomizeWindow.ShowModal (MODAL_SHADOW_GRAY)
-	return
