@@ -1,4 +1,3 @@
-# -*-python-*-
 # GemRB - Infinity Engine Emulator
 # Copyright (C) 2010 The GemRB Project
 #
@@ -17,7 +16,10 @@
 # Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 #
 
+import re
+
 import GemRB
+import CommonTables
 import GameCheck
 import GUICommon
 import Spellbook
@@ -41,11 +43,13 @@ SlotMap = None
 
 UpdateInventoryWindow = None
 
-def OnDragItemGround (btn, slot):
+def OnDragItemGround (btn):
 	"""Drops and item to the ground."""
 	
 	pc = GemRB.GameGetSelectedPCSingle ()
-	slot = slot + GemRB.GetVar ("TopIndex") - (47 if GameCheck.IsPST () else 68)
+	slot = btn.Value
+
+	GemRB.GetView ("MsgSys").SetText ("")
 
 	if GemRB.IsDraggingItem ()==0:
 		slot_item = GemRB.GetContainerItem (pc, slot)
@@ -76,19 +80,24 @@ def OnAutoEquip ():
 	UpdateInventoryWindow ()
 	return
 
-def OnDragItem (btn, slot):
+def OnDragItem (btn):
 	"""Updates dragging."""
 
 	#don't call when splitting items
 	if ItemAmountWindow != None:
 		return
 
+	slot = btn.Value
 	pc = GemRB.GameGetSelectedPCSingle ()
 	slot_item = GemRB.GetSlotItem (pc, slot)
-	
+
+	GemRB.GetView ("MsgSys").SetText ("")
+
 	if not GemRB.IsDraggingItem ():
 		item = GemRB.GetItem (slot_item["ItemResRef"])
 		GemRB.DragItem (pc, slot, item["ItemIcon"], 0, 0)
+		if slot == 2: # reset disguise
+			GemRB.SetGlobal ("APPEARANCE", "GLOBAL", 0)
 	else:
 		SlotType = GemRB.GetSlotType (slot, pc)
 		#special monk check
@@ -120,7 +129,16 @@ def OnDragItem (btn, slot):
 					#leave (save) store
 					GemRB.LeaveStore()
 
+			item = GemRB.GetSlotItem (0, 0)
+			itemData = GemRB.GetItem (item['ItemResRef'])
 			GemRB.DropDraggedItem (pc, slot)
+
+			# handle pst disguises
+			if slot == 2 and item['ItemResRef'] == 'dustrobe':
+				GemRB.SetGlobal ("APPEARANCE", "GLOBAL", 2)
+			elif slot < 21 and itemData['AnimationType'] != '':
+				GemRB.SetGlobal ("APPEARANCE", "GLOBAL", 0)
+
 			# drop item if it caused us to disable the inventory view (example: cursed berserking sword)
 			if GemRB.GetPlayerStat (pc, IE_STATE_ID) & (STATE_BERSERK) and GemRB.IsDraggingItem ():
 				GemRB.DropDraggedItem (pc, -3)
@@ -133,6 +151,9 @@ def OnDropItemToPC (pc):
 
 	if pc > GemRB.GetPartySize ():
 		return
+
+	GemRB.GetView ("MsgSys").SetText ("")
+
 	#-3 : drop stuff in inventory (but not equippable slots)
 	GemRB.DropDraggedItem (pc, -3)
 	UpdateInventoryWindow ()
@@ -160,7 +181,7 @@ def IncreaseStackAmount ():
 	Text.SetText (str (number))
 	return
 
-def DragItemAmount ():
+def DragItemAmount (slot_item, location = "inventory"):
 	"""Drag a split item."""
 
 	pc = GemRB.GameGetSelectedPCSingle ()
@@ -170,35 +191,33 @@ def DragItemAmount ():
 		GemRB.DropDraggedItem (pc, UsedSlot)
 		UpdateSlot (pc, UsedSlot-1)
 
-	slot_item = GemRB.GetSlotItem (pc, UsedSlot)
-
 	#if dropping didn't help, don't die if slot_item isn't here
 	if slot_item:
 		Text = ItemAmountWindow.GetControl (6)
-		Amount = Text.QueryText ()
+		Amount = Text.QueryInteger ()
 		item = GemRB.GetItem (slot_item["ItemResRef"])
-		GemRB.DragItem (pc, UsedSlot, item["ItemIcon"], int ("0"+Amount), 0)
+		GemRB.DragItem (pc, UsedSlot, item["ItemIcon"], Amount, location == "ground")
 	ItemAmountWindow.Close()
 	return
 
-def MouseEnterSlot (btn, slot):
+def MouseEnterSlot (Button):
 	pc = GemRB.GameGetSelectedPCSingle ()
 
 	if GemRB.IsDraggingItem ()==1:
-		drag_item = GemRB.GetSlotItem (0,0)
-		SlotType = UpdateSlot (pc, slot-1)
+		drag_item = GemRB.GetSlotItem (0, 0)
+		SlotType = UpdateSlot (pc, Button.Value - 1)
 
 		if GemRB.CanUseItemType (SlotType["Type"], drag_item["ItemResRef"]):
-			btn.SetState (IE_GUI_BUTTON_SELECTED)
+			Button.SetState (IE_GUI_BUTTON_SELECTED)
 		else:
-			btn.SetState (IE_GUI_BUTTON_ENABLED)
+			Button.SetState (IE_GUI_BUTTON_ENABLED)
 		
 	return
 
-def MouseLeaveSlot (btn, slot):
+def MouseLeaveSlot (Button):
 	pc = GemRB.GameGetSelectedPCSingle ()
 
-	UpdateSlot (pc, slot-1)
+	UpdateSlot (pc, Button.Value - 1)
 	return
 
 def MouseEnterGround (Button):
@@ -213,10 +232,26 @@ def MouseLeaveGround (Button):
 
 def CloseItemInfoWindow ():
 	if ItemInfoWindow:
-		ItemInfoWindow.Unload ()
+		ItemInfoWindow.Close ()
 	UpdateInventoryWindow ()
 	return
 
+def GetItemDescription (item, itemtype):
+	text = item["ItemDescIdentified"]
+	if (itemtype & 2):
+		text = item["ItemDesc"]
+
+	if not GameCheck.IsPST ():
+		return text
+
+	# PST had "NOTE:" handling hardcoded, see string 49156 for explanation
+	# we support translated versions visible at least in Czech as well (POZN)
+	text = GemRB.GetString (text)
+	searchRE = re.compile(r'^([A-Z][A-Z][A-Z][A-Z]+[: ]..[a-z"].*?($|\r?\n(\r?\n)?))', re.MULTILINE | re.DOTALL)
+	replacement = r"[color=ffffff]\1[/color]"
+	text = searchRE.sub(replacement, text)
+	return text
+
 def DisplayItem (slotItem, itemtype):
 	global ItemInfoWindow
 
@@ -260,20 +295,15 @@ def DisplayItem (slotItem, itemtype):
 	#middle button
 	Button = Window.GetControl (4)
 	Button.SetText (strrefs[0])
-	Button.SetEvent (IE_GUI_BUTTON_ON_PRESS, CloseItemInfoWindow)
+	Button.OnPress (CloseItemInfoWindow)
 	Button.MakeDefault()
 
 	#textarea
 	Text = Window.GetControl (5)
 	if GameCheck.IsBG2(): # I believe only BG2 has special initials
 		Text.SetColor (ColorWhitish, TA_COLOR_INITIALS)
-	if (itemtype & 2):
-		text = item["ItemDesc"]
-	else:
-		text = item["ItemDescIdentified"]
-
-	Text.Clear ()
-	Text.Append (text)
+	text = GetItemDescription (item, itemtype)
+	Text.SetText (text)
 	
 	Window.SetEventProxy(Text)
 
@@ -283,17 +313,17 @@ def DisplayItem (slotItem, itemtype):
 
 	if itemtype & 2:
 		Button.SetText (strrefs[1])
-		Button.SetEvent (IE_GUI_BUTTON_ON_PRESS, IdentifyItemWindow)
+		Button.OnPress (IdentifyItemWindow)
 		Button.SetFlags (IE_GUI_BUTTON_PICTURE, OP_SET)
 	elif select and not GameCheck.IsPST():
 		Button.SetText (strrefs[2])
-		Button.SetEvent (IE_GUI_BUTTON_ON_PRESS, AbilitiesItemWindow)
+		Button.OnPress (AbilitiesItemWindow)
 		Button.SetFlags (IE_GUI_BUTTON_PICTURE, OP_SET)
 	else:
 		Button.SetText ("")
 		Button.SetState (IE_GUI_BUTTON_LOCKED)
 		Button.SetFlags (IE_GUI_BUTTON_NO_IMAGE, OP_SET)
-		Button.SetEvent (IE_GUI_BUTTON_ON_PRESS, None)
+		Button.OnPress (None)
 
 	# description icon (not present in iwds)
 	if not GameCheck.IsIWD1() and not GameCheck.IsIWD2():
@@ -310,13 +340,18 @@ def DisplayItem (slotItem, itemtype):
 	Button.SetFlags (IE_GUI_BUTTON_PICTURE, OP_SET)
 	drink = (itemtype & 1) and (item["Function"]&ITM_F_DRINK)
 	read = (itemtype & 1) and (item["Function"]&ITM_F_READ)
-	# sorcerers cannot learn spells
+	# only mages and bards can learn spells
 	pc = GemRB.GameGetSelectedPCSingle ()
-	if Spellbook.HasSorcererBook (pc):
+	ClassName = GUICommon.GetClassRowName (pc)
+	SpellBookType = CommonTables.ClassSkills.GetValue (ClassName, "MAGESPELL", GTV_STR)
+	if SpellBookType == "*" or SpellBookType == "MXSPLSRC":
 		read = 0
+	# furthermore there might be school exclusions present
+	if read:
+		read = GemRB.CanUseItemType (SLOT_ALL, slotItem['ItemResRef'], pc, False)
 	container = (itemtype & 1) and (item["Function"]&ITM_F_CONTAINER)
 	dialog = (itemtype & 1) and (item["Dialog"]!="" and item["Dialog"]!="*")
-	familiar = (itemtype & 1) and (item["Type"] == 38)
+	familiar = (itemtype & 1) and (item["Type"] == 38) and (item["Flags"] & IE_INV_ITEM_UNDROPPABLE)
 
 	# The "conversable" bit in PST actually means "usable", eg clot charm
 	# unlike BG2 (which only has the bit set on SW2H14 Lilarcor)
@@ -338,10 +373,10 @@ def DisplayItem (slotItem, itemtype):
 	if drink and not dialog:
 		# Standard consumable item
 		Button.SetText (strrefs[3])
-		Button.SetEvent (IE_GUI_BUTTON_ON_PRESS, ConsumeItem)
+		Button.OnPress (ConsumeItem)
 	elif read:
 		Button.SetText (strrefs[4])
-		Button.SetEvent (IE_GUI_BUTTON_ON_PRESS, ReadItemWindow)
+		Button.OnPress (ReadItemWindow)
 	elif container:
 		# Just skip the redundant info page and go directly to the container
 		if GemRB.GetVar("GUIEnhancements")&GE_ALWAYS_OPEN_CONTAINER_ITEMS:
@@ -354,7 +389,7 @@ def DisplayItem (slotItem, itemtype):
 		else:
 			# a fallback, since the originals have nothing appropriate from not having any bags
 			Button.SetText ("Open container")
-		Button.SetEvent (IE_GUI_BUTTON_ON_PRESS, OpenItemWindow)
+		Button.OnPress (OpenItemWindow)
 	elif dialog:
 		if drink:
 			# Dialog item that is 'used'
@@ -362,17 +397,17 @@ def DisplayItem (slotItem, itemtype):
 		else:
 			# Dialog item that is 'talked to'
 			Button.SetText (strrefs[5])
-		Button.SetEvent (IE_GUI_BUTTON_ON_PRESS, DialogItemWindow)
+		Button.OnPress (DialogItemWindow)
 	elif familiar and not GameCheck.IsPST():
 		# PST earings share a type with familiars, so no
 		# mods that allow familiars would be possible in PST
 		Button.SetText (4373)
-		Button.SetEvent (IE_GUI_BUTTON_ON_PRESS, ReleaseFamiliar)
+		Button.OnPress (ReleaseFamiliar)
 	else:
 		Button.SetState (IE_GUI_BUTTON_LOCKED)
 		Button.SetFlags (IE_GUI_BUTTON_NO_IMAGE, OP_SET)
 		Button.SetText ("")
-		Button.SetEvent (IE_GUI_BUTTON_ON_PRESS, None)
+		Button.OnPress (None)
 
 	Label = Window.GetControl(0x1000000b)
 	if Label:
@@ -387,11 +422,11 @@ def DisplayItem (slotItem, itemtype):
 
 		#left scroll
 		Button = Window.GetControl (13)
-		Button.SetEvent (IE_GUI_BUTTON_ON_PRESS, lambda: CycleDisplayItem(-1))
+		Button.OnPress (lambda: CycleDisplayItem(-1))
 
 		#right scroll
 		Button = Window.GetControl (14)
-		Button.SetEvent (IE_GUI_BUTTON_ON_PRESS, lambda: CycleDisplayItem(1))
+		Button.OnPress (lambda: CycleDisplayItem(1))
 
 	ItemInfoWindow.ShowModal(MODAL_SHADOW_GRAY)
 	return
@@ -418,9 +453,9 @@ def CycleDisplayItem(direction):
 		GemRB.SetVar('ItemButton', slot)
 
 	if slot_item:
-		OpenItemInfoWindow (None, slot)
+		OpenItemInfoWindow (slot)
 
-def OpenItemInfoWindow (btn, slot):
+def OpenItemInfoWindow (slot):
 	pc = GemRB.GameGetSelectedPCSingle ()
 
 	slotItem = GemRB.GetSlotItem (pc, slot)
@@ -451,11 +486,11 @@ def TryAutoIdentification(pc, item, slot, slot_item, enabled=0):
 		return True
 	return False
 
-def OpenGroundItemInfoWindow (btn, slot):
+def OpenGroundItemInfoWindow (btn):
 	global ItemInfoWindow
 
 	pc = GemRB.GameGetSelectedPCSingle ()
-	slot = slot + GemRB.GetVar ("TopIndex") - (47 if GameCheck.IsPST () else 68)
+	slot = btn.Value
 	slot_item = GemRB.GetContainerItem (pc, slot)
 
 	#the ground items are only displayable
@@ -466,10 +501,9 @@ def OpenGroundItemInfoWindow (btn, slot):
 	DisplayItem(slot_item, value)
 	return
 
-# TODO: implement, reuse OpenItemAmountWindow, but be careful about any other uses of ItemButton
-def OpenGroundItemAmountWindow ():
-	pass
-	
+def OpenGroundItemAmountWindow (btn):
+	OpenItemAmountWindow (btn, "ground")
+
 def ItemAmountWindowClosed(win):
 	global ItemAmountWindow, UsedSlot
 
@@ -477,8 +511,7 @@ def ItemAmountWindowClosed(win):
 	UsedSlot = None
 	UpdateInventoryWindow()
 
-# TODO: can the GUISTORE be consolidate with this one?
-def OpenItemAmountWindow (btn, slot):
+def OpenItemAmountWindow (btn, location = "inventory"):
 	"""Open the split window."""
 
 	global UsedSlot, OverSlot
@@ -486,7 +519,7 @@ def OpenItemAmountWindow (btn, slot):
 
 	pc = GemRB.GameGetSelectedPCSingle ()
 
-	UsedSlot = slot
+	UsedSlot = btn.Value
 	if GemRB.IsDraggingItem ()==1:
 		GemRB.DropDraggedItem (pc, UsedSlot)
 		#redraw slot
@@ -495,7 +528,10 @@ def OpenItemAmountWindow (btn, slot):
 		if GemRB.IsDraggingItem () == 1:
 			return
 
-	slot_item = GemRB.GetSlotItem (pc, UsedSlot)
+	if location == "inventory":
+		slot_item = GemRB.GetSlotItem (pc, UsedSlot)
+	else:
+		slot_item = GemRB.GetContainerItem (pc, UsedSlot)
 
 	if slot_item:
 		StackAmount = slot_item["Usages0"]
@@ -522,8 +558,6 @@ def OpenItemAmountWindow (btn, slot):
 
 	# item amount
 	Text = Window.GetControl (6)
-	# FIXME: use a proper size
-	# FIXME: fix it for all the games
 	if GameCheck.IsIWD2():
 		Text.SetSize (40, 40)
 	Text.SetText (str (StackAmount//2))
@@ -532,24 +566,24 @@ def OpenItemAmountWindow (btn, slot):
 
 	# Decrease
 	Button = Window.GetControl (4)
-	Button.SetEvent (IE_GUI_BUTTON_ON_PRESS, DecreaseStackAmount)
+	Button.OnPress (DecreaseStackAmount)
 	Button.SetActionInterval (200)
 
 	# Increase
 	Button = Window.GetControl (3)
-	Button.SetEvent (IE_GUI_BUTTON_ON_PRESS, IncreaseStackAmount)
+	Button.OnPress (IncreaseStackAmount)
 	Button.SetActionInterval (200)
 
 	# Done
 	Button = Window.GetControl (2)
 	Button.SetText (strings['Done'])
-	Button.SetEvent (IE_GUI_BUTTON_ON_PRESS, DragItemAmount)
+	Button.OnPress (lambda: DragItemAmount(slot_item, location))
 	Button.MakeDefault()
 
 	# Cancel
 	Button = Window.GetControl (1)
 	Button.SetText (strings['Cancel'])
-	Button.SetEvent (IE_GUI_BUTTON_ON_PRESS, lambda: Window.Close())
+	Button.OnPress (Window.Close)
 	Button.MakeEscape()
 
 	Window.ShowModal (MODAL_SHADOW_GRAY)
@@ -614,12 +648,12 @@ def UpdateSlot (pc, slot):
 
 	if slot_item:
 		Button.SetAction(OnDragItem, IE_ACT_DRAG_DROP_CRT)
-		Button.SetEvent (IE_GUI_BUTTON_ON_PRESS, OnDragItem)
-		Button.SetEvent (IE_GUI_BUTTON_ON_RIGHT_PRESS, OpenItemInfoWindow)
-		Button.SetEvent (IE_GUI_BUTTON_ON_SHIFT_PRESS, OpenItemAmountWindow)
+		Button.OnPress (OnDragItem)
+		Button.OnRightPress (lambda: OpenItemInfoWindow (slot + 1))
+		Button.OnShiftPress (OpenItemAmountWindow)
 		#If the slot is being used to display the 'default' weapon, disable dragging.
 		if SlotType["ID"] == 10 and using_fists:
-			Button.SetEvent (IE_GUI_BUTTON_ON_PRESS, None)
+			Button.OnPress (None)
 			#dropping is ok, because it will drop in the quick weapon slot and not the default weapon slot.
 	else:
 		if SlotType["ResRef"]=="*":
@@ -644,10 +678,10 @@ def UpdateSlot (pc, slot):
 			Button.SetTooltip ("")
 			itemname = ""
 
-		Button.SetEvent (IE_GUI_BUTTON_ON_PRESS, None)
-		Button.SetEvent (IE_GUI_BUTTON_ON_RIGHT_PRESS, None)
-		Button.SetEvent (IE_GUI_BUTTON_ON_SHIFT_PRESS, None)
-		Button.SetEvent (IE_GUI_BUTTON_ON_DOUBLE_PRESS, OpenItemAmountWindow)
+		Button.OnPress (None)
+		Button.OnRightPress (None)
+		Button.OnShiftPress (None)
+		Button.OnDoublePress (OpenItemAmountWindow)
 
 	if (SlotType["Type"]&SLOT_INVENTORY) or not GemRB.CanUseItemType (SlotType["Type"], itemname):
 		Button.SetState (IE_GUI_BUTTON_ENABLED)
@@ -662,7 +696,7 @@ def UpdateSlot (pc, slot):
 def CancelColor():
 	global ColorPicker
 	if ColorPicker:
-		ColorPicker.Unload ()
+		ColorPicker.Close ()
 	InventoryWindow = GemRB.GetView ("WIN_INV")
 	InventoryWindow.Focus ()
 	return
@@ -673,7 +707,7 @@ def ColorDonePress():
 	pc = GemRB.GameGetSelectedPCSingle ()
 
 	if ColorPicker:
-		ColorPicker.Unload ()
+		ColorPicker.Close ()
 
 	ColorTable = GemRB.LoadTable ("clowncol")
 	PickedColor=ColorTable.GetValue (ColorIndex, GemRB.GetVar ("Selected"))
@@ -738,7 +772,7 @@ def GetColor():
 	GemRB.SetVar ("Selected",-1)
 	if GameCheck.IsIWD2 () or GameCheck.IsGemRBDemo ():
 		Button = ColorPicker.GetControl (35)
-		Button.SetEvent (IE_GUI_BUTTON_ON_PRESS, CancelColor)
+		Button.OnPress (CancelColor)
 		Button.SetText (103)
 		if GameCheck.IsIWD2 ():
 			Button.SetText (13727)
@@ -758,7 +792,7 @@ def GetColor():
 			Button.SetFlags (IE_GUI_BUTTON_PICTURE|IE_GUI_BUTTON_RADIOBUTTON, OP_OR)
 			Button.SetState (IE_GUI_BUTTON_ENABLED)
 		Button.SetVarAssoc ("Selected",i)
-		Button.SetEvent (IE_GUI_BUTTON_ON_PRESS, ColorDonePress)
+		Button.OnPress (ColorDonePress)
 	ColorPicker.Focus()
 	return
 
@@ -794,7 +828,7 @@ def OpenErrorWindow (strref):
 		Button.SetText (1403)
 	else:
 		Button.SetText (11973)
-	Button.SetEvent (IE_GUI_BUTTON_ON_PRESS, CloseErrorWindow)
+	Button.OnPress (CloseErrorWindow)
 	Button.MakeDefault()
 
 	TextArea = Window.GetControl (3)
@@ -804,7 +838,7 @@ def OpenErrorWindow (strref):
 
 def CloseErrorWindow ():
 	if ErrorWindow:
-		ErrorWindow.Unload ()
+		ErrorWindow.Close ()
 	UpdateInventoryWindow ()
 	return
 
@@ -845,19 +879,30 @@ def ReadItemWindow ():
 	if ret == LSR_OK:
 		GemRB.PlaySound ("GAM_44") # success!
 		if GameCheck.IsPST():
-			strref = 4249
+			strref = GetPSTPersonalizedRef (pc, 35645) # 4249 - "You succeeded in learning this spell!" / 35645 - "Spell has been copied"
+			GemRB.GetString (strref, 2) # play the attached sound
 		else:
 			strref = 10830
 	else:
 		GemRB.PlaySound ("EFF_M10") # failure!
 		if GameCheck.IsPST():
-			strref = 4250
+			strref = GetPSTPersonalizedRef (pc, 35629) # 4250 - "You failed to learn this spell!", but 35629 - "Spell has failed to have been copied" has sounds
+			GemRB.GetString (strref, 2) # play the attached sound
 		else:
 			strref = 10831
 
 	CloseItemInfoWindow ()
 	OpenErrorWindow (strref)
 
+def GetPSTPersonalizedRef(pc, baseRef):
+	"""Calculate strref based on base strref and party member"""
+
+	# figure out PC index (TNO, Morte, Annah, Dakkon, FFG, Nordom, Ignus, Vhailor)
+	# the original compared script names, but specifics should be fine
+	spec2offset = [ 0, 7, 5, 6, 4, 3, 2, 1 ]
+	pcOffset = spec2offset[GemRB.GetPlayerStat (pc, IE_SPECIFIC) - 2]
+	return baseRef + pcOffset
+
 def OpenItemWindow ():
 	"""Displays information about the item."""
 
@@ -865,7 +910,7 @@ def OpenItemWindow ():
 	GemRB.SetVar ("Inventory", 1)
 	slot = GemRB.GetVar ("ItemButton") #get this before closing win
 	if ItemInfoWindow:
-		ItemInfoWindow.Unload ()
+		ItemInfoWindow.Close ()
 
 	pc = GemRB.GameGetSelectedPCSingle ()
 	slot_item = GemRB.GetSlotItem (pc, slot)
@@ -886,7 +931,7 @@ def DialogItemWindow ():
 	item = GemRB.GetItem (ResRef)
 	dialog=item["Dialog"]
 	if ItemInfoWindow:
-		ItemInfoWindow.Unload ()
+		ItemInfoWindow.Close ()
 
 	GemRB.ExecuteString ("StartDialogOverride(\""+dialog+"\",Myself,0,0,1)", pc)
 	return
@@ -899,13 +944,17 @@ def IdentifyUseSpell ():
 	pc = GemRB.GameGetSelectedPCSingle ()
 	slot = GemRB.GetVar ("ItemButton")
 	if ItemIdentifyWindow:
-		ItemIdentifyWindow.Unload ()
+		ItemIdentifyWindow.Close ()
 	GemRB.HasSpecialSpell (pc, SP_IDENTIFY, 1)
 	if ItemInfoWindow:
-		ItemInfoWindow.Unload ()
+		ItemInfoWindow.Close ()
 	GemRB.ChangeItemFlag (pc, slot, IE_INV_ITEM_IDENTIFIED, OP_OR)
-	GemRB.PlaySound(DEF_IDENTIFY)
-	OpenItemInfoWindow(None, slot)
+	if GameCheck.IsPST ():
+		strRef = GetPSTPersonalizedRef (pc, 35685)
+		GemRB.GetString (strRef, 2) # play the attached sound
+	else:
+		GemRB.PlaySound (DEF_IDENTIFY)
+	OpenItemInfoWindow(slot)
 	return
 
 def IdentifyUseScroll ():
@@ -916,20 +965,24 @@ def IdentifyUseScroll ():
 	pc = GemRB.GameGetSelectedPCSingle ()
 	slot = GemRB.GetVar ("ItemButton")
 	if ItemIdentifyWindow:
-		ItemIdentifyWindow.Unload ()
+		ItemIdentifyWindow.Close ()
 	if ItemInfoWindow:
-		ItemInfoWindow.Unload ()
+		ItemInfoWindow.Close ()
 	if GemRB.HasSpecialItem (pc, 1, 1):
 		GemRB.ChangeItemFlag (pc, slot, IE_INV_ITEM_IDENTIFIED, OP_OR)
-	GemRB.PlaySound(DEF_IDENTIFY)
-	OpenItemInfoWindow(None, slot)
+	if GameCheck.IsPST ():
+		strRef = GetPSTPersonalizedRef (pc, 35685)
+		GemRB.GetString (strRef, 2) # play the attached sound
+	else:
+		GemRB.PlaySound (DEF_IDENTIFY)
+	OpenItemInfoWindow(slot)
 	return
 
 def CloseIdentifyItemWindow ():
 	global ItemIdentifyWindow, ItemInfoWindow
 
 	if ItemIdentifyWindow:
-		ItemIdentifyWindow.Unload ()
+		ItemIdentifyWindow.Close ()
 		ItemIdentifyWindow = None
 	if ItemInfoWindow:
 		ItemInfoWindow.ShowModal (MODAL_SHADOW_GRAY)
@@ -946,7 +999,7 @@ def IdentifyItemWindow ():
 		Button.SetText (4259)
 	else:
 		Button.SetText (17105)
-	Button.SetEvent (IE_GUI_BUTTON_ON_PRESS, IdentifyUseSpell)
+	Button.OnPress (IdentifyUseSpell)
 	if not GemRB.HasSpecialSpell (pc, SP_IDENTIFY, 0):
 		Button.SetState (IE_GUI_BUTTON_DISABLED)
 
@@ -955,7 +1008,7 @@ def IdentifyItemWindow ():
 		Button.SetText (4260)
 	else:
 		Button.SetText (17106)
-	Button.SetEvent (IE_GUI_BUTTON_ON_PRESS, IdentifyUseScroll)
+	Button.OnPress (IdentifyUseScroll)
 	if not GemRB.HasSpecialItem (pc, 1, 0):
 		Button.SetState (IE_GUI_BUTTON_DISABLED)
 
@@ -964,7 +1017,7 @@ def IdentifyItemWindow ():
 		Button.SetText (4196)
 	else:
 		Button.SetText (13727)
-	Button.SetEvent (IE_GUI_BUTTON_ON_PRESS, CloseIdentifyItemWindow)
+	Button.OnPress (CloseIdentifyItemWindow)
 	Button.MakeEscape()
 
 	TextArea = Window.GetControl (3)
@@ -986,7 +1039,7 @@ def CloseAbilitiesItemWindow ():
 	global ItemAbilitiesWindow, ItemInfoWindow
 
 	if ItemAbilitiesWindow:
-		ItemAbilitiesWindow.Unload ()
+		ItemAbilitiesWindow.Close ()
 		ItemAbilitiesWindow = None
 	if ItemInfoWindow:
 		ItemInfoWindow.ShowModal (MODAL_SHADOW_GRAY)
@@ -1006,7 +1059,7 @@ def AbilitiesItemWindow ():
 	GemRB.SetVar ("Ability", slot_item["Header"])
 	for i in range(3):
 		Button = Window.GetControl (i+1)
-		if GameCheck.IsBG2(): # TODO: check pst
+		if GameCheck.IsBG2():
 			Button.SetSprites ("GUIBTBUT",i,0,1,2,0)
 		Button.SetFlags (IE_GUI_BUTTON_RADIOBUTTON, OP_OR)
 		Button.SetVarAssoc ("Ability",i)
@@ -1025,12 +1078,12 @@ def AbilitiesItemWindow ():
 
 	Button = Window.GetControl (7)
 	Button.SetText (11973)
-	Button.SetEvent (IE_GUI_BUTTON_ON_PRESS, DoneAbilitiesItemWindow)
+	Button.OnPress (DoneAbilitiesItemWindow)
 	Button.MakeDefault()
 
 	Button = Window.GetControl (10)
 	Button.SetText (13727)
-	Button.SetEvent (IE_GUI_BUTTON_ON_PRESS, CloseAbilitiesItemWindow)
+	Button.OnPress (CloseAbilitiesItemWindow)
 	Button.MakeEscape()
 	Window.ShowModal (MODAL_SHADOW_GRAY)
 	return
@@ -1040,10 +1093,9 @@ def UpdateInventorySlot (pc, Button, Slot, Type, Equipped=False):
 
 	color = {'r' : 128, 'g' : 128, 'b' : 255, 'a' : 64}
 	Button.SetBorder (0, color, 0,1)
-	color = {'r' : 32, 'g' : 32, 'b' : 255, 'a' : 255}
-	Button.SetBorder (1, color, 0,0, Button.GetInsetFrame(2))
-	color = {'r' : 255, 'g' : 128, 'b' : 128, 'a' : 64}
-	Button.SetBorder (2, color, 0,1)
+	colorUnusable = {'r' : 255, 'g' : 128, 'b' : 128, 'a' : 64}
+	Button.SetBorder (2, colorUnusable, 0, 1)
+	colorUMD = {'r' : 255, 'g' : 255, 'b' : 0, 'a' : 64}
 
 	Button.SetText ("")
 	Button.SetFlags (IE_GUI_BUTTON_ALIGN_RIGHT | IE_GUI_BUTTON_ALIGN_BOTTOM | IE_GUI_BUTTON_PICTURE, OP_OR)
@@ -1055,40 +1107,42 @@ def UpdateInventorySlot (pc, Button, Slot, Type, Equipped=False):
 			tooltips = { "inventory": 82, "ground": 83, "container": "" }
 		Button.SetTooltip (tooltips[Type])
 		Button.EnableBorder (0, 0)
-		Button.EnableBorder (1, 0)
 		Button.EnableBorder (2, 0)
+		return
+
+	item = GemRB.GetItem (Slot['ItemResRef'])
+	identified = Slot["Flags"] & IE_INV_ITEM_IDENTIFIED
+	magical = item["Enchantment"] > 0
+
+	# MaxStackAmount holds the *maximum* item count in the stack while Usages0 holds the actual
+	if item["MaxStackAmount"] > 1:
+		Button.SetText (str (Slot["Usages0"]))
+
+	# auto-identify mundane items; the actual indentification will happen on transfer
+	if not identified and item["LoreToID"] == 0:
+		identified = True
+
+	if not identified or item["ItemNameIdentified"] == -1:
+		Button.SetTooltip (item["ItemName"])
+		Button.EnableBorder (0, 1)
 	else:
-		item = GemRB.GetItem (Slot['ItemResRef'])
-		identified = Slot["Flags"] & IE_INV_ITEM_IDENTIFIED
-		magical = item["Enchantment"] > 0
+		Button.SetTooltip (item["ItemNameIdentified"])
+		Button.EnableBorder (0, 0)
 
-		# MaxStackAmount holds the *maximum* item count in the stack while Usages0 holds the actual
-		if item["MaxStackAmount"] > 1:
-			Button.SetText (str (Slot["Usages0"]))
+	usable = GemRB.CanUseItemType (SLOT_ALL, Slot['ItemResRef'], pc, Equipped)
+	if usable:
+		# enable yellow overlay for "use magical device"
+		if usable & 0x100000:
+			Button.SetBorder (2, colorUMD, 1, 1)
 		else:
-			Button.SetText ("")
-
-		# auto-identify mundane items; the actual indentification will happen on transfer
-		if not identified and item["LoreToID"] == 0:
-			identified = True
-
-		if not identified or item["ItemNameIdentified"] == -1:
-			Button.SetTooltip (item["ItemName"])
-			Button.EnableBorder (0, 1)
-			Button.EnableBorder (1, 0)
-		else:
-			Button.SetTooltip (item["ItemNameIdentified"])
-			Button.EnableBorder (0, 0)
-			if magical and not GameCheck.IsPST():
-				Button.EnableBorder (1, 1)
-			else:
-				Button.EnableBorder (1, 0)
-
-		if GemRB.CanUseItemType (SLOT_ALL, Slot['ItemResRef'], pc, Equipped):
 			Button.EnableBorder (2, 0)
-		else:
-			Button.EnableBorder (2, 1)
+	else:
+		Button.SetBorder (2, colorUnusable, 1, 1)
 
+	if magical and GameCheck.IsIWD2 ():
+		Button.SetFlags (IE_GUI_BUTTON_HORIZONTAL, OP_OR)
+		Button.SetItemIcon (Slot['ItemResRef'], 0, 0, 0, "", "STORTIN3")
+	else:
+		Button.SetFlags (IE_GUI_BUTTON_HORIZONTAL, OP_NAND)
 		Button.SetItemIcon (Slot['ItemResRef'], 0)
-
 	return
