@@ -1,4 +1,3 @@
-# -*-python-*-
 # GemRB - Infinity Engine Emulator
 # Copyright (C) 2009 The GemRB Project
 #
@@ -98,6 +97,9 @@ def CanLevelUp(actor):
 	Multi = GUICommon.IsMultiClassed (actor, 1)
 	Dual = GUICommon.IsDualClassed (actor, 1)
 
+	if Class == "":
+		return 0
+
 	# get all the levels and overall xp here
 	xp = GemRB.GetPlayerStat (actor, IE_XP)
 	Levels = [GemRB.GetPlayerStat (actor, IE_LEVEL), GemRB.GetPlayerStat (actor, IE_LEVEL2),\
@@ -109,8 +111,8 @@ def CanLevelUp(actor):
 	if GameCheck.IsIWD2():
 		import GUIREC
 		levelsum = GemRB.GetPlayerStat (actor, IE_CLASSLEVELSUM)
-		next = GUIREC.GetNextLevelExp(levelsum, GUIREC.GetECL(actor))
-		return next <= xp
+		nextXP = GUIREC.GetNextLevelExp (levelsum, GUIREC.GetECL (actor))
+		return nextXP <= xp
 
 	# hardcoded special case to handle TNO, who is usually a single class 
 	# but with three separate Levels/XP values and the ability to switch between them
@@ -386,7 +388,7 @@ def SetupHP (pc, Level=None, LevelDiff=None):
 	ClassName = None
 	if Kit and not Dual[0] and Multi[0]<2:
 		KitName = CommonTables.KitList.GetValue (Kit, 0, GTV_STR)
-		if CommonTables.Classes.GetRowIndex (KitName) >= 0:
+		if CommonTables.Classes.GetRowIndex (KitName) != None:
 			ClassName = KitName
 
 	# determine the minimum hp roll
@@ -459,8 +461,7 @@ def SetupHP (pc, Level=None, LevelDiff=None):
 
 	#update our hp values
 	GemRB.SetPlayerStat (pc, IE_MAXHITPOINTS, CurrentHP+OldHP)
-	# HACK: account also for the new constitution bonus for the current hitpoints
-	GemRB.SetPlayerStat (pc, IE_HITPOINTS, GemRB.GetPlayerStat (pc, IE_HITPOINTS, 1)+CurrentHP+5)
+	GemRB.SetPlayerStat (pc, IE_HITPOINTS, CurrentHP + GemRB.GetPlayerStat (pc, IE_HITPOINTS))
 	return
 
 def ApplyFeats(MyChar):
