@@ -1,4 +1,3 @@
-# -*-python-*-
 # GemRB - Infinity Engine Emulator
 # Copyright (C) 2011 The GemRB Project
 #
@@ -52,16 +51,17 @@ def GetUsableMemorizedSpells(actor, BookType):
 			if not Spell0["Flags"]:
 				# depleted, so skip
 				continue
-			if Spell0["SpellResRef"] in spellResRefs:
+			resref = Spell0["SpellResRef"].lower()
+			if resref in spellResRefs:
 				# add another one, so we can get the count more cheaply later
-				spellResRefs.append (Spell0["SpellResRef"])
+				spellResRefs.append (resref)
 				continue
-			spellResRefs.append (Spell0["SpellResRef"])
-			Spell = GemRB.GetSpell(Spell0["SpellResRef"])
+			spellResRefs.append (resref)
+			Spell = GemRB.GetSpell(resref)
 			Spell['BookType'] = BookType # just another sorting key
 			Spell['SpellIndex'] = GemRB.GetSpelldataIndex (actor, Spell["SpellResRef"], 1<<BookType) # crucial!
 			if Spell['SpellIndex'] == -1:
-				GemRB.Log (LOG_ERROR, "GetUsableMemorizedSpells", "Memorized spell not found! " + Spell["SpellResRef"] + " of type " + 1<<BookType)
+				GemRB.Log (LOG_ERROR, "GetUsableMemorizedSpells", "Memorized spell not found! " + Spell["SpellResRef"] + " of type " + str(1<<BookType))
 			Spell['SpellIndex'] += 1000 * 1<<BookType
 			memorizedSpells.append (Spell)
 
@@ -71,10 +71,11 @@ def GetUsableMemorizedSpells(actor, BookType):
 	# count and remove the duplicates
 	memorizedSpells2 = []
 	for spell in memorizedSpells:
-		if spell["SpellResRef"] in spellResRefs:
-			spell['MemoCount'] = spellResRefs.count(spell["SpellResRef"])
-			while spell["SpellResRef"] in spellResRefs:
-				spellResRefs.remove(spell["SpellResRef"])
+		resref = spell["SpellResRef"].lower()
+		if resref in spellResRefs:
+			spell['MemoCount'] = spellResRefs.count(resref)
+			while resref in spellResRefs:
+				spellResRefs.remove(resref)
 			memorizedSpells2.append(spell)
 
 	return memorizedSpells2
@@ -194,6 +195,12 @@ def SortUsableSpells(memorizedSpells):
 def SetupSpellIcons(Window, BookType, Start=0, Offset=0):
 	actor = GemRB.GameGetFirstSelectedActor ()
 
+	# bardsongs weren't saved in iwd1, so learn them now if needed
+	if GameCheck.IsIWD1 () and BookType == (1 << IE_SPELL_TYPE_SONG) and HasSpell (actor, IE_SPELL_TYPE_SONG, 0, "SPIN151") == -1:
+		import GUICommon
+		level = GemRB.GetPlayerStat (actor, IE_LEVEL)
+		GUICommon.AddClassAbilities (actor, "clabbard", level, level)
+
 	# check if we're dealing with a temporary spellbook
 	if GemRB.GetVar("ActionLevel") == UAW_2DASPELLS:
 		allSpells = GetSpellinfoSpells (actor, BookType)
@@ -248,7 +255,7 @@ def SetupSpellIcons(Window, BookType, Start=0, Offset=0):
 	buttonCount = 12 - More * 2 # GUIBT_COUNT in PCStatsStruct
 	for i in range (buttonCount):
 		Button = Window.GetControl (i+Offset+More)
-		Button.SetEvent (IE_GUI_BUTTON_ON_RIGHT_PRESS, None)
+		Button.OnRightPress (None)
 
 		if i+Start >= len(memorizedSpells):
 			Button.SetState (IE_GUI_BUTTON_DISABLED)
@@ -272,13 +279,17 @@ def SetupSpellIcons(Window, BookType, Start=0, Offset=0):
 		specialSpell = (specialSpell & SP_IDENTIFY) or ((specialSpell & SP_SURGE) and actionLevel == UAW_ALLMAGE)
 		if specialSpell & SP_SILENCE and Spell['HeaderFlags'] & 0x2000000: # SF_IGNORES_SILENCE
 			specialSpell ^= SP_SILENCE
-		if specialSpell or (disabled_spellcasting&spellType):
+
+		disabled = disabled_spellcasting & spellType
+		if disabled_spellcasting == 16: # ignore type
+			disabled = not (Spell["HeaderFlags"] & 0x4000) # SF_HLA
+		if specialSpell or disabled:
 			Button.SetState (IE_GUI_BUTTON_DISABLED)
 			Button.EnableBorder(1, 0)
 		else:
 			Button.SetState (IE_GUI_BUTTON_UNPRESSED)
-			Button.SetEvent (IE_GUI_BUTTON_ON_PRESS, GUICommonWindows.SpellPressed)
-			Button.SetEvent (IE_GUI_BUTTON_ON_SHIFT_PRESS, GUICommonWindows.SpellShiftPressed)
+			Button.OnPress (GUICommonWindows.SpellPressed)
+			Button.OnShiftPress (GUICommonWindows.SpellShiftPressed)
 
 		if Spell['SpellResRef']:
 			Button.SetSprites ("guibtbut", 0, 0,1,2,3)
@@ -372,7 +383,8 @@ def GetMageSpells (Kit, Alignment, Level, baseClass = -1):
 		return GetIWD2Spells (Kit, Usability, Level, baseClass)
 
 	SpellsTable = GemRB.LoadTable ("spells")
-	for i in range(SpellsTable.GetValue ("MAGE", str(Level), GTV_INT)):
+	SpellCount = SpellsTable.GetValue ("MAGE", str(Level), GTV_INT)
+	for i in range(SpellCount):
 		SpellName = "SPWI%d%02d"%(Level,i+1)
 		ms = GemRB.GetSpell (SpellName, 1)
 		if ms == None:
@@ -413,7 +425,7 @@ def GetLearnableDomainSpells (pc, Level, baseClassName = -1):
 	BaseClassIndex = CommonTables.Classes.GetRowIndex (baseClassName)
 	# columns correspond to kits in the same order
 	KitIndex = GUICommonWindows.GetKitIndex (pc, BaseClassIndex)
-	if KitIndex == -1:
+	if KitIndex is None:
 		print("GetLearnableDomainSpells: couldn't determine the kit, bailing out!")
 		return Learnable
 	# calculate the offset from the first cleric kit
@@ -543,7 +555,7 @@ def CannotLearnSlotSpell ():
 		booktype = IE_IWD2_SPELL_WIZARD
 
 	slot_item = GemRB.GetSlotItem (pc, GemRB.GetVar ("ItemButton"))
-	spell_ref = GemRB.GetItem (slot_item['ItemResRef'], pc)['Spell']
+	spell_ref = GemRB.GetItem (slot_item['ItemResRef'])['Spell']
 	spell = GemRB.GetSpell (spell_ref)
 	level = spell['SpellLevel']
 
@@ -565,6 +577,17 @@ def CannotLearnSlotSpell ():
 
 	return 0
 
+def LearnPriestSpell (pc, spell, bookType, level):
+	if GameCheck.IsIWD2():
+		if bookType == IE_IWD2_SPELL_DOMAIN:
+			GemRB.LearnSpell (pc, spell, 0, 1 << bookType, level)
+		else:
+			# perhaps forcing would be fine here too, but it's untested and
+			# iwd2 cleric schools grant certain spells at different levels
+			GemRB.LearnSpell (pc, spell, 0, 1 << bookType)
+	else:
+		GemRB.LearnSpell (pc, spell)
+
 def LearnPriestSpells (pc, level, mask, baseClassName = -1):
 	"""Learns all the priest spells through the given spell level.
 
@@ -590,15 +613,7 @@ def LearnPriestSpells (pc, level, mask, baseClassName = -1):
 		for spell in learnable:
 			# if the spell isn't learned, learn it
 			if HasSpell (pc, booktype, i, spell) < 0:
-				if GameCheck.IsIWD2():
-					if booktype == IE_IWD2_SPELL_DOMAIN:
-						GemRB.LearnSpell (pc, spell, 0, 1<<booktype, i)
-					else:
-						# perhaps forcing would be fine here too, but it's untested and
-						# iwd2 cleric schools grant certain spells at different levels
-						GemRB.LearnSpell (pc, spell, 0, 1<<booktype)
-				else:
-					GemRB.LearnSpell (pc, spell)
+				LearnPriestSpell (pc, spell, booktype, i)
 	return
 
 
@@ -686,7 +701,7 @@ def LearnSpell(pc, spellref, booktype, level, count, flags=0):
 	if SpellIndex < 0:
 		ret = GemRB.LearnSpell (pc, spellref, flags, booktype)
 		if ret != LSR_OK and ret != LSR_KNOWN:
-			raise RuntimeError ("Failed learning spell: " + str(spellref))
+			raise RuntimeError ("Failed ({}) learning spell: {}".format(ret, spellref))
 
 		SpellIndex = HasSpell (pc, booktype, level, spellref)
 		count -= 1
@@ -696,14 +711,14 @@ def LearnSpell(pc, spellref, booktype, level, count, flags=0):
 
 	if SpellIndex == -1:
 		# should never happen
-		raise RuntimeError("LearnSpell: Severe spellbook problems: %s" + str(spellref))
+		raise RuntimeError("LearnSpell: Severe spellbook problems: " + str(spellref))
 
 	for j in range(count):
 		GemRB.MemorizeSpell (pc, booktype, level, SpellIndex, flags&LS_MEMO)
 
 def LearnFromScroll (pc, slot):
 	slot_item = GemRB.GetSlotItem (pc, slot)
-	spell_ref = GemRB.GetItem (slot_item['ItemResRef'], pc)['Spell']
+	spell_ref = GemRB.GetItem (slot_item['ItemResRef'])['Spell']
 
 	ret = GemRB.LearnSpell (pc, spell_ref, LS_STATS|LS_ADDXP)
 
