@@ -1,4 +1,3 @@
-# -*-python-*-
 # GemRB - Infinity Engine Emulator
 # Copyright (C) 2003-2005 The GemRB Project
 #
@@ -20,6 +19,7 @@
 ###################################################
 
 import GemRB
+import GUICommon
 import MessageWindow
 from ie_restype import RES_2DA
 from GameCheck import MAX_PARTY_SIZE
@@ -27,18 +27,16 @@ from GameCheck import MAX_PARTY_SIZE
 def EnterGame():
 	GemRB.GameSetPartySize(MAX_PARTY_SIZE)
 	GemRB.GameSetProtagonistMode(1)
+	GUICommon.SetSaveDir ()
 
 	MessageWindow.OnLoad()
 
 #upgrade savegame to next version
 #this is not necessary if TotSC was already installed before starting the game
 def GameExpansion():
-	if not HasTOTSC():
-		return
-
-	#reload world map if it doesn't have AR1000
-	GemRB.UpdateWorldMap("WORLDMAP", "AR1000")
-	return
+	if HasTOTSC():
+		#reload world map if it doesn't have AR1000
+		GemRB.UpdateWorldMap("WORLDMAP", "AR1000")
 
 def HasTOTSC ():
 	return GemRB.HasResource ("toscst", RES_2DA)
