@@ -1,4 +1,3 @@
-# -*-python-*-
 # GemRB - Infinity Engine Emulator
 # Copyright (C) 2003-2005 The GemRB Project
 #
@@ -22,12 +21,14 @@
 ###################################################
 
 import GemRB
+import GUICommon
 import MessageWindow
 from GameCheck import MAX_PARTY_SIZE
 
 def EnterGame():
 	GemRB.GameSetPartySize(MAX_PARTY_SIZE)
 	GemRB.GameSetProtagonistMode (2)
+	GUICommon.SetSaveDir ()
 
 	MessageWindow.OnLoad()
 	GemRB.GamePause (0, 0)
