@@ -1,4 +1,3 @@
-# -*-python-*-
 # GemRB - Infinity Engine Emulator
 # Copyright (C) 2003-2004 The GemRB Project
 #
@@ -35,7 +34,6 @@ from ie_stats import *
 from ie_restype import *
 from ie_feats import FEAT_ARMORED_ARCANA
 
-SelectWindow = 0
 Topic = None
 HelpTable = None
 DescTable = None
@@ -62,7 +60,7 @@ def Exportable(pc):
 	return True
 
 def InitRecordsWindow (Window):
-	global RecordsWindow, SelectWindow
+	global RecordsWindow
 	global BonusSpellTable, HateRaceTable
 
 	RecordsWindow = Window
@@ -75,57 +73,57 @@ def InitRecordsWindow (Window):
 	#information (help files)
 	Button = Window.GetControl (1)
 	Button.SetText (11946)
-	Button.SetEvent (IE_GUI_BUTTON_ON_PRESS, OpenHelpWindow)
+	Button.OnPress (OpenHelpWindow)
 
 	#biography
 	Button = Window.GetControl (59)
 	Button.SetText (18003)
-	Button.SetEvent (IE_GUI_BUTTON_ON_PRESS, GUIRECCommon.OpenBiographyWindow)
+	Button.OnPress (GUIRECCommon.OpenBiographyWindow)
 
 	#export
 	Button = Window.GetControl (36)
 	Button.SetText (13956)
-	Button.SetEvent (IE_GUI_BUTTON_ON_PRESS, GUIRECCommon.OpenExportWindow)
+	Button.OnPress (GUIRECCommon.OpenExportWindow)
 
 	#customize
 	Button = Window.GetControl (50)
 	Button.SetText (10645)
-	Button.SetEvent (IE_GUI_BUTTON_ON_PRESS, GUIRECCommon.OpenCustomizeWindow)
+	Button.OnPress (GUIRECCommon.OpenCustomizeWindow)
 
 	#general
-	GemRB.SetVar ("SelectWindow", 1)
-
 	Button = Window.GetControl (60)
 	Button.SetTooltip (40316)
 	Button.SetVarAssoc ("SelectWindow", 1)
 	Button.SetFlags (IE_GUI_BUTTON_RADIOBUTTON, OP_OR)
-	Button.SetEvent (IE_GUI_BUTTON_ON_PRESS, lambda: UpdateRecordsWindow(Window))
+	Button.OnPress (lambda: UpdateRecordsWindow(Window))
 
 	#weapons and armour
 	Button = Window.GetControl (61)
 	Button.SetTooltip (40317)
 	Button.SetVarAssoc ("SelectWindow", 2)
 	Button.SetFlags (IE_GUI_BUTTON_RADIOBUTTON, OP_OR)
-	Button.SetEvent (IE_GUI_BUTTON_ON_PRESS, lambda: UpdateRecordsWindow(Window))
+	Button.OnPress (lambda: UpdateRecordsWindow(Window))
 
 	#skills and feats
 	Button = Window.GetControl (62)
 	Button.SetTooltip (40318)
 	Button.SetVarAssoc ("SelectWindow", 3)
 	Button.SetFlags (IE_GUI_BUTTON_RADIOBUTTON, OP_OR)
-	Button.SetEvent (IE_GUI_BUTTON_ON_PRESS, lambda: UpdateRecordsWindow(Window))
+	Button.OnPress (lambda: UpdateRecordsWindow(Window))
 
 	#miscellaneous
 	Button = Window.GetControl (63)
 	Button.SetTooltip (33500)
 	Button.SetVarAssoc ("SelectWindow", 4)
 	Button.SetFlags (IE_GUI_BUTTON_RADIOBUTTON, OP_OR)
-	Button.SetEvent (IE_GUI_BUTTON_ON_PRESS, lambda: UpdateRecordsWindow(Window))
+	Button.OnPress (lambda: UpdateRecordsWindow(Window))
+
+	GemRB.SetVar ("SelectWindow", 1)
 
 	#level up
 	Button = Window.GetControl (37)
 	Button.SetText (7175)
-	Button.SetEvent (IE_GUI_BUTTON_ON_PRESS, OpenLevelUpWindow)
+	Button.OnPress (OpenLevelUpWindow)
 
 	return
 
@@ -138,14 +136,14 @@ def ColorDiff (Window, Label, diff):
 	else:
 		Color = {'r' : 255, 'g' : 255, 'b' : 255}
 
-	Label.SetTextColor (Color)
+	Label.SetColor (Color)
 	return
 
 def ColorDiff2 (Window, Label, diff):
 	if diff:
-		Label.SetTextColor ({'r' : 255, 'g' : 255, 'b' : 0})
+		Label.SetColor ({'r' : 255, 'g' : 255, 'b' : 0})
 	else:
-		Label.SetTextColor ({'r' : 255, 'g' : 255, 'b' : 255})
+		Label.SetColor ({'r' : 255, 'g' : 255, 'b' : 255})
 	return
 
 def GetBonusSpells (pc):
@@ -211,13 +209,13 @@ def DisplayFavouredEnemy (pc, RangerLevel, second, RecordsTextArea):
 		RaceID = GemRB.GetPlayerStat(pc, IE_HATEDRACE2+second)
 	if RaceID:
 		FavouredIndex = HateRaceTable.FindValue (1, RaceID)
-		if FavouredIndex == -1:
+		if FavouredIndex is None:
 			return
 		FavouredName = HateRaceTable.GetValue (FavouredIndex, 0)
 		if second == -1:
-			RecordsTextArea.Append (DelimitedText(FavouredName, PlusMinusStat((RangerLevel+4)/5), 0))
+			RecordsTextArea.Append (DelimitedText(FavouredName, PlusMinusStat((RangerLevel + 4) // 5), 0))
 		else:
-			RecordsTextArea.Append (DelimitedText(FavouredName, PlusMinusStat((RangerLevel+4)/5-second-1), 0))
+			RecordsTextArea.Append (DelimitedText(FavouredName, PlusMinusStat((RangerLevel + 4) // 5 - second - 1), 0))
 
 def GetFavoredClass (pc, code):
 	if GemRB.GetPlayerStat (pc, IE_SEX)==1:
@@ -341,12 +339,8 @@ def DisplayGeneral (pc, targetTextArea):
 		RecordsTextArea.Append ("\n[color=ffff00]" + GemRB.GetString(32052) + "[/color]\n")
 		StateTable = GemRB.LoadTable ("statdesc")
 		for c in effects:
-			c2 = str(c)
-			if isinstance(c, str):
-				c2 = c
-				c = ord(c)
 			tmp = StateTable.GetValue (str(c - 66), "DESCRIPTION", GTV_REF)
-			RecordsTextArea.Append ("[cap]" + c2 + "%[/cap][p]" + tmp + "[/p]")
+			RecordsTextArea.Append ("[cap][int=" + str(c) + "]%[/cap][p]" + tmp + "[/p]")
 
 	# TODO: Active Feats (eg. Power attack 4)
 
@@ -756,26 +750,26 @@ def DisplayWeapons (pc):
 		failure = GemRB.GetSpellFailure (pc)
 		total = failure["Total"]
 		arcana = 5*GemRB.HasFeat(pc, FEAT_ARMORED_ARCANA)
-		other = 5*(failure["Armor"] + failure["Shield"]) - arcana
+		armor = 5 * (failure["Armor"] + failure["Shield"])
+		other = armor - arcana
 		# account for armored arcana only if it doesn't cause a negative value (eating into the misc bonus)
-		verbose = 0
 		if other < 0:
 			other = total
 		else:
 			other = total - other
-			verbose = 1
+
 		if total < 0:
 			total = 0
 		RecordsTextArea.Append (DelimitedText (41390, str (total)+"%", 0))
 		# Armor Penalty (same as for skills and everything else)
-		if verbose and failure["Armor"]:
+		if failure["Armor"]:
 			AddIndent()
 			RecordsTextArea.Append (DelimitedText (39816, PlusMinusStat(5*failure["Armor"])+"%", 0))
 		# Shield Penalty
-		if verbose and failure["Shield"]:
+		if failure["Shield"]:
 			AddIndent()
 			RecordsTextArea.Append (DelimitedText (39822, PlusMinusStat(5*failure["Shield"])+"%", 0))
-		if arcana:
+		if armor and arcana:
 			AddIndent()
 			RecordsTextArea.Append (DelimitedText (36352, PlusMinusStat(-arcana)+"%", 0))
 		# Other, just a guess to show the remainder
@@ -884,14 +878,14 @@ def DisplayMisc (pc):
 
 	# Total Experience Value in Party
 	if TotalPartyExp:
-		val = stat['KillsTotalXP']*100/TotalPartyExp
+		val = stat['KillsTotalXP'] * 100 // TotalPartyExp
 	else:
 		val = 0
 	RecordsTextArea.Append (DelimitedText (11951, str(val) + "%", 0))
 
 	# Percentage of Total Kills in Party
 	if TotalPartyExp:
-		val = stat['KillsTotalCount']*100/TotalCount
+		val = stat['KillsTotalCount'] * 100 // TotalCount
 	else:
 		val = 0
 	RecordsTextArea.Append (DelimitedText (11952, str(val) + "%", 0))
@@ -1056,7 +1050,7 @@ def OpenHelpWindow ():
 		if title:
 			Label.SetText (title)
 			Button.SetState (IE_GUI_BUTTON_LOCKED)
-			Button.SetEvent (IE_GUI_BUTTON_ON_PRESS, UpdateHelpWindow)
+			Button.OnPress (UpdateHelpWindow)
 		else:
 			Label.SetText ("")
 			Button.SetState (IE_GUI_BUTTON_DISABLED)
@@ -1064,7 +1058,7 @@ def OpenHelpWindow ():
 	#done
 	Button = Window.GetControl (1)
 	Button.SetText (11973)
-	Button.SetEvent (IE_GUI_BUTTON_ON_PRESS, CloseHelpWindow)
+	Button.OnPress (CloseHelpWindow)
 	Window.ShowModal (MODAL_SHADOW_GRAY)
 	UpdateHelpWindow ()
 	return
@@ -1083,9 +1077,9 @@ def UpdateHelpWindow ():
 		#Button = Window.GetControl (i+27)
 		Label = Window.GetControl (i+0x10000004)
 		if Topic==i:
-			Label.SetTextColor ({'r' : 255, 'g' : 255, 'b' : 0})
+			Label.SetColor ({'r' : 255, 'g' : 255, 'b' : 0})
 		else:
-			Label.SetTextColor ({'r' : 255, 'g' : 255, 'b' : 255})
+			Label.SetColor ({'r' : 255, 'g' : 255, 'b' : 255})
 
 	resource = HelpTable.GetValue (Topic, 1)
 	if DescTable:
@@ -1104,7 +1098,7 @@ def UpdateHelpWindow ():
 
 	ScrollBar = Window.GetControl (4)
 	ScrollBar.SetVarAssoc ("TopIndex", i)
-	ScrollBar.SetEvent (IE_GUI_SCROLLBAR_ON_CHANGE, RefreshHelpWindow)
+	ScrollBar.OnChange (RefreshHelpWindow)
 
 	RefreshHelpWindow ()
 	return
@@ -1125,15 +1119,15 @@ def RefreshHelpWindow ():
 		Label = Window.GetControl (i+0x10000030)
 
 		if i+TopIndex==Selected:
-			Label.SetTextColor ({'r' : 255, 'g' : 255, 'b' : 0})
+			Label.SetColor ({'r' : 255, 'g' : 255, 'b' : 0})
 		else:
-			Label.SetTextColor ({'r' : 255, 'g' : 255, 'b' : 255})
+			Label.SetColor ({'r' : 255, 'g' : 255, 'b' : 255})
 		if DescTable:
 			title = DescTable.GetValue (i+startrow+TopIndex, titlecol)
 			Label.SetText (title)
 			Button.SetState (IE_GUI_BUTTON_LOCKED)
 			Button.SetVarAssoc ("Selected", i+TopIndex)
-			Button.SetEvent (IE_GUI_BUTTON_ON_PRESS, RefreshHelpWindow)
+			Button.OnPress (RefreshHelpWindow)
 		else:
 			Label.SetText ("")
 			Button.SetState (IE_GUI_BUTTON_DISABLED)
@@ -1154,11 +1148,11 @@ def CloseLUWindow ():
 	global LUWindow, LUKitWindow
 
 	if LUKitWindow:
-		LUKitWindow.Unload ()
+		LUKitWindow.Close ()
 		LUKitWindow = None
 
 	if LUWindow:
-		LUWindow.Unload ()
+		LUWindow.Close ()
 		LUWindow = None
 
 def OpenLevelUpWindow ():
@@ -1179,7 +1173,7 @@ def OpenLevelUpWindow ():
 	# next
 	Button = Window.GetControl (0)
 	Button.SetText (36789)
-	Button.SetEvent (IE_GUI_BUTTON_ON_PRESS, OpenLUKitWindow)
+	Button.OnPress (OpenLUKitWindow)
 	Button.SetState (IE_GUI_BUTTON_DISABLED)
 	Button.MakeDefault ()
 
@@ -1210,7 +1204,7 @@ def OpenLevelUpWindow ():
 		else:
 			Button.SetState (IE_GUI_BUTTON_ENABLED)
 		Button.SetVarAssoc ("LUClass", i-2)
-		Button.SetEvent (IE_GUI_BUTTON_ON_PRESS, LUClassPress)
+		Button.OnPress (LUClassPress)
 		Button.SetFlags (IE_GUI_BUTTON_RADIOBUTTON, OP_OR)
 
 	# description
@@ -1224,7 +1218,7 @@ def OpenLevelUpWindow ():
 	# cancel
 	Button = Window.GetControl (27)
 	Button.SetText (13727)
-	Button.SetEvent (IE_GUI_BUTTON_ON_PRESS, CloseLUWindow)
+	Button.OnPress (CloseLUWindow)
 
 	Window.ShowModal (MODAL_SHADOW_NONE)
 
@@ -1326,10 +1320,10 @@ def OpenLUKitWindow ():
 	LUClassID = CommonTables.Classes.GetValue (LUClassName, "ID")
 	hasKits = CommonTables.Classes.FindValue ("CLASS", LUClassID)
 	kitIndex = GUICommonWindows.GetKitIndex (pc, LUClass)
-	if hasKits == -1 or kitIndex > 0:
+	if hasKits is None or kitIndex > 0:
 		kitName = CommonTables.Classes.GetRowName (kitIndex)
 		kitID = CommonTables.Classes.GetValue (kitName, "ID", GTV_INT)
-		if hasKits == -1:
+		if hasKits is None:
 			kitID = 0
 		HandleSpecFlagExclusion(pc, LUClassID, kitID)
 		LUNextPress ()
@@ -1344,7 +1338,7 @@ def OpenLUKitWindow ():
 	# next
 	Button = Window.GetControl (0)
 	Button.SetText (36789)
-	Button.SetEvent (IE_GUI_BUTTON_ON_PRESS, LUNextPress)
+	Button.OnPress (LUNextPress)
 	Button.SetState (IE_GUI_BUTTON_DISABLED)
 	Button.MakeDefault ()
 
@@ -1374,7 +1368,7 @@ def OpenLUKitWindow ():
 		Button.SetText (kitTitle)
 		Button.SetState (IE_GUI_BUTTON_ENABLED)
 		Button.SetVarAssoc ("LUKit", i)
-		Button.SetEvent (IE_GUI_BUTTON_ON_PRESS, LUKitPress)
+		Button.OnPress (LUKitPress)
 		Button.SetFlags (IE_GUI_BUTTON_RADIOBUTTON, OP_OR)
 
 	Window.ShowModal (MODAL_SHADOW_NONE)
@@ -1458,9 +1452,9 @@ def FinishLevelUp():
 	BABTable = CommonTables.Classes.GetValue (LUClassName, "TOHIT")
 	BABTable = GemRB.LoadTable (BABTable)
 	currentBAB = GemRB.GetPlayerStat (pc, IE_TOHIT, 1)
-	oldBAB = BABTable.GetValue (str(oldLevel), "BASE_ATTACK")
+	oldBAB = BABTable.GetValue (str(oldLevel), "BASE_ATTACK", GTV_INT)
 	newLevel = oldLevel + LevelDiff
-	newBAB = BABTable.GetValue (str(newLevel), "BASE_ATTACK")
+	newBAB = BABTable.GetValue (str(newLevel), "BASE_ATTACK", GTV_INT)
 	GemRB.SetPlayerStat (pc, IE_TOHIT, currentBAB + newBAB - oldBAB)
 
 	# class level
@@ -1471,6 +1465,8 @@ def FinishLevelUp():
 	GemRB.SetVar ("LUClass", -1)
 	GemRB.SetVar ("LUKit", 0)
 
+	UpdateRecordsWindow (RecordsWindow)
+
 
 ###################################################
 # End of file GUIREC.py
