@@ -21,25 +21,23 @@
 #ifndef ACTORMGR_H
 #define ACTORMGR_H
 
+#include "ie_types.h"
+
 #include "Plugin.h"
 
 namespace GemRB {
 
 class Actor;
-class DataStream;
 
-class GEM_EXPORT ActorMgr : public Plugin {
+class GEM_EXPORT ActorMgr : public ImporterBase {
 public:
-	ActorMgr(void);
-	~ActorMgr(void) override;
-	virtual bool Open(DataStream* stream) = 0;
 	virtual Actor* GetActor(unsigned char is_in_party) = 0;
-  virtual int FindSpellType(char *name, unsigned short &level, unsigned int clsmsk, unsigned int kit) const = 0;
+	virtual ieWord FindSpellType(const ResRef& name, unsigned short &level, unsigned int clsMask, unsigned int kit) const = 0;
 
 	//returns saved size, updates internal offsets before save
-	virtual int GetStoredFileSize(Actor *ac) = 0;
+	virtual int GetStoredFileSize(const Actor *ac) = 0;
 	//saves file
-	virtual int PutActor(DataStream *stream, Actor *actor, bool chr=false) = 0;
+	virtual int PutActor(DataStream *stream, const Actor *actor, bool chr = false) = 0;
 };
 
 }
