@@ -25,23 +25,7 @@
 
 namespace GemRB {
 
-Ambient::Ambient()
-{
-	name[0] = 0;
-	radius = 0;
-	gain = gainVariance = interval = intervalVariance = 0;
-	pitchVariance = appearance = flags = 0;
-}
-
-Ambient::~Ambient()
-{
-	unsigned int i=sounds.size();
-	while(i--) {
-		free(sounds[i]);
-	}
-}
-
-ieWord Ambient::getTotalGain() const
+ieWord Ambient::GetTotalGain() const
 {
 	ieWord g = gain;
 	if (gainVariance != 0) {
@@ -51,9 +35,9 @@ ieWord Ambient::getTotalGain() const
 	return g;
 }
 
-ieDword Ambient::getTotalInterval() const
+tick_t Ambient::GetTotalInterval() const
 {
-	ieDword i = interval;
+	tick_t i = interval;
 	if (intervalVariance != 0) {
 		ieWord var = std::min(intervalVariance, interval / 2);
 		i += RAND(0, 2 * var) - var;
@@ -61,16 +45,13 @@ ieDword Ambient::getTotalInterval() const
 	return i;
 }
 
-ieDword Ambient::getTotalPitch() const
+ieDword Ambient::GetTotalPitch() const
 {
 	ieDword p = 100;
 	if (pitchVariance != 0) {
-		p += RAND(0, 2 * pitchVariance) - pitchVariance;
+		p += RAND(0u, 2 * pitchVariance) - pitchVariance;
 	}
 	return p;
 }
 
-void Ambient::setActive() { flags |= IE_AMBI_ENABLED; }
-void Ambient::setInactive() { flags &= ~IE_AMBI_ENABLED; }
-
 }
