@@ -22,19 +22,22 @@
 #define CALENDAR_H
 
 #include "exports.h"
+#include "ie_types.h"
 
 namespace GemRB {
 
 class GEM_EXPORT Calendar {
 private:
-	int daysinyear;
-	int monthnamecount;
-	int *days;
-	int *monthnames;
+	int daysinyear = 0;
+	size_t monthnamecount = 0;
+	int *days = nullptr;
+	ieStrRef *monthnames = nullptr;
 
 public:
 	Calendar(void);
+	Calendar(const Calendar&) = delete;
 	~Calendar(void);
+	Calendar& operator=(const Calendar&) = delete;
 	void GetMonthName(int dayandmonth) const;
 	int GetCalendarDay(int date) const;
 };
