@@ -30,12 +30,12 @@ using Callback = std::function<R(ARGS...)>;
 
 using EventHandler = std::function<void()>;
 
-// std::function has an exlicitly deleted operator== so we nee to write our own comparitor
+// std::function has an explicitly deleted operator== so we need to write our own comparitor
 // that makes sense for our purposes
 template <typename R, typename... ARGS>
 bool FunctionTargetsEqual(const std::function<R(ARGS...)>& lhs, const std::function<R(ARGS...)>& rhs)
 {
-	typedef R(fnType)(ARGS...);
+	using fnType = R(*)(ARGS...);
 	return lhs.template target<fnType*>() == rhs.template target<fnType*>();
 }
 
