@@ -30,35 +30,28 @@
 
 #include "Game.h"
 #include "GameData.h"
+#include "Logging/Logging.h"
 #include "Palette.h"
 
-#include <cmath>
+#include <utility>
 
 #define IS_PORTRAIT (Picture && ((flags&IE_GUI_BUTTON_PORTRAIT) == IE_GUI_BUTTON_PORTRAIT))
 
 namespace GemRB {
 
-Button::Button(Region& frame)
-	: Control(frame),
-	buttonImages()
+Button::Button(const Region& frame)
+	: Control(frame)
 {
 	ControlType = IE_GUI_BUTTON;
-	State = IE_GUI_BUTTON_UNPRESSED;
 	HotKeyCallback = METHOD_CALLBACK(&Button::HandleHotKey, this);
 
-	hasText = false;
 	SetFont(core->GetButtonFont());
-	SetFlags(IE_GUI_BUTTON_NORMAL, OP_OR);
-	ToggleState = false;
-	pulseBorder = false;
-	Picture = NULL;
-	Clipping = 1.0;
-
-	PushOffset = Point(2, 2);
+	SetFlags(IE_GUI_BUTTON_NORMAL, BitOp::OR);
 }
 
 Button::~Button()
 {
+	delete animation;
 	SetImage(BUTTON_IMAGE_NONE, NULL);
 	ClearPictureList();
 
@@ -83,17 +76,17 @@ void Button::UnregisterHotKey()
 void Button::SetImage(BUTTON_IMAGE_TYPE type, Holder<Sprite2D> img)
 {
 	if (type >= BUTTON_IMAGE_TYPE_COUNT) {
-		Log(ERROR, "Button", "Trying to set a button image index out of range: %d", type);
+		Log(ERROR, "Button", "Trying to set a button image index out of range: {}", type);
 		return;
 	}
 
 	if (type <= BUTTON_IMAGE_NONE) {
-		for (int i = 0; i < BUTTON_IMAGE_TYPE_COUNT; i++) {
-			buttonImages[i] = nullptr;
+		for (auto& buttonImage : buttonImages) {
+			buttonImage = nullptr;
 		}
 		flags &= IE_GUI_BUTTON_NO_IMAGE;
 	} else {
-		buttonImages[type] = img;
+		buttonImages[type] = std::move(img);
 		/*
 		 currently IE_GUI_BUTTON_NO_IMAGE cannot be infered from the presence or lack of images
 		 leaving this here commented out in case it may be useful in the future.
@@ -120,13 +113,24 @@ void Button::SetImage(BUTTON_IMAGE_TYPE type, Holder<Sprite2D> img)
 
 void Button::WillDraw(const Region& /*drawFrame*/, const Region& /*clip*/)
 {
+	if (animation && animation->HasEnded()) {
+		SetAnimation(nullptr);
+	}
+}
+
+void Button::DidDraw(const Region& /*drawFrame*/, const Region& /*clip*/)
+{
+	tick_t time = GetMilliseconds();
 	if (overlayAnim) {
-		overlayAnim.Next(GetTicks());
+		overlayAnim.Next(time);
+	}
+	if (animation) {
+		animation->Next(time);
 	}
 }
 
 /** Draws the Control on the Output Display */
-void Button::DrawSelf(Region rgn, const Region& /*clip*/)
+void Button::DrawSelf(const Region& rgn, const Region& /*clip*/)
 {
 	Video * video = core->GetVideoDriver();
 
@@ -134,16 +138,16 @@ void Button::DrawSelf(Region rgn, const Region& /*clip*/)
 	if (!( flags & IE_GUI_BUTTON_NO_IMAGE )) {
 		Holder<Sprite2D> Image;
 
-		switch (State) {
-			case IE_GUI_BUTTON_FAKEPRESSED:
-			case IE_GUI_BUTTON_PRESSED:
+		switch (ButtonState) {
+			case FAKEPRESSED:
+			case PRESSED:
 				Image = buttonImages[BUTTON_IMAGE_PRESSED];
 				break;
-			case IE_GUI_BUTTON_SELECTED:
+			case SELECTED:
 				Image = buttonImages[BUTTON_IMAGE_SELECTED];
 				break;
-			case IE_GUI_BUTTON_DISABLED:
-			case IE_GUI_BUTTON_FAKEDISABLED:
+			case DISABLED:
+			case FAKEDISABLED:
 				Image = buttonImages[BUTTON_IMAGE_DISABLED];
 				break;
 			default:
@@ -153,11 +157,11 @@ void Button::DrawSelf(Region rgn, const Region& /*clip*/)
 		if (Image) {
 			// FIXME: maybe it's useless...
 			Point offset((frame.w / 2) - (Image->Frame.w / 2), (frame.h / 2) - (Image->Frame.h / 2));
-			video->BlitSprite(Image, rgn.Origin() + offset);
+			video->BlitSprite(Image, rgn.origin + offset);
 		}
 	}
 
-	if (State == IE_GUI_BUTTON_PRESSED) {
+	if (ButtonState == PRESSED) {
 		//shift the writing/border a bit
 		rgn.x += PushOffset.x;
 		rgn.y += PushOffset.y;
@@ -170,7 +174,7 @@ void Button::DrawSelf(Region rgn, const Region& /*clip*/)
 		picPos.x = (rgn.w / 2) - (Picture->Frame.w / 2) + rgn.x;
 		picPos.y = (rgn.h / 2) - (Picture->Frame.h / 2) + rgn.y;
 		if (flags & IE_GUI_BUTTON_HORIZONTAL) {
-			picPos += Picture->Frame.Origin();
+			picPos += Picture->Frame.origin;
 
 			// Clipping: 0 = overlay over full button, 1 = no overlay
 			int overlayHeight = Picture->Frame.h * (1.0 - Clipping);
@@ -187,23 +191,24 @@ void Button::DrawSelf(Region rgn, const Region& /*clip*/)
 			}
 
 			Region rb = Region(picPos.x, picPos.y, Picture->Frame.w, buttonHeight);
-			video->BlitSprite( Picture, rb.Origin(), &rb );
+			video->BlitSprite( Picture, rb.origin, &rb );
 		} else {
 			Region r(picPos.x, picPos.y, (Picture->Frame.w * Clipping), Picture->Frame.h);
-			video->BlitSprite(Picture, Picture->Frame.Origin() + r.Origin(), &r);
+			video->BlitSprite(Picture, Picture->Frame.origin + r.origin, &r);
 		}
 	}
 
 	// Button animation
-	if (AnimPicture) {
+	if (animation && animation->Current()) {
+		auto AnimPicture = animation->Current();
 		int xOffs = ( frame.w / 2 ) - ( AnimPicture->Frame.w / 2 );
 		int yOffs = ( frame.h / 2 ) - ( AnimPicture->Frame.h / 2 );
 		Region r( rgn.x + xOffs, rgn.y + yOffs, int(AnimPicture->Frame.w * Clipping), AnimPicture->Frame.h );
 
 		if (flags & IE_GUI_BUTTON_CENTER_PICTURES) {
-			video->BlitSprite( AnimPicture, r.Origin() + AnimPicture->Frame.Origin(), &r );
+			video->BlitSprite( AnimPicture, r.origin + AnimPicture->Frame.origin, &r );
 		} else {
-			video->BlitSprite( AnimPicture, r.Origin(), &r );
+			video->BlitSprite( AnimPicture, r.origin, &r );
 		}
 	}
 
@@ -223,8 +228,12 @@ void Button::DrawSelf(Region rgn, const Region& /*clip*/)
 			offset.y = frame.h / 2 - (*iter)->Frame.h/2 + (*iter)->Frame.y;
 		}
 
+		BlitFlags blitFlags = BlitFlags::NONE;
+		if (flags & IE_GUI_BUTTON_HORIZONTAL) {
+			blitFlags = BlitFlags::HALFTRANS;
+		}
 		for (; iter != PictureList.end(); ++iter) {
-			video->BlitSprite(*iter, rgn.Origin() + offset);
+			video->BlitSprite(*iter, rgn.origin + offset, nullptr, blitFlags);
 		}
 	}
 
@@ -260,7 +269,7 @@ void Button::DrawSelf(Region rgn, const Region& /*clip*/)
 			r.w -= Anchor.x;
 			r.h -= Anchor.y;
 		} else {
-			Font::StringSizeMetrics metrics {r.Dimensions(), 0, 0, false};
+			Font::StringSizeMetrics metrics {r.size, 0, 0, false};
 			font->StringSize(Text, &metrics);
 
 			if (metrics.numLines == 1 && (IE_GUI_BUTTON_ALIGNMENT_FLAGS & flags)) {
@@ -273,7 +282,7 @@ void Button::DrawSelf(Region rgn, const Region& /*clip*/)
 		}
 		
 		Color c = textColor;
-		if (State == IE_GUI_BUTTON_DISABLED || IsDisabled()) {
+		if (ButtonState == DISABLED || IsDisabled()) {
 			c.r *= 0.66;
 			c.g *= 0.66;
 			c.b *= 0.66;
@@ -284,12 +293,12 @@ void Button::DrawSelf(Region rgn, const Region& /*clip*/)
 	}
 
 	if (! (flags&IE_GUI_BUTTON_NO_IMAGE)) {
-		for (int i = 0; i < MAX_NUM_BORDERS; i++) {
-			ButtonBorder *fr = &borders[i];
+		for (const auto& border : borders) {
+			const ButtonBorder *fr = &border;
 			if (! fr->enabled) continue;
 
 			const Region& frRect = fr->rect;
-			Region r = Region( rgn.Origin() + frRect.Origin(), frRect.Dimensions() );
+			Region r = Region( rgn.origin + frRect.origin, frRect.size );
 			if (pulseBorder && !fr->filled) {
 				Color mix = GlobalColorCycle.Blend(ColorWhite, fr->color);
 				video->DrawRect( r, mix, fr->filled, BlitFlags::BLENDED );
@@ -299,24 +308,45 @@ void Button::DrawSelf(Region rgn, const Region& /*clip*/)
 		}
 	}
 }
-/** Sets the Button State */
-void Button::SetState(unsigned char state)
+
+void Button::FlagsChanged(unsigned int /*oldflags*/)
 {
-	if (state > IE_GUI_BUTTON_LOCKED_PRESSED) {// If wrong value inserted
+	if (animation) {
+		animation->flags = SpriteAnimation::PLAY_NORMAL;
+		if (flags & IE_GUI_BUTTON_PLAYRANDOM) {
+			animation->flags |= SpriteAnimation::PLAY_RANDOM;
+		}
+		if (flags & IE_GUI_BUTTON_PLAYONCE) {
+			animation->flags |= SpriteAnimation::PLAY_ONCE;
+		}
+		if (flags & IE_GUI_BUTTON_PLAYALWAYS) {
+			animation->flags |= SpriteAnimation::PLAY_ALWAYS;
+		}
+	}
+}
+
+/** Sets the Button State */
+void Button::SetState(State state)
+{
+	if (state > LOCKED_PRESSED) {// If wrong value inserted
 		return;
 	}
 
 	// FIXME: we should properly consolidate IE_GUI_BUTTON_DISABLED with the view Disabled flag
-	SetDisabled(state == IE_GUI_BUTTON_DISABLED);
+	SetDisabled(state == DISABLED);
 
-	if (State != state) {
+	if (ButtonState != state) {
 		MarkDirty();
-		State = state;
+		ButtonState = state;
 	}
 }
 
 bool Button::IsAnimated() const
 {
+	if (animation) {
+		return true;
+	}
+
 	if (overlayAnim) {
 		return true;
 	}
@@ -325,14 +355,24 @@ bool Button::IsAnimated() const
 		return true;
 	}
 
-	return Control::IsAnimated();
+	return false;
 }
 
 bool Button::IsOpaque() const
 {
-	return Picture != NULL
-		&& !(flags&IE_GUI_BUTTON_NO_IMAGE)
-		&& Picture->HasTransparency() == false;
+	if (animation) {
+		// we are dirty every frame anyway
+		// so no need to get fancy
+		// just redraw everything
+		return false;
+	}
+
+	bool opaque = View::IsOpaque();
+	if (!opaque && Picture) {
+		opaque = !(flags&IE_GUI_BUTTON_NO_IMAGE) && !Picture->HasTransparency();
+	}
+	
+	return opaque;
 }
 
 void Button::SetBorder(int index, const Region& rgn, const Color &color, bool enabled, bool filled)
@@ -341,7 +381,7 @@ void Button::SetBorder(int index, const Region& rgn, const Color &color, bool en
 		return;
 
 	ButtonBorder fr = { rgn, color, filled, enabled };
-	borders[index] = fr;
+	borders[index] = std::move(fr);
 
 	MarkDirty();
 }
@@ -368,9 +408,7 @@ String Button::TooltipText() const
 		return L"";
 	}
 
-	ieDword showHotkeys = 0;
-	core->GetDictionary()->Lookup("Hotkeys On Tooltips", showHotkeys);
-
+	ieDword showHotkeys = core->GetVariable("Hotkeys On Tooltips", 0);
 	if (showHotkeys && hotKey) {
 		String s;
 		switch (hotKey.key) {
@@ -388,7 +426,6 @@ String Button::TooltipText() const
 				s += L"PgDn";
 				break;
 			default:
-				// TODO: check if there are more possible keys
 				if (hotKey.key >= GEM_FUNCTIONX(1) && hotKey.key <= GEM_FUNCTIONX(16)) {
 					s.push_back(L'F');
 					int offset = hotKey.key - GEM_FUNCTIONX(0);
@@ -398,7 +435,7 @@ String Button::TooltipText() const
 					}
 					s.push_back(L'0' + offset);
 				} else {
-					s.push_back(toupper(hotKey.key));
+					s.push_back(towupper(hotKey.key));
 				}
 				break;
 		}
@@ -412,7 +449,7 @@ String Button::TooltipText() const
 Holder<Sprite2D> Button::Cursor() const
 {
 	if (IS_PORTRAIT) {
-		GameControl* gc = core->GetGameControl();
+		const GameControl* gc = core->GetGameControl();
 		if (gc) {
 			Holder<Sprite2D> cur = gc->GetTargetActionCursor();
 			if (cur) return cur;
@@ -458,16 +495,16 @@ Holder<Sprite2D> Button::DragCursor() const
 bool Button::OnMouseDown(const MouseEvent& me, unsigned short mod)
 {
 	ActionKey key(Action::DragDropDest);
-    if (core->GetDraggedItem() && !SupportsAction(key)) {
-        return true;
-    }
+	if (core->GetDraggedItem() && !SupportsAction(key)) {
+		return true;
+	}
     
 	if (me.button == GEM_MB_ACTION) {
-		if (State == IE_GUI_BUTTON_LOCKED) {
-			SetState( IE_GUI_BUTTON_LOCKED_PRESSED );
+		if (ButtonState == LOCKED) {
+			SetState(LOCKED_PRESSED);
 			return true;
 		}
-		SetState( IE_GUI_BUTTON_PRESSED );
+		SetState(PRESSED);
 		if (flags & IE_GUI_BUTTON_SOUND) {
 			core->PlaySound(DS_BUTTON_PRESSED, SFX_CHAN_GUI);
 		}
@@ -490,17 +527,10 @@ bool Button::OnMouseUp(const MouseEvent& me, unsigned short mod)
 		}
 	}
 
-	switch (State) {
-	case IE_GUI_BUTTON_PRESSED:
-		if (ToggleState) {
-			SetState( IE_GUI_BUTTON_SELECTED );
-		} else {
-			SetState( IE_GUI_BUTTON_UNPRESSED );
-		}
-		break;
-	case IE_GUI_BUTTON_LOCKED_PRESSED:
-		SetState( IE_GUI_BUTTON_LOCKED );
-		break;
+	if (ButtonState == LOCKED_PRESSED) {
+		SetState(LOCKED);
+	} else {
+		SetState(UNPRESSED);
 	}
 
 	DoToggle();
@@ -509,7 +539,7 @@ bool Button::OnMouseUp(const MouseEvent& me, unsigned short mod)
 
 bool Button::OnMouseOver(const MouseEvent& me)
 {
-	if (State == IE_GUI_BUTTON_LOCKED) {
+	if (ButtonState == LOCKED) {
 		return true;
 	}
 
@@ -521,11 +551,11 @@ void Button::OnMouseEnter(const MouseEvent& me, const DragOp* dop)
 	Control::OnMouseEnter(me, dop);
 
 	if (IsFocused() && me.ButtonState(GEM_MB_ACTION)) {
-		SetState( IE_GUI_BUTTON_PRESSED );
+		SetState(PRESSED);
 	}
 
-	for (int i = 0; i < MAX_NUM_BORDERS; i++) {
-		ButtonBorder *fr = &borders[i];
+	for (const auto& border : borders) {
+		const ButtonBorder *fr = &border;
 		if (fr->enabled) {
 			pulseBorder = !fr->filled;
 			MarkDirty();
@@ -538,8 +568,8 @@ void Button::OnMouseLeave(const MouseEvent& me, const DragOp* dop)
 {
 	Control::OnMouseLeave(me, dop);
 
-	if (State == IE_GUI_BUTTON_PRESSED && (dop == nullptr || dop->dragView == this)) {
-		SetState( IE_GUI_BUTTON_UNPRESSED );
+	if (ButtonState == PRESSED && (dop == nullptr || dop->dragView == this)) {
+		SetState(UNPRESSED);
 	}
 
 	if (pulseBorder) {
@@ -549,10 +579,10 @@ void Button::OnMouseLeave(const MouseEvent& me, const DragOp* dop)
 }
 
 /** Sets the Text of the current control */
-void Button::SetText(const String& string)
+void Button::SetText(String string)
 {
-	Text = string;
-	if (string.length()) {
+	Text = std::move(string);
+	if (Text.length()) {
 		if (flags&IE_GUI_BUTTON_LOWERCASE)
 			StringToLower( Text );
 		else if (flags&IE_GUI_BUTTON_CAPS)
@@ -564,72 +594,54 @@ void Button::SetText(const String& string)
 	MarkDirty();
 }
 
+BitOp Button::GetDictOp() const noexcept
+{
+	if (flags & IE_GUI_BUTTON_CHECKBOX) {
+		return BitOp::XOR;
+	}
+	
+	return BitOp::SET;
+}
+
 /** Refresh a button from a given radio button group */
-void Button::UpdateState(unsigned int Sum)
+void Button::UpdateState(value_t Sum)
 {
 	if (IsDisabled()) {
+		// FIXME: buttons should be able to both be disabled and reflect their state
 		return;
 	}
-
+	
+	State state = UNPRESSED;
 	if (flags & IE_GUI_BUTTON_RADIOBUTTON) {
 		//radio button, exact value
-		ToggleState = ( Sum == GetValue() );
+		state = Sum == GetValue() ? SELECTED : UNPRESSED;
 	} else if (flags & IE_GUI_BUTTON_CHECKBOX) {
 		//checkbox, bitvalue
-		ToggleState = !!( Sum & GetValue() );
+		state = bool(Sum & GetValue()) ? SELECTED : UNPRESSED;
 	} else {
 		//other buttons, nothing to redraw
 		return;
 	}
 
-	if (ToggleState) {
-		SetState(IE_GUI_BUTTON_SELECTED);
-	} else {
-		SetState(IE_GUI_BUTTON_UNPRESSED);
-	}
+	SetState(state);
 }
 
 void Button::DoToggle()
 {
-	if (flags & IE_GUI_BUTTON_CHECKBOX) {
-		//checkbox
-		ToggleState = !ToggleState;
-		if (ToggleState)
-			SetState( IE_GUI_BUTTON_SELECTED );
-		else
-			SetState( IE_GUI_BUTTON_UNPRESSED );
-		if (VarName[0] != 0) {
-			ieDword tmp = 0;
-			core->GetDictionary()->Lookup( VarName, tmp );
-			tmp ^= GetValue();
-			core->GetDictionary()->SetAt( VarName, tmp );
-			window->RedrawControls( VarName, tmp );
-		}
-	} else {
-		if (flags & IE_GUI_BUTTON_RADIOBUTTON) {
-			//radio button
-			ToggleState = true;
-			SetState( IE_GUI_BUTTON_SELECTED );
-		}
-		if (VarName[0] != 0) {
-			ieDword val = GetValue();
-			core->GetDictionary()->SetAt( VarName, val );
-			window->RedrawControls( VarName, val );
-		}
-	}
+	UpdateDictValue();
 }
 
 /** Sets the Picture */
 void Button::SetPicture(Holder<Sprite2D> newpic)
 {
 	ClearPictureList();
-	Picture = newpic;
+	Picture = std::move(newpic);
 	if (Picture) {
 		// try fitting to width if rescaling is possible, otherwise we automatically crop
-		unsigned int ratio = round((float) Picture->Frame.w / (float) frame.w);
+		unsigned int ratio = CeilDiv(Picture->Frame.w, frame.w);
 		if (ratio > 1) {
 			Holder<Sprite2D> img = core->GetVideoDriver()->SpriteScaleDown(Picture, ratio);
-			Picture = img;
+			Picture = std::move(img);
 		}
 		flags |= IE_GUI_BUTTON_PICTURE;
 	} else {
@@ -638,6 +650,23 @@ void Button::SetPicture(Holder<Sprite2D> newpic)
 	MarkDirty();
 }
 
+void Button::SetAnimation(SpriteAnimation* anim)
+{
+	// if this button says the resource is the same
+	// we wanted to set, we don't reset it
+	// but we must reinitialize it, if it was play once
+	if (animation && animation->SameResource(anim) && !(animation->flags & SpriteAnimation::PLAY_ONCE)) {
+		delete anim;
+		return;
+	} else {
+		delete animation;
+		animation = anim;
+		FlagsChanged(flags); // sync animation flags
+	}
+	
+	MarkDirty();
+}
+
 /** Clears the list of Pictures */
 void Button::ClearPictureList()
 {
@@ -646,9 +675,9 @@ void Button::ClearPictureList()
 }
 
 /** Add picture to the end of the list of Pictures */
-void Button::StackPicture(Holder<Sprite2D> Picture)
+void Button::StackPicture(const Holder<Sprite2D>& newPicture)
 {
-	PictureList.push_back(Picture);
+	PictureList.push_back(newPicture);
 	MarkDirty();
 	flags |= IE_GUI_BUTTON_PICTURE;
 }
@@ -660,7 +689,7 @@ bool Button::HitTest(const Point& p) const
 		// some buttons have hollow Image frame filled w/ Picture
 		// some buttons in BG2 are text only (if BAM == 'GUICTRL')
 		Holder<Sprite2D> Unpressed = buttonImages[BUTTON_IMAGE_UNPRESSED];
-		if (Picture || PictureList.size() || !Unpressed) return true;
+		if (Picture || !PictureList.empty() || !Unpressed) return true;
 
 		Point off;
 		off.x = (frame.w / 2) - (Unpressed->Frame.w / 2) + Unpressed->Frame.x;
@@ -688,12 +717,12 @@ void Button::SetHorizontalOverlay(double clip, const Color& src, const Color &de
 	MarkDirty();
 }
 
-void Button::SetAnchor(ieWord x, ieWord y)
+void Button::SetAnchor(int x, int y)
 {
 	Anchor = Point(x,y);
 }
 
-void Button::SetPushOffset(ieWord x, ieWord y)
+void Button::SetPushOffset(int x, int y)
 {
 	PushOffset = Point(x,y);
 }
@@ -713,7 +742,7 @@ bool Button::SetHotKey(KeyboardKey key, short mod, bool global)
 			hotKey.global = true;
 			return true;
 		}
-	} else if (window->RegisterHotKeyCallback(HotKeyCallback, key)) { // FIXME: this doesnt respect mod param
+	} else if (window->RegisterHotKeyCallback(HotKeyCallback, key)) { // FIXME: this doesn't respect mod param
 		hotKey.key = key;
 		hotKey.mod = mod;
 		return true;
