@@ -21,45 +21,45 @@
 #include "TextArea.h"
 
 #include "Interface.h"
-#include "Variables.h"
+#include "Logging/Logging.h"
+
 #include "GUI/EventMgr.h"
 #include "GUI/ScrollBar.h"
 #include "GUI/Window.h"
 
+#include <utility>
+
 namespace GemRB {
 	
 TextArea::SpanSelector::SpanSelector(TextArea& ta, const std::vector<const String*>& opts, bool numbered, Margin m)
 : ContentContainer(Region(0, 0, ta.Frame().w, 0)), ta(ta)
 {
-	SetFlags(RESIZE_WIDTH, OP_NAND);
-
-	selectedSpan = NULL;
-	hoverSpan = NULL;
+	SetFlags(RESIZE_WIDTH, BitOp::NAND);
 
 	size = opts.size();
 
 	SetMargin(m);
 
 	Size flexFrame(-1, 0); // flex frame for hanging indent after optnum
-	String format = L". - ";
-	int numWidth = int(ta.ftext->StringSizeWidth(std::to_wstring(opts.size()) + format, 0)) + 3; // good guess at max width
+	String format = L". - "; // no dash in pst, but we don't bother
+	int numWidth = int(ta.ftext->StringSizeWidth(fmt::to_wstring(opts.size()) + format, 0)) + 3; // good guess at max width
 	Size numFrame(numWidth, ta.ftext->LineHeight); // size for the numerical prefix so they stay aligned
 	Point origin(margin.left, margin.top);
 	Region r(origin, Dimensions());
 	r.w = std::max(r.w - margin.left - margin.right, 0);
 	r.h = std::max(r.h - margin.top - margin.bottom, 0);
 	
-	Font::PrintColors colors {ta.colors[COLOR_OPTIONS], ta.colors[COLOR_BACKGROUND]};
+	Font::PrintColors optionColors {ta.colors[COLOR_OPTIONS], ta.colors[COLOR_BACKGROUND]};
 	Font::PrintColors selectedCol {ta.colors[COLOR_SELECTED], ta.colors[COLOR_BACKGROUND]};
 
 	for (size_t i = 0; i < opts.size(); i++) {
-		TextContainer* selOption = new OptSpan(r, ta.ftext, colors.fg, colors.bg);
-		selOption->SetAutoResizeFlags(ResizeHorizontal, OP_SET);
+		TextContainer* selOption = new OptSpan(r, ta.ftext, optionColors.fg, optionColors.bg);
+		selOption->SetAutoResizeFlags(ResizeHorizontal, BitOp::SET);
 
 		if (numbered) {
 			// TODO: as per the original PALETTE_SELECTED should be updated to the PC color (same color their name is rendered in)
 			// but that should probably actually be done by the dialog handler, not here.
-			auto ts = new TextSpan(std::to_wstring(i + 1) + format, nullptr, selectedCol, &numFrame);
+			auto ts = new TextSpan(fmt::to_wstring(i + 1) + format, nullptr, selectedCol, &numFrame);
 			ts->Alignment = IE_FONT_ALIGN_RIGHT;
 			selOption->AppendContent(ts);
 		}
@@ -145,7 +145,7 @@ void TextArea::SpanSelector::ClearHover()
 	}
 }
 
-void TextArea::SpanSelector::MakeSelection(size_t idx)
+void TextArea::SpanSelector::MakeSelection(Option_t idx)
 {
 	TextContainer* optspan = TextAtIndex(idx);
 
@@ -163,17 +163,18 @@ void TextArea::SpanSelector::MakeSelection(size_t idx)
 		selectedSpan->SetColors(ta.colors[COLOR_SELECTED], ta.colors[COLOR_BACKGROUND]);
 	}
 
+	selected = idx;
 	// beware, this will recursively call this function.
-	ta.UpdateState(static_cast<unsigned int>(idx));
+	ta.UpdateStateWithSelection(idx);
 }
-	
+
 TextContainer* TextArea::SpanSelector::TextAtPoint(const Point& p)
 {
 	// container only has text, so...
 	return static_cast<TextContainer*>(SubviewAt(p, true, false));
 }
 	
-TextContainer* TextArea::SpanSelector::TextAtIndex(size_t idx)
+TextContainer* TextArea::SpanSelector::TextAtIndex(Option_t idx)
 {
 	if (subViews.empty() || idx > subViews.size() - 1) {
 		return NULL;
@@ -203,7 +204,7 @@ bool TextArea::SpanSelector::OnMouseOver(const MouseEvent& me)
 bool TextArea::SpanSelector::OnMouseUp(const MouseEvent& me, unsigned short /*Mod*/)
 {
 	Point p = ConvertPointFromScreen(me.Pos());
-	TextContainer* span = TextAtPoint(p);
+	const TextContainer* span = TextAtPoint(p);
 	
 	if (span) {
 		std::list<View*>::reverse_iterator it = subViews.rbegin();
@@ -236,22 +237,18 @@ TextArea::TextArea(const Region& frame, Font* text, Font* caps)
 	assert(ftext && finit);
 
 	ControlType = IE_GUI_TEXTAREA;
-	strncpy(VarName, "Selected", sizeof(VarName));
 
-	selectOptions = NULL;
-	textContainer = NULL;
-	historyTimer = NULL;
-	
-	AddSubviewInFrontOfView(&scrollview);
+	Control::AddSubviewInFrontOfView(&scrollview);
 
 	// initialize the Text containers
 	ClearSelectOptions();
 	ClearText();
-	SetAnimPicture(NULL);
 
 	scrollview.SetScrollIncrement(LineHeight());
-	scrollview.SetAutoResizeFlags(ResizeAll, OP_SET);
-	scrollview.SetFlags(View::IgnoreEvents, (Flags()&View::IgnoreEvents) ? OP_OR : OP_NAND);
+	scrollview.SetAutoResizeFlags(ResizeAll, BitOp::SET);
+	scrollview.SetFlags(View::IgnoreEvents, (Flags()&View::IgnoreEvents) ? BitOp::OR : BitOp::NAND);
+	
+	BindDictVariable("Selected", Control::INVALID_VALUE);
 }
 
 TextArea::~TextArea()
@@ -259,23 +256,23 @@ TextArea::~TextArea()
 	ClearHistoryTimer();
 }
 
-void TextArea::DrawSelf(Region drawFrame, const Region& /*clip*/)
+void TextArea::DrawSelf(const Region& drawFrame, const Region& /*clip*/)
 {
-	if (AnimPicture) {
-		// speaker portrait
-		core->GetVideoDriver()->BlitSprite(AnimPicture, drawFrame.Origin());
+	if (speakerPic) {
+		core->GetVideoDriver()->BlitSprite(speakerPic, drawFrame.origin);
 	}
 }
 
-void TextArea::SetAnimPicture(Holder<Sprite2D> pic)
+void TextArea::SetSpeakerPicture(Holder<Sprite2D> pic)
 {
-	if (core->HasFeature(GF_DIALOGUE_SCROLLS)) {
+	if (core->HasFeature(GFFlags::DIALOGUE_SCROLLS)) {
 		// FIXME: there isnt a specific reason why animatied dialog couldnt also use pics
 		// However, PST does not and the animation makes the picture spaz currently
 		return;
 	}
 
-	Control::SetAnimPicture(pic);
+	speakerPic = std::move(pic);
+	MarkDirty();
 
 	assert(textContainer);
 	UpdateTextFrame();
@@ -293,7 +290,7 @@ void TextArea::SetMargins(ContentContainer::Margin m)
 		textContainer->SetMargin(textMargins);
 }
 
-ieDword TextArea::LineCount() const
+int TextArea::LineCount() const
 {
 	int rowHeight = LineHeight();
 	if (rowHeight > 0)
@@ -310,9 +307,9 @@ Region TextArea::UpdateTextFrame()
 		r.w = cr.w + cr.x;
 		r.h = 0; // auto grow
 
-		if (AnimPicture) {
+		if (speakerPic) {
 			// shrink and shift the container to accommodate the image
-			r.x = AnimPicture->Frame.w + 5;
+			r.x = speakerPic->Frame.w + 5;
 			r.w -= r.x;
 		} else {
 			r.x = 0;
@@ -341,9 +338,9 @@ void TextArea::UpdateScrollview()
 		ieDword anim = 0;
 		int y = 0;
 
-		if (core->HasFeature(GF_DIALOGUE_SCROLLS)) {
+		if (core->HasFeature(GFFlags::DIALOGUE_SCROLLS)) {
 			anim = 500;
-			y = -9999999; // FIXME: properly calculate the "bottom"?
+			y = 9999999; // FIXME: properly calculate the "bottom"?
 		} else {
 			int blankH = frame.h - LineHeight() - nodeBounds.h - optH;
 			if (blankH > 0) {
@@ -359,7 +356,7 @@ void TextArea::UpdateScrollview()
 		// FIXME: must update before the scroll, but this should be automaticly done as a reaction to changing sizes/origins of subviews
 		scrollview.Update();
 		scrollview.ScrollTo(Point(0, -y), anim);
-	} else if (!core->HasFeature(GF_DIALOGUE_SCROLLS)) {
+	} else if (!core->HasFeature(GFFlags::DIALOGUE_SCROLLS)) {
 		scrollview.Update();
 	}
 	
@@ -374,29 +371,29 @@ void TextArea::UpdateScrollview()
 void TextArea::FlagsChanged(unsigned int oldflags)
 {
 	if (Flags()&View::IgnoreEvents) {
-		scrollview.SetFlags(View::IgnoreEvents, OP_OR);
+		scrollview.SetFlags(View::IgnoreEvents, BitOp::OR);
 	} else if (oldflags&View::IgnoreEvents) {
-		scrollview.SetFlags(View::IgnoreEvents, OP_NAND);
+		scrollview.SetFlags(View::IgnoreEvents, BitOp::NAND);
 	}
 
 	if (Flags()&Editable) {
 		assert(textContainer);
-		textContainer->SetFlags(View::IgnoreEvents, OP_NAND);
+		textContainer->SetFlags(View::IgnoreEvents, BitOp::NAND);
 		textContainer->SetEventProxy(NULL);
 		SetEventProxy(textContainer);
 	} else if (oldflags&Editable) {
 		assert(textContainer);
-		textContainer->SetFlags(View::IgnoreEvents, OP_OR);
+		textContainer->SetFlags(View::IgnoreEvents, BitOp::OR);
 		textContainer->SetEventProxy(&scrollview);
 		SetEventProxy(&scrollview);
 	}
 }
 
 /** Sets the Actual Text */
-void TextArea::SetText(const String& text)
+void TextArea::SetText(String text)
 {
 	ClearText();
-	AppendText(text);
+	AppendText(std::move(text));
 }
 
 void TextArea::SetColor(const Color& color, COLOR_TYPE idx)
@@ -404,6 +401,7 @@ void TextArea::SetColor(const Color& color, COLOR_TYPE idx)
 	assert(idx < COLOR_TYPE_COUNT);
 	colors[idx] = color;
 	parser.ResetAttributes(ftext, {colors[COLOR_NORMAL], colors[COLOR_BACKGROUND]}, finit, {colors[COLOR_INITIALS], colors[COLOR_BACKGROUND]});
+	textContainer->SetColors(colors[COLOR_NORMAL], colors[COLOR_BACKGROUND]);
 }
 
 void TextArea::SetColor(const Color* color, COLOR_TYPE idx)
@@ -442,9 +440,9 @@ void TextArea::TrimHistory(size_t lines)
 	ClearHistoryTimer();
 }
 
-void TextArea::AppendText(const String& text)
+void TextArea::AppendText(String text)
 {
-	if ((flags&ClearHistory)) {
+	if (flags & ClearHistory) {
 		ClearHistoryTimer();
 
 		int heightLimit = (ftext->LineHeight * 100); // 100 lines of content
@@ -466,7 +464,7 @@ void TextArea::AppendText(const String& text)
 	} else if (text.length()) {
 		if (finit != ftext) {
 			// append cap spans
-			size_t textpos = text.find_first_not_of(WHITESPACE_STRING);
+			size_t textpos = text.find_first_not_of(WHITESPACE_STRING_W);
 			if (textpos != String::npos) {
 				// first append the white space as its own span
 				textContainer->AppendText(text.substr(0, textpos));
@@ -493,7 +491,7 @@ void TextArea::AppendText(const String& text)
 			}
 			textContainer->AppendText(text.substr(textpos));
 		} else {
-			textContainer->AppendText(text);
+			textContainer->AppendText(std::move(text));
 		}
 	}
 
@@ -517,7 +515,7 @@ int TextArea::InsertText(const char* text, int pos)
 }
 */
 
-ieWord TextArea::LineHeight() const
+int TextArea::LineHeight() const
 {
 	return ftext->LineHeight;
 }
@@ -538,7 +536,7 @@ void TextArea::ScrollToY(int y, ieDword duration)
 	scrollview.ScrollTo(Point(0, y), duration);
 }
 
-void TextArea::UpdateState(unsigned int optIdx)
+void TextArea::UpdateState(value_t opt)
 {
 	if (!selectOptions) {
 		// no selectable options present
@@ -547,25 +545,36 @@ void TextArea::UpdateState(unsigned int optIdx)
 		return;
 	}
 
-	if (!VarName[0]) {
-		return;
-	}
-	
-	if (optIdx >= selectOptions->NumOpts()) {
-		SetValue(-1);
+	auto it = std::find(values.begin(), values.end(), opt);
+	if (it == values.end()) {
+		SetValue(INVALID_VALUE);
 		selectOptions->MakeSelection(-1);
 		return;
 	}
+	
+	Option_t optIdx = std::distance(values.begin(), it);
+	
+	// only run the handlers if the context has changed, or the selection is different
+	// note that handlers can trigger reentrancy back here
+	if (optionContext != selectOptions->Context() || optIdx != selectOptions->SelectionIdx()) {
+		optionContext = selectOptions->Context();
 
-	assert(optIdx < values.size());
-	// always run the TextAreaOnSelect handler even if the value hasnt changed
-	// the *context* of the value can change (dialog) and the handler will want to know 
-	SetValue( values[optIdx] );
+		// this can be called from elsewhere (GUIScript), so we need to make sure we update the selected span
+		selectOptions->MakeSelection(optIdx);
+	
+		SetValue(values[optIdx]);
+		PerformAction(Action::Select);
+	}
+}
 
-	// this can be called from elsewhere (GUIScript), so we need to make sure we update the selected span
-	selectOptions->MakeSelection(optIdx);
-
-	PerformAction(Action::Select);
+void TextArea::UpdateStateWithSelection(Option_t optIdx)
+{
+	assert(selectOptions);
+	if (optIdx < selectOptions->NumOpts()) {
+		UpdateState(values[optIdx]);
+	} else {
+		UpdateState(INVALID_VALUE);
+	}
 }
 
 void TextArea::DidFocus()
@@ -586,17 +595,17 @@ void TextArea::AddSubviewInFrontOfView(View* front, const View* back)
 {
 	// we dont have a way of retrieving a TextArea's scrollview so
 	// we have no direct way of placing subviews in front of it so we let NULL represent it
-	const View* target = (back) ? back : &scrollview;
+	const View* target = back ? back : &scrollview;
 	View::AddSubviewInFrontOfView(front, target);
 }
 	
 int TextArea::TextHeight() const
 {
-	return (textContainer) ? textContainer->Dimensions().h : 0;
+	return textContainer ? textContainer->Dimensions().h : 0;
 }
 int TextArea::OptionsHeight() const
 {
-	return (selectOptions) ? selectOptions->Dimensions().h : 0;
+	return selectOptions ? selectOptions->Dimensions().h : 0;
 }
 
 int TextArea::ContentHeight() const
@@ -626,8 +635,9 @@ void TextArea::ClearSelectOptions()
 	delete scrollview.RemoveSubview(selectOptions);
 	dialogBeginNode = NULL;
 	selectOptions = NULL;
+	optionContext = OptionContext();
 
-	if (!core->HasFeature(GF_DIALOGUE_SCROLLS)) {
+	if (!core->HasFeature(GFFlags::DIALOGUE_SCROLLS)) {
 		UpdateScrollview();
 	}
 }
@@ -652,7 +662,7 @@ void TextArea::SetScrollbar(ScrollBar* sb)
 	SetFrame(combined);
 	SetMargins(margins);
 	
-	Point origin = ConvertPointFromWindow(sb->Frame().Origin());
+	Point origin = ConvertPointFromWindow(sb->Frame().origin);
 	sb->SetFrameOrigin(origin);
 
 	scrollview.SetVScroll(sb);
@@ -675,9 +685,9 @@ void TextArea::SetSelectOptions(const std::vector<SelectOption>& opts, bool numb
 	}
 
 	ContentContainer::Margin m;
-	size_t selectIdx = -1;
+	Option_t selectIdx = -1;
 	if (dialogBeginNode) {
-		if (AnimPicture)
+		if (speakerPic)
 			m = ContentContainer::Margin(10, 20);
 		else
 			m = ContentContainer::Margin(LineHeight(), 40, 10);
@@ -695,7 +705,7 @@ void TextArea::SetSelectOptions(const std::vector<SelectOption>& opts, bool numb
 	UpdateScrollview();
 }
 
-void TextArea::SelectAvailableOption(size_t idx)
+void TextArea::SelectAvailableOption(Option_t idx)
 {
 	if (selectOptions) {
 		selectOptions->MakeSelection(idx);
@@ -717,10 +727,10 @@ void TextArea::ClearText()
 	textContainer->SetMargin(textMargins);
 	textContainer->callback = METHOD_CALLBACK(&TextArea::TextChanged, this);
 	if (Flags()&Editable) {
-		textContainer->SetFlags(View::IgnoreEvents, OP_NAND);
+		textContainer->SetFlags(View::IgnoreEvents, BitOp::NAND);
 		SetEventProxy(textContainer);
 	} else {
-		textContainer->SetFlags(View::IgnoreEvents, OP_OR);
+		textContainer->SetFlags(View::IgnoreEvents, BitOp::OR);
 		textContainer->SetEventProxy(&scrollview);
 		SetEventProxy(&scrollview);
 	}
