@@ -21,35 +21,27 @@
 #include "Interface.h"
 #include "Palette.h"
 #include "Sprite2D.h"
-#include "System/String.h"
-#include "Video.h"
+#include "Strings/String.h"
+#include "Video/Video.h"
 
 #include <algorithm>
 #include <climits>
 
 namespace GemRB {
 
-Content::Content(const Size& size)
-	: frame(Point(0, 0), size)
-{
-	parent = NULL;
-}
-
 LayoutRegions Content::LayoutForPointInRegion(Point p, const Region& rgn) const
 {
-	return { std::make_shared<LayoutRegion>(Region(rgn.Origin() + p, frame.Dimensions())) };
+	return { std::make_shared<LayoutRegion>(Region(rgn.origin + p, frame.size)) };
 }
 
-TextSpan::TextSpan(const String& string, const Font* fnt, const Size* frame)
-	: Content((frame) ? *frame : Size()), text(string), font(fnt)
+TextSpan::TextSpan(String string, const Font* fnt, const Size* frame)
+	: Content(frame ? *frame : Size()), text(std::move(string)), font(fnt)
 {
-	Alignment = IE_FONT_ALIGN_LEFT;
 }
 
-TextSpan::TextSpan(const String& string, const Font* fnt, Font::PrintColors cols, const Size* frame)
-	: Content((frame) ? *frame : Size()), text(string), font(fnt), colors(new Font::PrintColors(cols))
+TextSpan::TextSpan(String string, const Font* fnt, Font::PrintColors cols, const Size* frame)
+	: Content(frame ? *frame : Size()), text(std::move(string)), font(fnt), colors(new Font::PrintColors(cols))
 {
-	Alignment = IE_FONT_ALIGN_LEFT;
 }
 
 TextSpan::~TextSpan()
@@ -72,35 +64,39 @@ inline const Font* TextSpan::LayoutFont() const
 {
 	if (font) return font;
 
-	TextContainer* container = static_cast<TextContainer*>(parent);
+	const TextContainer* container = static_cast<const TextContainer*>(parent);
 	if (container) {
 		return container->TextFont();
 	}
-	return NULL;
+	return nullptr;
 }
 
 inline Region TextSpan::LayoutInFrameAtPoint(const Point& p, const Region& rgn) const
 {
 	const Font* layoutFont = LayoutFont();
-	Size maxSize = frame.Dimensions();
-	Region drawRegion = Region(p + rgn.Origin(), maxSize);
+	Size maxSize = frame.size;
+	Region drawRegion = Region(p + rgn.origin, maxSize);
 	if (maxSize.w <= 0) {
 		if (maxSize.w == -1) {
 			// take remainder of parent width
 			drawRegion.w = rgn.w - p.x;
 			maxSize.w = drawRegion.w;
+			assert(drawRegion.w);
 		} else {
 			Font::StringSizeMetrics metrics = {maxSize, 0, 0, true};
 			drawRegion.w = layoutFont->StringSize(text, &metrics).w;
+			assert(drawRegion.w);
 		}
 	}
 	if (maxSize.h <= 0) {
 		if (maxSize.h == -1) {
 			// take remainder of parent height
 			drawRegion.h = rgn.w - p.y;
+			assert(drawRegion.h);
 		} else {
 			Font::StringSizeMetrics metrics = {maxSize, 0, 0, true};
 			drawRegion.h = layoutFont->StringSize(text, &metrics).h;
+			assert(drawRegion.h);
 		}
 	}
 	return drawRegion;
@@ -109,11 +105,11 @@ inline Region TextSpan::LayoutInFrameAtPoint(const Point& p, const Region& rgn)
 LayoutRegions TextSpan::LayoutForPointInRegion(Point layoutPoint, const Region& rgn) const
 {
 	LayoutRegions layoutRegions;
-	const Point& drawOrigin = rgn.Origin();
+	const Point& drawOrigin = rgn.origin;
 	const Font* layoutFont = LayoutFont();
 	assert(layoutFont);
 
-	if (frame.Dimensions().IsZero()) {
+	if (frame.size.IsZero()) {
 		// this means we get to wrap :)
 		// calculate each line and print line by line
 		int lineheight = layoutFont->LineHeight;
@@ -122,7 +118,7 @@ LayoutRegions TextSpan::LayoutForPointInRegion(Point layoutPoint, const Region&
 		lineRgn.y -= lineheight;
 		Region lineSegment;
 
-#define LINE_REMAINDER lineRgn.w - lineSegment.x;
+#define LINE_REMAINDER (lineRgn.w - lineSegment.x)
 		const Region* excluded = NULL;
 		size_t numPrinted = 0;
 		bool newline = true;
@@ -133,7 +129,7 @@ LayoutRegions TextSpan::LayoutForPointInRegion(Point layoutPoint, const Region&
 				lineRgn.x = drawOrigin.x;
 				lineRgn.y += lineheight;
 				lineRgn.w = rgn.w;
-				layoutPoint = lineRgn.Origin();
+				layoutPoint = lineRgn.origin;
 
 				lineSegment = lineRgn;
 			}
@@ -146,10 +142,9 @@ LayoutRegions TextSpan::LayoutForPointInRegion(Point layoutPoint, const Region&
 				excluded = parent->ContentRegionForRect(lineSegment);
 				if (!excluded) {
 					// now check if we already used this region ourselves
-					std::vector<Region>::const_iterator it;
-					for (it = lineExclusions.begin(); it != lineExclusions.end(); ++it) {
-						if (lineSegment.IntersectsRegion(*it)) {
-							excluded = &*it;
+					for (const auto& lineEx : lineExclusions) {
+						if (lineSegment.IntersectsRegion(lineEx)) {
+							excluded = &lineEx;
 							break;
 						}
 					}
@@ -193,7 +188,7 @@ LayoutRegions TextSpan::LayoutForPointInRegion(Point layoutPoint, const Region&
 						subLen = nextLine - numPrinted + 1; // +1 for the \n
 					}
 					const String& substr = text.substr(numPrinted, subLen);
-					Font::StringSizeMetrics metrics = {lineSegment.Dimensions(), 0, 0, lineSegment.w == lineRgn.w};
+					Font::StringSizeMetrics metrics = {lineSegment.size, 0, 0, lineSegment.w == lineRgn.w};
 					Size printSize = layoutFont->StringSize(substr, &metrics);
 					numOnLine = metrics.numChars;
 					assert(numOnLine || !metrics.forceBreak);
@@ -217,13 +212,13 @@ LayoutRegions TextSpan::LayoutForPointInRegion(Point layoutPoint, const Region&
 				}
 				numPrinted += numOnLine;
 			}
-			assert(!lineSegment.Dimensions().IsEmpty());
+			assert(!lineSegment.size.IsInvalid());
 			lineExclusions.push_back(lineSegment);
 
 		newline:
 			if (newline || numPrinted == text.length()) {
 				// must claim the lineExclusions as part of the layout
-				// just because we didnt fit doesnt mean somethng else wont...
+				// just because we didnt fit doesn't mean something else won't...
 				Region lineLayout = Region::RegionEnclosingRegions(lineExclusions);
 				assert(lineLayout.h % lineheight == 0);
 				layoutRegions.emplace_back(std::make_shared<TextLayoutRegion>(lineLayout, begin, end));
@@ -239,10 +234,10 @@ LayoutRegions TextSpan::LayoutForPointInRegion(Point layoutPoint, const Region&
 		// FIXME: we ought to be able to set an alignment for "blocks" of text
 		// probably the way to do this is have alignment on the container
 		// then maybe another Draw method that takes an alignment argument?
-
-		Region drawRegion = LayoutInFrameAtPoint(layoutPoint, rgn);
-		assert(drawRegion.h && drawRegion.w);
-		layoutRegions.emplace_back(std::make_shared<TextLayoutRegion>(drawRegion, 0, text.length()));
+		if (!text.empty()) { // kachiko.dlg has a completely void response 224
+			Region drawRegion = LayoutInFrameAtPoint(layoutPoint, rgn);
+			layoutRegions.emplace_back(std::make_shared<TextLayoutRegion>(drawRegion, 0, text.length()));
+		}
 	}
 	return layoutRegions;
 }
@@ -256,7 +251,7 @@ void TextSpan::DrawContentsInRegions(const LayoutRegions& rgns, const Point& off
 		drawRect.y += offset.y;
 		const Font* printFont = LayoutFont();
 		const Font::PrintColors* pc = colors;
-		TextContainer* container = static_cast<TextContainer*>(parent);
+		const TextContainer* container = static_cast<const TextContainer*>(parent);
 		if (!pc && container) {
 			pc = container->TextColors();
 		}
@@ -282,8 +277,8 @@ void TextSpan::DrawContentsInRegions(const LayoutRegions& rgns, const Point& off
 	}
 }
 
-ImageSpan::ImageSpan(Holder<Sprite2D> im)
-	: Content(im->Frame.Dimensions())
+ImageSpan::ImageSpan(const Holder<Sprite2D>& im)
+	: Content(im->Frame.size)
 {
 	image = im;
 }
@@ -294,7 +289,7 @@ void ImageSpan::DrawContentsInRegions(const LayoutRegions& rgns, const Point& of
 	Region r = rgns.front()->region;
 	r.x += offset.x;
 	r.y += offset.y;
-	core->GetVideoDriver()->BlitSprite(image, r.Origin(), &r);
+	core->GetVideoDriver()->BlitSprite(image, r.origin, &r);
 }
 
 ContentContainer::ContentContainer(const Region& frame)
@@ -349,7 +344,7 @@ void ContentContainer::DidDraw(const Region& /*drawFrame*/, const Region& clip)
 	core->GetVideoDriver()->SetScreenClip(&clip);
 }
 
-void ContentContainer::DrawSelf(Region drawFrame, const Region& clip)
+void ContentContainer::DrawSelf(const Region& drawFrame, const Region& clip)
 {
 	Video* video = core->GetVideoDriver();
 	if (core->InDebugMode(ID_TEXT)) {
@@ -366,7 +361,7 @@ void ContentContainer::DrawSelf(Region drawFrame, const Region& clip)
 
 	// layout shouldn't be empty unless there is no content anyway...
 	if (layout.empty()) return;
-	Point dp = drawFrame.Origin() + Point(margin.left, margin.top);
+	Point dp = drawFrame.origin + Point(margin.left, margin.top);
 	
 	ContentLayout::const_iterator it = layout.begin();
 	for (; it != layout.end(); ++it) {
@@ -377,9 +372,9 @@ void ContentContainer::DrawSelf(Region drawFrame, const Region& clip)
 	}
 }
 
-void ContentContainer::DrawContents(const Layout& layout, Point point)
+void ContentContainer::DrawContents(const Layout& contentLayout, Point point)
 {
-	layout.content->DrawContentsInRegions(layout.regions, point);
+	contentLayout.content->DrawContentsInRegions(contentLayout.regions, point);
 }
 
 void ContentContainer::AppendContent(Content* content)
@@ -407,10 +402,10 @@ void ContentContainer::InsertContentAfter(Content* newContent, const Content* ex
 void ContentContainer::SizeChanged(const Size& /*oldSize*/)
 {
 	if (frame.w <= 0) {
-		SetFlags(RESIZE_WIDTH, OP_OR);
+		SetFlags(RESIZE_WIDTH, BitOp::OR);
 	}
 	if (frame.h <= 0) {
-		SetFlags(RESIZE_HEIGHT, OP_OR);
+		SetFlags(RESIZE_HEIGHT, BitOp::OR);
 	}
 	LayoutContentsFrom(contents.begin());
 }
@@ -442,7 +437,7 @@ Content* ContentContainer::RemoveContent(const Content* span, bool doLayout)
 		content->parent = NULL;
 		layout.erase(std::find(layout.begin(), layout.end(), content));
 
-		layoutPoint = Point(); // reset cached layoutPoint
+		layoutPoint.reset(); // reset cached layoutPoint
 		if (doLayout) {
 			LayoutContentsFrom(it);
 		}
@@ -459,7 +454,7 @@ ContentContainer::EraseContent(ContentList::iterator it)
 	Content* content = *it;
 	content->parent = NULL;
 	layout.erase(std::find(layout.begin(), layout.end(), content));
-	layoutPoint = Point(); // reset cached layoutPoint
+	layoutPoint.reset(); // reset cached layoutPoint
 	ContentRemoved(content);
 	delete content;
 
@@ -477,11 +472,11 @@ ContentContainer::EraseContent(ContentList::iterator beg, ContentList::iterator
 
 Content* ContentContainer::ContentAtPoint(const Point& p) const
 {
-	const Layout* layout = LayoutAtPoint(p);
-	if (layout) {
+	const Layout* contentLayout = LayoutAtPoint(p);
+	if (contentLayout) {
 		// i know we are casting away const.
 		// we could return std::find(contents.begin(), contents.end(), *it) instead, but whats the point?
-		return (Content*)(layout->content);
+		return (Content*)(contentLayout->content);
 	}
 	return NULL;
 }
@@ -507,7 +502,7 @@ const ContentContainer::Layout* ContentContainer::LayoutAtPoint(const Point& p)
 			++it;
 			count -= step + 1;
 		} else {
-			std::advance(it, -step);
+			std::advance(it, -std::make_signed<size_t>::type(step));
 			count = step;
 		}
 	}
@@ -581,7 +576,7 @@ void ContentContainer::LayoutContentsFrom(ContentList::const_iterator it)
 	for (; clearit != contents.end(); ++clearit) {
 		ContentLayout::iterator i = std::find(layout.begin(), layout.end(), *clearit);
 		if (i != layout.end()) {
-			layoutPoint = Point(); // reset cached layoutPoint
+			layoutPoint.reset(); // reset cached layoutPoint
 			// since 'layout' is sorted alongsize 'contents' we should be able clear everyting following 'i' and bail
 			layout.erase(i, layout.end());
 			break;
@@ -601,7 +596,7 @@ void ContentContainer::LayoutContentsFrom(ContentList::const_iterator it)
 		layoutFrame.h -= margin.top + margin.bottom;
 	}
 
-	assert(!layoutFrame.Dimensions().IsEmpty());
+	assert(!layoutFrame.size.IsInvalid());
 	while (it != contents.end()) {
 		const Content* content = *it++;
 		while (exContent) {
@@ -626,7 +621,8 @@ void ContentContainer::LayoutContentsFrom(ContentList::const_iterator it)
 			assert(exContent != content);
 		}
 		const LayoutRegions& rgns = content->LayoutForPointInRegion(layoutPoint, layoutFrame);
-		layout.push_back(Layout(content, rgns));
+		if (rgns.empty()) return;
+		layout.emplace_back(content, rgns);
 		exContent = content;
 
 		ieDword flags = Flags();
@@ -649,13 +645,13 @@ void ContentContainer::LayoutContentsFrom(ContentList::const_iterator it)
 	ResizeSubviews(oldSize);
 }
 
-void ContentContainer::DeleteContentsInRect(Region exclusion)
+void ContentContainer::DeleteContentsInRect(const Region& exclusion)
 {
 	int top = exclusion.y;
 	int bottom = top;
 	const Content* content;
 	while (const Region* rgn = ContentRegionForRect(exclusion)) {
-		content = ContentAtPoint(rgn->Origin());
+		content = ContentAtPoint(rgn->origin);
 		assert(content);
 
 		top = (rgn->y < top) ? rgn->y : top;
@@ -678,10 +674,6 @@ void ContentContainer::DeleteContentsInRect(Region exclusion)
 TextContainer::TextContainer(const Region& frame, Font* fnt)
 	: ContentContainer(frame), font(fnt)
 {
-	alignment = IE_FONT_ALIGN_LEFT;
-	textLen = 0;
-	cursorPos = 0;
-	printPos = 0;
 }
 
 TextContainer::~TextContainer()
@@ -689,21 +681,22 @@ TextContainer::~TextContainer()
 	delete colors;
 }
 
-void TextContainer::AppendText(const String& text)
+void TextContainer::AppendText(String text)
 {
-	AppendText(text, nullptr, colors);
+	AppendText(std::move(text), nullptr, colors);
 }
 
-void TextContainer::AppendText(const String& text, const Font* fnt, const Font::PrintColors* cols)
+void TextContainer::AppendText(String text, const Font* fnt, const Font::PrintColors* cols)
 {
-	if (text.length()) {
-		TextSpan* span = new TextSpan(text, fnt);
+	size_t len = text.length();
+	if (len) {
+		TextSpan* span = new TextSpan(std::move(text), fnt);
 		if (cols) {
 			span->SetColors(cols->fg, cols->bg);
 		}
 		span->Alignment = alignment;
 		AppendContent(span);
-		textLen += text.length();
+		textLen += len;
 
 		MarkDirty();
 	}
@@ -753,7 +746,7 @@ String TextContainer::TextFrom(ContentList::const_iterator it) const
 	return text;
 }
 
-void TextContainer::DrawSelf(Region drawFrame, const Region& clip)
+void TextContainer::DrawSelf(const Region& drawFrame, const Region& clip)
 {
 	printPos = 0;
 	ContentContainer::DrawSelf(drawFrame, clip);
@@ -766,7 +759,7 @@ void TextContainer::DrawSelf(Region drawFrame, const Region& clip)
 }
 
 LayoutRegions::const_iterator
-TextContainer::FindCursorRegion(const Layout& layout)
+TextContainer::FindCursorRegion(const Layout& layout) const
 {
 	auto end = layout.regions.end();
 	for (auto it = layout.regions.begin(); it != end; ++it) {
@@ -785,9 +778,9 @@ void TextContainer::DrawContents(const Layout& layout, Point dp)
 
 	const TextSpan* ts = (const TextSpan*)layout.content;
 	const String& text = ts->Text();
-	size_t textLen = ts->Text().length();
+	size_t textLength = ts->Text().length();
 
-	if (Editable() && printPos <= cursorPos && printPos + textLen >= cursorPos) {
+	if (Editable() && printPos <= cursorPos && printPos + textLength >= cursorPos) {
 		const Font* printFont = ts->LayoutFont();
 		
 		auto it = FindCursorRegion(layout);
@@ -795,7 +788,7 @@ void TextContainer::DrawContents(const Layout& layout, Point dp)
 			auto cursorRegion = static_cast<const TextLayout&>(**it);
 			size_t begin = cursorRegion.beginCharIdx;
 			const Region& rect = cursorRegion.region;
-			cursorPoint = rect.Origin();
+			cursorPoint = rect.origin;
 			const String& substr = text.substr(begin, cursorPos - begin);
 			cursorPoint.x += printFont->StringSizeWidth(substr, 0);
 		}
@@ -804,7 +797,7 @@ void TextContainer::DrawContents(const Layout& layout, Point dp)
 		dp.y += cursor->Frame.y;
 		core->GetVideoDriver()->BlitSprite(cursor, cursorPoint + dp);
 	}
-	printPos += textLen;
+	printPos += textLength;
 }
 
 void TextContainer::SizeChanged(const Size& oldSize)
@@ -850,7 +843,7 @@ void TextContainer::MoveCursorToPoint(const Point& p)
 			} else {
 				// not in this one so we need to consume some text
 				// TODO: this could be faster if we stored it while calculating layout
-				metrics.size = rect.Dimensions();
+				metrics.size = rect.size;
 				printFont->StringSize(text.substr(numChars), &metrics);
 				numChars += metrics.numChars;
 			}
@@ -1003,12 +996,12 @@ TextContainer::ContentIndex TextContainer::FindContentForChar(size_t idx)
 	size_t charCount = 0;
 	ContentList::iterator it = contents.begin();
 	while (it != contents.end()) {
-		TextSpan* ts = static_cast<TextSpan*>(*it);
-		size_t textLen = ts->Text().length();
-		if (charCount + textLen >= idx) {
+		const TextSpan* ts = static_cast<const TextSpan*>(*it);
+		size_t textLength = ts->Text().length();
+		if (charCount + textLength >= idx) {
 			break;
 		}
-		charCount += textLen;
+		charCount += textLength;
 		++it;
 	}
 	return std::make_pair(charCount, it);
@@ -1027,7 +1020,7 @@ void TextContainer::InsertText(const String& text)
 	}
 
 	EraseContent(idx.second, contents.end());
-	AppendText(newtext);
+	AppendText(std::move(newtext));
 	AdvanceCursor(int(text.length()));
 
 	if (callback) {
@@ -1045,7 +1038,7 @@ void TextContainer::DeleteText(size_t len)
 	}
 
 	EraseContent(idx.second, contents.end());
-	AppendText(newtext);
+	AppendText(std::move(newtext));
 	AdvanceCursor(-int(len));
 
 	if (callback) {
