@@ -24,14 +24,16 @@
 #include "GUI/TextSystem/Font.h"
 #include "Interface.h"
 #include "Sprite2D.h"
-#include "Video.h"
+#include "Video/Video.h"
 
 #include <typeinfo>
+#include <utility>
+
 
 namespace GemRB {
 
 View::DragOp::DragOp(View* v, Holder<Sprite2D> cursor)
-: dragView(v), cursor(cursor)
+: dragView(v), cursor(std::move(cursor))
 {}
 
 View::DragOp::~DragOp() {
@@ -40,15 +42,7 @@ View::DragOp::~DragOp() {
 
 View::View(const Region& frame)
 	: frame(frame)
-{
-	eventProxy = NULL;
-	superView = NULL;
-	window = NULL;
-
-	dirty = true;
-	flags = 0;
-	autoresizeFlags = ResizeNone;
-}
+{}
 
 View::~View()
 {
@@ -57,17 +51,16 @@ View::~View()
 	if (superView) {
 		superView->RemoveSubview(this);
 	}
-	std::list<View*>::iterator it;
-	for (it = subViews.begin(); it != subViews.end(); ++it) {
-		View* view = *it;
-		view->superView = NULL;
-		delete view;
+
+	for (const auto& subView : subViews) {
+		subView->superView = nullptr;
+		delete subView;
 	}
 }
 
 void View::SetBackground(Holder<Sprite2D> bg, const Color* c)
 {
-	background = bg;
+	background = std::move(bg);
 	if (c) backgroundColor = *c;
 
 	MarkDirty();
@@ -75,7 +68,7 @@ void View::SetBackground(Holder<Sprite2D> bg, const Color* c)
 
 void View::SetCursor(Holder<Sprite2D> c)
 {
-	cursor = c;
+	cursor = std::move(c);
 }
 
 void View::SetEventProxy(View* proxy)
@@ -95,29 +88,27 @@ void View::MarkDirty(const Region* rgn)
 	// TODO: we could implement partial redraws by storing the dirty region
 	// not much to gain at the moment, however
 
-	if (dirty == false) {
-		dirty = true;
+	if (dirty) return;
 
-		if (superView && !IsOpaque()) {
-			superView->DirtyBGRect(frame);
-		}
+	dirty = true;
 
-		std::list<View*>::iterator it;
-		for (it = subViews.begin(); it != subViews.end(); ++it) {
-			View* view = *it;
-			if (rgn) {
-				Region intersect = view->frame.Intersect(*rgn);
-				const Size& idims = intersect.Dimensions();
-				if (!idims.IsEmpty()) {
-					Point p = view->ConvertPointFromSuper(intersect.Origin());
-					Region r = Region(p, idims);
-					view->MarkDirty(&r);
-				}
-			} else {
-				Point p = view->ConvertPointFromSuper(Point());
-				Region r = Region(p, Dimensions());
+	if (superView && !IsOpaque()) {
+		superView->DirtyBGRect(frame);
+	}
+
+	for (const auto& view : subViews) {
+		if (rgn) {
+			Region intersect = view->frame.Intersect(*rgn);
+			const Size& idims = intersect.size;
+			if (!idims.IsInvalid()) {
+				Point p = view->ConvertPointFromSuper(intersect.origin);
+				Region r = Region(p, idims);
 				view->MarkDirty(&r);
 			}
+		} else {
+			Point p = view->ConvertPointFromSuper(Point());
+			Region r = Region(p, Dimensions());
+			view->MarkDirty(&r);
 		}
 	}
 }
@@ -130,7 +121,7 @@ void View::MarkDirty()
 bool View::NeedsDraw() const
 {
 	// cull anything that can't be seen
-	if (frame.Dimensions().IsEmpty() || (flags&Invisible)) return false;
+	if (frame.size.IsInvalid() || (flags&Invisible)) return false;
 
 	// check ourselves
 	if (dirty || IsAnimated()) {
@@ -180,32 +171,34 @@ bool View::IsReceivingEvents() const
 	return getEvents;
 }
 
-void View::DirtyBGRect(const Region& r)
+void View::DirtyBGRect(const Region& r, bool force) noexcept
 {
 	// no need to draw the parent BG for opaque views
 	if (superView && !IsOpaque()) {
-		Region rgn = frame.Intersect(Region(ConvertPointToSuper(r.Origin()), r.Dimensions()));
-		superView->DirtyBGRect(rgn);
+		Region rgn = frame.Intersect(Region(ConvertPointToSuper(r.origin), r.size));
+		superView->DirtyBGRect(rgn, force);
 	}
 
 	// if we are going to draw the entire BG, no need to compute and store this
-	if (NeedsDrawRecursive())
+	if (!force && NeedsDrawRecursive())
 		return;
 
 	// do we want to intersect this too?
 	//Region bgRgn = Region(background->Frame.x, background->Frame.y, background->Frame.w, background->Height);
 	Region clip(Point(), Dimensions());
-	Region dirty = r.Intersect(clip);
-	dirtyBGRects.push_back(dirty);
+	Region dirtyRect = r.Intersect(clip);
+	dirtyBGRects.push_back(dirtyRect);
 	
-	MarkDirty(&dirty);
+	MarkDirty(&dirtyRect);
 }
 
-void View::DrawSubviews() const
+void View::DrawSubviews()
 {
-	std::list<View*>::const_iterator it;
-	for (it = subViews.begin(); it != subViews.end(); ++it) {
-		(*it)->Draw();
+	for (View* subview : subViews) {
+		subview->Draw();
+		if (subview->IsAnimated() && !subview->IsOpaque()) {
+			DirtyBGRect(subview->frame, true);
+		}
 	}
 }
 
@@ -219,11 +212,11 @@ void View::DrawBackground(const Region* rgn) const
 	Video* video = core->GetVideoDriver();
 	if (backgroundColor.a > 0) {
 		if (rgn) {
-			Point p = ConvertPointToWindow(rgn->Origin());
-			video->DrawRect(Region(p, rgn->Dimensions()), backgroundColor, true);
+			Point p = ConvertPointToWindow(rgn->origin);
+			video->DrawRect(Region(p, rgn->size), backgroundColor, true);
 		} else if (window) {
 			assert(superView);
-			Point p = superView->ConvertPointToWindow(frame.Origin());
+			Point p = superView->ConvertPointToWindow(frame.origin);
 			video->DrawRect(Region(p, Dimensions()), backgroundColor, true);
 		} else {
 			// FIXME: this is a Window and we need this hack becasue Window::WillDraw() changed the coordinate system
@@ -238,9 +231,9 @@ void View::DrawBackground(const Region* rgn) const
 	if (background) {
 		if (rgn) {
 			Region intersect = rgn->Intersect(background->Frame);
-			Point screenPt = ConvertPointToWindow(intersect.Origin());
-			Region toClip(screenPt, intersect.Dimensions());
-			video->BlitSprite(background, intersect, toClip, BlitFlags::BLENDED);
+			Point screenPt = ConvertPointToWindow(intersect.origin);
+			Region toClip(screenPt, intersect.size);
+			video->BlitSprite(background, intersect, std::move(toClip), BlitFlags::BLENDED);
 		} else {
 			Point dp = ConvertPointToWindow(Point(background->Frame.x, background->Frame.y));
 			video->BlitSprite(background, dp);
@@ -256,7 +249,7 @@ void View::Draw()
 	const Region clip = video->GetScreenClip();
 	const Region& drawFrame = DrawingFrame();
 	const Region& intersect = clip.Intersect(drawFrame);
-	if (intersect.Dimensions().IsEmpty()) return; // outside the window/screen
+	if (intersect.size.IsInvalid()) return; // outside the window/screen
 
 	// clip drawing to the view bounds, then restore after drawing
 	video->SetScreenClip(&intersect);
@@ -297,22 +290,24 @@ void View::Draw()
 		if (debuginfo) {
 			const ViewScriptingRef* ref = GetScriptingRef();
 			if (ref) {
-				Font* fnt = core->GetTextFont();
+				const Font* fnt = core->GetTextFont();
 				ScriptingId id = ref->Id;
 				id &= 0x00000000ffffffff; // control id is lower 32bits
 
-				wchar_t string[256];
-				swprintf(string, 255, L"id: %lu  grp: %s  \nflgs: %u\ntype:%s",
-					 id, ref->ScriptingGroup().CString(), flags, typeid(*this).name());
+				std::string formatted = fmt::format("id: {}  grp: {}  \nflgs: {}\ntype:{}", id, ref->ScriptingGroup(), flags, typeid(*this).name());
+				const String* string = StringFromCString(formatted.c_str());
+				assert(string);
+				
 				Region r = drawFrame;
-				r.w = (win) ? win->Frame().w - r.x : Frame().w - r.x;
-				Font::StringSizeMetrics metrics = {r.Dimensions(), 0, 0, true};
-				fnt->StringSize(string, &metrics);
+				r.w = win ? win->Frame().w - r.x : Frame().w - r.x;
+				Font::StringSizeMetrics metrics = {r.size, 0, 0, true};
+				fnt->StringSize(*string, &metrics);
 				r.h = metrics.size.h;
 				r.w = metrics.size.w;
 				video->SetScreenClip(nullptr);
 				video->DrawRect(r, ColorBlack, true);
-				fnt->Print(r, string, IE_FONT_ALIGN_TOP|IE_FONT_ALIGN_LEFT, {ColorWhite, ColorBlack});
+				fnt->Print(r, *string, IE_FONT_ALIGN_TOP|IE_FONT_ALIGN_LEFT, {ColorWhite, ColorBlack});
+				delete string;
 			}
 		}
 	}
@@ -371,59 +366,63 @@ Point View::ConvertPointFromScreen(const Point& p) const
 
 Region View::ConvertRegionToSuper(Region r) const
 {
-	r.SetOrigin(ConvertPointToSuper(r.Origin()));
+	r.origin = ConvertPointToSuper(r.origin);
 	return r;
 }
 
 Region View::ConvertRegionFromSuper(Region r) const
 {
-	r.SetOrigin(ConvertPointFromSuper(r.Origin()));
+	r.origin = ConvertPointFromSuper(r.origin);
 	return r;
 }
 
 Region View::ConvertRegionToWindow(Region r) const
 {
-	r.SetOrigin(ConvertPointToWindow(r.Origin()));
+	r.origin = ConvertPointToWindow(r.origin);
 	return r;
 }
 
 Region View::ConvertRegionFromWindow(Region r) const
 {
-	r.SetOrigin(ConvertPointFromWindow(r.Origin()));
+	r.origin = ConvertPointFromWindow(r.origin);
 	return r;
 }
 
 Region View::ConvertRegionToScreen(Region r) const
 {
-	r.SetOrigin(ConvertPointToScreen(r.Origin()));
+	r.origin = ConvertPointToScreen(r.origin);
 	return r;
 }
 
 Region View::ConvertRegionFromScreen(Region r) const
 {
-	r.SetOrigin(ConvertPointFromScreen(r.Origin()));
+	r.origin = ConvertPointFromScreen(r.origin);
 	return r;
 }
 
 void View::AddSubviewInFrontOfView(View* front, const View* back)
 {
-	if (front == NULL) return;
+	if (front == nullptr) return;
+	
+	// remember, being "in front" means coming after in the list
+	auto it = subViews.begin();
+	if (back != nullptr) {
+		it = std::find(subViews.begin(), subViews.end(), back);
+		assert(it != subViews.end());
+		++it;
+	}
 
-	std::list<View*>::iterator it;
-	it = std::find(subViews.begin(), subViews.end(), back);
-
-	View* super = front->superView;
+	const View* super = front->superView;
 	if (super == this) {
 		// already here, but may need to move the view
-		std::list<View*>::iterator cur;
-		cur = std::find(subViews.begin(), subViews.end(), front);
+		auto cur = std::find(subViews.begin(), subViews.end(), front);
 		subViews.splice(it, subViews, cur);
 	} else {
-		if (super != NULL) {
+		if (super != nullptr) {
 			front->superView->RemoveSubview(front);
 		}
-		// remember, being "in front" means coming after in the list
-		subViews.insert(++it, front);
+		
+		subViews.insert(it, front);
 	}
 
 	front->superView = this;
@@ -438,7 +437,7 @@ void View::AddSubviewInFrontOfView(View* front, const View* back)
 	front->AddedToView(this);
 }
 
-View* View::RemoveSubview(const View* view)
+View* View::RemoveSubview(const View* view) noexcept
 {
 	if (!view || view->superView != this) {
 		return NULL;
@@ -477,10 +476,8 @@ View* View::RemoveFromSuperview()
 void View::AddedToWindow(Window* newwin)
 {
 	window = newwin;
-	std::list<View*>::iterator it;
-	for (it = subViews.begin(); it != subViews.end(); ++it) {
-		View* subview = *it;
-		subview->AddedToWindow(newwin);
+	for (const auto& subView : subViews) {
+		subView->AddedToWindow(newwin);
 	}
 }
 
@@ -495,9 +492,9 @@ void View::AddedToView(View* view)
 	}
 }
 
-void View::RemovedFromView(View*)
+void View::RemovedFromView(const View*)
 {
-	window = NULL;
+	window = nullptr;
 }
 
 bool View::IsOpaque() const
@@ -511,6 +508,10 @@ bool View::IsOpaque() const
 
 bool View::HitTest(const Point& p) const
 {
+	if (flags & (IgnoreEvents | Invisible)) {
+		return false;
+	}
+
 	Region r(Point(), Dimensions());
 	if (!r.PointInside(p)) {
 		return false;
@@ -525,14 +526,13 @@ bool View::HitTest(const Point& p) const
 View* View::SubviewAt(const Point& p, bool ignoreTransparency, bool recursive)
 {
 	// iterate backwards because the backmost control would be drawn on top
-	std::list<View*>::reverse_iterator it;
-	for (it = subViews.rbegin(); it != subViews.rend(); ++it) {
+	for (auto it = subViews.rbegin(); it != subViews.rend(); ++it) {
 		View* v = *it;
 		Point subP = v->ConvertPointFromSuper(p);
 		if ((ignoreTransparency && v->frame.PointInside(p)) || v->HitTest(subP)) {
 			if (recursive) {
 				View* subV = v->SubviewAt(subP, ignoreTransparency, recursive);
-				v = (subV) ? subV : v;
+				v = subV ? subV : v;
 			}
 			return v;
 		}
@@ -550,13 +550,11 @@ bool View::ContainsView(const View* view) const
 		return true;
 	}
 
-	std::list<View*>::const_iterator it;
-	for (it = subViews.begin(); it != subViews.end(); ++it) {
-		const View* subview = *it;
-		if (subview == view) {
+	for (const auto& subView : subViews) {
+		if (subView == view) {
 			return true;
 		}
-		if (subview->ContainsView(view)) {
+		if (subView->ContainsView(view)) {
 			return true;
 		}
 	}
@@ -570,71 +568,68 @@ Window* View::GetWindow() const
 
 	if (superView) {
 		Window* win = dynamic_cast<Window*>(superView);
-		return (win) ? win : superView->GetWindow();
+		return win ? win : superView->GetWindow();
 	}
 	return NULL;
 }
 
 void View::ResizeSubviews(const Size& oldSize)
 {
-	std::list<View*>::iterator it;
-	for (it = subViews.begin(); it != subViews.end(); ++it) {
-		View* subview = *it;
-		unsigned short flags = subview->AutoResizeFlags();
+	for (const auto& subView : subViews) {
+		unsigned short arFlags = subView->AutoResizeFlags();
 
-		if (flags == ResizeNone)
+		if (arFlags == ResizeNone)
 			continue;
 
-		Region newSubFrame = subview->Frame();
+		Region newSubFrame = subView->Frame();
 		int delta = frame.w - oldSize.w;
 
-		if (flags&ResizeRight) {
-			if (flags&ResizeLeft) {
+		if (arFlags & ResizeRight) {
+			if (arFlags & ResizeLeft) {
 				newSubFrame.w += delta;
 			} else {
 				newSubFrame.x += delta;
 			}
-		} else if (flags&ResizeLeft) {
+		} else if (arFlags & ResizeLeft) {
 			newSubFrame.x += delta;
 		}
 
 		delta = frame.h - oldSize.h;
-		if (flags&ResizeBottom) {
-			if (flags&ResizeTop) {
+		if (arFlags & ResizeBottom) {
+			if (arFlags & ResizeTop) {
 				newSubFrame.h += delta;
 			} else {
 				newSubFrame.y += delta;
 			}
-		} else if (flags&ResizeTop) {
+		} else if (arFlags & ResizeTop) {
 			newSubFrame.y += delta;
 		}
 
-		subview->SetFrame(newSubFrame);
+		subView->SetFrame(newSubFrame);
 	}
 	MarkDirty();
 }
 
 void View::SetFrame(const Region& r)
 {
-	SetFrameOrigin(r.Origin());
-	SetFrameSize(r.Dimensions());
+	SetFrameOrigin(r.origin);
+	SetFrameSize(r.size);
 }
 
 void View::SetFrameOrigin(const Point& p)
 {
-	Point oldP = frame.Origin();
+	Point oldP = frame.origin;
 	if (oldP == p) return;
 	
 	MarkDirty(); // refresh the old position in the superview
-	frame.x = p.x;
-	frame.y = p.y;
+	frame.origin = p;
 	
 	OriginChanged(oldP);
 }
 
 void View::SetFrameSize(const Size& s)
 {
-	const Size oldSize = frame.Dimensions();
+	const Size oldSize = frame.size;
 	if (oldSize == s) return;
 
 	MarkDirty(); // refresh the old position in the superview
@@ -646,7 +641,7 @@ void View::SetFrameSize(const Size& s)
 	SizeChanged(oldSize);
 }
 
-bool View::SetFlags(unsigned int arg_flags, int opcode)
+bool View::SetFlags(unsigned int arg_flags, BitOp opcode)
 {
 	unsigned int oldflags = flags;
 	bool ret = SetBits(flags, arg_flags, opcode);
@@ -665,7 +660,7 @@ bool View::SetFlags(unsigned int arg_flags, int opcode)
 	return ret;
 }
 	
-bool View::SetAutoResizeFlags(unsigned short arg_flags, int opcode)
+bool View::SetAutoResizeFlags(unsigned short arg_flags, BitOp opcode)
 {
 	return SetBits(autoresizeFlags, arg_flags, opcode);
 }
@@ -903,7 +898,7 @@ bool View::OnControllerButtonUp(const ControllerEvent& ce)
 	return OnKeyRelease(ke, 0);
 }
 
-const ViewScriptingRef* View::ReplaceScriptingRef(const ViewScriptingRef* old, ScriptingId id, ResRef group)
+const ViewScriptingRef* View::ReplaceScriptingRef(const ViewScriptingRef* old, ScriptingId id, const ScriptingGroup_t& group)
 {
 	std::vector<ViewScriptingRef*>::iterator it = std::find(scriptingRefs.begin(), scriptingRefs.end(), old);
 	if (it != scriptingRefs.end()) {
@@ -929,12 +924,11 @@ const ViewScriptingRef* View::RemoveScriptingRef(const ViewScriptingRef* ref)
 	return ReplaceScriptingRef(ref, id++, "__DEL__");
 }
 	
-void View::ClearScriptingRefs()
+void View::ClearScriptingRefs() noexcept
 {
-	std::vector<ViewScriptingRef*>::iterator rit;
-	for (rit = scriptingRefs.begin(); rit != scriptingRefs.end();) {
+	for (auto rit = scriptingRefs.begin(); rit != scriptingRefs.end();) {
 		ViewScriptingRef* ref = *rit;
-		assert(GetView(ref) == this);
+		assert(ref->GetObject() == this);
 		bool unregistered = ScriptEngine::UnregisterScriptingRef(ref);
 		assert(unregistered);
 		delete ref;
@@ -942,12 +936,12 @@ void View::ClearScriptingRefs()
 	}
 }
 	
-ViewScriptingRef* View::CreateScriptingRef(ScriptingId id, ResRef group)
+ViewScriptingRef* View::CreateScriptingRef(ScriptingId id, ScriptingGroup_t group)
 {
 	return new ViewScriptingRef(this, id, group);
 }
 	
-const ViewScriptingRef* View::AssignScriptingRef(ScriptingId id, ResRef group)
+const ViewScriptingRef* View::AssignScriptingRef(ScriptingId id, const ScriptingGroup_t& group)
 {
 	ViewScriptingRef* ref = CreateScriptingRef(id, group);
 	if (ScriptEngine::RegisterScriptingRef(ref)) {
@@ -959,7 +953,7 @@ const ViewScriptingRef* View::AssignScriptingRef(ScriptingId id, ResRef group)
 	}
 }
 
-const ViewScriptingRef* View::GetScriptingRef(ScriptingId id, ResRef group) const
+const ViewScriptingRef* View::GetScriptingRef(ScriptingId id, ScriptingGroup_t group) const
 {
 	auto it = std::find_if(scriptingRefs.begin(), scriptingRefs.end(), [&](const ViewScriptingRef* ref) {
 		return ref->Id == id && ref->ScriptingGroup() == group;
