@@ -22,6 +22,7 @@
 #include "Interface.h"
 #include "ImageMgr.h"
 #include "Window.h"
+#include "GUI/GameControl.h"
 
 #include "defsounds.h"
 
@@ -30,8 +31,8 @@ std::find(windows.begin(), windows.end(), w)
 
 namespace GemRB {
 
-int WindowManager::ToolTipDelay = 500;
-unsigned long WindowManager::TooltipTime = 0;
+tick_t WindowManager::ToolTipDelay = 500;
+tick_t WindowManager::TooltipTime = 0;
 
 Holder<Sprite2D> WindowManager::CursorMouseUp;
 Holder<Sprite2D> WindowManager::CursorMouseDown;
@@ -41,17 +42,13 @@ void WindowManager::SetTooltipDelay(int delay)
 	ToolTipDelay = delay;
 }
 
-WindowManager::WindowManager(Video* vid)
+WindowManager::WindowManager(const std::shared_ptr<Video>& vid)
 : tooltip(core->CreateTooltip())
 {
 	assert(vid);
 
 	cursorFeedback = MOUSE_ALL;
 
-	hoverWin = NULL;
-	modalWin = NULL;
-	trackingWin = NULL;
-
 	EventMgr::EventCallback cb = METHOD_CALLBACK(&WindowManager::DispatchEvent, this);
 	eventMgr.RegisterEventMonitor(cb);
 
@@ -64,7 +61,7 @@ WindowManager::WindowManager(Video* vid)
 	vid->SetEventMgr(&eventMgr);
 
 	gameWin = new Window(screen, *this);
-	gameWin->SetFlags(Window::Borderless|View::Invisible, OP_SET);
+	gameWin->SetFlags(Window::Borderless|View::Invisible, BitOp::SET);
 	gameWin->SetFrame(screen);
 
 	HUDBuf = vid->CreateBuffer(screen, Video::BufferFormat::DISPLAY_ALPHA);
@@ -77,26 +74,32 @@ WindowManager::WindowManager(Video* vid)
 	// TODO: how do we get notified if the Video driver changes size?
 }
 
+void WindowManager::MarkAllDirty() const
+{
+	for (auto& window : windows) {
+		window->MarkDirty();
+	}
+}
+
 WindowManager::~WindowManager()
 {
-	DestroyWindows(closedWindows);
+	CloseAllWindows();
+	
+	DestroyClosedWindows();
 	assert(closedWindows.empty());
-	DestroyWindows(windows);
-	assert(windows.empty());
 
-	video.release();
 	delete gameWin;
 }
 
-void WindowManager::DestroyWindows(WindowList& list)
+void WindowManager::DestroyClosedWindows()
 {
-	WindowList::iterator it = list.begin();
-	for (; it != list.end();) {
+	WindowList::iterator it = closedWindows.begin();
+	while (it != closedWindows.end()) {
 		Window* win = *it;
-		// IMPORTANT: ensure the window (a control subview) isnt executing a callback before deleting it
+		// IMPORTANT: ensure the window (a control subview) isn't executing a callback before deleting it
 		if (win->InActionHandler() == false) {
 			delete win;
-			it = list.erase(it);
+			it = closedWindows.erase(it);
 		} else {
 			++it;
 		}
@@ -109,21 +112,20 @@ bool WindowManager::IsOpenWindow(Window* win) const
 	return it != windows.end();
 }
 
-bool WindowManager::IsPresentingModalWindow() const
+Window* WindowManager::ModalWindow() const
 {
-	return modalWin != NULL;
+	Window* fwin = windows.front();
+	return fwin->Flags() & Window::Modal ? fwin : nullptr;
 }
 
 /** Show a Window in Modal Mode */
-bool WindowManager::PresentModalWindow(Window* win, ModalShadow shadow)
+bool WindowManager::PresentModalWindow(Window* win)
 {
 	if (!IsOpenWindow( win )) return false;
 
 	OrderFront(win);
 	win->SetDisabled(false);
-	win->SetFlags(Window::Modal, OP_OR);
-	modalWin = win;
-	modalShadow = shadow;
+	win->SetFlags(Window::Modal, BitOp::OR);
 
 	if (win->Flags() & Window::Borderless && !(win->Flags() & Window::NoSounds)) {
 		core->PlaySound(DS_WINDOW_OPEN, SFX_CHAN_GUI);
@@ -141,7 +143,17 @@ WindowManager::CursorFeedback WindowManager::SetCursorFeedback(CursorFeedback fe
 /** Sets a Window on the Top */
 bool WindowManager::FocusWindow(Window* win)
 {
-	if (!IsPresentingModalWindow() && OrderFront(win)) {
+	// we can only focus the window if it is above the topmost modal window
+	const Window* modalWin = ModalWindow();
+	if (modalWin && win != modalWin) {
+		auto modalIt = WIN_IT(modalWin);
+		auto winIt = std::find(modalIt, windows.end(), win);
+		if (winIt == windows.end()) {
+			return false;
+		}
+	}
+	
+	if (OrderFront(win)) {
 		if (gameWin == win) {
 			core->SetEventFlag(EF_CONTROL);
 		}
@@ -153,14 +165,14 @@ bool WindowManager::FocusWindow(Window* win)
 
 bool WindowManager::OrderFront(Window* win)
 {
-	assert(windows.size()); // win should be contained in windows
+	assert(!windows.empty()); // win should be contained in windows
 	win->SetVisible(true);
 	return OrderRelativeTo(win, windows.front(), true);
 }
 
 bool WindowManager::OrderBack(Window* win)
 {
-	assert(windows.size()); // win should be contained in windows
+	assert(!windows.empty()); // win should be contained in windows
 	return OrderRelativeTo(win, windows.back(), false);
 }
 
@@ -170,18 +182,19 @@ bool WindowManager::OrderRelativeTo(Window* win, Window* win2, bool front)
 		return false;
 	}
 	// FIXME: this should probably account for modal windows
-	// shouldnt beable to move non modals in front of modals, nor one modal to infront of another
+	// shouldn't be able to move non modals in front of modals, nor one modal to in front of another
 
 	Window* oldFront = windows.front();
 	// if we only have one window, or the 2 windows are the same it is an automatic success
 	if (windows.size() > 1 && win != win2) {
-		WindowList::iterator it = WIN_IT(win), it2 = WIN_IT(win2);
+		WindowList::iterator it = WIN_IT(win);
+		WindowList::iterator it2 = WIN_IT(win2);
 		if (it == windows.end() || it2 == windows.end()) return false;
 
 		windows.erase(it);
 		// it2 may have become invalid after erase
 		it2 = WIN_IT(win2);
-		windows.insert((front) ? it2 : ++it2, win);
+		windows.insert(front ? it2 : ++it2, win);
 	}
 
 	Window* frontWin = windows.front();
@@ -213,18 +226,20 @@ bool WindowManager::OrderRelativeTo(Window* win, Window* win2, bool front)
 
 Window* WindowManager::MakeWindow(const Region& rgn)
 {
-	DestroyWindows(closedWindows);
+	DestroyClosedWindows();
 
 	Window* win = new Window(rgn, *this);
 	windows.push_back(win);
 	return win;
 }
 
-//this function won't delete the window, just queue it for deletion
-//it will be deleted when another window is opened
-//regardless, the window deleted is inaccessible for gui scripts and
-//other high level functions from now
-// this is a caching mechanisim in case the window is reopened
+// this function won't delete the window, just queue it for deletion
+// it will be deleted when another window is opened
+// regardless, the closed window is inaccessible for gui scripts and
+// other high level functions from now
+// this is a necessity to prevent crashing when a control callback results
+// in the parent window closing which would delete the control before
+// returning from the callback
 void WindowManager::CloseWindow(Window* win)
 {
 	if (win == nullptr || win == gameWin) {
@@ -234,22 +249,12 @@ void WindowManager::CloseWindow(Window* win)
 	WindowList::iterator it = WIN_IT(win);
 	if (it == windows.end()) return;
 
-	if (win == modalWin) {
+	if (win == ModalWindow()) {
 		if (win->Flags() & Window::Borderless && !(win->Flags() & Window::NoSounds)) {
 			core->PlaySound(DS_WINDOW_CLOSE, SFX_CHAN_GUI);
 		}
 
-		WindowList::iterator mit = it;
-		win->SetFlags(Window::Modal, OP_NAND);
-		// find the next modal window
-		modalWin = NULL;
-		while (++mit != windows.end()) {
-			if ((*mit)->Flags() & Window::Modal) {
-				modalWin = *mit;
-				modalWin->FocusGained();
-				break;
-			}
-		}
+		win->SetFlags(Window::Modal, BitOp::NAND);
 	}
 
 	if (win == hoverWin) {
@@ -279,18 +284,16 @@ void WindowManager::CloseWindow(Window* win)
 	win->SetDisabled(true);
 }
 
-void WindowManager::DestroyAllWindows()
+void WindowManager::CloseAllWindows() const
 {
-	WindowList::iterator it = windows.begin();
-	for (; it != windows.end(); ++it) {
-		Window* win = *it;
-		win->SetFlags(Window::DestroyOnClose, OP_OR); // force delete
+	for (Window* win : WindowList(windows)) {
+		win->SetFlags(Window::DestroyOnClose, BitOp::OR); // force delete
 		win->Close();
-		if (windows.empty()) break;
 	}
+	assert(windows.empty());
 }
 
-bool WindowManager::HotKey(const Event& event)
+bool WindowManager::HotKey(const Event& event) const
 {
 	if (event.type == Event::KeyDown && event.keyboard.repeats == 1) {
 		switch (event.keyboard.keycode) {
@@ -309,8 +312,8 @@ bool WindowManager::HotKey(const Event& event)
 
 Window* WindowManager::GetFocusWindow() const
 {
-	if (IsPresentingModalWindow()) {
-		return modalWin;
+	if (Window* mwin = ModalWindow()) {
+		return mwin;
 	}
 
 	for (Window* win : windows) {
@@ -330,18 +333,18 @@ Window* WindowManager::GetFocusWindow() const
 Window* WindowManager::NextEventWindow(const Event& event, WindowList::const_iterator& current)
 {
 	if (current == windows.end()) {
-		// we already we through them all and returned gameWin or modalWin once. there is no target window after gameWin
+		// we already went through them all and returned gameWin or modalWin once. There is no target window after gameWin
 		return NULL;
 	}
 
-	if (IsPresentingModalWindow()) {
+	if (Window* mwin = ModalWindow()) {
 		// modal win is always the target for all events no matter what
-		// if the window shouldnt handle sreen events outside its bounds (ie negative coords etc)
+		// if the window shouldn't handle screen events outside its bounds (ie. negative coords)
 		// then the Window class should be responsible for bounds checking
 
 		// the NULL return is so that if this is called again after returning modalWindow there is no NextTarget
 		current = windows.end(); // invalidate the iterator, no other target is possible.
-		return modalWin;
+		return mwin;
 	}
 	
 	if (event.isScreen) {
@@ -365,17 +368,20 @@ Window* WindowManager::NextEventWindow(const Event& event, WindowList::const_ite
 
 bool WindowManager::DispatchEvent(const Event& event)
 {
+	if (event.type == Event::EventType::RedrawRequest) {
+		MarkAllDirty();
+		return true;
+	}
+
 	if (eventMgr.MouseDown() == false && eventMgr.FingerDown() == false) {
 		if (event.type == Event::MouseUp || event.type == Event::TouchUp) {
-			if (trackingWin) {
-				if (trackingWin->IsDisabled() == false) {
-					trackingWin->DispatchEvent(event);
-				}
+			// we don't deliver mouse up events if there isn't a corresponding mouse down (no trackingWin).
+			if (!trackingWin) return false;
 
-				trackingWin = NULL;
+			if (!trackingWin->IsDisabled() && trackingWin->IsVisible()) {
+				trackingWin->DispatchEvent(event);
 			}
-
-			// we don't deliver mouse up events if there isn't a corresponding mouse down (trackingWin == NULL).
+			trackingWin = nullptr;
 			return false;
 		}
 
@@ -392,7 +398,7 @@ bool WindowManager::DispatchEvent(const Event& event)
 	if (windows.empty()) return false;
 
 	if (event.EventMaskFromType(event.type) & Event::AllMouseMask) {
-		TooltipTime = GetTicks();
+		TooltipTime = GetMilliseconds();
 		
 		// handle when mouse leaves the window
 		if (hoverWin && HIT_TEST(event, hoverWin) == false) {
@@ -401,7 +407,7 @@ bool WindowManager::DispatchEvent(const Event& event)
 		}
 	// handled here instead of as a hotkey, so also gamecontrol can do its thing
 	} else if (event.type == Event::KeyDown && event.keyboard.keycode == GEM_TAB) {
-		if (TooltipTime + ToolTipDelay > GetTicks()) {
+		if (TooltipTime + ToolTipDelay > GetMilliseconds()) {
 			TooltipTime -= ToolTipDelay;
 		}
 	}
@@ -417,7 +423,7 @@ bool WindowManager::DispatchEvent(const Event& event)
 				}
 			} else if ((target->Flags()&(View::IgnoreEvents|View::Disabled)) == View::Disabled
 					   && event.type == Event::KeyDown && event.keyboard.keycode == GEM_ESCAPE) {
-				// force close disabled windows if they arent also ignoreing events
+				// force close disabled windows if they aren't also ignoring events
 				target->Close();
 			}
 			return true;
@@ -434,9 +440,19 @@ void WindowManager::DrawMouse() const
 	if (cursorFeedback == MOUSE_NONE)
 		return;
 
-	Point pos = eventMgr.MousePos();
-	DrawCursor(pos);
-	DrawTooltip(pos);
+	Point cursorPos = eventMgr.MousePos();
+	Point tooltipPos = cursorPos;
+
+	// pst displays actor name tooltips overhead, not at the mouse position
+	const GameControl* gc = core->GetGameControl();
+	if (core->HasFeature(GFFlags::ONSCREEN_TEXT) && gc) {
+		tooltipPos.y -= gc->GetOverheadOffset();
+	}
+
+	if (tooltip.tt.TextSize().IsZero() || core->HasFeature(GFFlags::ONSCREEN_TEXT)) {
+		DrawCursor(cursorPos);
+	}
+	DrawTooltip(tooltipPos);
 }
 
 void WindowManager::DrawCursor(const Point& pos) const
@@ -463,7 +479,7 @@ void WindowManager::DrawCursor(const Point& pos) const
 	assert(cur); // must have a cursor
 
 	if (hoverWin && hoverWin->IsDisabledCursor()) {
-		// draw greayed cursor
+		// draw greyed cursor
 		video->BlitGameSprite(cur, pos, BlitFlags::GREY|BlitFlags::BLENDED, ColorGray);
 	} else {
 		// draw normal cursor
@@ -477,15 +493,15 @@ void WindowManager::DrawTooltip(Point pos) const
 		return;
 	}
 
-	if (trackingWin) // if the mouse is held down we dont want tooltips
-		TooltipTime = GetTicks();
+	if (trackingWin) // if the mouse is held down we don't want tooltips
+		TooltipTime = GetMilliseconds();
 
 	if (tooltip.time != TooltipTime + ToolTipDelay) {
 		tooltip.time = TooltipTime + ToolTipDelay;
 		tooltip.reset = true;
 	}
 
-	if (hoverWin && TooltipTime && GetTicks() >= tooltip.time) {
+	if (hoverWin && TooltipTime && GetMilliseconds() >= tooltip.time) {
 		if (tooltip.reset) {
 			// reset the tooltip and restart the sound
 			const String& text = hoverWin->TooltipText();
@@ -507,16 +523,18 @@ void WindowManager::DrawTooltip(Point pos) const
 		pos.y = Clamp<int>(pos.y, halfW, screen.h - halfH);
 
 		tooltip.tt.Draw(pos);
+	} else {
+		tooltip.tt.SetText(L"");
 	}
 }
 
 void WindowManager::DrawWindowFrame(BlitFlags flags) const
 {
-	// the window buffers dont have room for the frame
+	// the window buffers don't have room for the frame
 	// we also only need to draw the frame *once* (even if it applies to multiple windows)
 	// therefore, draw the frame on its own buffer (above everything else)
 	// ... I'm not 100% certain this works for all use cases.
-	// if it doesnt... i think it might be better to just forget about the window frames once the game is loaded
+	// if it doesn't... i think it might be better to just forget about the window frames once the game is loaded
 
 	video->SetScreenClip( NULL );
 
@@ -541,7 +559,7 @@ void WindowManager::DrawWindowFrame(BlitFlags flags) const
 	}
 }
 
-WindowManager::HUDLock WindowManager::DrawHUD()
+WindowManager::HUDLock WindowManager::DrawHUD() const
 {
 	return HUDLock(*this);
 }
@@ -550,11 +568,11 @@ void WindowManager::DrawWindows() const
 {
 	HUDBuf->Clear();
 
-	if (!windows.size()) {
+	if (windows.empty()) {
 		return;
 	}
 
-	// draw the game window now (beneath everything else); its not part of the windows collection
+	// draw the game window now (beneath everything else); it's not part of the windows collection
 	if (gameWin->IsVisible()) {
 		gameWin->Draw();
 	} else {
@@ -566,7 +584,8 @@ void WindowManager::DrawWindows() const
 	}
 
 	bool drawFrame = false;
-	Window* frontWin = windows.front();
+	const Window* frontWin = windows.front();
+	Window* modalWin = ModalWindow();
 	const Region& frontWinFrame = frontWin->Frame();
 	// we have to draw windows from the bottom up so the front window is drawn last
 	WindowList::const_reverse_iterator rit = windows.rbegin();
@@ -600,7 +619,7 @@ void WindowManager::DrawWindows() const
 
 		if (win->IsDisabled() && win->NeedsDraw()) {
 			// Important to only draw if the window itself is dirty
-			// controls on greyed out windows shouldnt be updating anyway
+			// controls on greyed out windows shouldn't be updating anyway
 			win->Draw();
 			Region winrgn(Point(), win->Dimensions());
 			video->DrawRect(winrgn, ColorBlack, true, BlitFlags::HALFTRANS|BlitFlags::BLENDED);
@@ -613,8 +632,8 @@ void WindowManager::DrawWindows() const
 
 	BlitFlags frame_flags = BlitFlags::NONE;
 	if (modalWin) {
-		if (modalShadow != ShadowNone) {
-			if (modalShadow == ShadowGray) {
+		if (modalWin->modalShadow != Window::ModalShadow::None) {
+			if (modalWin->modalShadow == Window::ModalShadow::Gray) {
 				frame_flags |= BlitFlags::HALFTRANS;
 			}
 			video->DrawRect(screen, ColorBlack, true, frame_flags);
@@ -628,7 +647,7 @@ void WindowManager::DrawWindows() const
 	}
 	
 	if (core->InDebugMode(ID_WINDOWS)) {
-		// ensure this is drawin over the window frames
+		// ensure this is drawing over the window frames
 		if (trackingWin) {
 			Region r = trackingWin->Frame();
 			r.ExpandAllSides(5);
@@ -656,7 +675,7 @@ void WindowManager::DrawWindows() const
 Holder<Sprite2D> WindowManager::GetScreenshot(Window* win)
 {
 	Holder<Sprite2D> screenshot;
-	if (win) { // we dont really care if we are managing the window
+	if (win) { // we don't really care if we are managing the window
 		// only a screen shot of passed win
 		auto& winBuf = win->DrawWithoutComposition();
 		screenshot = video->GetScreenshot( Region(Point(), win->Dimensions()), winBuf );
@@ -674,41 +693,40 @@ Holder<Sprite2D> WindowManager::GetScreenshot(Window* win)
 
 Holder<Sprite2D> WindowManager::WinFrameEdge(int edge) const
 {
-	std::string refstr = "STON";
+	ResRef refstr = "STON";
 
 	// we probably need a HasResource("CSTON" + width + height) call
 	// to check for a custom resource
 
 	if (screen.w >= 800 && screen.w < 1024)
-		refstr += "08";
+		refstr.Append("08");
 	else if (screen.w >= 1024)
-		refstr += "10";
+		refstr.Append("10");
 
 	switch (edge) {
 		case 0:
-			refstr += "L";
+			refstr.Append("L");
 			break;
 		case 1:
-			refstr += "R";
+			refstr.Append("R");
 			break;
 		case 2:
-			refstr += "T";
+			refstr.Append("T");
 			break;
 		case 3:
-			refstr += "B";
+			refstr.Append("B");
 			break;
 	}
 
-	ResRef ref = refstr.c_str();
 	Holder<Sprite2D> frame;
-	if (winframes.find(ref) != winframes.end()) {
-		frame = winframes[ref];
+	if (winframes.find(refstr) != winframes.end()) {
+		frame = winframes[refstr];
 	} else {
-		ResourceHolder<ImageMgr> im = GetResourceHolder<ImageMgr>(ref);
+		auto im = gamedata->GetResourceHolder<ImageMgr>(refstr);
 		if (im) {
 			frame = im->GetSprite2D();
 		}
-		winframes.insert(std::make_pair(ref, frame));
+		winframes.emplace(refstr, frame);
 	}
 
 	return frame;
