@@ -23,28 +23,19 @@
 
 #include "exports.h"
 
+#include "Interface.h"
 #include "SymbolMgr.h"
-#include "Variables.h"
 #include "Scriptable/Actor.h"
-#include "System/DataStream.h"
+#include "Streams/DataStream.h"
 
 #include <cstdio>
 #include <vector>
 
-// absent from msvc6
-#ifdef _MSC_VER
-#ifndef __FUNCTION__
-#define __FUNCTION__ "no message"
-#endif
-#endif
-
 namespace GemRB {
 
 class Action;
 class GameScript;
 
-class StringBuffer;
-
 //escapearea flags
 #define EA_DESTROY 1        //destroy actor at the exit (otherwise move to new place)
 #define EA_NOSEE   2        //no need to see the exit
@@ -130,7 +121,13 @@ class StringBuffer;
 #define SC_AURA_CHECK   32
 #define SC_NOINTERRUPT  64
 
-#define ACF_REALLOW_SCRIPTS 1
+#define ACF_OVERRIDE 1 // was this action invoked via ActionOverride?
+#define ACF_REALLOW_SCRIPTS 0x1000 // gemrb internal
+#define ACF_FOLLOW_DONE 0x10000000 // Bubb: written during CGameSprite::Follow(), I believe it means the MoveToPoint() ended with ACTION_DONE
+// NOTE: if it ever becomes useful, this is where it came into play
+// CGameSprite::Follow() is used in the Follow action, as well as when Leader() is used in
+// combination with FollowObjectFormation(), Formation(), MoveToObject(), and MoveToPoint().
+// Leader() is also used internally when processing the "Follow the Leader" group formation.
 
 //trigger flags stored in triggers in .bcs files
 #define TF_NEGATE  1   //negate trigger result
@@ -140,28 +137,21 @@ class StringBuffer;
 #define MAX_OBJECT_FIELDS	10
 #define MAX_NESTING		5
 
-typedef std::vector<ieDword> SrcVector;
+using StringParam = FixedSizeString<64, strnicmp>;
+static_assert(std::is_standard_layout<StringParam>::value, "Fixed Size String must be standard layout for use in unions");
 
 struct targettype {
 	Scriptable *actor; //hmm, could be door
 	unsigned int distance;
 };
 
-typedef std::list<targettype> targetlist;
+using targetlist = std::list<targettype>;
 
 class GEM_EXPORT Targets {
-public:
-	Targets()
-	{
-	}
-
-	~Targets()
-	{
-		Clear();
-	}
-private:
 	targetlist objects;
 public:
+	Targets() noexcept {};
+	
 	int Count() const;
 	void dump() const;
 	targettype *RemoveTargetAt(targetlist::iterator &m);
@@ -178,11 +168,13 @@ class Canary {
 private:
 	volatile unsigned long canary;
 protected:
-	Canary() // protected constructor
+	Canary() noexcept // protected constructor
 	{
 		canary = (unsigned long) 0xdeadbeef;
 	}
-	~Canary() // protected destructor
+	Canary(const Canary&) = delete;
+	Canary& operator=(const Canary&) = delete;
+	virtual ~Canary() noexcept // protected destructor
 	{
 		AssertCanary("Destroying Canary");
 		canary = 0xdddddddd;
@@ -190,7 +182,7 @@ protected:
 	void AssertCanary(const char* msg) const
 	{
 		if (!CheckCanary()) {
-			error("Canary Died", "Canary([0x%08lX]) != 0xdeadbeef. Message: %s\n", canary, msg);
+			error("Canary Died", "Canary([{:#10x}]) != 0xdeadbeef. Message: {}", canary, msg);
 		}
 	}
 	bool CheckCanary() const
@@ -201,21 +193,20 @@ protected:
 
 class GEM_EXPORT Object : protected Canary {
 public:
-	Object()
-	{
-		memset( objectName, 0, 65 );
-		memset( objectFields, 0, MAX_OBJECT_FIELDS * sizeof( int ) );
-		memset( objectFilters, 0, MAX_NESTING * sizeof( int ) );
-	}
-public:
-	int objectFields[MAX_OBJECT_FIELDS];
-	int objectFilters[MAX_NESTING];
+	int objectFields[MAX_OBJECT_FIELDS]{};
+	int objectFilters[MAX_NESTING]{};
 	Region objectRect{};
-	char objectName[65];
+	
+	union {
+		StringParam objectName;
+		ieVariable objectNameVar;
+		ResRef objectNameRef;
+	};
 
 public:
-	void dump() const;
-	void dump(StringBuffer&) const;
+	Object() noexcept : objectName() {};
+
+	std::string dump(bool print = true) const;
 	void Release()
 	{
 		delete this;
@@ -223,41 +214,39 @@ public:
 	bool isNull() const;
 };
 
-class GEM_EXPORT Trigger : protected Canary {
+class GEM_EXPORT Trigger final : protected Canary {
 public:
-	Trigger()
-	{
-		triggerID = 0;
-		flags = 0;
-		objectParameter = NULL;
-		memset(string0Parameter, 0, 65);
-		memset(string1Parameter, 0, 65);
-		int0Parameter = 0;
-		int1Parameter = 0;
-		int2Parameter = 0;
-		pointParameter.null();
-	}
-	~Trigger()
+	Trigger() noexcept : string0Parameter(), string1Parameter() {};
+	~Trigger() final
 	{
 		if (objectParameter) {
 			objectParameter->Release();
-			objectParameter = NULL;
+			objectParameter = nullptr;
 		}
 	}
 	int Evaluate(Scriptable *Sender) const;
 
-	unsigned short triggerID;
-	int int0Parameter;
-	int flags;
-	int int1Parameter;
-	int int2Parameter;
+	unsigned short triggerID = 0;
+	int int0Parameter = 0;
+	int flags = 0;
+	int int1Parameter = 0;
+	int int2Parameter = 0;
 	Point pointParameter;
-	char string0Parameter[65];
-	char string1Parameter[65];
-	Object* objectParameter;
+	Object* objectParameter = nullptr;
+	
+	union {
+		StringParam string0Parameter;
+		ieVariable variable0Parameter;
+		ResRef resref0Parameter;
+	};
+	
+	union {
+		StringParam string1Parameter;
+		ieVariable variable1Parameter;
+		ResRef resref1Parameter;
+	};
 
-	void dump() const;
-	void dump(StringBuffer&) const;
+	std::string dump() const;
 
 	void Release()
 	{
@@ -265,14 +254,14 @@ public:
 	}
 };
 
-class GEM_EXPORT Condition : protected Canary {
+class GEM_EXPORT Condition final : protected Canary {
 public:
-	~Condition()
+	~Condition() noexcept
 	{
-		for (size_t c = 0; c < triggers.size(); ++c) {
-			if (triggers[c]) {
-				triggers[c]->Release();
-				triggers[c] = NULL;
+		for (auto& trigger : triggers) {
+			if (trigger) {
+				trigger->Release();
+				trigger = nullptr;
 			}
 		}
 	}
@@ -285,62 +274,62 @@ public:
 	std::vector<Trigger*> triggers;
 };
 
-class GEM_EXPORT Action : protected Canary {
+class GEM_EXPORT Action final : protected Canary {
 public:
-	Action(bool autoFree)
+	explicit Action(bool autoFree) noexcept
+	: string0Parameter(), string1Parameter()
 	{
-		actionID = 0;
-		objects[0] = NULL;
-		objects[1] = NULL;
-		objects[2] = NULL;
-		memset(string0Parameter, 0, 65);
-		memset(string1Parameter, 0, 65);
-		int0Parameter = 0;
-		pointParameter.null();
-		int1Parameter = 0;
-		int2Parameter = 0;
 		//changed now
 		if (autoFree) {
 			RefCount = 0; //refcount will be increased by each AddAction
 		} else {
-			RefCount = 1; //one reference hold by the script
+			RefCount = 1; //one reference held by the script
 		}
-		flags = 0;
 	}
-	~Action()
+	~Action() noexcept
 	{
-		for (int c = 0; c < 3; c++) {
-			if (objects[c]) {
-				objects[c]->Release();
-				objects[c] = NULL;
+		for (auto& object : objects) {
+			if (object) {
+				object->Release();
+				object = nullptr;
 			}
 		}
 	}
 
-	unsigned short actionID;
-	Object* objects[3];
-	int int0Parameter;
+	unsigned short actionID = 0;
+	Object* objects[3]{};
+	int int0Parameter = 0;
 	Point pointParameter;
-	int int1Parameter;
-	int int2Parameter;
-	char string0Parameter[65];
-	char string1Parameter[65];
-	unsigned short flags;
+	int int1Parameter = 0;
+	int int2Parameter = 0;
+	
+	union {
+		StringParam string0Parameter; // keep largest type first to 0 fill everythings
+		ieVariable variable0Parameter;
+		ResRef resref0Parameter;
+	};
+	
+	union {
+		StringParam string1Parameter; // keep largest type first to 0 fill everythings
+		ieVariable variable1Parameter;
+		ResRef resref1Parameter;
+	};
+
+	uint32_t flags = 0;
 private:
-	int RefCount;
+	int RefCount = 0;
 public:
-	int GetRef() {
+	int GetRef() const {
 		return RefCount;
 	}
 
-	void dump() const;
-	void dump(StringBuffer&) const;
+	std::string dump() const;
 
 	void Release()
 	{
-		AssertCanary(__FUNCTION__);
+		AssertCanary(__func__);
 		if (!RefCount) {
-			error("GameScript", "WARNING!!! Double Freeing in %s: Line %d\n", __FILE__,
+			error("GameScript", "WARNING!!! Double Freeing in {}: Line {}", __FILE__,
 				__LINE__);
 		}
 		RefCount--;
@@ -350,30 +339,27 @@ public:
 	}
 	void IncRef()
 	{
-		AssertCanary(__FUNCTION__);
+		AssertCanary(__func__);
 		RefCount++;
 		if (RefCount >= 65536) {
-			error("GameScript", "Refcount increased to: %d in action %d\n", RefCount,
+			error("GameScript", "Refcount increased to: {} in action {}", RefCount,
 				actionID);
 		}
 	}
 };
 
-class GEM_EXPORT Response : protected Canary {
+class GEM_EXPORT Response final : protected Canary {
 public:
-	Response()
+	Response() noexcept = default;
+	~Response() noexcept
 	{
-		weight = 0;
-	}
-	~Response()
-	{
-		for (size_t c = 0; c < actions.size(); c++) {
-			if (actions[c]) {
-				if (actions[c]->GetRef()>2) {
-					print("Residue action %d with refcount %d", actions[c]->actionID, actions[c]->GetRef());
+		for (auto& action : actions) {
+			if (action) {
+				if (action->GetRef() > 2) {
+					Log(DEBUG, "GameScript", "Residue action {} with refcount {}", action->actionID, action->GetRef());
 				}
-				actions[c]->Release();
-				actions[c] = NULL;
+				action->Release();
+				action = nullptr;
 			}
 		}
 	}
@@ -383,17 +369,17 @@ public:
 	}
 	int Execute(Scriptable* Sender);
 
-	unsigned char weight;
+	unsigned char weight = 0;
 	std::vector<Action*> actions;
 };
 
-class GEM_EXPORT ResponseSet : protected Canary {
+class GEM_EXPORT ResponseSet final : protected Canary {
 public:
-	~ResponseSet()
+	~ResponseSet() final
 	{
-		for (size_t b = 0; b < responses.size(); b++) {
-			responses[b]->Release();
-			responses[b] = NULL;
+		for (auto& response : responses) {
+			response->Release();
+			response = nullptr;
 		}
 	}
 	void Release()
@@ -405,22 +391,18 @@ public:
 	std::vector<Response*> responses;
 };
 
-class GEM_EXPORT ResponseBlock : protected Canary {
+class GEM_EXPORT ResponseBlock final : protected Canary {
 public:
-	ResponseBlock()
-	{
-		condition = NULL;
-		responseSet = NULL;
-	}
-	~ResponseBlock()
+	ResponseBlock() noexcept = default;
+	~ResponseBlock() noexcept
 	{
 		if (condition) {
 			condition->Release();
-			condition = NULL;
+			condition = nullptr;
 		}
 		if (responseSet) {
 			responseSet->Release();
-			responseSet = NULL;
+			responseSet = nullptr;
 		}
 	}
 	void Release()
@@ -428,18 +410,18 @@ public:
 		delete this;
 	}
 
-	Condition* condition;
-	ResponseSet* responseSet;
+	Condition* condition = nullptr;
+	ResponseSet* responseSet = nullptr;
 };
 
-class GEM_EXPORT Script : protected Canary {
+class GEM_EXPORT Script final : protected Canary {
 public:
-	~Script()
+	~Script() noexcept
 	{
-		for (unsigned int i = 0; i < responseBlocks.size(); i++) {
-			if (responseBlocks[i]) {
-				responseBlocks[i]->Release();
-				responseBlocks[i] = NULL;
+		for (auto& responseBlock : responseBlocks) {
+			if (responseBlock) {
+				responseBlock->Release();
+				responseBlock = nullptr;
 			}
 		}
 	}
@@ -452,10 +434,10 @@ public:
 	}
 };
 
-typedef int (* TriggerFunction)(Scriptable *, const Trigger *);
-typedef void (* ActionFunction)(Scriptable*, Action*);
-typedef Targets *(* ObjectFunction)(const Scriptable *, Targets*, int ga_flags);
-typedef int (* IDSFunction)(const Actor *, int parameter);
+using TriggerFunction = int (*)(Scriptable*, const Trigger*);
+using ActionFunction = void (*)(Scriptable*, Action*);
+using ObjectFunction = Targets* (*)(const Scriptable*, Targets*, int ga_flags);
+using IDSFunction = int (*)(const Actor*, int parameter);
 
 #define TF_NONE		0
 #define TF_CONDITION    1 //this isn't a trigger, just a condition (0x4000)
@@ -515,6 +497,13 @@ struct TriggerLink {
 #define AF_DLG_INSTANT  4096 //instant dialog actions
 #define AF_SCR_INSTANT  8192 //instant script actions
 #define AF_INSTANT      (AF_DLG_INSTANT|AF_SCR_INSTANT) //only iwd2 treats them separately; 12288
+#define AF_IWD2_OVERRIDE 16384 // marking actions that require special attention when clearing during ActionOverride
+
+enum RunAwayFlags {
+	LeaveArea = 1,
+	NoInterrupt = 2,
+	UsePoint = 4
+};
 
 struct ActionLink {
 	const char* Name;
@@ -537,27 +526,35 @@ struct IDSLink {
 #define MAX_OBJECTS			256
 #define AI_SCRIPT_LEVEL 4             //the script level of special ai scripts
 
-extern void ScriptDebugLog(int bit, const char* message, ...);
+template<typename... ARGS>
+extern void ScriptDebugLog(int bit, const char* message, ARGS&&... args)
+{
+	if (!core->InDebugMode(bit)) return;
+
+	Log(DEBUG, "GameScript", message, std::forward<ARGS>(args)...);
+}
 extern int RandomNumValue;
 
 class GEM_EXPORT GameScript {
 public:
-	GameScript(const ieResRef ResRef, Scriptable* Myself,
+	GameScript(const ResRef& ResRef, Scriptable* Myself,
 		int ScriptLevel = 0, bool AIScript = false);
+	GameScript(const GameScript&) = delete;
 	~GameScript();
+	GameScript& operator=(const GameScript&) = delete;
 
 	bool dead = false;      // Script replaced itself with another and should be deleted when done running
 	bool running = false;   // Script is currently running so defer any deletion to caller
 
-	const char *GetName() const { return Name; }
-	static void ExecuteString(Scriptable* Sender, const char* String);
-	static int EvaluateString(Scriptable* Sender, char* String);
+	ResRef GetName() const { return Name; }
+	static void ExecuteString(Scriptable* Sender, std::string string);
+	static int EvaluateString(Scriptable* Sender, const char* String);
 	static void ExecuteAction(Scriptable* Sender, Action* aC);
 
 	bool Update(bool *continuing = NULL, bool *done = NULL);
-	void EvaluateAllBlocks();
+	void EvaluateAllBlocks(bool testConditions = false);
 private: //Internal Functions
-	Script* CacheScript(ieResRef ResRef, bool AIScript);
+	Script* CacheScript(const ResRef& ResRef, bool AIScript);
 	ResponseBlock* ReadResponseBlock(DataStream* stream);
 	ResponseSet* ReadResponseSet(DataStream* stream);
 	Response* ReadResponse(DataStream* stream);
@@ -566,9 +563,9 @@ private: //Internal Functions
 
 	// Internal variables
 	Scriptable* const MySelf;
-	ieResRef Name;
+	ResRef Name;
 	Script* script;
-	unsigned int lastAction;
+	size_t lastAction = -1;
 	int scriptlevel;
 public: //Script Functions
 	static int ID_Alignment(const Actor *actor, int parameter);
@@ -595,6 +592,7 @@ public: //Script Functions
 	static int AnyPCOnMap(Scriptable *Sender, const Trigger *parameters);
 	static int AnyPCSeesEnemy(Scriptable *Sender, const Trigger *parameters);
 	static int AreaCheck(Scriptable *Sender, const Trigger *parameter);
+	static int AreaCheckAllegiance(Scriptable* Sender, const Trigger* parameters);
 	static int AreaCheckObject(Scriptable *Sender, const Trigger *parameter);
 	static int AreaFlag(Scriptable *Sender, const Trigger *parameter);
 	static int AreaRestDisabled(Scriptable *Sender, const Trigger *parameter);
@@ -610,10 +608,13 @@ public: //Script Functions
 	static int BitGlobal_Trigger(Scriptable *Sender, const Trigger *parameters);
 	static int BouncingSpellLevel(Scriptable *Sender, const Trigger *parameters);
 	static int BreakingPoint(Scriptable *Sender, const Trigger *parameters);
+	static int ButtonDisabled(Scriptable* Sender, const Trigger* parameters);
 	static int CalendarDay(Scriptable *Sender, const Trigger *parameters);
 	static int CalendarDayGT(Scriptable *Sender, const Trigger *parameters);
 	static int CalendarDayLT(Scriptable *Sender, const Trigger *parameters);
 	static int CalledByName(Scriptable *Sender, const Trigger *parameters);
+	static int CanEquipRanged(Scriptable* Sender, const Trigger* /*parameters*/);
+	static int CanTurn(Scriptable* Sender, const Trigger* parameters);
 	static int ChargeCount(Scriptable *Sender, const Trigger *parameters);
 	static int CharName(Scriptable *Sender, const Trigger *parameters);
 	static int CheckAreaDiffLevel(Scriptable */*Sender*/, const Trigger *parameters);
@@ -641,6 +642,7 @@ public: //Script Functions
 	static int CreatureHidden( Scriptable* Sender, const Trigger *parameters);
 	static int CurrentAmmo(Scriptable *Sender, const Trigger *parameters);
 	static int CurrentAreaIs(Scriptable *Sender, const Trigger *parameters);
+	static int CutSceneBroken(Scriptable* /*Sender*/, const Trigger* /*parameters*/);
 	static int DamageTaken(Scriptable *Sender, const Trigger *parameters);
 	static int DamageTakenGT(Scriptable *Sender, const Trigger *parameters);
 	static int DamageTakenLT(Scriptable *Sender, const Trigger *parameters);
@@ -704,6 +706,7 @@ public: //Script Functions
 	static int HasImmunityEffects(Scriptable *Sender, const Trigger *parameters);
 	static int HasInnateAbility(Scriptable *Sender, const Trigger *parameters);
 	static int HasItem(Scriptable *Sender, const Trigger *parameters);
+	static int HasItemCategory(Scriptable* Sender, const Trigger* parameters);
 	static int HasItemEquipped(Scriptable *Sender, const Trigger *parameters);
 	static int HasItemSlot(Scriptable *Sender, const Trigger *parameters);
 	static int HasItemTypeSlot(Scriptable *Sender, const Trigger *parameters);
@@ -742,6 +745,7 @@ public: //Script Functions
 	static int InTrap(Scriptable *Sender, const Trigger *parameters);
 	static int InventoryFull(Scriptable *Sender, const Trigger *parameter);
 	static int InWeaponRange(Scriptable *Sender, const Trigger *parameter);
+	static int INI(Scriptable* /*Sender*/, const Trigger* parameters);
 	static int IsAClown(Scriptable *Sender, const Trigger *parameters);
 	static int IsActive(Scriptable *Sender, const Trigger *parameters);
 	static int IsCreatureAreaFlag( Scriptable* Sender, const Trigger *parameters);
@@ -750,6 +754,7 @@ public: //Script Functions
 	static int IsExtendedNight(Scriptable *Sender, const Trigger *parameters);
 	static int IsFacingObject(Scriptable *Sender, const Trigger *parameters);
 	static int IsFacingSavedRotation(Scriptable *Sender, const Trigger *parameters);
+	static int IsForcedRandomEncounterActive(Scriptable* /*Sender*/, const Trigger* /*parameters*/);
 	static int IsInGuardianMantle(Scriptable *Sender, const Trigger *parameters);
 	static int IsLocked(Scriptable *Sender, const Trigger *parameters);
 	static int IsMarkedSpell(Scriptable *Sender, const Trigger *parameters);
@@ -826,6 +831,9 @@ public: //Script Functions
 	static int NumMirrorImages(Scriptable *Sender, const Trigger *parameters);
 	static int NumMirrorImagesGT(Scriptable *Sender, const Trigger *parameters);
 	static int NumMirrorImagesLT(Scriptable *Sender, const Trigger *parameters);
+	static int NumKilledByParty(Scriptable* /*Sender*/, const Trigger* parameters);
+	static int NumKilledByPartyGT(Scriptable* /*Sender*/, const Trigger* parameters);
+	static int NumKilledByPartyLT(Scriptable* /*Sender*/, const Trigger* parameters);
 	static int NumTimesInteracted(Scriptable *Sender, const Trigger *parameters);
 	static int NumTimesInteractedGT(Scriptable *Sender, const Trigger *parameters);
 	static int NumTimesInteractedLT(Scriptable *Sender, const Trigger *parameters);
@@ -887,6 +895,7 @@ public: //Script Functions
 	static int ReputationGT(Scriptable *Sender, const Trigger *parameters);
 	static int ReputationLT(Scriptable *Sender, const Trigger *parameters);
 	static int School(Scriptable *Sender, const Trigger *parameters);
+	static int SecretDoorDetected(Scriptable* Sender, const Trigger* parameters);
 	static int See(Scriptable *Sender, const Trigger *parameters);
 	static int Sequence(Scriptable *Sender, const Trigger *parameters);
 	static int SetLastMarkedObject(Scriptable *Sender, const Trigger *parameters);
@@ -906,6 +915,7 @@ public: //Script Functions
 	static int SummoningLimit(Scriptable *Sender, const Trigger *parameters);
 	static int SummoningLimitGT(Scriptable *Sender, const Trigger *parameters);
 	static int SummoningLimitLT(Scriptable *Sender, const Trigger *parameters);
+	static int Switch(Scriptable*/*Sender*/, const Trigger* /*parameters*/);
 	static int SystemVariable_Trigger(Scriptable *Sender, const Trigger *parameters);
 	static int TargetUnreachable(Scriptable *Sender, const Trigger *parameters);
 	static int Team(Scriptable *Sender, const Trigger *parameters);
@@ -940,6 +950,8 @@ public: //Script Functions
 	static int Vacant(Scriptable *Sender, const Trigger *parameters);
 	static int WalkedToTrigger(Scriptable *Sender, const Trigger *parameters);
 	static int WasInDialog(Scriptable *Sender, const Trigger *parameters);
+	static int WeaponCanDamage(Scriptable* Sender, const Trigger* parameters);
+	static int WeaponEffectiveVs(Scriptable* Sender, const Trigger* parameters);
 	static int Xor(Scriptable *Sender, const Trigger *parameters);
 	static int XP(Scriptable *Sender, const Trigger *parameters);
 	static int XPGT(Scriptable *Sender, const Trigger *parameters);
@@ -960,12 +972,14 @@ public: //Script Functions
 	static void AddKit(Scriptable* Sender, Action* parameters);
 	static void AddMapnote(Scriptable* Sender, Action* parameters);
 	static void AddSpecialAbility(Scriptable* Sender, Action* parameters);
+	static void AddStoreItem(Scriptable* /*Sender*/, Action* parameters);
 	static void AddSuperKit(Scriptable* Sender, Action* parameters);
 	static void AddWayPoint(Scriptable* Sender, Action* parameters);
 	static void AddWorldmapAreaFlag(Scriptable* /*Sender*/, Action* parameters);
 	static void AddXP2DA(Scriptable *Sender, Action* parameters);
 	static void AddXPObject(Scriptable *Sender, Action* parameters);
 	static void AddXPVar(Scriptable *Sender, Action* parameters);
+	static void AddXPWorth(Scriptable* Sender, Action* parameters);
 	static void AdvanceTime(Scriptable *Sender, Action* parameters);
 	static void Ally(Scriptable* Sender, Action* parameters);
 	static void AmbientActivate(Scriptable* Sender, Action* parameters);
@@ -997,7 +1011,6 @@ public: //Script Functions
 	static void ChangeAnimationNoEffect(Scriptable* Sender, Action* parameters);
 	static void ChangeClass(Scriptable* Sender, Action* parameters);
 	static void ChangeColor(Scriptable* Sender, Action* parameters);
-//	static void ChangeCurrentScript(Scriptable* Sender, Action* parameters);
 	static void ChangeDestination(Scriptable* Sender, Action* parameters);
 	static void ChangeDialogue(Scriptable* Sender, Action* parameters);
 	static void ChangeGender(Scriptable* Sender, Action* parameters);
@@ -1054,6 +1067,7 @@ public: //Script Functions
 	static void DestroyAllEquipment(Scriptable* Sender, Action* parameters);
 	static void DestroyAllFragileEquipment(Scriptable* Sender, Action* parameters);
 	static void DestroyGold(Scriptable* Sender, Action* parameters);
+	static void DestroyGroundPiles(Scriptable* Sender, Action* /*parameters*/);
 	static void DestroyItem(Scriptable* Sender, Action* parameters);
 	static void DestroyPartyGold(Scriptable* Sender, Action* parameters);
 	static void DestroyPartyItem(Scriptable* Sender, Action* parameters);
@@ -1068,6 +1082,7 @@ public: //Script Functions
 	static void DisplayMessage(Scriptable* Sender, Action* parameters);
 	static void DisplayString(Scriptable* Sender, Action* parameters);
 	static void DisplayStringHead(Scriptable* Sender, Action* parameters);
+	static void DisplayStringHeadNoLog(Scriptable* Sender, Action* parameters);
 	static void DisplayStringHeadOwner(Scriptable* Sender, Action* parameters);
 	static void DisplayStringNoName(Scriptable* Sender, Action* parameters);
 	static void DisplayStringNoNameHead(Scriptable* Sender, Action* parameters);
@@ -1122,6 +1137,7 @@ public: //Script Functions
 	static void ForceHide(Scriptable* Sender, Action* parameters);
 	static void ForceLeaveAreaLUA(Scriptable* Sender, Action* parameters);
 	static void ForceMarkedSpell(Scriptable* Sender, Action* parameters);
+	static void ForceRandomEncounter(Scriptable* Sender, Action* parameters);
 	static void ForceSpell(Scriptable* Sender, Action* parameters);
 	static void ForceSpellRange(Scriptable* Sender, Action* parameters);
 	static void ForceSpellPoint(Scriptable* Sender, Action* parameters);
@@ -1134,6 +1150,7 @@ public: //Script Functions
 	static void GetItem(Scriptable* Sender, Action* parameters);
 	static void GetStat(Scriptable* Sender, Action* parameters);
 	static void GiveItem(Scriptable* Sender, Action* parameters);
+	static void GiveObjectGoldGlobal(Scriptable* Sender, Action* parameters);
 	static void GiveOrder(Scriptable* Sender, Action* parameters);
 	static void GivePartyAllEquipment(Scriptable* Sender, Action* parameters);
 	static void GivePartyGold(Scriptable* Sender, Action* parameters);
@@ -1189,6 +1206,7 @@ public: //Script Functions
 	static void Lock(Scriptable* Sender, Action* parameters);
 	static void LockScroll(Scriptable* Sender, Action* parameters);
 	static void MakeGlobal(Scriptable* Sender, Action* parameters);
+	static void MakeGlobalOverride(Scriptable* Sender, Action* parameters);
 	static void MakeUnselectable(Scriptable* Sender, Action* parameters);
 	static void MarkObject(Scriptable* Sender, Action* parameters);
 	static void MarkSpellAndObject(Scriptable* Sender, Action* parameters);
@@ -1198,12 +1216,14 @@ public: //Script Functions
 	static void MoraleSet(Scriptable* Sender, Action* parameters);
 	static void MoveBetweenAreas(Scriptable* Sender, Action* parameters);
 	static void MoveBetweenAreasEffect(Scriptable* Sender, Action* parameters);
+	static void MoveContainerContents(Scriptable* Sender, Action* parameters);
 	static void MoveCursorPoint(Scriptable* Sender, Action* parameters);
 	static void MoveGlobal(Scriptable* Sender, Action* parameters);
 	static void MoveGlobalObject(Scriptable* Sender, Action* parameters);
 	static void MoveGlobalObjectOffScreen(Scriptable* Sender, Action* parameters);
 	static void MoveGlobalsTo(Scriptable* Sender, Action* parameters);
 	static void MoveInventory(Scriptable *Sender, Action* parameters);
+	static void MoveToCampaign(Scriptable* /*Sender*/, Action* parameters);
 	static void MoveToCenterOfScreen(Scriptable* Sender, Action* parameters);
 	static void MoveToExpansion(Scriptable* Sender, Action* parameters);
 	static void MoveToObject(Scriptable* Sender, Action* parameters);
@@ -1221,7 +1241,6 @@ public: //Script Functions
 	static void NIDSpecial1(Scriptable* Sender, Action* parameters);
 	static void NIDSpecial2(Scriptable* Sender, Action* parameters);
 	static void NoAction(Scriptable* Sender, Action* parameters);
-	static void NoActionAtAll(Scriptable* Sender, Action* parameters);
 	static void OpenDoor(Scriptable* Sender, Action* parameters);
 	static void Panic(Scriptable* Sender, Action* parameters);
 	static void PauseGame(Scriptable *Sender, Action* parameters);
@@ -1265,10 +1284,13 @@ public: //Script Functions
 	static void RemovePaladinHood(Scriptable* Sender, Action* parameters);
 	static void RemoveRangerHood(Scriptable* Sender, Action* parameters);
 	static void RemoveSpell(Scriptable* Sender, Action* parameters);
+	static void RemoveStoreItem(Scriptable* /*Sender*/, Action* parameters);
 	static void RemoveTraps(Scriptable* Sender, Action* parameters);
 	static void RemoveWorldmapAreaFlag(Scriptable* /*Sender*/, Action* parameters);
 	static void ReputationInc(Scriptable* Sender, Action* parameters);
 	static void ReputationSet(Scriptable* Sender, Action* parameters);
+	static void ResetMorale(Scriptable* Sender, Action* parameters);
+	static void ResetPlayerAI(Scriptable* Sender, Action* parameters);
 	static void RestorePartyLocation(Scriptable *Sender, Action* parameters);
 	static void Rest(Scriptable *Sender, Action* parameters);
 	static void RestNoSpells(Scriptable *Sender, Action* parameters);
@@ -1280,6 +1302,7 @@ public: //Script Functions
 	static void RevealAreaOnMap(Scriptable* Sender, Action* parameters);
 	static void RunAwayFrom(Scriptable* Sender, Action* parameters);
 	static void RunAwayFromNoInterrupt(Scriptable* Sender, Action* parameters);
+	static void RunAwayFromNoInterruptNoLeaveArea(Scriptable* Sender, Action* parameters);
 	static void RunAwayFromNoLeaveArea(Scriptable* Sender, Action* parameters);
 	static void RunFollow(Scriptable* Sender, Action* parameters);
 	static void RunningAttack(Scriptable* Sender, Action* parameters);
@@ -1297,12 +1320,14 @@ public: //Script Functions
 	static void SetApparentName(Scriptable* Sender, Action* parameters);
 	static void SetAreaFlags(Scriptable* Sender, Action* parameters);
 	static void SetAreaRestFlag(Scriptable* Sender, Action* parameters);
+	static void SetAreaScript(Scriptable* Sender, Action* parameters);
 	static void SetArmourLevel(Scriptable* Sender, Action* parameters);
 	static void SetBeenInPartyFlags(Scriptable* Sender, Action* parameters);
 	static void SetBestWeapon(Scriptable *Sender, Action *parameters);
 	static void SetCursorState(Scriptable* Sender, Action* parameters);
 	static void SetCreatureAreaFlag(Scriptable* Sender, Action* parameters);
 	static void SetCriticalPathObject(Scriptable* Sender, Action* parameters);
+	static void SetCutSceneBreakable(Scriptable* /*Sender*/, Action* parameters);
 	static void SetDialogue(Scriptable* Sender, Action* parameters);
 	static void SetDialogueRange(Scriptable* Sender, Action* parameters);
 	static void SetDoorFlag(Scriptable* Sender, Action* parameters);
@@ -1310,6 +1335,7 @@ public: //Script Functions
 	static void SetEncounterProbability(Scriptable* Sender, Action* parameters);
 	static void SetExtendedNight(Scriptable* Sender, Action* parameters);
 	static void SetFaction(Scriptable* Sender, Action* parameters);
+	static void SetPCStatsTokens(Scriptable* Sender, Action* parameters);
 	static void SetGabber(Scriptable* Sender, Action* parameters);
 	static void SetGlobal(Scriptable* Sender, Action* parameters);
 	static void SetGlobalRandom(Scriptable* Sender, Action* parameters);
@@ -1359,6 +1385,7 @@ public: //Script Functions
 	static void SetupWish(Scriptable* Sender, Action* parameters);
 	static void SetupWishObject(Scriptable* Sender, Action* parameters);
 	static void SetVisualRange(Scriptable* Sender, Action* parameters);
+	static void SetWorldmap(Scriptable* /*Sender*/, Action* parameters);
 	static void SG(Scriptable* Sender, Action* parameters);
 	static void Shout(Scriptable* Sender, Action* parameters);
 	static void SmallWait(Scriptable* Sender, Action* parameters);
@@ -1402,10 +1429,12 @@ public: //Script Functions
 	static void StorePartyLocation(Scriptable *Sender, Action* parameters);
 	static void Swing(Scriptable* Sender, Action* parameters);
 	static void SwingOnce(Scriptable* Sender, Action* parameters);
+	static void TakeCreatureItems(Scriptable* Sender, Action* parameters);
 	static void TakeItemList(Scriptable* Sender, Action* parameters);
 	static void TakeItemListParty(Scriptable* Sender, Action* parameters);
 	static void TakeItemListPartyNum(Scriptable* Sender, Action* parameters);
 	static void TakeItemReplace(Scriptable* Sender, Action* parameters);
+	static void TakeObjectGoldGlobal(Scriptable* Sender, Action* parameters);
 	static void TakePartyGold(Scriptable* Sender, Action* parameters);
 	static void TakePartyItem(Scriptable* Sender, Action* parameters);
 	static void TakePartyItemAll(Scriptable* Sender, Action* parameters);
@@ -1553,9 +1582,9 @@ public: //Script Functions
 	static Targets *Player10Fill(const Scriptable *Sender, Targets *parameters, int ga_flags);
 };
 
-GEM_EXPORT Action* GenerateAction(const char* String);
-Action *GenerateActionDirect(const char *String, const Scriptable *object);
-GEM_EXPORT Trigger* GenerateTrigger(char* String);
+GEM_EXPORT Action* GenerateAction(std::string String);
+GEM_EXPORT Action *GenerateActionDirect(std::string string, const Scriptable *object);
+GEM_EXPORT Trigger* GenerateTrigger(std::string string);
 
 void InitializeIEScript();
 
