@@ -48,7 +48,8 @@ Targets *GameScript::Myself(const Scriptable *Sender, Targets* parameters, int g
 		// just in case we're in the middle of a move
 		snd = core->GetGame()->GetActorByGlobalID(Sender->GetGlobalID());
 	}
-	parameters->AddTarget(snd, 0, ga_flags);
+	// you are always visible to yourself
+	parameters->AddTarget(snd, 0, ga_flags & ~GA_NO_UNSCHEDULED);
 	return parameters;
 }
 
@@ -110,7 +111,7 @@ Targets *GameScript::Protagonist(const Scriptable *Sender, Targets *parameters,
 {
 	parameters->Clear();
 	//this sucks but IWD2 is like that...
-	static bool charnameisgabber = core->HasFeature(GF_CHARNAMEISGABBER);
+	static bool charnameisgabber = core->HasFeature(GFFlags::CHARNAMEISGABBER);
 	if (charnameisgabber) {
 		const GameControl* gc = core->GetGameControl();
 		if (gc) {
@@ -159,11 +160,9 @@ Targets *GameScript::LastTrigger(const Scriptable *Sender, Targets *parameters,
 
 Targets *GameScript::LastMarkedObject(const Scriptable *Sender, Targets *parameters, int ga_flags)
 {
-	const Actor *actor = (Actor *) parameters->GetTarget(0, ST_ACTOR);
-	if (!actor) {
-		if (Sender->Type==ST_ACTOR) {
-			actor = (const Actor *) Sender;
-		}
+	const Actor *actor = static_cast<Actor*>(parameters->GetTarget(0, ST_ACTOR));
+	if (!actor && Sender->Type == ST_ACTOR) {
+		actor = static_cast<const Actor*>(Sender);
 	}
 	parameters->Clear();
 	if (actor) {
@@ -177,11 +176,9 @@ Targets *GameScript::LastMarkedObject(const Scriptable *Sender, Targets *paramet
 
 Targets *GameScript::SpellTarget(const Scriptable *Sender, Targets *parameters, int ga_flags)
 {
-	const Actor *actor = (Actor *) parameters->GetTarget(0, ST_ACTOR);
-	if (!actor) {
-		if (Sender->Type==ST_ACTOR) {
-			actor = (const Actor *) Sender;
-		}
+	const Actor *actor = static_cast<Actor*>(parameters->GetTarget(0, ST_ACTOR));
+	if (!actor && Sender->Type == ST_ACTOR) {
+		actor = static_cast<const Actor*>(Sender);
 	}
 	parameters->Clear();
 	if (actor) {
@@ -196,11 +193,9 @@ Targets *GameScript::SpellTarget(const Scriptable *Sender, Targets *parameters,
 //actions should always use LastMarkedObject, because LastSeen could be deleted
 Targets *GameScript::LastSeenBy(const Scriptable *Sender, Targets *parameters, int ga_flags)
 {
-	const Actor *actor = (Actor *) parameters->GetTarget(0, ST_ACTOR);
-	if (!actor) {
-		if (Sender->Type==ST_ACTOR) {
-			actor = (const Actor *) Sender;
-		}
+	const Actor *actor = static_cast<Actor*>(parameters->GetTarget(0, ST_ACTOR));
+	if (!actor && Sender->Type == ST_ACTOR) {
+		actor = static_cast<const Actor*>(Sender);
 	}
 	parameters->Clear();
 	if (actor) {
@@ -214,11 +209,9 @@ Targets *GameScript::LastSeenBy(const Scriptable *Sender, Targets *parameters, i
 
 Targets *GameScript::LastHelp(const Scriptable *Sender, Targets *parameters, int ga_flags)
 {
-	const Actor *actor = (Actor *) parameters->GetTarget(0, ST_ACTOR);
-	if (!actor) {
-		if (Sender->Type==ST_ACTOR) {
-			actor = (const Actor *) Sender;
-		}
+	const Actor *actor = static_cast<Actor*>(parameters->GetTarget(0, ST_ACTOR));
+	if (!actor && Sender->Type == ST_ACTOR) {
+		actor = static_cast<const Actor*>(Sender);
 	}
 	parameters->Clear();
 	if (actor) {
@@ -232,11 +225,9 @@ Targets *GameScript::LastHelp(const Scriptable *Sender, Targets *parameters, int
 
 Targets *GameScript::LastHeardBy(const Scriptable *Sender, Targets *parameters, int ga_flags)
 {
-	const Actor *actor = (Actor *) parameters->GetTarget(0, ST_ACTOR);
-	if (!actor) {
-		if (Sender->Type==ST_ACTOR) {
-			actor = (const Actor *) Sender;
-		}
+	const Actor *actor = static_cast<Actor*>(parameters->GetTarget(0, ST_ACTOR));
+	if (!actor && Sender->Type == ST_ACTOR) {
+		actor = static_cast<const Actor*>(Sender);
 	}
 	parameters->Clear();
 	if (actor) {
@@ -252,11 +243,9 @@ Targets *GameScript::LastHeardBy(const Scriptable *Sender, Targets *parameters,
 //object selector for everyone with the same specifics as the current object
 Targets *GameScript::GroupOf(const Scriptable *Sender, Targets *parameters, int ga_flags)
 {
-	const Actor *actor = (Actor *) parameters->GetTarget(0, ST_ACTOR);
-	if (!actor) {
-		if (Sender->Type==ST_ACTOR) {
-			actor = (const Actor *) Sender;
-		}
+	const Actor *actor = static_cast<Actor*>(parameters->GetTarget(0, ST_ACTOR));
+	if (!actor && Sender->Type == ST_ACTOR) {
+		actor = static_cast<const Actor*>(Sender);
 	}
 	parameters->Clear();
 	if (actor) {
@@ -276,24 +265,12 @@ Targets *GameScript::GroupOf(const Scriptable *Sender, Targets *parameters, int
 /*this one is tough, but done */
 Targets *GameScript::ProtectorOf(const Scriptable *Sender, Targets *parameters, int ga_flags)
 {
-	const Actor *actor = (Actor *) parameters->GetTarget(0, ST_ACTOR);
-	if (!actor) {
-		if (Sender->Type==ST_ACTOR) {
-			actor = (const Actor *) Sender;
-		}
+	const Actor *actor = static_cast<Actor*>(parameters->GetTarget(0, ST_ACTOR));
+	if (!actor && Sender->Type == ST_ACTOR) {
+		actor = static_cast<const Actor*>(Sender);
 	}
 	parameters->Clear();
-	/*if (actor) {
-		ieWord tmp = actor->LastProtected;
-		Map *cm = Sender->GetCurrentArea();
-		int i = cm->GetActorCount(true);
-		while (i--) {
-			Actor *target=cm->GetActor(i,true);
-			if (target && (target->LastProtected ==tmp) ) {
-				parameters->AddTarget(target, 0, ga_flags);
-			}
-		}
-	}*/
+
 	if (actor) {
 		Actor *target = actor->GetCurrentArea()->GetActorByGlobalID(actor->LastProtector);
 		if (target) {
@@ -305,11 +282,9 @@ Targets *GameScript::ProtectorOf(const Scriptable *Sender, Targets *parameters,
 
 Targets *GameScript::ProtectedBy(const Scriptable *Sender, Targets *parameters, int ga_flags)
 {
-	const Actor *actor = (Actor *) parameters->GetTarget(0, ST_ACTOR);
-	if (!actor) {
-		if (Sender->Type==ST_ACTOR) {
-			actor = (const Actor *) Sender;
-		}
+	const Actor *actor = static_cast<Actor*>(parameters->GetTarget(0, ST_ACTOR));
+	if (!actor && Sender->Type == ST_ACTOR) {
+		actor = static_cast<const Actor*>(Sender);
 	}
 	parameters->Clear();
 	if (actor) {
@@ -323,11 +298,9 @@ Targets *GameScript::ProtectedBy(const Scriptable *Sender, Targets *parameters,
 
 Targets *GameScript::LastCommandedBy(const Scriptable *Sender, Targets *parameters, int ga_flags)
 {
-	const Actor *actor = (Actor *) parameters->GetTarget(0, ST_ACTOR);
-	if (!actor) {
-		if (Sender->Type==ST_ACTOR) {
-			actor = (const Actor *) Sender;
-		}
+	const Actor *actor = static_cast<Actor*>(parameters->GetTarget(0, ST_ACTOR));
+	if (!actor && Sender->Type == ST_ACTOR) {
+		actor = static_cast<const Actor*>(Sender);
 	}
 	parameters->Clear();
 	if (actor) {
@@ -359,17 +332,15 @@ Targets *GameScript::MyTarget(const Scriptable *Sender, Targets *parameters, int
 
 Targets *GameScript::LastTargetedBy(const Scriptable *Sender, Targets *parameters, int ga_flags)
 {
-	const Actor *actor = (Actor *) parameters->GetTarget(0, ST_ACTOR);
+	const Actor *actor = static_cast<Actor*>(parameters->GetTarget(0, ST_ACTOR));
 	return GetMyTarget(Sender, actor, parameters, ga_flags);
 }
 
 Targets *GameScript::LastAttackerOf(const Scriptable *Sender, Targets *parameters, int ga_flags)
 {
-	const Actor *actor = (Actor *) parameters->GetTarget(0, ST_ACTOR);
-	if (!actor) {
-		if (Sender->Type==ST_ACTOR) {
-			actor = (const Actor *) Sender;
-		}
+	const Actor *actor = static_cast<Actor*>(parameters->GetTarget(0, ST_ACTOR));
+	if (!actor && Sender->Type == ST_ACTOR) {
+		actor = static_cast<const Actor*>(Sender);
 	}
 	parameters->Clear();
 	if (actor) {
@@ -383,11 +354,9 @@ Targets *GameScript::LastAttackerOf(const Scriptable *Sender, Targets *parameter
 
 Targets *GameScript::LastHitter(const Scriptable *Sender, Targets *parameters, int ga_flags)
 {
-	const Actor *actor = (Actor *) parameters->GetTarget(0, ST_ACTOR);
-	if (!actor) {
-		if (Sender->Type==ST_ACTOR) {
-			actor = (const Actor *) Sender;
-		}
+	const Actor *actor = static_cast<Actor*>(parameters->GetTarget(0, ST_ACTOR));
+	if (!actor && Sender->Type == ST_ACTOR) {
+		actor = static_cast<const Actor*>(Sender);
 	}
 	parameters->Clear();
 	if (actor) {
@@ -401,11 +370,9 @@ Targets *GameScript::LastHitter(const Scriptable *Sender, Targets *parameters, i
 
 Targets *GameScript::LeaderOf(const Scriptable *Sender, Targets *parameters, int ga_flags)
 {
-	const Actor *actor = (Actor *) parameters->GetTarget(0, ST_ACTOR);
-	if (!actor) {
-		if (Sender->Type==ST_ACTOR) {
-			actor = (const Actor *) Sender;
-		}
+	const Actor *actor = static_cast<Actor*>(parameters->GetTarget(0, ST_ACTOR));
+	if (!actor && Sender->Type == ST_ACTOR) {
+		actor = static_cast<const Actor*>(Sender);
 	}
 	parameters->Clear();
 	if (actor) {
@@ -419,11 +386,9 @@ Targets *GameScript::LeaderOf(const Scriptable *Sender, Targets *parameters, int
 
 Targets *GameScript::LastTalkedToBy(const Scriptable *Sender, Targets *parameters, int ga_flags)
 {
-	const Actor *actor = (Actor *) parameters->GetTarget(0, ST_ACTOR);
-	if (!actor) {
-		if (Sender->Type==ST_ACTOR) {
-			actor = (const Actor *) Sender;
-		}
+	const Actor *actor = static_cast<Actor*>(parameters->GetTarget(0, ST_ACTOR));
+	if (!actor && Sender->Type == ST_ACTOR) {
+		actor = static_cast<const Actor*>(Sender);
 	}
 	parameters->Clear();
 	if (actor) {
@@ -437,11 +402,9 @@ Targets *GameScript::LastTalkedToBy(const Scriptable *Sender, Targets *parameter
 
 Targets *GameScript::LastSummonerOf(const Scriptable *Sender, Targets *parameters, int ga_flags)
 {
-	const Actor *actor = (Actor *) parameters->GetTarget(0, ST_ACTOR);
-	if (!actor) {
-		if (Sender->Type==ST_ACTOR) {
-			actor = (const Actor *) Sender;
-		}
+	const Actor *actor = static_cast<Actor*>(parameters->GetTarget(0, ST_ACTOR));
+	if (!actor && Sender->Type == ST_ACTOR) {
+		actor = static_cast<const Actor*>(Sender);
 	}
 	parameters->Clear();
 	if (actor) {
@@ -1011,16 +974,12 @@ int GameScript::ID_Alignment(const Actor *actor, int parameter)
 {
 	int value = actor->GetStat( IE_ALIGNMENT );
 	int a = parameter&15;
-	if (a) {
-		if (a != ( value & 15 )) {
-			return 0;
-		}
+	if (a && a != (value & 15)) {
+		return 0;
 	}
 	a = parameter & 240;
-	if (a) {
-		if (a != ( value & 240 )) {
-			return 0;
-		}
+	if (a && a != (value & 240)) {
+		return 0;
 	}
 	return 1;
 }
@@ -1124,11 +1083,11 @@ inline bool idclass(const Actor *actor, int parameter, bool iwd2) {
 
 int GameScript::ID_Class(const Actor *actor, int parameter)
 {
-	if (core->HasFeature(GF_3ED_RULES)) {
+	if (core->HasFeature(GFFlags::RULES_3ED)) {
 		//iwd2 has different values, see also the note for AVClass
-		return idclass(actor, parameter, 1);
+		return idclass(actor, parameter, true);
 	}
-	return idclass(actor, parameter, 0);
+	return idclass(actor, parameter, false);
 }
 
 // IE_CLASS holds only one class, not a bitmask like with iwd2 kits. The ids values
@@ -1151,7 +1110,7 @@ int GameScript::ID_ClassMask(const Actor *actor, int parameter)
 // iwd2's class.ids is different than the rest, while class20 is identical (remnant)
 int GameScript::ID_AVClass(const Actor *actor, int parameter)
 {
-	return idclass(actor, parameter, 0);
+	return idclass(actor, parameter, false);
 }
 
 int GameScript::ID_Race(const Actor *actor, int parameter)
