@@ -22,10 +22,12 @@
 
 #include "Interface.h"
 
+#include <utility>
+
 namespace GemRB {
 
-ImageFactory::ImageFactory(const char* ResRef, Holder<Sprite2D> bitmap_)
-	: FactoryObject( ResRef, IE_BMP_CLASS_ID ), bitmap(bitmap_)
+ImageFactory::ImageFactory(const ResRef &resref, Holder<Sprite2D> bitmap_)
+: FactoryObject(resref, IE_BMP_CLASS_ID), bitmap(std::move(bitmap_))
 {
 
 }
