@@ -21,15 +21,12 @@
 
 #include "Plugin.h"
 #include "Sprite2D.h"
-#include "System/DataStream.h"
+#include "Streams/DataStream.h"
 
 namespace GemRB {
 
 class GEM_EXPORT ImageWriter : public Plugin {
 public:
-	ImageWriter(void);
-	~ImageWriter(void) override;
-
 	/** Writes an Sprite2D to a stream and frees the sprite. */
 	virtual void PutImage(DataStream *output, Holder<Sprite2D> sprite) = 0;
 };
