@@ -26,15 +26,21 @@
 #include "Scriptable/PCStatStruct.h"
 
 #include "exports.h"
+#include "ie_stats.h" // using definitions as described in stats.ids
 #include "ie_types.h"
+#include "strrefs.h"
 
 #include "Audio.h"
 #include "CombatInfo.h"
 #include "EffectQueue.h"
+#include "Game.h"
+#include "Inventory.h"
 #include "Palette.h"
 #include "Polygon.h"
-#include "Video.h"
+#include "Spellbook.h"
+#include "Video/Video.h"
 
+#include <array>
 #include <map>
 #include <set>
 #include <vector>
@@ -47,20 +53,22 @@ class CharAnimations;
 class DataFileMgr;
 class Map;
 class ScriptedAnimation;
-class StringBuffer;
 class ToHitStats;
 struct PolymorphCache;
 
 }
 
-/** USING DEFINITIONS AS DESCRIBED IN STATS.IDS */
-#include "ie_stats.h"
-
-#include "Inventory.h"
-#include "Spellbook.h"
-
 namespace GemRB {
 
+enum CREVersion {
+	GemRB,
+	V1_0 = 10, // bg1
+	V1_1 = 11, // bg2 (still V1.0)
+	V1_2 = 12, // pst
+	V2_2 = 22, // iwd2
+	V9_0 = 90 // iwd
+};
+
 #define MAX_STATS 256
 #define MAX_LEVEL 128
 #define MAX_FEATS 96 //3*sizeof(ieDword)
@@ -166,6 +174,7 @@ namespace GemRB {
 
 #define GA_ONLY_BUMPABLE 32768
 #define GA_CAN_BUMP 65536
+#define GA_BIGBAD 131072
 
 #define VCONST_COUNT 100
 
@@ -231,6 +240,8 @@ namespace GemRB {
 // used for distinguishing damage immunity from high damage resistance
 #define DR_IMMUNE 999999
 
+constexpr int STYLE_STAR_MAX = 3;
+
 // wild surge target change type
 #define WSTC_SETTYPE   1 // change to this target type
 #define WSTC_ADDTYPE   2 // affect also this target type
@@ -245,26 +256,42 @@ struct WildSurgeSpellMods {
 	int saving_throw_mod;
 };
 
-typedef ieByte ActionButtonRow[GUIBT_COUNT];
+using ActionButtonRow = std::array<ieByte, GUIBT_COUNT>;
 struct ActionButtonRow2 {
 	ActionButtonRow buttons;
 	ieByte clss;
 };
 
+// internal flags for calculating to hit and similar
+#define WEAPON_FIST        0
+#define WEAPON_MELEE       1
+#define WEAPON_RANGED      2
+#define WEAPON_STYLEMASK   15
+#define WEAPON_LEFTHAND    16
+#define WEAPON_USESTRENGTH 32
+#define WEAPON_USESTRENGTH_DMG 64
+#define WEAPON_USESTRENGTH_HIT 128
+#define WEAPON_FINESSE     256
+#define WEAPON_BYPASS      0x10000
+#define WEAPON_KEEN        0x20000
+#define WEAPON_BREAKABLE   0x40000
+
 struct WeaponInfo {
-	int slot;
-	ieDword enchantment;
-	unsigned int range;
-	ieDword itemtype;
-	ieDword itemflags;
-	ieDword prof;
-	bool backstabbing;
-	ieDword wflags;
-	int critmulti; //critical hit multiplier (usually 2)
-	int critrange; // the lower value of the critical range (eg. 19 in 19-20/x3)
-	int profdmgbon;
-	int launcherdmgbon;
-	WeaponInfo(): slot(0), enchantment(0), range(0), itemtype(0), itemflags(0), prof(0), backstabbing(false), wflags(0), critmulti(0), critrange(0), profdmgbon(0), launcherdmgbon(0) {};
+	int slot = 0;
+	ieDword enchantment = 0;
+	unsigned int range = 0;
+	ieDword itemtype = 0;
+	ieDword itemflags = 0;
+	ieByte prof = 0;
+	bool backstabbing = false;
+	ieDword wflags = 0;
+	int critmulti = 0; // critical hit multiplier (usually 2)
+	int critrange = 0; // the lower value of the critical range (eg. 19 in 19-20/x3)
+	int profdmgbon = 0;
+	int launcherDmgBonus = 0;
+	int launcherTHAC0Bonus = 0;
+	const Item* item = nullptr;
+	const ITMExtHeader* extHeader = nullptr;
 };
 
 struct BABTable {
@@ -274,25 +301,45 @@ struct BABTable {
 };
 
 struct ModalStatesStruct {
-	ieResRef spell;
-	char action[16];
-	unsigned int entering_str;
-	unsigned int leaving_str;
-	unsigned int failed_str;
+	ResRef spell;
+	ieVariable action;
+	ieStrRef entering_str;
+	ieStrRef leaving_str;
+	ieStrRef failed_str;
 	unsigned int aoe_spell;
 	unsigned int repeat_msg;
 };
 
 struct ModalState {
-	ieDword State;
-	ieResRef Spell;             //apply this spell once per round
-	ieResRef LingeringSpell;    //apply this spell once per round if the effects are lingering
-	char LingeringCount;        //the count of rounds for which the modal spell will be reapplied after the state ends
-	ieDword LastApplyTime;      //last time the modal effect used
-	bool FirstApply;            //running for the first time?
+	ieDword State = MS_NONE;
+	ResRef Spell; // apply this (modal) spell once per round
+	ResRef LingeringSpell; // apply this spell once per round if the effects are lingering
+	char LingeringCount = 0; // the count of rounds for which the modal spell will be reapplied after the state ends
+	ieDword LastApplyTime = 0; // last time the modal spell was applied
+	bool FirstApply = true; // running for the first time?
+};
+
+struct SaveInfo {
+	ieByte savingThrow[5]{};
+	// for feedback rate limiting
+	ieDword prevType = ieDword(-1);
+	int prevRoll = -1;
+};
+
+enum DamageFlags {
+	DrainFromTarget = 0,
+	DrainFromSource = 1,
+	FistOnly = 2,
+	DrainFromTargetNC = 4,
+	DrainFromSourceNC = 8,
+	NoFeedback = 0x10,
+	CapToTarget = 0x20,
+	CapToSource = 0x40,
+	SaveForHalf = 0x80,
+	FailForHalf = 0x100,
+	NoAwake = 0x200
 };
 
-extern void ReleaseMemoryActor();
 GEM_EXPORT void UpdateActorConfig(); //call this from guiscripts when some variable has changed
 
 bool VVCSort(const ScriptedAnimation* lhs, const ScriptedAnimation* rhs);
@@ -301,88 +348,97 @@ using vvcDict = std::multimap<ResRef, ScriptedAnimation*>;
 
 class GEM_EXPORT Actor : public Movable {
 public:
+	using stat_t = ieDword;
+	using stats_t = std::array<stat_t, MAX_STATS>;
 	//CRE DATA FIELDS
-	ieDword BaseStats[MAX_STATS];
-	ieDword Modified[MAX_STATS];
-	ieDword *PrevStats;
-	ieByteSigned DeathCounters[4];   //PST specific (good, law, lady, murder)
+	stats_t BaseStats {};
+	stats_t Modified {};
+	stat_t* PrevStats = nullptr;
+	ieByteSigned DeathCounters[4]{}; // PST specific (good, law, lady, murder)
 
-	ieResRef BardSong;               //custom bard song (updated by fx)
-	ieResRef BackstabResRef;         //apply on successful backstab
+	ResRef BardSong;               //custom bard song (updated by fx)
+	ResRef BackstabResRef = "*";         //apply on successful backstab
 
-	PCStatsStruct*  PCStats;
-	ieResRef SmallPortrait;
-	ieResRef LargePortrait;
+	PCStatsStruct* PCStats = nullptr;
+	PCStatsStruct::StateArray previousStates;
+	ResRef SmallPortrait;
+	ResRef LargePortrait;
 	/** 0: NPC, 1-8 party slot */
-	ieByte InParty;
-	//32 is the maximum possible length of the actor name in the original games
-	char LongName[33], ShortName[33];
-	ieStrRef ShortStrRef, LongStrRef;
-	ieStrRef StrRefs[VCONST_COUNT];
+	ieByte InParty = 0;
+	
+	ieStrRef ShortStrRef = ieStrRef(-1);
+	ieStrRef LongStrRef = ieStrRef(-1);
+	ieStrRef StrRefs[VCONST_COUNT]{};
 
-	ieDword AppearanceFlags;
+	ieDword AppearanceFlags = 0;
 
 	ieVariable KillVar; //this second field is present in pst, iwd1 and iwd2
 	ieVariable IncKillVar; // iwd1, iwd2
 
-	ieByte SetDeathVar, IncKillCount, UnknownField; // boolean fields from iwd1 and iwd2
+	// boolean fields from iwd1 and iwd2
+	ieByte SetDeathVar = 0;
+	ieByte IncKillCount = 0;
+	ieByte UnknownField = 0;
 
 	Inventory inventory;
 	Spellbook spellbook;
 	//savefile version (creatures embedded in area)
-	int version;
+	int creVersion = 0;
 	//in game or area actor header
-	ieDword TalkCount;
-	ieDword RemovalTime;
+	ieDword TalkCount = 0;
+	ieDword RemovalTime = 0;
 	//FIXME: this is definitely not the same in bg2, in bg2 there are joinable npcs
 	//which keep a matrix of counters
-	ieDword InteractCount; //this is accessible in iwd2, probably exists in other games too
-	ieDword appearance;
+	ieDword InteractCount = 0; // this is accessible in iwd2, probably exists in other games too
+	ieDword appearance = 0xffffff; // schedule
 	ArmorClass AC;
 	ToHitStats ToHit;
-	ModalState Modal;
+	WeaponInfo weaponInfo[2]{};
+	ModalState Modal{};
 
-	ieDword LastExit;    //the global ID of the exit to be used
+	bool usedLeftHand = false; // which weaponInfo index was used in an attack last
+	ieDword LastExit = 0;    // the global ID of the exit to be used
 	ieVariable UsedExit; // name of the exit, since global id is not stable after loading a new area
-	ieResRef LastArea;
-	char ShieldRef[2];
-	char HelmetRef[2];
-	char WeaponRef[2];
-	int WeaponType;
-	ieDword multiclass;
-	bool GotLUFeedback;
-	int WMLevelMod;
+	ResRef LastArea;
+	AnimRef ShieldRef;
+	AnimRef HelmetRef;
+	AnimRef WeaponRef;
+	unsigned char WeaponType = 0;
+	ieDword multiclass = 0;
+	bool GotLUFeedback = false;
+	int WMLevelMod = 0;
 
-	int LastDamageType;
-	int LastDamage;
+	int LastDamageType = 0;
+	int LastDamage = 0;
 	Point FollowOffset;//follow lastfollowed at this offset
-	bool Spawned;      //has been created by a spawn point
+	bool Spawned = false; // has been created by a spawn point
 
-	ieDword TargetDoor;
+	ieDword TargetDoor = 0;
 
 	EffectQueue fxqueue;
 	
 	vvcDict vfxDict;
 	vvcSet vfxQueue = vvcSet(VVCSort); // sorted so we can distinguish effects infront and behind
-	ieDword *projectileImmunity; //classic bitfield
+	std::vector<bool> projectileImmunity; // classic bitfield
 	Holder<SoundHandle> casting_sound;
-	ieDword roundTime;           //these are timers for attack rounds
-	ieDword panicMode;           //runaway, berserk or randomwalk
-	ieDword nextComment;         //do something random (area comment, interaction)
-	ieDword nextBored;           //do something when bored
-	int FatigueComplaintDelay;   // stagger tired messages
-	ieDword lastInit;
+	ieDword roundTime = 0;           // these are timers for attack rounds
+	ieDword panicMode = PANIC_NONE;  // runaway, berserk or randomwalk
+	ieDword nextComment = 0;         // do something random (area comment, interaction)
+	ieDword nextBored = 0;           // do something when bored
+	int FatigueComplaintDelay = 0;   // stagger tired messages
+	ieDword lastInit = 0;
 	//how many attacks left in this round, must be public for cleave opcode
-	int attackcount;
+	int attackcount = 0;
 
-	PolymorphCache *polymorphCache; // fx_polymorph etc
-	WildSurgeSpellMods wildSurgeMods;
-	ieByte DifficultyMargin;
-	ieDword *spellStates;
+	PolymorphCache* polymorphCache = nullptr; // fx_polymorph etc
+	WildSurgeSpellMods wildSurgeMods{};
+	ieByte DifficultyMargin = 0;
+	ieDword* spellStates = nullptr;
+	// delay all maxhp checks until we completely load all effects
 	// set after modifying maxhp, adjusts hp next tick
-	int checkHP;
+	int checkHP = 2;
 	// to determine that a tick has passed
-	ieDword checkHPTime;
+	ieDword checkHPTime = 0;
 	/**
 	 * We don't know how to profit of them, but PST needs them saved.
 	 * Otherwise, some actors are badly drawn, like TNO but not Morte.
@@ -390,12 +446,14 @@ public:
 	 * bit 1 is for enabling pulsating for the particular color range (we store them in IE_COLOR*)
 	 *   it periodically reduces brightness to ~50% and back to full
 	 */
-	ieByte pstColorBytes[10];
+	ieByte pstColorBytes[10]{};
 	
 	Region drawingRegion;
 private:
-	//this stuff doesn't get saved
-	CharAnimations* anims;
+	String LongName;
+	String ShortName;
+
+	CharAnimations* anims = nullptr;
 	
 	using AnimationPart = std::pair<Animation*, PaletteHolder>;
 	struct {
@@ -403,28 +461,30 @@ private:
 		std::vector<AnimationPart> shadow;
 	} currentStance;
 
-	ieByte SavingThrow[5];
-	ieByte weapSlotCount;
+	SaveInfo lastSave;
+	ieByte weapSlotCount = 4;
 	int walkScale = 0;
 	// true when command has been played after select
-	bool playedCommandSound;
+	bool playedCommandSound = false;
 	//true every second round of attack
-	bool secondround;
-	int attacksperround;
+	bool secondround = false;
+	int attacksperround = 0;
 	//time of our next attack
-	ieDword nextattack;
-	ieDword nextWalk;
-	ieDword lastattack;
+	ieDword nextattack = 0;
+	ieDword nextWalk = 0;
+	ieDword lastattack = 0;
 	//trap we're trying to disarm
-	ieDword disarmTrap;
-	ieDword InTrap;
-	char AttackStance;
+	ieDword disarmTrap = 0;
+	ieDword InTrap = 0;
+	char AttackStance = 0;
 	/*The projectile bringing the current attack*/
-	Projectile* attackProjectile ;
-	ieDword TicksLastRested;
-	ieDword LastFatigueCheck;
-	unsigned int remainingTalkSoundTime;
-	unsigned int lastTalkTimeCheckAt;
+	Projectile* attackProjectile = nullptr;
+	tick_t TicksLastRested = 0;
+	tick_t LastFatigueCheck = 0;
+	tick_t remainingTalkSoundTime = 0;
+	tick_t lastTalkTimeCheckAt = 0;
+	ieDword lastScriptCheck = 0;
+	int lastConBonus;
 	/** paint the actor itself. Called internally by Draw() */
 	void DrawActorSprite(const Point& p, BlitFlags flags,
 						 const std::vector<AnimationPart>& anims, const Color& tint) const;
@@ -432,35 +492,36 @@ private:
 	/** fixes the palette */
 	void SetupColors();
 	/** debugging function, gets the scripting name of an actor referenced by a global ID */
-	const char* GetActorNameByID(ieDword ID) const;
+	ieVariable GetActorNameByID(ieDword ID) const;
 	/* checks a weapon quick slot and resets it to fist if it is empty */
-	void CheckWeaponQuickSlot(unsigned int which);
+	void CheckWeaponQuickSlot(unsigned int which) const;
 	/* helper for usability checks */
-	int CheckUsability(const Item *item) const;
+	HCStrings CheckUsability(const Item* item) const;
 	/* Set up all the missing stats on load time, or after level up */
 	void CreateDerivedStatsBG();
 	/* Set up all the missing stats on load time, or after level up */
 	void CreateDerivedStatsIWD2();
 	/* Gets the given ISCLASS level */
-	ieDword GetClassLevel (const ieDword isclass) const;
+	ieDword GetClassLevel(ieDword isClass) const;
 	/* Returns true if the dual class is backwards */
 	bool IsDualSwap() const;
 	/* returns the weapon proficiency stat of the actor */
-	int GetProficiency(int proftype) const;
+	int GetProficiency(ieByte proftype) const;
+	int GetNonProficiencyPenalty(int stars) const;
+	int GetProficiencyBonus(int& style, bool leftOrRight, int& damageBonus, int& speedBonus, int& criticalBonus) const;
 	/** Re/Inits the Modified vector for PCs/NPCs */
 	void RefreshPCStats();
 	void RefreshHP();
 	bool ShouldDrawCircle() const;
 	bool HasBodyHeat() const;
-	void SetupFistData() const;
 	void UpdateFatigue();
 	int GetSneakAttackDamage(Actor *target, WeaponInfo &wi, int &multiplier, bool weaponImmunity);
-	int GetBackstabDamage(Actor *target, WeaponInfo &wi, int multiplier, int damage) const;
+	int GetBackstabDamage(const Actor *target, WeaponInfo &wi, int multiplier, int damage) const;
 	/** for IE_EXISTANCEDELAY */
 	void PlayExistenceSounds();
-	ieDword GetKitIndex (ieDword kit, ieDword baseclass=0) const;
+	TableMgr::index_t GetKitIndex (ieDword kit, ieDword baseclass=0) const;
 	char GetArmorCode() const;
-	const char* GetArmorSound() const;
+	ResRef GetArmorSound() const;
 
 	bool AdvanceAnimations();
 	void UpdateDrawingRegion();
@@ -469,21 +530,22 @@ private:
 
 	int CalculateSpeedFromRate(bool feedback) const;
 	int CalculateSpeedFromINI(bool feedback) const;
-	ieDword IncrementDeathVariable(Variables *vars, const char *format, const char *name, ieDword start = 0) const;
+	void IncrementDeathVariable(Game::kaputz_t& vars, const char *format, StringView name) const;
+
+	stats_t ResetStats(bool init);
+	void RefreshEffects(bool init, const stats_t& prev);
 
 public:
 	Actor(void);
-	~Actor(void) override;
-	/** releases memory */
-	static void ReleaseMemory();
+	Actor(const Actor&) = delete;
+	~Actor() override;
+	Actor& operator=(const Actor&) = delete;
 	/** sets game specific parameter (which stat should determine the fist weapon type */
 	static void SetFistStat(ieDword stat);
 	/** sets game specific default data about action buttons */
 	static void SetDefaultActions(int qslot, ieByte slot1, ieByte slot2, ieByte slot3);
 	/** prints useful information on console */
-	void dump() const;
-	/** prints useful information to given buffer */
-	void dump(StringBuffer&) const;
+	std::string dump() const;
 	/** fixes the feet circle */
 	void SetCircleSize();
 	/** places the actor on the map */
@@ -499,7 +561,8 @@ public:
 	/** some hardcoded effects in puppetmaster based on puppet type */
 	void CheckPuppet(Actor *puppet, ieDword type);
 	/** Re/Inits the Modified vector */
-	void RefreshEffects(EffectQueue *eqfx);
+	void RefreshEffects();
+	void AddEffects(EffectQueue&& eqfx);
 	/** gets saving throws */
 	void RollSaves();
 	/** returns a saving throw */
@@ -507,13 +570,13 @@ public:
 	/** Returns true if the actor is targetable */
 	bool ValidTarget(int ga_flags, const Scriptable *checker = NULL) const;
 	/** Clamps a stat value to the valid range for the respective stat */
-	ieDword ClampStat(unsigned int StatIndex, ieDword Value) const;
+	stat_t ClampStat(unsigned int StatIndex, stat_t Value) const;
 	/** Returns a Stat value */
-	ieDword GetStat(unsigned int StatIndex) const;
+	stat_t GetStat(unsigned int StatIndex) const;
 	/** Returns a safe Stat value, one, that is not partially computed */
-	ieDword GetSafeStat(unsigned int StatIndex) const;
+	stat_t GetSafeStat(unsigned int StatIndex) const;
 	/** Sets a Stat Value (unsaved) */
-	bool SetStat(unsigned int StatIndex, ieDword Value, int pcf);
+	bool SetStat(unsigned int StatIndex, stat_t Value, int pcf);
 	/** Returns the difference */
 	int GetMod(unsigned int StatIndex) const;
 	/** Returns a Stat Base Value */
@@ -527,29 +590,31 @@ public:
 	int NewStat(unsigned int StatIndex, ieDword ModifierValue, ieDword ModifierType);
 	/** Modifies the base stat value in different ways, returns difference */
 	int NewBase(unsigned int StatIndex, ieDword ModifierValue, ieDword ModifierType);
-	void SetLeader(Actor *actor, int xoffset=0, int yoffset=0);
+	void SetLeader(const Actor* actor, int offset = 0);
 	/** Sets the Icon ResRef */
 	//Which - 0 both, 1 Large, 2 Small
-	void SetPortrait(const char* ResRef, int Which=0);
-	void SetSoundFolder(const char *soundset);
+	void SetPortrait(const ResRef&, int Which=0);
+	void SetSoundFolder(const ieVariable& soundset) const;
 	/* Use overrideSet to replace PCStats->SoundSet */
-	void GetSoundFolder(char *soundset, int flag, ieResRef overrideSet = 0) const;
+	std::string GetSoundFolder(int flag, const ResRef& overrideSet) const;
+	
 	/** Gets the Character Long Name/Short Name */
-	const char* GetName(int which) const override
+	const String& GetShortName() const
 	{
-		if(which==-1) which=TalkCount;
-		if (which) {
-			return LongName;
-		}
 		return ShortName;
 	}
-	/** Gets the DeathVariable */
-	const char* GetScriptName(void) const
+	
+	const String& GetLongName() const
 	{
-		return scriptName;
+		return LongName;
 	}
-	/** Gets a Script ResRef */
-	const char* GetScript(int ScriptIndex) const;
+	
+	const String& GetDefaultName() const
+	{
+		return TalkCount ? GetLongName() : GetShortName();
+	}
+	
+	ResRef GetScript(int ScriptIndex) const;
 	/** Gets the Character's level for XP calculations */
 	ieDword GetXPLevel(int modified) const;
 	/** Guesses the (base) casting level */
@@ -562,8 +627,8 @@ public:
 	int CastingLevelBonus(int level, int type);
 
 	/** Gets the Dialog ResRef */
-	const char* GetDialog(int flags=GD_NORMAL) const;
-	void SetDialog(const ieResRef resref);
+	ResRef GetDialog(int flags=GD_NORMAL) const;
+	void SetDialog(const ResRef &resref);
 	/** Gets the Portrait */
 	Holder<Sprite2D> CopyPortrait(int which) const;
 
@@ -572,8 +637,8 @@ public:
 	{
 		return attackProjectile;
 	}
-	void SetName(const char* ptr, unsigned char type);
-	void SetName(int strref, unsigned char type);
+	void SetName(String str, unsigned char type);
+	void SetName(ieStrRef strref, unsigned char type);
 	/* Returns by how much movement speed should be divided to account for loot weight */
 	int GetEncumbranceFactor(bool feedback) const;
 	/* calculates speed, encumbrance etc */
@@ -585,7 +650,7 @@ public:
 	/* receives undead turning message */
 	void Turn(Scriptable *cleric, ieDword turnlevel);
 	/* call this on gui selects */
-	void PlaySelectionSound();
+	void PlaySelectionSound(bool force = false);
 	/* play a roar if the setting isn't disabled */
 	bool PlayWarCry(int range) const;
 	/* call this when adding actions via gui */
@@ -597,56 +662,54 @@ public:
 	/* sets the actor in panic (turn/morale break) */
 	void Panic(const Scriptable *attacker, int panicmode);
 	/* sets a multi class flag (actually this is a lot of else too) */
-	void SetMCFlag(ieDword bitmask, int op);
+	void SetMCFlag(ieDword bitmask, BitOp op);
 	/* inlined dialogue start */
 	void Interact(int type) const;
 	/* returns a remapped verbal constant strref */
-	ieStrRef GetVerbalConstant(int index) const;
+	ieStrRef GetVerbalConstant(size_t index) const;
 	/* returns a random remapped verbal constant strref */
 	ieStrRef GetVerbalConstant(int start, int count) const;
 	/* displaying a random verbal constant */
 	bool VerbalConstant(int start, int count=1, int flags=0) const;
 	/* display string or verbal constant depending on what is available */
-	void DisplayStringOrVerbalConstant(int str, int vcstat, int vccount=1) const;
+	void DisplayStringOrVerbalConstant(HCStrings str, int vcstat, int vccount = 1) const;
 	/* inlined dialogue response */
 	void Response(int type) const;
 	/* called when someone died in the party */
-	bool HasSpecialDeathReaction(const char *deadname) const;
-	void ReactToDeath(const char *deadname);
+	bool HasSpecialDeathReaction(const ieVariable& deadname) const;
+	void ReactToDeath(const ieVariable& deadname);
 	/* sends trigger_died to everyone in visual range */
 	void SendDiedTrigger() const;
 	/* called when someone talks to Actor */
 	void DialogInterrupt() const;
 	/* called when actor was hit */
-	void GetHit(int damage=0, int spellLevel=0);
+	void GetHit(int damage = 0, bool killingBlow = false);
 	/* checks whether taking damage should disrupt spellcasting */
-	bool CheckSpellDisruption(int damage, int spellLevel) const;
+	bool CheckSpellDisruption(int damage) const;
 	/* called when actor starts to cast a spell*/
-	bool HandleCastingStance(const ieResRef SpellResRef, bool deplete, bool instant);
-	/* check if the actor should be just knocked out by a lethal hit */
-	bool AttackIsStunning(int damagetype) const;
+	bool HandleCastingStance(const ResRef& spellResRef, bool deplete, bool instant);
 	/* check if the actor is silenced - for casting purposes */
 	bool CheckSilenced() const;
 	/* check and perform a cleave movement */
 	void CheckCleave();
 	/* deals damage to this actor */
-	int Damage(int damage, int damagetype, Scriptable *hitter, int modtype=MOD_ADDITIVE, int critical=0, int saveflags=0);
+	int Damage(int damage, int damagetype, Scriptable* hitter, int modtype = MOD_ADDITIVE, int critical = 0, int saveflags = 0, int specialFlags = 0);
 	/* displays the damage taken and other details (depends on the game type) */
-	void DisplayCombatFeedback (unsigned int damage, int resisted, int damagetype, Scriptable *hitter);
+	void DisplayCombatFeedback(unsigned int damage, int resisted, int damagetype, const Scriptable *hitter);
 	/* play a random footstep sound */
 	void PlayWalkSound();
 	/* play the proper hit sound (in pst) */
-	void PlayHitSound(DataFileMgr *resdata, int damagetype, bool suffix) const;
-	void PlaySwingSound(WeaponInfo &wi) const;
+	void PlayHitSound(const DataFileMgr *resdata, int damagetype, bool suffix) const;
+	void PlaySwingSound(const WeaponInfo &wi) const;
 	/* drops items from inventory to current spot */
-	void DropItem(const ieResRef resref, unsigned int flags);
+	void DropItem(const ResRef& resref, unsigned int flags);
 	void DropItem(int slot, unsigned int flags);
 	/* returns item information in quickitem slot */
-	void GetItemSlotInfo(ItemExtHeader *item, int which, int header);
+	void GetItemSlotInfo(ItemExtHeader *item, int which, int header) const;
 	/* returns spell information in quickspell slot */
 	void GetSpellSlotInfo(SpellExtHeader *spell, int which);
 	/* updates quickslots */
-	void ReinitQuickSlots();
+	void ReinitQuickSlots() const;
 	/* actor is in trap */
 	void SetInTrap(ieDword tmp);
 	/* sets some of the internal flags */
@@ -655,9 +718,9 @@ public:
 	/* applies the kit abilities, returns false if kit is not applicable */
 	bool ApplyKit(bool remove, ieDword baseclass=0, int diff=0);
 	/* applies the class abilities*/
-	void ApplyClab(const char *clab, ieDword max, int remove, int diff);
+	void ApplyClab(const ResRef& clab, ieDword max, int remove, int diff);
 	/* calls InitQuickSlot in PCStatStruct */
-	void SetupQuickSlot(unsigned int which, int slot, int headerindex);
+	void SetupQuickSlot(unsigned int which, ieWord slot, ieWord headerIndex) const;
 	/* returns true if the actor is PC/joinable*/
 	bool Persistent() const;
 	/* assigns actor to party slot, 0 = NPC, areas won't remove it */
@@ -676,32 +739,26 @@ public:
 	void GetNextStance();
 	void ClearCurrentStanceAnims();
 	/* learns the given spell, possibly receive XP */
-	int LearnSpell(const ieResRef resref, ieDword flags, int bookmask=-1, int level=-1);
-	/* returns the ranged weapon header associated with the currently equipped projectile */
-	ITMExtHeader *GetRangedWeapon(WeaponInfo &wi) const;
+	int LearnSpell(const ResRef& resref, ieDword flags, int bookmask=-1, int level=-1);
 	/* Returns current weapon range and extended header
 	if range is nonzero, then which is valid */
-	ITMExtHeader* GetWeapon(WeaponInfo &wi, bool leftorright=false) const;
+	const ITMExtHeader* GetWeapon(bool leftOrRight) const;
 	/* Creates player statistics */
 	void CreateStats();
 	/* Heals actor */
 	void Heal(int hp);
 	/* Receive experience (handle dual/multi class) */
 	void AddExperience(int exp, int combat);
-	/* Calculate experience bonus */
-	int CalculateExperience(int type, int level) const;
 	/* Sets the modal state after checks */
-	void SetModal(ieDword newstate, bool force=1);
+	void SetModal(ieDword newstate, bool force = true);
 	/* Sets the modal spell after checks */
-	void SetModalSpell(ieDword state, const char *spell);
+	void SetModalSpell(ieDword state, const ResRef& spell);
 	/* casts the modal spell if any */
-	void ApplyModal(ieResRef modalSpell);
-	/* returns current attack style */
-	int GetAttackStyle() const;
+	void ApplyModal(const ResRef& modalSpell);
 	/* adds the combatants to the attackers list */
 	void AttackedBy(const Actor *actor);
 	/* reorients to face target (for immediate attack) */
-	void FaceTarget(Scriptable *actor);
+	void FaceTarget(const Scriptable *actor);
 	/* returns the number of attacks (handles monk barehanded bonus) */
 	ieDword GetNumberOfAttacks();
 	/* starts combat round*/
@@ -713,15 +770,16 @@ public:
 	void GetTHAbilityBonus(ieDword Flags);
 	/* gets the defense against an attack */
 	int GetDefense(int DamageType, ieDword wflags, const Actor *attacker) const;
-	/* checks if something is wrong with the weapon we are using for the attack */
-	bool WeaponIsUsable(bool leftorright, ITMExtHeader *header=NULL) const;
+	/* returns the number of allocated proficiency points (stars) */
+	int GetStars(stat_t proficiency) const;
+	static bool IsCriticalEffectEligible(const WeaponInfo& wi, const Effect* fx);
 	/* get the current hit bonus */
-	bool GetCombatDetails(int &tohit, bool leftorright, WeaponInfo &wi, ITMExtHeader *&header, ITMExtHeader *&hittingheader,\
-		int &DamageBonus, int &speed, int &CriticalBonus, int &style, const Actor *target);
+	bool GetCombatDetails(int& tohit, bool leftorright, \
+		int& DamageBonus, int& speed, int& CriticalBonus, int& style, const Actor* target);
 	/* performs attack against target */
 	void PerformAttack(ieDword gameTime);
 	/* returns the adjusted weapon range, since items have odd values stored */
-	int GetWeaponRange(const WeaponInfo &wi) const;
+	unsigned int GetWeaponRange(bool leftOrRight) const;
 	/* filter out any damage reduction that is cancelled by high weapon enchantment and return the resulting resistance */
 	int GetDamageReduction(int resist_stat, ieDword weaponEnchantment) const;
 	/* calculates strength (dexterity) based damage adjustment */
@@ -744,10 +802,10 @@ public:
 	/* overridden method, won't walk if dead */
 	void WalkTo(const Point &Des, ieDword flags, int MinDistance = 0);
 	/* resolve string constant (sound will be altered) */
-	void ResolveStringConstant(ieResRef& sound, unsigned int index) const;
-	bool GetSoundFromFile(ieResRef &Sound, unsigned int index) const;
-	bool GetSoundFromINI(ieResRef &Sound, unsigned int index) const;
-	bool GetSoundFrom2DA(ieResRef &Sound, unsigned int index) const;
+	void GetVerbalConstantSound(ResRef& sound, size_t index) const;
+	bool GetSoundFromFile(ResRef& Sound, TableMgr::index_t index) const;
+	bool GetSoundFromINI(ResRef& Sound, TableMgr::index_t index) const;
+	bool GetSoundFrom2DA(ResRef &Sound, TableMgr::index_t index) const;
 	/* generate area specific oneliner */
 	void GetAreaComment(int areaflag) const;
 	/* handle oneliner interaction, -1: unsuccessful (may comment area), 0: dialog banter, 1: oneliner */
@@ -757,11 +815,11 @@ public:
 	/* generate party banter, return true if successful */
 	bool GetPartyComment();
 	/* sets the quick slots */
-	void SetActionButtonRow(ActionButtonRow &ar);
+	void SetActionButtonRow(const ActionButtonRow &ar) const;
 	/* updates the quick slots */
 	void GetActionButtonRow(ActionButtonRow &qs);
 	/* converts the iwd2 qslot index to our internal representation */
-	int IWD2GemrbQslot (int slotindex) const;
+	ieByte IWD2GemrbQslot(int slotindex) const;
 	int Gemrb2IWD2Qslot(ieByte actslot, int slotindex) const;
 	void dumpQSlots() const;
 
@@ -771,9 +829,9 @@ public:
 	/* update internal per frame state and return true if state is suitable for drawing the actor */
 	bool UpdateDrawingState();
 	Region DrawingRegion() const override;
-	uint8_t GetElevation() const;
+	int GetElevation() const;
 	bool ShouldDrawReticle() const;
-	void DoStep(unsigned int walkScale, ieDword time = 0) override;
+	void DoStep(unsigned int newWalkScale, ieDword time = 0) override;
 	void Draw(const Region &screen, Color baseTint, Color tint, BlitFlags flags) const;
 
 	/* add mobile vvc (spell effects) to actor's list */
@@ -791,7 +849,7 @@ public:
 
 	void SetLockedPalette(const ieDword *gradients);
 	void UnlockPalette();
-	void AddAnimation(const ieResRef resource, int gradient, int height, int flags);
+	void AddAnimation(const ResRef& resource, int gradient, int height, int flags);
 	/* plays damage animation, if hit is not set, then plays only the splash part */
 	void PlayDamageAnimation(int x, bool hit=true);
 	void PlayCritDamageAnimation(int x);
@@ -802,56 +860,56 @@ public:
 	/* returns the monk wisdom AC adjusted by armor (iwd2) */
 	int GetWisdomAC() const;
 	/* PST specific criticals */
-	int GetCriticalType() const;
+	ieWord GetCriticalType() const;
 	/* restores a spell of maximum maxlevel level, type is a mask of disabled spells */
 	int RestoreSpellLevel(ieDword maxlevel, ieDword typemask);
 	/* rememorizes spells, cures fatigue, etc */
 	void Rest(int hours);
 	int GetConHealAmount() const;
-	/* returns the portrait icons list */
-	const unsigned char *GetStateString() const;
 	/* adds a state icon to the list */
-	void AddPortraitIcon(ieByte icon);
+	void AddPortraitIcon(ieByte icon) const;
 	/* disables a state icon in the list, doesn't remove it! */
-	void DisablePortraitIcon(ieByte icon);
+	void DisablePortraitIcon(ieByte icon) const;
 	/* returns which slot belongs to the quickweapon slot */
 	int GetQuickSlot(int slot) const;
 	/* Sets equipped Quick slot, if header is -1, then use the current one */
-	int SetEquippedQuickSlot(int slot, int header);
+	HCStrings SetEquippedQuickSlot(int slot, int header);
 	/* Uses an item on the target or point */
+	bool TryUsingMagicDevice(const Item* item, ieDword header);
+	bool RequiresUMD(const Item* item) const;
 	bool UseItemPoint(ieDword slot, ieDword header, const Point &point, ieDword flags);
-	bool UseItem(ieDword slot, ieDword header, Scriptable *target, ieDword flags, int damage = 0);
+	bool UseItem(ieDword slot, ieDword header, const Scriptable *target, ieDword flags, int damage = 0);
 	/* Deducts a charge from an item */
-	void ChargeItem(ieDword slot, ieDword header, CREItem *item, Item *itm, bool silent, bool expend = true);
+	void ChargeItem(ieDword slot, ieDword header, CREItem *item, const Item *itm, bool silent, bool expend = true);
 	/* If it returns true, then default AC=10 and the lesser the better */
 	static int IsReverseToHit();
 	/* initialize the action buttons based on class. If forced, it will override
 		previously customized or set buttons. */
-	void InitButtons(ieDword cls, bool forced);
+	void InitButtons(ieDword cls, bool forced) const;
 	int GetMaxEncumbrance() const;
 	int GetAbilityBonus(unsigned int ability, int value = -1) const;
 	int GetSkillStat(unsigned int skill) const;
 	int GetSkill(unsigned int skill, bool ids=false) const;
 	int GetFeat(unsigned int feat) const;
-	void SetFeat(unsigned int feat, int mode);
+	void SetFeat(unsigned int feat, BitOp mode);
 	void SetFeatValue(unsigned int feat, int value, bool init = true);
-	void SetUsedWeapon(const char (&AnimationType)[2], ieWord *MeleeAnimation,
-		int WeaponType=-1);
-	void SetUsedShield(const char (&AnimationType)[2], int WeaponType=-1);
-	void SetUsedHelmet(const char (&AnimationType)[2]);
+	void SetUsedWeapon(AnimRef, const ieWord *MeleeAnimation,
+		unsigned char WeaponType = IE_ANI_WEAPON_INVALID);
+	void SetUsedShield(AnimRef, unsigned char WeaponType = IE_ANI_WEAPON_INVALID);
+	void SetUsedHelmet(AnimRef);
 	void SetupFist();
 	/* Returns nonzero if the caster is held */
 	int Immobile() const;
 	/* Returns strref if the item is unusable due to name/type restrictions */
-	ieStrRef Disabled(const ieResRef name, ieDword type) const;
+	ieStrRef Disabled(const ResRef& name, ieDword type) const;
 	/* Returns constant string if the item is unusable */
-	int Unusable(const Item *item) const;
+	HCStrings Unusable(const Item* item) const;
 	/* Sets all clown colour to the given gradient */
 	void SetGradient(ieDword gradient);
 	/* Enables an overlay */
 	void SetOverlay(unsigned int overlay);
 	/* Checks and sets a spellstate if it wasn't set yet */
-	bool SetSpellState(unsigned int spellstate);
+	bool SetSpellState(unsigned int spellstate) const;
 	/* Checks a spellstate */
 	bool HasSpellState(unsigned int spellstate) const;
 	/* Checks a feat */
@@ -874,9 +932,9 @@ public:
 	bool IsDualClassed() const;
 	/* Returns an exact copy of this actor */
 	Actor *CopySelf(bool mislead) const;
-	static ieDword GetClassID (const ieDword isclass);
-	const char *GetClassName(ieDword classID) const;
-	const char *GetKitName(ieDword kitID) const;
+	static ieDword GetClassID(ieDword isClass);
+	const std::string& GetClassName(ieDword classID) const;
+	const std::string& GetKitName(ieDword kitID) const;
 	/* Returns the actor's level of the given class */
 	ieDword GetFighterLevel() const { return GetClassLevel(ISFIGHTER); }
 	ieDword GetMageLevel() const { return GetClassLevel(ISMAGE); }
@@ -902,7 +960,7 @@ public:
 	void UseExit(ieDword exitID);
 	//int GetReaction() const;
 	/* Similar to Roll, but takes luck into account */
-	int LuckyRoll(int dice, int size, int add, ieDword flags=LR_CRITICAL, Actor* opponent=NULL) const;
+	int LuckyRoll(int dice, int size, int add, ieDword flags = LR_CRITICAL, const Actor* opponent = nullptr) const;
 	/* removes normal invisibility (type 0) */
 	void CureInvisibility();
 	/* removes sanctuary */
@@ -910,7 +968,7 @@ public:
 	/* resets the invisibility, sanctuary and modal states */
 	void ResetState();
 	/* checks whether the actor is behind the target */
-	bool IsBehind(Actor* target) const;
+	bool IsBehind(const Actor* target) const;
 	/* checks whether the target is the actor's racial enemy */
 	int GetRacialEnemyBonus(const Actor *target) const;
 	/* checks whether the actor can stay in the current modal state */
@@ -923,14 +981,14 @@ public:
 	/* checks if the alignment matches one of the masking constants */
 	//bool MatchesAlignmentMask(ieDword mask);
 	/** untargetable by spells/attack due to invisibility or sanctuary */
-	bool Untargetable(ieResRef spellRef) const;
+	bool Untargetable(const ResRef& spellRef) const;
 	/* returns true if this it is futile to try to harm actor (dead/sanctuaried) */
 	bool InvalidSpellTarget() const;
 	/* returns true if the spell is useless to cast on target
 	or the spell's range is smaller than range */
 	bool InvalidSpellTarget(int spellnum, Actor *caster, int range) const;
 	/* function to get a class level used by scripting */
-	ieDword GetLevelInClass (const ieDword classid) const;
+	ieDword GetLevelInClass(ieDword classid) const;
 	/* computes the actor's classmask (iwd2) */
 	int GetClassMask() const;
 	/* computes the actor's usable books (iwd2) */
@@ -950,7 +1008,7 @@ public:
 	bool IsDead() const;
 	bool IsInvisibleTo(const Scriptable *checker) const;
 	int UpdateAnimationID(bool derived);
-	void MovementCommand(char *command);
+	void MovementCommand(std::string command);
 	/* shows hp/maxhp as overhead text */
 	bool HasVisibleHP() const;
 	void DisplayHeadHPRatio();
@@ -963,22 +1021,23 @@ public:
 	ieDword GetDisarmingTrap() const { return disarmTrap; }
 	void ReleaseCurrentAction() override;
 	bool ConcentrationCheck() const;
-	void ApplyEffectCopy(Effect *oldfx, EffectRef &newref, Scriptable *Owner, ieDword param1, ieDword param2);
-	ieDword GetLastRested() { return TicksLastRested; }
+	void ApplyEffectCopy(const Effect *oldfx, EffectRef &newref, Scriptable *Owner, ieDword param1, ieDword param2);
+	tick_t GetLastRested() const { return TicksLastRested; }
 	void IncreaseLastRested(int inc) { TicksLastRested += inc; LastFatigueCheck += inc; }
 	bool WasClass(ieDword oldClassID) const;
 	ieDword GetActiveClass() const;
 	bool IsKitInactive() const;
-	const char* GetRaceName() const;
+	const std::string& GetRaceName() const;
 	unsigned int GetSubRace() const;
 	std::list<int> ListLevels() const;
 	void ChangeSorcererType (ieDword classIdx);
-	unsigned int GetAdjustedTime(unsigned int time) const;
-	void SetAnimatedTalking(unsigned int);
+	tick_t GetAdjustedTime(tick_t time) const;
+	void SetAnimatedTalking(tick_t);
 	bool HasPlayerClass() const;
 	void PlayArmorSound() const;
 	bool ShouldModifyMorale() const;
 	bool HibernateIfAble();
+	bool ForceScriptCheck();
 };
 }
 
