@@ -22,9 +22,11 @@
 #define SCRIPTABLE_H
 
 #include "exports.h"
+#include "ie_cursors.h"
+#include "ie_types.h"
 
 #include "CharAnimations.h"
-#include "Variables.h"
+#include "OverHeadText.h"
 
 #include <list>
 #include <map>
@@ -43,7 +45,8 @@ class InfoPoint;
 class Map;
 class Movable;
 class Object;
-struct PathNode;
+struct PathListNode;
+class Projectile;
 class Scriptable;
 class Selectable;
 class Spell;
@@ -77,7 +80,7 @@ class Sprite2D;
 //#define TRAP_ENEMY	 16 // "trap set off by enemy" in NI, unused
 #define TRAP_TUTORIAL	 32 //active only when in tutorial mode
 #define TRAP_NPC	64 // "trap set off by NPC"
-//#define TRAP_SILENT	128 // "trigger silent", unused
+#define TRAP_SILENT	128 // "trigger silent" / "no string", used in pst
 #define TRAP_DEACTIVATED  256
 #define _TRAVEL_NONPC      512
 #define _TRAP_USEPOINT       1024 //override usage point of travel regions (used for sound in PST traps)
@@ -127,13 +130,14 @@ class Sprite2D;
 #define XP_LOCKPICK   0
 #define XP_DISARM     1
 #define XP_LEARNSPELL 2
+#define XP_PICKPOCKET  3 // gemrb extension
 
 #define MAX_PATH_TRIES 8
 #define MAX_BUMP_BACK_TRIES 16
 #define MAX_RAND_WALK 10
 
-typedef enum ScriptableType { ST_ACTOR = 0, ST_PROXIMITY = 1, ST_TRIGGER = 2,
-ST_TRAVEL = 3, ST_DOOR = 4, ST_CONTAINER = 5, ST_AREA = 6, ST_GLOBAL = 7 } ScriptableType;
+using ScriptableType = enum ScriptableType { ST_ACTOR = 0, ST_PROXIMITY = 1, ST_TRIGGER = 2,
+	ST_TRAVEL = 3, ST_DOOR = 4, ST_CONTAINER = 5, ST_AREA = 6, ST_GLOBAL = 7 };
 
 enum {
 	trigger_acquired = 0x1, // unused and broken in the original
@@ -203,7 +207,8 @@ enum {
 	trigger_namelessbitthedust = 0xab, // pst
 	trigger_failedtoopen = 0xaf, // pst
 	trigger_tookdamage = 0xcc, // bg2
-	trigger_walkedtotrigger = 0xd6 // bg2
+	trigger_walkedtotrigger = 0xd6, // bg2
+	trigger_secreddoordetected = 0x100 // ees
 };
 
 // flags for TriggerEntry
@@ -214,115 +219,128 @@ enum {
 };
 
 struct TriggerEntry {
-	TriggerEntry(unsigned short id) : triggerID(id), param1(0), param2(0), flags(0) { }
-	TriggerEntry(unsigned short id, ieDword p1) : triggerID(id), param1(p1), param2(0), flags(0) { }
-	TriggerEntry(unsigned short id, ieDword p1, ieDword p2) : triggerID(id), param1(p1), param2(p2), flags(0) { }
+	explicit TriggerEntry(unsigned short id) : triggerID(id) { }
+	TriggerEntry(unsigned short id, ieDword p1) : triggerID(id), param1(p1) { }
+	TriggerEntry(unsigned short id, ieDword p1, ieDword p2) : triggerID(id), param1(p1), param2(p2) { }
 
 	unsigned short triggerID;
-	ieDword param1;
-	ieDword param2;
-	unsigned int flags;
+	ieDword param1 = 0;
+	ieDword param2 = 0;
+	unsigned int flags = 0;
 };
 
-//typedef std::list<ieDword *> TriggerObjects;
-
-//#define SEA_RESET		0x00000002
-//#define SEA_PARTY_REQUIRED	0x00000004
-
 class GEM_EXPORT Scriptable {
 public:
-	Scriptable(ScriptableType type);
-	virtual ~Scriptable(void);
+	explicit Scriptable(ScriptableType type);
+	Scriptable(const Scriptable&) = delete;
+	virtual ~Scriptable();
+	Scriptable& operator=(const Scriptable&) = delete;
 private:
-	unsigned long WaitCounter;
+	tick_t WaitCounter = 0;
 	std::map<ieDword,ieDword> script_timers;
-	ieDword globalID;
+	ieDword globalID = 0;
 protected: //let Actor access this
 	std::list<TriggerEntry> triggers;
-	Map *area;
+	Map *area = nullptr;
 	ieVariable scriptName;
-	ieDword InternalFlags; //for triggers
-	ieResRef Dialog;
+	ieDword InternalFlags = 0; // for triggers
+	ResRef Dialog;
 	std::list< Action*> actionQueue;
-	Action* CurrentAction;
-
-	// Variables for overhead text.
-	Point overHeadTextPos;
-	bool overheadTextDisplaying;
-	unsigned long timeStartDisplaying;
-	String OverheadText;
+	Action* CurrentAction = nullptr;
 public:
 	Region BBox;
 	// State relating to the currently-running action.
-	int CurrentActionState;
-	ieDword CurrentActionTarget;
-	bool CurrentActionInterruptable;
-	ieDword CurrentActionTicks;
+	int CurrentActionState = 0;
+	ieDword CurrentActionTarget = 0;
+	bool CurrentActionInterruptable = true;
+	ieDword CurrentActionTicks = 0;
 
 	// The number of times this was updated.
-	ieDword Ticks;
+	ieDword Ticks = 0;
 	// The same, after adjustment for being slowed/hasted.
-	ieDword AdjustedTicks;
-	// The number of times UpdateActions() was run.
-	ieDword ScriptTicks;
-	// The number of times since UpdateActions() tried to do anything.
-	ieDword IdleTicks;
+	ieDword AdjustedTicks = 0;
+	// The number of times TickScripting() was run.
+	ieDword ScriptTicks = 0;
+	// The number of times since TickScripting() tried to do anything.
+	ieDword IdleTicks = 0;
 	// The number of ticks since the last spellcast
-	ieDword AuraTicks;
+	ieDword AuraCooldown = 0;
 	// The countdown for forced activation by triggers.
-	ieDword TriggerCountdown;
+	ieDword TriggerCountdown = 0;
 
-	Variables* locals;
-	ScriptableType Type;
+	ieVarsMap locals;
+	ScriptableType Type = ST_ACTOR;
 	Point Pos;
 
-	ieStrRef DialogName;
+	ieStrRef DialogName = ieStrRef::INVALID;
+	OverHeadText overHead{this};
 
-	GameScript* Scripts[MAX_SCRIPTS];
-	int scriptlevel;
+	GameScript* Scripts[MAX_SCRIPTS] = {};
+	int scriptlevel = 0;
 
-	ieDword UnselectableTimer;
+	ieDword UnselectableTimer = 0;
 
 	// Stored objects.
-	ieDword LastAttacker;
-	ieDword LastCommander;
-	ieDword LastProtector;
-	ieDword LastProtectee;
-	ieDword LastTargetedBy;
-	ieDword LastHitter;
-	ieDword LastHelp;
-	ieDword LastTrigger;
-	ieDword LastSeen;
-	ieDword LastTalker;
-	ieDword LastHeard;
-	ieDword LastSummoner;
-	ieDword LastFollowed; // gemrb extension (LeaderOf)
-	ieDword LastMarked; // iwd2
+	ieDword LastAttacker = 0;
+	ieDword LastCommander = 0;
+	ieDword LastProtector = 0;
+	ieDword LastProtectee = 0;
+	ieDword LastTargetedBy = 0;
+	ieDword LastHitter = 0;
+	ieDword LastHelp = 0;
+	ieDword LastTrigger = 0;
+	ieDword LastSeen = 0;
+	ieDword LastTalker = 0;
+	ieDword LastHeard = 0;
+	ieDword LastSummoner = 0;
+	ieDword LastFollowed = 0; // gemrb extension (LeaderOf)
+	ieDword LastMarked = 0; // iwd2
 	ieDword MyTarget = 0; // iwd2, has nothing to do with LastTarget
 
-	int LastMarkedSpell; // iwd2
+	int LastMarkedSpell = 0; // iwd2
 
 	// this is used by GUIScript :(
-	ieDword LastSpellOnMe;  //Last spell cast on this scriptable
+	ieDword LastSpellOnMe = 0xffffffff;  // Last spell cast on this scriptable
 
-	ieDword LastTarget, LastSpellTarget;
-	ieDword LastTargetPersistent; // gemrb extension, persists across actions; remove if LastTarget ever gets the same persistence
+	ieDword LastTarget = 0;
+	ieDword LastSpellTarget = 0;
+	ieDword LastTargetPersistent = 0; // gemrb extension, persists across actions; remove if LastTarget ever gets the same persistence
 	Point LastTargetPos;
-	int SpellHeader;
-	ieResRef SpellResRef;
-	bool InterruptCasting;
+	int SpellHeader = 0;
+	ResRef SpellResRef;
+	bool InterruptCasting = false;
+	unsigned char weightsAsCases = 0;
 public:
+	
+	template <class RETURN, class PARAM>
+	static constexpr auto As(PARAM* obj)
+	-> typename std::conditional<std::is_const<PARAM>::value, const RETURN*, RETURN*>::type {
+		static_assert(std::is_base_of<Scriptable, RETURN>::value, "Attempted bad Scriptable cast!");
+		// dynamic_cast will return nullptr if the cast is invalid
+		return dynamic_cast<decltype(As<RETURN, PARAM>(obj))>(obj);
+	}
+
+	template <class RETURN>
+	RETURN* As() {
+		return Scriptable::As<RETURN>(this);
+	}
+	
+	template <class RETURN>
+	constexpr RETURN* As() const {
+		return Scriptable::As<RETURN>(this);
+	}
+
 	/** Gets the Dialog ResRef */
-	const char* GetDialog(void) const
+	ResRef GetDialog() const
 	{
 		return Dialog;
 	}
-	void SetDialog(const char *resref);
+	void SetDialog(const ResRef &resref);
 	void SetFloatingText(char*);
-	void SetScript(const ieResRef aScript, int idx, bool ai=false);
-	void SetSpellResRef(ieResRef resref);
-	void SetWait(unsigned long time);
-	unsigned long GetWait() const;
+	void SetScript(const ResRef &aScript, int idx, bool ai = false);
+	void SetSpellResRef(const ResRef& resref);
+	void SetWait(tick_t time);
+	tick_t GetWait() const;
 	void LeftDialog();
 	void Interrupt();
 	void NoInterrupt();
@@ -332,16 +350,11 @@ public:
 	void Deactivate();
 	void PartyRested();
 	ieDword GetInternalFlag() const;
-	void SetInternalFlag(unsigned int value, int mode);
-	const char* GetScriptName() const;
+	void SetInternalFlag(unsigned int value, BitOp mode);
+	const ieVariable& GetScriptName() const;
 	Map* GetCurrentArea() const;
 	void SetMap(Map *map);
-	void SetOverheadText(const String& text, bool display = true);
-	const String& GetOverheadText() const { return OverheadText; };
-	bool DisplayOverheadText(bool);
-	bool OverheadTextIsDisplaying() const { return overheadTextDisplaying; }
-	void FixHeadTextPos();
-	void SetScriptName(const char* text);
+	void SetScriptName(const ieVariable& text);
 	//call this to enable script running as soon as possible
 	void ImmediateEvent();
 	bool IsPC() const;
@@ -353,8 +366,8 @@ public:
 	Action* GetCurrentAction() const { return CurrentAction; }
 	Action* GetNextAction() const;
 	Action* PopNextAction();
-	void ClearActions();
-	virtual void Stop();
+	void ClearActions(int skipFlags = 0);
+	virtual void Stop(int flags = 0);
 	virtual void ReleaseCurrentAction();
 	bool InMove() const;
 	void ProcessActions();
@@ -362,8 +375,8 @@ public:
 	//true condition (whole triggerblock returned true)
 	void ClearTriggers();
 	void AddTrigger(TriggerEntry trigger);
-	void SetLastTrigger(ieDword triggerID, ieDword globalID);
-	bool MatchTrigger(unsigned short id, ieDword param = 0);
+	void SetLastTrigger(ieDword triggerID, ieDword scriptableID);
+	bool MatchTrigger(unsigned short id, ieDword param = 0) const;
 	bool MatchTriggerWithObject(short unsigned int id, const Object *obj, ieDword param = 0) const;
 	const TriggerEntry *GetMatchingTrigger(unsigned short id, unsigned int notflags = 0) const;
 	void SendTriggerToAll(TriggerEntry entry);
@@ -371,48 +384,51 @@ public:
 	void DrawOverheadText();
 	virtual Region DrawingRegion() const;
 	/* check if casting is allowed at all */
-	int CanCast(const ieResRef SpellRef, bool verbose = true);
+	int CanCast(const ResRef& SpellRef, bool verbose = true);
 	/* check for and trigger a wild surge */
 	int CheckWildSurge();
-	void SpellcraftCheck(const Actor *caster, const ieResRef SpellRef);
+	void SpellcraftCheck(const Actor *caster, const ResRef& spellRef);
 	/* internal spellcasting shortcuts */
-	void DirectlyCastSpellPoint(const Point &target, ieResRef spellref, int level, int no_stance, bool deplete);
-	void DirectlyCastSpell(Scriptable *target, ieResRef spellref, int level, int no_stance, bool deplete);
+	void DirectlyCastSpellPoint(const Point& target, const ResRef& spellRef, int level, bool keepStance, bool deplete);
+	void DirectlyCastSpell(Scriptable* target, const ResRef& spellRef, int level, bool keepStance, bool deplete);
 	/* actor/scriptable casts spell */
-	int CastSpellPoint( const Point &Target, bool deplete, bool instant = false, bool nointerrupt = false );
-	int CastSpell( Scriptable* Target, bool deplete, bool instant = false, bool nointerrupt = false );
+	int CastSpellPoint(const Point& target, bool deplete, bool instant = false, bool noInterrupt = false, int level = 0);
+	int CastSpell(Scriptable* target, bool deplete, bool instant = false, bool noInterrupt = false, int level = 0);
 	/* spellcasting finished */
-	void CastSpellPointEnd(int level, int no_stance);
-	void CastSpellEnd(int level, int no_stance);
+	void CastSpellPointEnd(int level, bool keepStance);
+	void CastSpellEnd(int level, bool keepStance);
 	ieDword GetGlobalID() const { return globalID; }
 	/** timer functions (numeric ID, not saved) */
 	bool TimerActive(ieDword ID);
 	bool TimerExpired(ieDword ID);
 	void StartTimer(ieDword ID, ieDword expiration);
-	virtual const char* GetName(int /*which*/) const { return NULL; }
+	String GetName() const;
 	bool AuraPolluted();
+	ieDword GetLocal(const ieVariable& key, ieDword fallback) const;
+	void DumpLocals() const;
 private:
 	/* used internally to handle start of spellcasting */
-	int SpellCast(bool instant, Scriptable *target = NULL);
+	int SpellCast(bool instant, Scriptable* target = nullptr, int level = 0);
 	/* also part of the spellcasting process, creating the projectile */
-	void CreateProjectile(const ieResRef SpellResRef, ieDword tgt, int level, bool fake);
+	void CreateProjectile(const ResRef& spellResRef, ieDword tgt, int level, bool fake);
 	/* do some magic for the weird/awesome wild surges */
-	bool HandleHardcodedSurge(ieResRef surgeSpellRef, Spell *spl, Actor *caster);
+	bool HandleHardcodedSurge(const ResRef& surgeSpell, const Spell *spl, Actor *caster);
+	void ModifyProjectile(Projectile* &pro, Spell* spl, ieDword tgt, int level);
 	void ResetCastingState(Actor* caster);
-	void DisplaySpellCastMessage(ieDword tgt, Spell *spl);
+	void DisplaySpellCastMessage(ieDword tgt, const Spell* spl) const;
 };
 
 class GEM_EXPORT Selectable : public Scriptable {
 public:
-	Selectable(ScriptableType type);
+	using Scriptable::Scriptable;
 public:
-	ieWord Selected; //could be 0x80 for unselectable
-	bool Over;
-	Color selectedColor;
-	Color overColor;
-	Holder<Sprite2D> circleBitmap[2];
-	int size;
-	float sizeFactor;
+	ieWord Selected = 0; // could be 0x80 for unselectable
+	bool Over = false;
+	Color selectedColor = ColorBlack;
+	Color overColor = ColorBlack;
+	Holder<Sprite2D> circleBitmap[2] = {};
+	int circleSize = 0;
+	float sizeFactor = 1.0f;
 public:
 	void SetBBox(const Region &newBBox);
 	void DrawCircle(const Point& p) const;
@@ -421,38 +437,35 @@ public:
 	bool IsSelected() const;
 	void Select(int Value);
 	void SetCircle(int size, float, const Color &color, Holder<Sprite2D> normal_circle, Holder<Sprite2D> selected_circle);
+	int CircleSize2Radius() const;
 };
 
 class GEM_EXPORT Highlightable : public Scriptable {
 public:
-	Highlightable(ScriptableType type);
+	using Scriptable::Scriptable;
 	virtual int TrapResets() const = 0;
 	virtual bool CanDetectTrap() const { return true; }
 	virtual bool PossibleToSeeTrap() const;
 public:
-	std::shared_ptr<Gem_Polygon> outline;
-	Color outlineColor;
-	ieDword Cursor;
-	bool Highlight;
-	Point TrapLaunch;
-	ieWord TrapDetectionDiff;
-	ieWord TrapRemovalDiff;
-	ieWord Trapped;
-	ieWord TrapDetected;
-	ieResRef KeyResRef;
+	std::shared_ptr<Gem_Polygon> outline = nullptr;
+	Color outlineColor = ColorBlack;
+	ieDword Cursor = IE_CURSOR_NORMAL;
+	bool Highlight = false;
+	Point TrapLaunch = Point(-1, -1);
+	ieWord TrapDetectionDiff = 0;
+	ieWord TrapRemovalDiff = 0;
+	ieWord Trapped = 0;
+	ieWord TrapDetected = 0;
+	ResRef KeyResRef;
 	//play this wav file when stepping on the trap (on PST)
-	ieResRef EnterWav;
+	ResRef EnterWav;
 public:
 	bool IsOver(const Point &Place) const;
 	void DrawOutline(Point origin) const;
 	void SetCursor(unsigned char CursorIndex);
-	const char* GetKey(void) const
-	{
-		if (KeyResRef[0]) return KeyResRef;
-		return NULL;
-	}
+
 	void SetTrapDetected(int x);
-	void TryDisarm(const Actor *actor);
+	void TryDisarm(Actor* actor);
 	//detect trap, set skill to 256 if you want sure fire
 	void DetectTrap(int skill, ieDword actorID);
 	//returns true if trap is visible, only_detected must be true
@@ -460,29 +473,30 @@ public:
 	bool VisibleTrap(int only_detected) const;
 	//returns true if trap has been triggered, tumble skill???
 	virtual bool TriggerTrap(int skill, ieDword ID);
-	bool TryUnlock(Actor *actor, bool removekey);
+	bool TryUnlock(Actor *actor, bool removekey) const;
 };
 
 class GEM_EXPORT Movable : public Selectable {
 private: //these seem to be sensitive, so get protection
-	unsigned char StanceID;
-	unsigned char Orientation, NewOrientation;
-	ieWord AttackMovements[3];
+	unsigned char StanceID = 0;
+	orient_t Orientation = S;
+	orient_t NewOrientation = S;
+	ieWord AttackMovements[3] = { 100, 0 , 0 };
 
-	PathNode* path; //whole path
-	PathNode* step; //actual step
-	unsigned int prevTicks;
-	int bumpBackTries;
-	bool pathAbandoned;
+	PathListNode* path = nullptr; // whole path
+	PathListNode* step = nullptr; // actual step
+	unsigned int prevTicks = 0;
+	int bumpBackTries = 0;
+	bool pathAbandoned = false;
 protected:
-	ieDword timeStartStep;
+	ieDword timeStartStep = 0;
 	//the # of previous tries to pick up a new walkpath
-	int pathTries;
-	int randomBackoff;
-	Point oldPos;
-	bool bumped;
-	int pathfindingDistance;
-	int randomWalkCounter;
+	int pathTries = 0;
+	int randomBackoff = 0;
+	Point oldPos = Pos;
+	bool bumped = false;
+	int pathfindingDistance = circleSize;
+	int randomWalkCounter = 0;
 public:
 	inline int GetRandomBackoff() const
 	{
@@ -493,26 +507,29 @@ public:
 	{
 		randomBackoff--;
 	}
-	Movable(ScriptableType type);
-	~Movable(void) override;
-	Point Destination;
-	ieResRef Area;
+	using Selectable::Selectable;
+	Movable(const Movable&) = delete;
+	~Movable() override;
+	Movable& operator=(const Movable&) = delete;
+
+	Point Destination = Pos;
+	ResRef Area;
 	Point HomeLocation;//spawnpoint, return here after rest
-	ieWord maxWalkDistance;//maximum random walk distance from home
+	ieWord maxWalkDistance = 0; // maximum random walk distance from home
 public:
 	inline void ImpedeBumping() { oldPos = Pos; bumped = false; }
 	void AdjustPosition();
 	void BumpAway();
 	void BumpBack();
 	inline bool IsBumped() const { return bumped; }
-	PathNode *GetNextStep(int x) const;
-	inline PathNode *GetPath() const { return path; };
+	PathListNode *GetNextStep(int x) const;
+	inline PathListNode *GetPath() const { return path; };
 	inline int GetPathTries() const	{ return pathTries; }
 	inline void IncrementPathTries() { pathTries++; }
 	inline void ResetPathTries() { pathTries = 0; }
 	int GetPathLength() const;
 //inliners to protect data consistency
-	inline PathNode * GetStep() {
+	inline PathListNode * GetStep() {
 		if (!step) {
 			DoStep((unsigned int) ~0);
 		}
@@ -523,9 +540,9 @@ public:
 		return (StanceID == IE_ANI_WALK || StanceID == IE_ANI_RUN);
 	}
 
-	unsigned char GetNextFace();
+	orient_t GetNextFace() const;
 
-	inline unsigned char GetOrientation() const {
+	inline orient_t GetOrientation() const {
 		return Orientation;
 	}
 
@@ -534,18 +551,20 @@ public:
 	}
 
 	void SetStance(unsigned int arg);
-	void SetOrientation(int value, bool slow);
-	void SetAttackMoveChances(ieWord *amc);
+	void SetOrientation(orient_t value, bool slow);
+	void SetOrientation(const Point& from, const Point& to, bool slow);
+	void SetAttackMoveChances(const ieWord *amc);
 	virtual void DoStep(unsigned int walkScale, ieDword time = 0);
 	void AddWayPoint(const Point &Des);
 	void RunAwayFrom(const Point &Des, int PathLength, bool noBackAway);
 	void RandomWalk(bool can_stop, bool run);
 	int GetRandomWalkCounter() const { return randomWalkCounter; };
-	void MoveLine(int steps, ieDword Orient);
+	void MoveLine(int steps, orient_t Orient);
 	void WalkTo(const Point &Des, int MinDistance = 0);
 	void MoveTo(const Point &Des);
-	void Stop() override;
+	void Stop(int flags = 0) override;
 	void ClearPath(bool resetDestination = true);
+	void HandleAnkhegStance(bool emerge);
 
 	/* returns the most likely position of this actor */
 	Point GetMostLikelyPosition() const;
@@ -557,19 +576,21 @@ public:
 //are they scriptable?
 class GEM_EXPORT TileObject {
 public:
-	TileObject(void);
-	~TileObject(void);
+	TileObject() noexcept = default;
+	TileObject(const TileObject&) = delete;
+	~TileObject();
+	TileObject& operator=(const TileObject&) = delete;
 	void SetOpenTiles(unsigned short *indices, int count);
 	void SetClosedTiles(unsigned short *indices, int count);
 
 public:
 	ieVariable Name;
-	ieResRef Tileset; //or wed door ID?
-	ieDword Flags;
-	unsigned short* opentiles;
-	ieDword opencount;
-	unsigned short* closedtiles;
-	ieDword closedcount;
+	ResRef Tileset; //or wed door ID?
+	ieDword Flags = 0;
+	unsigned short* opentiles = nullptr;
+	ieDword opencount = 0;
+	unsigned short* closedtiles = nullptr;
+	ieDword closedcount = 0;
 };
 
 }
