@@ -29,7 +29,7 @@
 
 #include "Plugin.h"
 #include "Spell.h"
-#include "System/DataStream.h"
+#include "Streams/DataStream.h"
 
 namespace GemRB {
 
@@ -40,8 +40,6 @@ namespace GemRB {
 
 class GEM_EXPORT SpellMgr : public Plugin {
 public:
-	SpellMgr(void);
-	~SpellMgr(void) override;
 	virtual bool Open(DataStream* stream) = 0;
 	virtual Spell* GetSpell(Spell *spl, bool silent=false) = 0;
 };
