@@ -30,7 +30,7 @@
 
 #include "Plugin.h"
 #include "Store.h"
-#include "System/DataStream.h"
+#include "Streams/DataStream.h"
 
 namespace GemRB {
 
@@ -41,8 +41,6 @@ namespace GemRB {
 
 class GEM_EXPORT StoreMgr : public Plugin {
 public:
-	StoreMgr(void);
-	~StoreMgr(void) override;
 	virtual bool Open(DataStream* stream) = 0;
 	virtual Store* GetStore(Store *s) = 0;
 
