@@ -23,15 +23,6 @@
 
 #ifdef HAVE_CONFIG_H
 	#include <config.h>
-#else
-	// we need this fallback for Android and anyone else skipping
-	// cmake, where the proper sizes are checked for
-	#ifndef SIZEOF_INT
-	#define SIZEOF_INT 4
-	#endif
-	#ifndef SIZEOF_LONG_INT
-	#define SIZEOF_LONG_INT 4
-	#endif
 #endif
 
 #include "exports.h"
@@ -40,6 +31,7 @@
 	#ifdef WIN32
 		#define _MAX_PATH 260
 	#else
+		#include <cstdio> // for FILENAME_MAX...
 		#define _MAX_PATH FILENAME_MAX
 	#endif
 #endif
@@ -50,10 +42,6 @@
 	#include <unistd.h>
 #endif
 
-#include <cstdio>
-#include <cstdlib>
-
-#include "System/Logging.h"
-#include "System/String.h"
+#include <cstddef>
 
 #endif  //! PLATFORM_H
