@@ -32,8 +32,8 @@
 #define UNICODE
 #define _UNICODE
 #define NOGDI
-#define _USE_MATH_DEFINES
 #define NOCOLOR
+#define NOUSER
 
 #ifdef _DEBUG
 #include <crtdbg.h>
