@@ -22,19 +22,16 @@
 #define _ACM_LAB_SUBBAND_DECODER_H
 
 #include <cstdlib>
-#if defined(sgi)
-#include <iostream>
-#endif
 
 class CSubbandDecoder {
 private:
 	int levels, block_size;
-	int* memory_buffer;
-	void sub_4d3fcc(short* memory, int* buffer, int sb_size, int blocks);
-	void sub_4d420c(int* memory, int* buffer, int sb_size, int blocks);
+	int* memory_buffer = nullptr;
+	void sub_4d3fcc(short* memory, int* buffer, int sb_size, int blocks) const;
+	void sub_4d420c(int* memory, int* buffer, int sb_size, int blocks) const;
 public:
-	CSubbandDecoder(int lev_cnt)
-		: levels( lev_cnt ), block_size( 1 << lev_cnt ), memory_buffer( NULL )
+	explicit CSubbandDecoder(int lev_cnt)
+		: levels(lev_cnt), block_size(1 << lev_cnt)
 	{
 	}
 	virtual ~CSubbandDecoder()
