@@ -31,14 +31,16 @@ class BAMFontManager : public FontManager
 private:
 	/** private data members */
 	BAMImporter* bamImp;
-	bool isStateFont;
+	bool isStateFont = false;
 	ResRef resRef;
 public:
 	/** public methods */
-	~BAMFontManager(void) override;
+	BAMFontManager(const BAMFontManager&) = delete;
+	~BAMFontManager() override;
+	BAMFontManager& operator=(const BAMFontManager&) = delete;
 	BAMFontManager();
 
-	bool Open(DataStream* stream) override;
+	bool Import(DataStream* stream) override;
 
 	Font* GetFont(ieWord pxSize, FontStyle style, bool background) override;
 };
