@@ -4,9 +4,7 @@ namespace GemRB {
 
 class BMPWriter : public ImageWriter {
 public:
-	BMPWriter(void);
-	~BMPWriter(void) override;
-
+	BMPWriter() noexcept = default;
 	void PutImage(DataStream *output, Holder<Sprite2D> sprite) override;
 };
 
