@@ -25,17 +25,45 @@
 
 namespace GemRB {
 
+enum class MOSVersion {
+	V1, V2
+};
+
+struct MOSV2DataBlock {
+	ieDword pvrzPage;
+	Point source;
+	Size size;
+	Point destination;
+};
+
 class MOSImporter : public ImageMgr {
 private:
-	ieWord Width, Height, Cols, Rows;
-	ieDword BlockSize, PalOffset;
+	MOSVersion version = MOSVersion::V1;
+	ieWord Cols = 0;
+	ieWord Rows = 0;
+	union U {
+		constexpr U() noexcept : v1{} {} // only for compiler happiness
+		struct {
+			ieDword BlockSize = 0;
+			ieDword PalOffset = 0;
+		} v1;
+		struct {
+			ieDword NumBlocks = 0;
+			ieDword BlockOffset = 0;
+		} v2;
+	} layout;
+
+	std::shared_ptr<ImageMgr> lastPVRZ;
+	ieDword lastPVRZPage = 0;
+
+	void Blit(const MOSV2DataBlock& dataBlock, uint8_t* data);
+	Holder<Sprite2D> GetSprite2Dv1();
+	Holder<Sprite2D> GetSprite2Dv2();
 public:
-	MOSImporter(void);
-	~MOSImporter(void) override;
-	bool Open(DataStream* stream) override;
+	MOSImporter() noexcept = default;
+	bool Import(DataStream* stream) override;
 	Holder<Sprite2D> GetSprite2D() override;
-	int GetWidth() override { return (int) Width; }
-	int GetHeight() override { return (int) Height; }
+	Holder<Sprite2D> GetSprite2D(Region&&) override { return {}; };
 };
 
 }
