@@ -20,7 +20,7 @@
 #define OGGREADER_H
 
 #include "SoundMgr.h"
-#include "System/DataStream.h"
+#include "Streams/DataStream.h"
 
 #include <cstdio>
 #include <cstdlib>
@@ -35,13 +35,14 @@ namespace GemRB {
 class OGGReader : public SoundMgr {
 private:
 	OggVorbis_File OggStream;
-	int samples_left; // count of unread samples
+	int samples_left = 0; // count of unread samples
 public:
 	OGGReader()
-		: samples_left( 0 )
 	{
 		memset(&OggStream, 0, sizeof(OggStream) );
 	}
+	OGGReader(const OGGReader&) = delete;
+	OGGReader& operator=(const OGGReader&) = delete;
 	~OGGReader() override
 	{
 		Close();
@@ -50,7 +51,7 @@ public:
 	{
 		ov_clear(&OggStream);
 	}
-	bool Open(DataStream* stream) override;
+	bool Import(DataStream* stream) override;
 	int read_samples(short* buffer, int count) override;
 };
 
