@@ -22,10 +22,16 @@
 
 #include "GameData.h"
 #include "Interface.h"
+#include "Logging/Logging.h"
 
 #include <cassert>
 #include <cstdio>
 
+#ifdef __clang__
+#pragma clang diagnostic push
+#pragma clang diagnostic ignored "-Wdeprecated-declarations"
+#endif
+
 using namespace GemRB;
 
 template <typename TO, typename FROM>
@@ -53,7 +59,7 @@ static LPALAUXILIARYEFFECTSLOTI alAuxiliaryEffectSloti = NULL;
 static bool checkALError(const char* msg, log_level level) {
 	int error = alGetError();
 	if (error != AL_NO_ERROR) {
-		Log(level, "OpenAL", "%s: 0x%x - %s", msg, error, alGetString(error));
+		Log(level, "OpenAL", "{}: {:#x} - {}", msg, error, alGetString(error));
 		return true;
 	}
 	return false;
@@ -62,17 +68,17 @@ static bool checkALError(const char* msg, log_level level) {
 static void showALCError(const char* msg, log_level level, ALCdevice *device) {
 	int error = alcGetError(device);
 	if (error != AL_NO_ERROR) {
-		Log(level, "OpenAL", "%s: 0x%x", msg, error);
+		Log(level, "OpenAL", "{}: {:#x}", msg, error);
 	} else {
-		Log(level, "OpenAL", "%s", msg);
+		Log(level, "OpenAL", "{}", msg);
 	}
 }
 
-void OpenALSoundHandle::SetPos(int XPos, int YPos) {
+void OpenALSoundHandle::SetPos(const Point& p) {
 	if (!parent) return;
 
 	ALfloat SourcePos[] = {
-		(float) XPos, (float) YPos, 0.0f
+		float(p.x), float(p.y), 0.0f
 	};
 
 	alSourcefv(parent->Source, AL_POSITION, SourcePos);
@@ -99,7 +105,7 @@ void OpenALSoundHandle::StopLooping() {
 	checkALError("Unable to stop audio loop", WARNING);
 }
 
-void AudioStream::ClearProcessedBuffers()
+void AudioStream::ClearProcessedBuffers() const
 {
 	ALint processed = 0;
 	alGetSourcei( Source, AL_BUFFERS_PROCESSED, &processed );
@@ -170,32 +176,21 @@ void AudioStream::ForceClear()
 
 OpenALAudioDriver::OpenALAudioDriver(void)
 {
-	alutContext = NULL;
-	MusicPlaying = false;
 	music_memory = (short*) malloc(ACM_BUFFERSIZE);
-	MusicSource = num_streams = 0;
-	memset(MusicBuffer, 0, MUSICBUFFERS*sizeof(ALuint));
-	ambim = NULL;
-	stayAlive = false;
-	hasReverbProperties = false;
-#ifdef HAVE_OPENAL_EFX_H
-	hasEFX = false;
-	efxEffectSlot = efxEffect = 0;
 	memset(&reverbProperties.reverbData, 0, sizeof(reverbProperties.reverbData));
 	reverbProperties.reverbDisabled = true;
-#endif
 }
 
-void OpenALAudioDriver::PrintDeviceList ()
+void OpenALAudioDriver::PrintDeviceList() const
 {
-	char *deviceList;
+	const char *deviceList;
 
 	if (alcIsExtensionPresent(nullptr, (const ALchar*) "ALC_ENUMERATION_EXT") == AL_TRUE) { // try out enumeration extension
 		Log(MESSAGE, "OpenAL", "Usable audio output devices:");
-		deviceList = (char *)alcGetString(NULL, ALC_DEVICE_SPECIFIER);
+		deviceList = alcGetString(nullptr, ALC_DEVICE_SPECIFIER);
 
 		while(deviceList && *deviceList) {
-			Log(MESSAGE,"OpenAL", "Devices: %s", deviceList);
+			Log(MESSAGE,"OpenAL", "Devices: {}", deviceList);
 			deviceList+=strlen(deviceList)+1;
 		}
 		return;
@@ -205,8 +200,11 @@ void OpenALAudioDriver::PrintDeviceList ()
 
 bool OpenALAudioDriver::Init(void)
 {
-	Log(MESSAGE, "OpenAL", "Initializing OpenAL driver:\nAL Version:%s\nAL Renderer:%s\nAL Vendor:%s",
-		alGetString(AL_VERSION), alGetString(AL_RENDERER), alGetString(AL_VENDOR));
+	const char* version = alGetString(AL_VERSION);
+	const char* renderer = alGetString(AL_RENDERER);
+	const char* vendor = alGetString(AL_VENDOR);
+	Log(MESSAGE, "OpenAL", "Initializing OpenAL driver:\nAL Version: {}\nAL Renderer: {}\nAL Vendor: {}",
+		version ? version : "", renderer ? renderer : "", vendor ? vendor : "");
 
 	ALCdevice *device;
 	ALCcontext *context;
@@ -237,18 +235,16 @@ bool OpenALAudioDriver::Init(void)
 	int sources = CountAvailableSources(MAX_STREAMS+1);
 	num_streams = sources - 1;
 
-	Log(MESSAGE, "OpenAL", "Allocated %d streams.%s",
+	Log(MESSAGE, "OpenAL", "Allocated {} streams.{}",
 		num_streams, (num_streams < MAX_STREAMS ? " (Fewer than desired.)" : "" ));
 
-	stayAlive = true;
-
 	musicThread = std::thread(&OpenALAudioDriver::MusicManager, this);
 
 	if (!InitEFX()) {
 		Log(MESSAGE, "OpenAL", "EFX not available.");
 	}
 
-	ambim = new AmbientMgrAL;
+	ambim = new AmbientMgr;
 	speech.free = true;
 	speech.ambient = false;
 	return true;
@@ -351,7 +347,6 @@ OpenALAudioDriver::~OpenALAudioDriver(void)
 	}
 	speech.ForceClear();
 	ResetMusics();
-	clearBufferCache(true);
 
 #ifdef HAVE_OPENAL_EFX_H
 	if (hasEFX) {
@@ -376,21 +371,18 @@ OpenALAudioDriver::~OpenALAudioDriver(void)
 	delete ambim;
 }
 
-ALuint OpenALAudioDriver::loadSound(const char *ResRef, unsigned int &time_length)
+ALuint OpenALAudioDriver::loadSound(StringView ResRef, tick_t &time_length)
 {
 	ALuint Buffer = 0;
 
-	CacheEntry *e;
-	void* p;
-
-	if (!ResRef[0]) {
+	if (ResRef.empty()) {
 		return 0;
 	}
-	if(buffercache.Lookup(ResRef, p))
-	{
-		e = (CacheEntry*) p;
-		time_length = e->Length;
-		return e->Buffer;
+
+	auto entry = buffercache.Lookup(ResRef);
+	if (entry != nullptr) {
+		time_length = entry->Length;
+		return entry->Buffer;
 	}
 
 	//no cache entry...
@@ -399,7 +391,7 @@ ALuint OpenALAudioDriver::loadSound(const char *ResRef, unsigned int &time_lengt
 		return 0;
 	}
 
-	ResourceHolder<SoundMgr> acm = GetResourceHolder<SoundMgr>(ResRef);
+	ResourceHolder<SoundMgr> acm = gamedata->GetResourceHolder<SoundMgr>(ResRef);
 	if (!acm) {
 		alDeleteBuffers( 1, &Buffer );
 		checkALError("Unable to delete buffer!", ERROR);
@@ -425,26 +417,17 @@ ALuint OpenALAudioDriver::loadSound(const char *ResRef, unsigned int &time_lengt
 		return 0;
 	}
 
-	e = new CacheEntry;
-	e->Buffer = Buffer;
-	e->Length = time_length;
+	buffercache.SetAt(ResRef, Buffer, time_length);
 
-	buffercache.SetAt(ResRef, (void*)e);
-	//print("LoadSound: added %s to cache: %d. Cache size now %d", ResRef, e->Buffer, buffercache.GetCount());
-
-	if (buffercache.GetCount() > BUFFER_CACHE_SIZE) {
-		evictBuffer();
-	}
 	return Buffer;
 }
 
-Holder<SoundHandle> OpenALAudioDriver::Play(const char* ResRef, unsigned int channel, int XPos, int YPos,
-	unsigned int flags, unsigned int *length)
+Holder<SoundHandle> OpenALAudioDriver::Play(StringView ResRef, unsigned int channel, const Point& p,
+	unsigned int flags, tick_t *length)
 {
 	ALuint Buffer;
-	unsigned int time_length;
 
-	if (ResRef == NULL || !ResRef[0]) {
+	if (ResRef.empty()) {
 		if((flags & GEM_SND_SPEECH) && (speech.Source && alIsSource(speech.Source))) {
 			//So we want him to be quiet...
 			alSourceStop( speech.Source );
@@ -454,6 +437,7 @@ Holder<SoundHandle> OpenALAudioDriver::Play(const char* ResRef, unsigned int cha
 		return Holder<SoundHandle>();
 	}
 
+	tick_t time_length;
 	Buffer = loadSound( ResRef, time_length );
 	if (Buffer == 0) {
 		return Holder<SoundHandle>();
@@ -464,7 +448,7 @@ Holder<SoundHandle> OpenALAudioDriver::Play(const char* ResRef, unsigned int cha
 	}
 
 	ALfloat SourcePos[] = {
-		(float) XPos, (float) YPos, 0.0f
+		float(p.x), float(p.y), 0.0f
 	};
 	ALfloat SourceVel[] = {
 		0.0f, 0.0f, 0.0f
@@ -489,7 +473,7 @@ Holder<SoundHandle> OpenALAudioDriver::Play(const char* ResRef, unsigned int cha
 			}
 		}
 
-		core->GetDictionary()->Lookup("Volume Voices", volume);
+		volume = core->GetVariable("Volume Voices", 100);
 
 		loop = 0; // Speech ignores GEM_SND_LOOPING
 	} else {
@@ -502,7 +486,7 @@ Holder<SoundHandle> OpenALAudioDriver::Play(const char* ResRef, unsigned int cha
 			}
 		}
 
-		core->GetDictionary()->Lookup("Volume SFX", volume);
+		volume = core->GetVariable("Volume SFX", 100);
 
 		if (stream == NULL) {
 			// Failed to assign new sound.
@@ -531,10 +515,9 @@ Holder<SoundHandle> OpenALAudioDriver::Play(const char* ResRef, unsigned int cha
 	checkALError("Unable to set audio parameters", WARNING);
 
 #ifdef HAVE_OPENAL_EFX_H
-	ieDword efxSetting;
-	core->GetDictionary()->Lookup("Environmental Audio", efxSetting);
+	ieDword efxSetting = core->GetVariable("Environmental Audio", 0);
 
-	if (efxSetting && hasReverbProperties && ((0 != XPos && 0 != YPos) || (flags & GEM_SND_RELATIVE))) {
+	if (efxSetting && hasReverbProperties && (!p.IsZero() || (flags & GEM_SND_RELATIVE))) {
 		alSource3i(Source, AL_AUXILIARY_SEND_FILTER, efxEffectSlot, 0, 0);
 	} else {
 		alSource3i(Source, AL_AUXILIARY_SEND_FILTER, 0, 0, 0);
@@ -550,25 +533,25 @@ Holder<SoundHandle> OpenALAudioDriver::Play(const char* ResRef, unsigned int cha
 		return Holder<SoundHandle>();
 	}
 
-	stream->handle = new OpenALSoundHandle(stream);
-	return stream->handle.get();
+	stream->handle = MakeHolder<OpenALSoundHandle>(stream);
+	return Holder<SoundHandle>(stream->handle.get()); // TODO: we need something like static_ptr_cast
 }
 
 void OpenALAudioDriver::UpdateVolume(unsigned int flags)
 {
-	ieDword volume;
+	ieDword volume = 0;
 
 	if (flags & GEM_SND_VOL_MUSIC) {
 		musicMutex.lock();
-		core->GetDictionary()->Lookup("Volume Music", volume);
+		volume = core->GetVariable("Volume Music", 0);
 		if (MusicSource && alIsSource(MusicSource))
 			alSourcef(MusicSource, AL_GAIN, volume * 0.01f);
 		musicMutex.unlock();
 	}
 
 	if (flags & GEM_SND_VOL_AMBIENTS) {
-		core->GetDictionary()->Lookup("Volume Ambients", volume);
-		((AmbientMgrAL*) ambim)->UpdateVolume(volume);
+		volume = core->GetVariable("Volume Ambients", volume);
+		ambim->UpdateVolume(volume);
 	}
 }
 
@@ -631,7 +614,7 @@ bool OpenALAudioDriver::Pause()
 	alSourcePause(MusicSource);
 	checkALError("Unable to pause music source", WARNING);
 	MusicPlaying = false;
-	((AmbientMgrAL*) ambim)->deactivate();
+	ambim->Deactivate();
 
 	return true;
 }
@@ -647,16 +630,16 @@ bool OpenALAudioDriver::Resume()
 		checkALError("Unable to resume music source", WARNING);
 		MusicPlaying = true;
 	}
-	((AmbientMgrAL*) ambim)->activate();
+	ambim->Activate();
 	return true;
 }
 
-int OpenALAudioDriver::CreateStream(Holder<SoundMgr> newMusic)
+int OpenALAudioDriver::CreateStream(std::shared_ptr<SoundMgr> newMusic)
 {
 	std::lock_guard<std::recursive_mutex> l(musicMutex);
 
 	// Free old MusicReader
-	MusicReader = newMusic;
+	MusicReader = std::move(newMusic);
 	if (!MusicReader) {
 		MusicPlaying = false;
 	}
@@ -682,8 +665,7 @@ int OpenALAudioDriver::CreateStream(Holder<SoundMgr> newMusic)
 			0.0f, 0.0f, 0.0f
 		};
 
-		ieDword volume;
-		core->GetDictionary()->Lookup( "Volume Music", volume );
+		ieDword volume = core->GetVariable("Volume Music", 0);
 		alSourcef( MusicSource, AL_PITCH, 1.0f );
 		alSourcef( MusicSource, AL_GAIN, 0.01f * volume );
 		alSourcei( MusicSource, AL_SOURCE_RELATIVE, 1 );
@@ -696,19 +678,18 @@ int OpenALAudioDriver::CreateStream(Holder<SoundMgr> newMusic)
 	return 0;
 }
 
-void OpenALAudioDriver::UpdateListenerPos(int XPos, int YPos )
+void OpenALAudioDriver::UpdateListenerPos(const Point& p)
 {
-	alListener3f( AL_POSITION, (float) XPos, (float) YPos, LISTENER_HEIGHT );
+	alListener3f(AL_POSITION, p.x, p.y, LISTENER_HEIGHT);
 	checkALError("Unable to update listener position.", WARNING);
 }
 
-void OpenALAudioDriver::GetListenerPos(int &XPos, int &YPos )
+Point OpenALAudioDriver::GetListenerPos()
 {
 	ALfloat listen[3];
 	alGetListenerfv( AL_POSITION, listen );
-	if (checkALError("Unable to get listener pos", ERROR)) return;
-	XPos = (int) listen[0];
-	YPos = (int) listen[1];
+	if (checkALError("Unable to get listener pos", ERROR)) return {};
+	return Point(listen[0], listen[1]);
 }
 
 bool OpenALAudioDriver::ReleaseStream(int stream, bool HardStop)
@@ -729,7 +710,7 @@ bool OpenALAudioDriver::ReleaseStream(int stream, bool HardStop)
 }
 
 //This one is used for movies and ambients.
-int OpenALAudioDriver::SetupNewStream( ieWord x, ieWord y, ieWord z,
+int OpenALAudioDriver::SetupNewStream(int x, int y, int z,
 		            ieWord gain, bool point, int ambientRange)
 {
 	// Find a free (or finished) stream for this sound
@@ -742,7 +723,7 @@ int OpenALAudioDriver::SetupNewStream( ieWord x, ieWord y, ieWord z,
 		}
 	}
 	if (stream == -1) {
-		Log(ERROR, "OpenAL", "No available audio streams out of %d", num_streams);
+		Log(ERROR, "OpenAL", "No available audio streams out of {}", num_streams);
 		return -1;
 	}
 
@@ -771,7 +752,7 @@ int OpenALAudioDriver::SetupNewStream( ieWord x, ieWord y, ieWord z,
 	return stream;
 }
 
-int OpenALAudioDriver::QueueAmbient(int stream, const char* sound)
+tick_t OpenALAudioDriver::QueueAmbient(int stream, const ResRef& sound)
 {
 	if (streams[stream].free || !streams[stream].ambient)
 		return -1;
@@ -781,10 +762,7 @@ int OpenALAudioDriver::QueueAmbient(int stream, const char* sound)
 	// first dequeue any processed buffers
 	streams[stream].ClearProcessedBuffers();
 
-	if (sound == 0)
-		return 0;
-
-	unsigned int time_length;
+	tick_t time_length;
 	ALuint Buffer = loadSound(sound, time_length);
 	if (0 == Buffer) {
 		return -1;
@@ -819,55 +797,6 @@ void OpenALAudioDriver::SetAmbientStreamPitch(int stream, int pitch)
 	checkALError("Unable to set ambient pitch", WARNING);
 }
 
-bool OpenALAudioDriver::evictBuffer()
-{
-	// Note: this function assumes the caller holds bufferMutex
-
-	// Room for optimization: this is O(n^2) in the number of buffers
-	// at the tail that are used. It can be O(n) if LRUCache supports it.
-
-	unsigned int n = 0;
-	void* p;
-	const char* k;
-	bool res;
-
-	while ((res = buffercache.getLRU(n, k, p)) == true) {
-		CacheEntry* e = (CacheEntry*)p;
-		alDeleteBuffers(1, &e->Buffer);
-		if (alGetError() == AL_NO_ERROR) {
-			// Buffer was unused. An error would have indicated
-			// the buffer was still attached to a source.
-
-			delete e;
-			buffercache.Remove(k);
-
-			//print("Removed buffer %s from ACMImp cache", k);
-			break;
-		}
-		++n;
-	}
-
-	return res;
-}
-
-void OpenALAudioDriver::clearBufferCache(bool force)
-{
-	// Room for optimization: any method of iterating over the buffers
-	// would suffice. It doesn't have to be in LRU-order.
-	void* p;
-	const char* k;
-	int n = 0;
-	while (buffercache.getLRU(n, k, p)) {
-		CacheEntry* e = (CacheEntry*)p;
-		alDeleteBuffers(1, &e->Buffer);
-		if (force || alGetError() == AL_NO_ERROR) {
-			delete e;
-			buffercache.Remove(k);
-		} else
-			++n;
-	}
-}
-
 ALenum OpenALAudioDriver::GetFormatEnum(int channels, int bits) const
 {
 	switch (channels) {
@@ -889,7 +818,6 @@ ALenum OpenALAudioDriver::GetFormatEnum(int channels, int bits) const
 int OpenALAudioDriver::MusicManager(void* arg)
 {
 	OpenALAudioDriver* driver = (OpenALAudioDriver*) arg;
-	ALuint buffersreturned = 0;
 	ALboolean bFinished = AL_FALSE;
 	while (driver->stayAlive) {
 		std::this_thread::sleep_for(std::chrono::milliseconds(30));
@@ -903,7 +831,7 @@ int OpenALAudioDriver::MusicManager(void* arg)
 			}
 			switch (state) {
 				default:
-					Log(ERROR, "OpenAL", "Unhandled Music state '%d'.", state);
+					Log(ERROR, "OpenAL", "Unhandled Music state '{}'.", state);
 				// intentional fallthrough
 				case AL_PAUSED:
 					driver->MusicPlaying = false;
@@ -913,9 +841,9 @@ int OpenALAudioDriver::MusicManager(void* arg)
 					// ensure that MusicSource has no buffers attached by passing "NULL" buffer
 					alSourcei(driver->MusicSource, AL_BUFFER, 0);
 					checkALError("Unable to detach buffers from music source.", WARNING);
-					for (int i = 0; i < MUSICBUFFERS; i++) {
+					for (unsigned int buffer : driver->MusicBuffer) {
 						driver->MusicReader->read_samples( driver->music_memory, ACM_BUFFERSIZE >> 1 );
-						alBufferData( driver->MusicBuffer[i], AL_FORMAT_STEREO16,
+						alBufferData(buffer, AL_FORMAT_STEREO16,
 							driver->music_memory, ACM_BUFFERSIZE,
 							driver->MusicReader->get_samplerate() );
 					}
@@ -952,7 +880,6 @@ int OpenALAudioDriver::MusicManager(void* arg)
 				return -1;
 			}
 			if (processed > 0) {
-				buffersreturned += processed;
 				while (processed) {
 					ALuint BufferID;
 					alSourceUnqueueBuffers( driver->MusicSource, 1, &BufferID );
@@ -1026,7 +953,7 @@ void OpenALAudioDriver::QueueBuffer(int stream, unsigned short bits,
 // Private Methods
 // !!!!!!!!!!!!!!!
 
-int OpenALAudioDriver::QueueALBuffer(ALuint source, ALuint buffer)
+int OpenALAudioDriver::QueueALBuffer(ALuint source, ALuint buffer) const
 {
 #ifdef DEBUG_AUDIO
 	ALint frequency, bits, channels;
@@ -1034,7 +961,7 @@ int OpenALAudioDriver::QueueALBuffer(ALuint source, ALuint buffer)
 	alGetBufferi(buffer, AL_BITS, &bits);
 	alGetBufferi(buffer, AL_CHANNELS, &channels);
 	checkALError("Error querying buffer properties.", WARNING);
-	Log(DEBUG, "OpenAL", "Attempting to buffer audio source:%d\nFrequency:%d\nBits:%d\nChannels:%d",
+	Log(DEBUG, "OpenAL", "Attempting to buffer audio source: {}\nFrequency: {}\nBits: {}\nChannels: {}",
 		source, frequency, bits, channels);
 #endif
 	ALint type;
@@ -1064,12 +991,11 @@ int OpenALAudioDriver::QueueALBuffer(ALuint source, ALuint buffer)
 	return GEM_OK;
 }
 
-#ifdef HAVE_OPENAL_EFX_H
-void OpenALAudioDriver::UpdateMapAmbient(MapReverb& mapReverb) {
+void OpenALAudioDriver::UpdateMapAmbient(const MapReverbProperties& props) {
 	if (hasEFX) {
-		mapReverb.getReverbProperties(reverbProperties);
+		reverbProperties = props;
 		hasReverbProperties = true;
-
+#ifdef HAVE_OPENAL_EFX_H
 		alDeleteEffects(1, &efxEffect);
 		alGenEffects(1, &efxEffect);
 
@@ -1094,11 +1020,16 @@ void OpenALAudioDriver::UpdateMapAmbient(MapReverb& mapReverb) {
 		}
 
 		alAuxiliaryEffectSloti(efxEffectSlot, AL_EFFECTSLOT_EFFECT, efxEffect);
+#else
+		// avoid warnings for unused members
+		(void)efxEffectSlot;
+		(void)efxEffect;
+#endif
 	}
 }
-#else
-void OpenALAudioDriver::UpdateMapAmbient(MapReverb&) {
-}
+
+#ifdef __clang__
+#pragma clang diagnostic pop
 #endif
 
 #include "plugindef.h"
