@@ -27,12 +27,15 @@ namespace GemRB {
 
 class PLTImporter : public PalettedImageMgr {
 private:
-	ieDword Width, Height;
-	void* pixels;
+	ieDword Width = 0;
+	ieDword Height = 0;
+	void* pixels = nullptr;
 public:
-	PLTImporter(void);
-	~PLTImporter(void) override;
-	bool Open(DataStream* stream) override;
+	PLTImporter() noexcept = default;
+	PLTImporter(const PLTImporter&) = delete;
+	~PLTImporter() override;
+	PLTImporter& operator=(const PLTImporter&) = delete;
+	bool Import(DataStream* stream) override;
 	Holder<Sprite2D> GetSprite2D(unsigned int type, ieDword col[8]) override;
 };
 
