@@ -31,17 +31,17 @@ class PNGImporter : public ImageMgr {
 private:
 	PNGInternal* inf;
 
-	ieDword Width, Height;
-	bool hasPalette;
+	bool hasPalette = false;
 public:
 	PNGImporter(void);
-	~PNGImporter(void) override;
+	PNGImporter(const PNGImporter&) = delete;
+	~PNGImporter() override;
+	PNGImporter& operator=(const PNGImporter&) = delete;
 	void Close();
-	bool Open(DataStream* stream) override;
+	bool Import(DataStream* stream) override;
 	Holder<Sprite2D> GetSprite2D() override;
+	Holder<Sprite2D> GetSprite2D(Region&&) override { return {}; };
 	int GetPalette(int colors, Color* pal) override;
-	int GetWidth() override { return (int) Width; }
-	int GetHeight() override { return (int) Height; }
 };
 
 }
