@@ -36,12 +36,14 @@ private:
 	int version = 0;
 
 public:
-	PROImporter(void);
-	~PROImporter(void) override;
+	PROImporter() noexcept = default;
+	PROImporter(const PROImporter&) = delete;
+	~PROImporter() override;
+	PROImporter& operator=(const PROImporter&) = delete;
 	bool Open(DataStream* stream) override;
 	Projectile* GetProjectile(Projectile *s) override;
 private:
-	void GetAreaExtension(ProjectileExtension *s);
+	Holder<ProjectileExtension> GetAreaExtension();
 };
 
 
