@@ -30,12 +30,15 @@
 #include "TableMgr.h"
 #include "TileMap.h"
 #include "VEFObject.h"
-#include "Video.h" //for tints
+#include "Video/Video.h" //for tints
+
+#include "GameScript/GSUtils.h"
 #include "Scriptable/Actor.h"
 #include "ScriptedAnimation.h"
 
 using namespace GemRB;
 
+// TODO: recheck 0x52 is Incite Berserk Attack and implement; currently mapped to bg1 SetAIScript in effect.ids
 int fx_retreat_from (Scriptable* Owner, Actor* target, Effect* fx);//6e
 int fx_set_status (Scriptable* Owner, Actor* target, Effect* fx);//ba
 int fx_play_bam_blended (Scriptable* Owner, Actor* target, Effect* fx);//bb
@@ -90,7 +93,7 @@ static EffectDesc effectnames[] = {
 	EffectDesc("StopAllAction", fx_stop_all_action, EFFECT_NO_ACTOR, -1 ), //cf
 	EffectDesc("TintScreen", fx_tint_screen, EFFECT_NO_ACTOR, -1 ), //c3
 	EffectDesc("TransferHP", fx_transfer_hp, EFFECT_DICED, -1 ), //c0
-	EffectDesc( NULL, NULL, 0, 0 ),
+	EffectDesc(nullptr, nullptr, 0, 0),
 };
 
 static void RegisterTormentOpcodes()
@@ -149,6 +152,17 @@ int fx_set_status (Scriptable* /*Owner*/, Actor* target, Effect* fx)
 	return FX_PERMANENT;
 }
 
+static void SetRGBMod(ieDword color, ScriptedAnimation* sca)
+{
+	RGBModifier rgb;
+
+	rgb.speed = -1;
+	rgb.phase = 0;
+	rgb.rgb = Color::FromABGR(color);
+	rgb.type = RGBModifier::TINT;
+	sca->AlterPalette(rgb);
+}
+
 //bb fx_play_bam_bb (play multi-part blended sticky animation)
 // 1 repeats
 // 2 not sticky (override default)
@@ -175,16 +189,7 @@ int fx_play_bam_blended (Scriptable* Owner, Actor* target, Effect* fx)
 	sca->SetBlend();
 	//the transparency is based on the original palette
 	if (fx->Parameter1) {
-		RGBModifier rgb;
-
-		rgb.speed=-1;
-		rgb.phase=0;
-		rgb.rgb.r=fx->Parameter1;
-		rgb.rgb.g=fx->Parameter1 >> 8;
-		rgb.rgb.b=fx->Parameter1 >> 16;
-		rgb.rgb.a=0;
-		rgb.type=RGBModifier::TINT;
-		sca->AlterPalette(rgb);
+		SetRGBMod(fx->Parameter1, sca);
 	}
 	if ((fx->TimingMode==FX_DURATION_INSTANT_LIMITED) && (fx->Parameter2&1) ) {
 		playonce=false;
@@ -196,7 +201,7 @@ int fx_play_bam_blended (Scriptable* Owner, Actor* target, Effect* fx)
 	} else {
 		if (fx->Parameter2&1) {
 			//four cycles, duration is in millisecond
-			sca->SetDefaultDuration(sca->GetSequenceDuration(AI_UPDATE_TIME));
+			sca->SetDefaultDuration(sca->GetSequenceDuration(core->Time.defaultTicksPerSec));
 		} else {
 			sca->SetDefaultDuration(fx->Duration-core->GetGame()->Ticks);
 		}
@@ -207,7 +212,7 @@ int fx_play_bam_blended (Scriptable* Owner, Actor* target, Effect* fx)
 	}
 
 	if (fx->Parameter2&2) {
-		sca->Pos = Point(fx->PosX, fx->PosY);
+		sca->Pos = fx->Pos;
 		area->AddVVCell( new VEFObject(sca));
 	} else {
 		assert(target);
@@ -267,13 +272,7 @@ int fx_play_bam_not_blended (Scriptable* Owner, Actor* target, Effect* fx)
 		break;
 	default:
 		if (fx->Parameter1) {
-			RGBModifier rgb;
-
-			rgb.speed=-1;
-			rgb.phase=0;
-			rgb.rgb = Color::FromABGR(fx->Parameter1);
-			rgb.type=RGBModifier::TINT;
-			sca->AlterPalette(rgb);
+			SetRGBMod(fx->Parameter1, sca);
 		}
 	}
 	if (fx->TimingMode==FX_DURATION_INSTANT_LIMITED) {
@@ -321,12 +320,12 @@ int fx_play_bam_not_blended (Scriptable* Owner, Actor* target, Effect* fx)
 			y = (tmp>>5)&31;
 		}
 		
-		sca->Pos = Point(fx->PosX, fx->PosY);
+		sca->Pos = fx->Pos;
 		sca->XOffset -= x;
 		sca->YOffset -= y;
 
 		if (twin) {
-			twin->Pos = Point(fx->PosX, fx->PosY);
+			twin->Pos = fx->Pos;
 			twin->XOffset -= x;
 			twin->YOffset -= y;
 			area->AddVVCell( new VEFObject(twin) );
@@ -379,7 +378,7 @@ int fx_transfer_hp (Scriptable* Owner, Actor* target, Effect* fx)
 		damage=(signed) fx->Parameter1;
 	}
 	if (damage) {
-		damage = donor->Damage(damage, fx->Parameter2, owner, fx->IsVariable, fx->SavingThrowType);
+		damage = donor->Damage(damage, fx->Parameter2, owner, MOD_ADDITIVE, fx->SavingThrowType);
 		receiver->NewBase( IE_HITPOINTS, damage, MOD_ADDITIVE );
 	}
 	return FX_NOT_APPLIED;
@@ -391,7 +390,7 @@ int fx_transfer_hp (Scriptable* Owner, Actor* target, Effect* fx)
 int fx_flash_screen (Scriptable* /*Owner*/, Actor* /*target*/, Effect* fx)
 {
 	// print("fx_flash_screen(%2d): Par2: %d", fx->Opcode, fx->Parameter2);
-	unsigned char* bytes = (unsigned char *)&fx->Parameter1;
+	const unsigned char* bytes = (const unsigned char *) &fx->Parameter1;
 	Color c(bytes[0], bytes[1], bytes[2], 0xff);
 	core->GetWindowManager()->FadeColor = c;
 	//this needs to be at least 2 for any effect
@@ -400,35 +399,211 @@ int fx_flash_screen (Scriptable* /*Owner*/, Actor* /*target*/, Effect* fx)
 }
 
 //0xc3 fx_tint_screen
-//FIXME: implement bit4 which would mean duration
-// param2 is mostly used with 1 or 5; only one occurrence of 100 (our spells excluded)
-// timing is mostly 16, with one 8, one 48 and one 64
+// NOTE: it actually ignored actors, not tinting them
+// Most modes are untested, the original only used 1 and 5
+// Users in the original:
+//
+// Resref      name type color dice duration
+// SPIN108.SPL raise dead: 1 grey 16d8 5
+// SPPR204.SPL spiritual hammer: 1 grey 8d6 0
+// SPPR502.SPL raise dead: 1 grey 16d8 5
+// SPPR601.SPL heal: 5 grey 16d8 7
+// SPWI113.SPL missile of patience:  1 grey 16d6 0
+// SPWI115.SPL blindness: 5 grey 16d6 3
+// SPWI120.SPL vilquar's eye: 5 grey 16d6 3
+// SPWI202.SPL black-barbed curse: 5 grey 16d6 3
+// SPWI203.SPL black-barbed shield: 5 grey 16d6 3
+// SPWI301.SPL ball lightning: 5 grey 16d(-1) 4
+// SPWI305.SPL elysium's tears: 1 grey 16d6 0
+// SPWI307.SPL hold undead: 5 grey 16d6 5
+// SPWI308.SPL tasha's: 5 grey 16d6 12
+// SPWI310.SPL ax of torment: 1 grey 16d6 10
+// SPWI313.SPL fiery rain: 1 grey 16d6 6
+// SPWI406.SPL improved strength: 5 grey 16d(-1) 4
+// SPWI408.SPL shroud of shadows: 5 grey 16d(-1) 4
+// SPWI502.SPL cone of cold: 5 grey 16d6 10
+// SPWI504.SPL enoll eva's: 5 grey 16d6 3
+// SPWI506.SPL fire and ice: 1 grey 16d6 0
+// SPWI601.SPL antimagic shell: 5 grey 16d2 3
+// SPWI602.SPL globe of inv.: 5 grey 16d2 7
+// SPWI603.SPL howl of pan.: 5 purple 16d6 10
+// SPWI604.SPL chain light. storm: 1 grey 64d6 0
+// SPWI701.SPL acid storm: 5 grey 16d6 10
+// SPWI704.SPL guardian mantle: 5 grey 16d6 5
+// SPWI805.SPL pw blind: 5 grey 16d6 6
+// SPWI905.SPL elysium's fires: 1 grey 16d6 6
+// SPWI909.SPL pw kill: 5 darkgrey 48d6 10
+
+static int ColorBoundInfringed(const Color& curGlobalTint, const Color& tintMin, const Color& tintMax)
+{
+	// not the typical luminance calculation the engine used elsewhere (ITU-R BT.601)
+	auto zip = [](const Color& c) {
+		return c.r * 77 + c.b * 154 + c.g * 25;
+	};
+
+	if (zip(curGlobalTint) >= zip(tintMin)) {
+		if (zip(curGlobalTint) <= zip(tintMax)) {
+			// continue normally
+			return 0;
+		} else {
+			// bound infringed (max)
+			return 2;
+		}
+	} else {
+		// bound infringed (min)
+		return 1;
+	}
+}
+
 int fx_tint_screen (Scriptable* /*Owner*/, Actor* /*target*/, Effect* fx)
 {
-	// print("fx_tint_screen(%2d): Par2: %d", fx->Opcode, fx->Parameter2);
-	// some effects contain garbage in DiceSides
-	// FIXME: should we really be just using DiceThrown?
-	int fromTime = fx->DiceThrown;
-	int toTime = fromTime;
-	switch(fx->Parameter2&6) {
-		case 0: toTime = 0; break;
-		case 2: fromTime = 0; break;
+	Color fadeColor = Color::FromABGR(fx->Parameter1); // usually gray
+	ieDword type = fx->Parameter2;
+	if (!core->HasFeature(GFFlags::HAS_EE_EFFECTS)) {
+		// the game clearly fades to a darker tint than what the light gray most spells use represents
+		// remove this block if we switch to actual tinting
+		fadeColor = ColorBlack;
+		fadeColor.a = 0x80;
 	}
 
-	// set the fade color (usually gray)
-	// it was a gentle effect, so use a high dose of transparency
-	// since it was so high, we just skip setting the fade color
-/*	int r, g, b;
-	r = fx->Parameter1&255;
-	g = (fx->Parameter1>>8)&255;
-	b = (fx->Parameter1>>16)&255;
-	core->GetVideoDriver()->SetFadeColor(r, g, b); // will be permanent!
-*/
+	Color tintMin;
+	Color tintMax;
+	static const Color tintNone{};
+	Color step; // the amount the engine adds to each color parameter of the current area tint every AI tick
+	if (type & 8) {
+		tintMin = ColorBlack;
+		tintMax = tintNone;
+		step = fadeColor;
+		if (type & 1) {
+			// step is set to negated param1 components, basically stepping the opposite way
+			step *= -1;
+		}
+	} else {
+		// ensure the step will be constant
+		Color initialTint;
+		if (fx->FirstApply) {
+			initialTint = core->GetWindowManager()->FadeColor; // current global lighting
+			fx->Parameter4 = fx->Duration - core->GetGame()->GameTime; // estimate amount of steps
+			fx->Parameter5 = initialTint.Packed();
+			fx->Parameter3 = fx->Parameter4;
+		} else {
+			initialTint = Color(fx->Parameter5);
+		}
+		tintMin = initialTint;
+		tintMax = fadeColor;
+		ieDword div = fx->IsVariable == 0 ? fx->Parameter4 : fx->IsVariable;
+		step = (fadeColor - initialTint) / div;
+// 		if (ColorBoundInfringed(core->GetWindowManager()->FadeColor + step, tintMin, tintMax) != 0) {
+// 			core->GetWindowManager()->FadeColor = tintNone;
+// 			return FX_NOT_APPLIED;
+// 		}
+	}
 
-	// order matters here, as SetFadeToColor resets fadeFromCounter
-	core->timer.SetFadeToColor(toTime, 2);
-	core->timer.SetFadeFromColor(fromTime, 2);
-	return FX_NOT_APPLIED;
+	// only update the color during the initial fade in
+	// NOTE: alpha channel will be overwritten by the fade code
+	if (fx->Parameter3 && ColorBoundInfringed(core->GetWindowManager()->FadeColor + step, tintMin, tintMax) == 0) {
+		core->GetWindowManager()->FadeColor += step;
+		fx->Parameter3--;
+	}
+	if (fx->FirstApply) core->GetAudioDrv()->PlayRelative(fx->Resource, SFX_CHAN_HITS);
+
+	// only some types actually use duration, the rest are permanent
+	tick_t fromTime = core->Time.defaultTicksPerSec;
+	tick_t toTime = fromTime;
+
+	// NOTE: not really treating them as bits
+	// the original only used type 1 and type 5
+	switch (type) {
+		case 0:
+		case 1: // quick fade from light to dark, then quick fade to light
+			// order matters here, as SetFadeToColor resets fadeFromCounter
+			core->timer.SetFadeToColor(toTime, 2);
+			core->timer.SetFadeFromColor(fromTime, 2);
+			// we don't have to worry about these bit 1 details:
+			// temporary durations maintain starting global lighting on bounds infringement.
+			// if duration expires before a bound is infringed, fades back to starting global lighting.
+			// permanent duration (1) terminates on bounds infringement.
+			return FX_NOT_APPLIED;
+		case 2:
+		case 3: // quick fade from light to dark, then instant light
+			core->timer.SetFadeToColor(toTime, 2);
+			core->timer.SetFadeFromColor(1, 2);
+			// we don't have to worry about these bit 2 details:
+			// temporary durations maintain opposite bound on bounds infringement. (sounds crazy)
+			// permanent duration (1) maintains bounds infringement.
+			// this mode is permanent and does not terminate on its own, unless the infringed bound
+			// happens to be the starting global lighting.
+			return FX_NOT_APPLIED;
+		case 4:
+		case 5:
+		case 6:
+		case 7: // fade from light to target color, goes for duration of effect, then fade to light
+			// extend the duration for some time of the fade-in
+			if (fx->FirstApply) {
+				core->timer.SetFadeToColor(toTime, 2);
+				fx->Parameter6 = static_cast<ieDword>(toTime) + 1;
+				fx->Duration += toTime;
+			} else if (fx->Parameter6 != 0) {
+				fx->Parameter6--;
+				fx->Duration++;
+			}
+
+			// maintain color after fade-in
+			if (!fx->Parameter6) {
+				core->GetWindowManager()->FadeColor = fadeColor;
+				if (fx->Duration == core->GetGame()->GameTime) core->timer.SetFadeFromColor(fromTime, 2);
+			}
+
+			// bit (4) Temporary durations maintain first bound infringement, then fade back to starting global
+			// lighting when duration is expired.
+			//
+			// Permanent duration (1) inverts stepping and fades to the opposite bound on bounds infringement.
+			// This mode is permanent and does not terminate on its own.
+
+			// bit (6) Temporary durations maintain first bound infringement, then fade back to starting global
+			// lighting when duration is expired.
+			//
+			// Permanent duration (1) maintains bounds infringement.
+			// This mode is permanent and does not terminate on its own, unless the infringed bound
+			// happens to be the starting global lighting.
+			break;
+		case 8: // no effect
+			return FX_NOT_APPLIED;
+		case 9: // very fast light to black shift and back / Instant inverted 'RGB Colour' for the duration of the effect
+			core->GetWindowManager()->FadeColor = ColorBlack;
+			core->timer.SetFadeToColor(core->Time.defaultTicksPerSec / 2);
+			core->timer.SetFadeFromColor(core->Time.defaultTicksPerSec / 2);
+			return FX_NOT_APPLIED;
+		case 10: // instant black for the duration of the effect, then instant light
+			core->GetWindowManager()->FadeColor = ColorBlack;
+			if (fx->FirstApply) core->timer.SetFadeToColor(1);
+			core->timer.SetFadeFromColor(1);
+			break;
+		case 100: // maintains starting global lighting for the specified duration
+			// Bug(?) for 100/101: If param1 is more luminous than the starting global lighting,
+			//                     fades to param1 and terminates regardless of timing mode
+			// NOTE: both also used fx->IsVariable somehow (beyond the above?)
+			core->GetWindowManager()->FadeColor = tintNone;
+			break;
+		case 101: // instantly set area tint to param1, then fade back to starting global lighting when duration is expired
+			core->timer.SetFadeToColor(1);
+			// permanent duration does not terminate on its own unless param1 happens to be the starting global lighting
+			if (fx->TimingMode == FX_DURATION_INSTANT_PERMANENT && tintMax != tintNone) {
+				return FX_NOT_APPLIED;
+			}
+			// about to expire
+			if (fx->Duration == core->GetGame()->GameTime) core->timer.SetFadeFromColor(core->Time.defaultTicksPerSec);
+			break;
+		case 200: // supposed to fade currently active area tint back to its starting global lighting, but just kills it
+			core->timer.SetFadeToColor(1);
+			core->timer.SetFadeFromColor(1);
+			return FX_NOT_APPLIED;
+		default:
+			Log(ERROR, "PSTOpcodes", "fx_tint_screen: Unknown type passed: {} through {} by {}!", type, fx->SourceRef, fx->CasterID);
+			return FX_NOT_APPLIED;
+	}
+
+	return FX_APPLIED;
 }
 
 //0xc4 fx_special_effect
@@ -443,24 +618,22 @@ int fx_special_effect (Scriptable* Owner, Actor* target, Effect* fx)
 	//  adds a play bam opcode to the projectile
 	//2 - raise dead projectile - the projectile itself has the effect (why is it so complicated)
 
-	//TODO: create the spells
 	switch(fx->Parameter2) {
 		case 0:
-			strnuprcpy(fx->Resource,"adder",8);
+			fx->Resource = "ADDER";
 			break;
 		case 1:
-			strnuprcpy(fx->Resource,"ball",8);
+			fx->Resource = "BALL";
 			break;
 		case 2:
-			strnuprcpy(fx->Resource,"rdead",8);
+			fx->Resource = "RDEAD";
 			break;
 	}
 
-	ieResRef OldSpellResRef;
+	ResRef OldSpellResRef = Owner->SpellResRef;
 
-	memcpy(OldSpellResRef, Owner->SpellResRef, sizeof(OldSpellResRef));
 	// flags: no deplete, instant, no interrupt
-	Owner->DirectlyCastSpell(target, fx->Resource, fx->CasterLevel, 1, false);
+	Owner->DirectlyCastSpell(target, fx->Resource, fx->CasterLevel, true, false);
 	Owner->SetSpellResRef(OldSpellResRef);
 
 	return FX_NOT_APPLIED;
@@ -497,8 +670,8 @@ int fx_multiple_vvc (Scriptable* Owner, Actor* /*target*/, Effect* fx)
 		ScriptedAnimation *sca = gamedata->GetScriptedAnimation(tab->QueryField(rows,2), true);
 		if (!sca) continue;
 		sca->SetBlend();
-		sca->SetDelay(AI_UPDATE_TIME*delay);
-		sca->SetDefaultDuration(AI_UPDATE_TIME*duration);
+		sca->SetDelay(core->Time.defaultTicksPerSec * delay);
+		sca->SetDefaultDuration(core->Time.defaultTicksPerSec * duration);
 		sca->Frame.x+=fx->PosX+offset.x;
 		sca->Frame.y+=fx->PosY+offset.y;
 		area->AddVVCell(sca);
@@ -542,21 +715,22 @@ static EffectRef fx_str_ref = { "StrengthModifier", -1 };
 
 static inline int DamageLastHitter(Effect *fx, Actor *target, int param1, int param2)
 {
-	if (fx->Parameter3) {
-		Map *map = target->GetCurrentArea();
-		Actor *actor = map->GetActorByGlobalID(target->LastHitter);
-		if (actor && PersonalDistance(target, actor)<30 ) {
-			const TriggerEntry *entry = target->GetMatchingTrigger(trigger_hitby, TEF_PROCESSED_EFFECTS);
-			if (entry) {
-				Effect *newfx = EffectQueue::CreateEffect( fx_damage_opcode_ref, param1, param2<<16, FX_DURATION_INSTANT_PERMANENT);
-				newfx->Target = FX_TARGET_PRESET;
-				newfx->Power = fx->Power;
-				memcpy(newfx->Source, fx->Source, sizeof(newfx->Source) );
-				core->ApplyEffect(newfx, actor, target);
-				if (fx->Parameter3!=0xffffffff) {
-					fx->Parameter3--;
-				}
-				delete newfx;
+	if (!fx->Parameter3) {
+		return FX_NOT_APPLIED;
+	}
+
+	const Map *map = target->GetCurrentArea();
+	Actor *actor = map->GetActorByGlobalID(target->LastHitter);
+	if (actor && PersonalDistance(target, actor) < 30) {
+		const TriggerEntry *entry = target->GetMatchingTrigger(trigger_hitby, TEF_PROCESSED_EFFECTS);
+		if (entry) {
+			Effect *newFX = EffectQueue::CreateEffect(fx_damage_opcode_ref, param1, param2 << 16, FX_DURATION_INSTANT_PERMANENT);
+			newFX->Target = FX_TARGET_PRESET;
+			newFX->Power = fx->Power;
+			newFX->Source = fx->Source;
+			core->ApplyEffect(newFX, actor, target);
+			if (fx->Parameter3 != 0xffffffff) {
+				fx->Parameter3--;
 			}
 		}
 	}
@@ -570,7 +744,7 @@ static inline int DamageLastHitter(Effect *fx, Actor *target, int param1, int pa
 static inline void ConvertTiming(Effect *fx, int Duration)
 {
 	// GameTime will be added in by EffectQueue
-	fx->Duration = Duration ? Duration * AI_UPDATE_TIME : 1;
+	fx->Duration = Duration ? Duration * core->Time.defaultTicksPerSec : 1;
 	if (fx->TimingMode == FX_DURATION_ABSOLUTE) {
 		fx->Duration += core->GetGame()->GameTime;
 	}
@@ -587,13 +761,15 @@ int fx_overlay (Scriptable* Owner, Actor* target, Effect* fx)
 	bool playonce = false;
 	ieDword tint = 0;
 	Effect *newfx;
+	ieDword tmp;
 
 	//special effects based on fx_param2
+	// PST:EE also exploded this into separate opcodes with more parameters and using a bam/vvc instead of an internal projectile
 	if (fx->FirstApply) {
 		switch(fx->Parameter2) {
 		case 0: //cloak of warding
 			ConvertTiming (fx, 5 * fx->CasterLevel);
-			fx->Parameter3 = core->Roll(3,4,fx->CasterLevel);
+			fx->Parameter3 = RAND(3, 12) + fx->CasterLevel;
 			break;
 		case 1: //shield
 			ConvertTiming(fx, 25 * fx->CasterLevel);
@@ -607,7 +783,13 @@ int fx_overlay (Scriptable* Owner, Actor* target, Effect* fx)
 
 			break;
 		case 2: //black barbed shield
-			ConvertTiming (fx, core->Roll(10,3,0));
+			// don't stack
+			tmp = fx->Projectile;
+			fx->Projectile = 0;
+			target->fxqueue.RemoveAllEffectsWithProjectile(tmp);
+			fx->Projectile = tmp;
+
+			ConvertTiming(fx, RAND(10, 30));
 			target->ApplyEffectCopy(fx, fx_armor_ref, Owner, 2, 0);
 			fx->Parameter3=0xffffffff;
 			break;
@@ -620,10 +802,11 @@ int fx_overlay (Scriptable* Owner, Actor* target, Effect* fx)
 			ConvertTiming (fx, 50 + 5 * fx->CasterLevel);
 			break;
 		case 5: //shroud of shadows
-
+			// should be 5d4 + 10 per level, but the original spell doesn't use the opcode
+			// unhardcoded in our sshadow.spl with 10s as the flat bonus instead of rolling
 			break;
 		case 6: //duplication
-			core->GetAudioDrv()->Play("magic02", SFX_CHAN_HITS, target->Pos.x, target->Pos.y);
+			core->GetAudioDrv()->Play("magic02", SFX_CHAN_HITS, target->Pos);
 			break;
 		case 7: //armor
 			target->ApplyEffectCopy(fx, fx_colorchange_ref, Owner, 0x825A2800, -1);
@@ -631,18 +814,15 @@ int fx_overlay (Scriptable* Owner, Actor* target, Effect* fx)
 			break;
 		case 8: //antimagic shell
 			{
-				int i;
-
 				newfx = EffectQueue::CreateEffectCopy(fx, fx_dispel_ref, 100, 0);
 				newfx->Power = 10;
 				core->ApplyEffect(newfx, target, Owner);
-				delete newfx;
 
-				for (i=0;i<2;i++) {
+				for (int i = 0; i < 2; i++) {
 					target->ApplyEffectCopy(fx, fx_miscast_ref, Owner, 100, i);
 				}
 
-				for (i=1;i<10;i++) {
+				for (int i = 1; i < 10; i++) {
 					target->ApplyEffectCopy(fx, fx_protection_ref, Owner, i, 0);
 				}
 				target->ApplyEffectCopy(fx, fx_magicdamage_ref, Owner, 100, 0);
@@ -653,15 +833,14 @@ int fx_overlay (Scriptable* Owner, Actor* target, Effect* fx)
 		default:
 			break;
 		case 11: //flame walk
-			ConvertTiming (fx, 10 * fx->CasterLevel);
+			ConvertTiming(fx, 10 + 10 * fx->CasterLevel);
 
 			target->ApplyEffectCopy(fx, fx_single_color_pulse_ref, Owner, 0xFF00, 0x400040);
 
 			newfx = EffectQueue::CreateEffectCopy(fx, fx_colorchange_ref, 0x64FA00, 0x50005);
-			//wtf is this
-			newfx->IsVariable = 0x23;
+			newfx->IsVariable = 0x23; // delay for 35 ticks
+			newfx->TimingMode = FX_DURATION_DELAY_LIMITED;
 			core->ApplyEffect(newfx, target, Owner);
-			delete newfx;
 
 			target->ApplyEffectCopy(fx, fx_resistfire_ref, Owner, 50, 1);
 			target->ApplyEffectCopy(fx, fx_resistmfire_ref, Owner, 50, 1);
@@ -678,7 +857,7 @@ int fx_overlay (Scriptable* Owner, Actor* target, Effect* fx)
 			//terminate = FX_NOT_APPLIED;
 			break;
 		case 13: //conflagration
-			ConvertTiming (fx, 50);
+			ConvertTiming(fx, 50); // NOTE: the description is conflicting with itself, stating both 50s and 5s/level
 			playonce = true;
 			break;
 		case 14: //infernal shield
@@ -717,16 +896,29 @@ int fx_overlay (Scriptable* Owner, Actor* target, Effect* fx)
 				bonus = std::max((ieDword) 0, target->Modified[IE_STR] + bonus - strLimit);
 			}
 
-			int duration = 60 * core->Time.hour_sec * fx->CasterLevel;
-			if (fx->SavingThrowBonus == 1) { // power of one
+			// don't stack strength spells
+			// potentially too agressive, since the durations vary a lot — did old ones just get suppressed?
+			int duration = core->Time.hour_sec * fx->CasterLevel;
+			if (fx->SavingThrowBonus == 1) { // power of one - 286
 				duration /= 2;
-			} else if (fx->SavingThrowBonus == 4) { // improved strength
+				target->fxqueue.RemoveAllEffectsWithProjectile(222);
+				target->fxqueue.RemoveAllEffectsWithProjectile(227);
+			} else if (fx->SavingThrowBonus == 4) { // improved strength - 227
 				duration = 5 * fx->CasterLevel;
+				target->fxqueue.RemoveAllEffectsWithProjectile(222);
+				target->fxqueue.RemoveAllEffectsWithProjectile(286);
+			} else { // strength - 222
+				target->fxqueue.RemoveAllEffectsWithProjectile(227);
+				target->fxqueue.RemoveAllEffectsWithProjectile(286);
 			}
+			tmp = fx->Projectile;
+			fx->Projectile = 0;
+			target->fxqueue.RemoveAllEffectsWithProjectile(tmp);
+			fx->Projectile = tmp;
 			ConvertTiming (fx, duration);
 
 			// improved strength also has a pulse we need to adjust
-			Effect *efx = target->fxqueue.HasEffectWithSource(fx_colorpulse_ref, fx->Source);
+			Effect *efx = const_cast<Effect*>(target->fxqueue.HasEffectWithSource(fx_colorpulse_ref, fx->SourceRef)); // FIXME: const_cast
 			if (efx) {
 				ConvertTiming (efx, duration);
 			}
@@ -740,14 +932,7 @@ int fx_overlay (Scriptable* Owner, Actor* target, Effect* fx)
 			ScriptedAnimation *sca = gamedata->GetScriptedAnimation(fx->Resource, true);
 			if (sca) {
 				if (tint) {
-					RGBModifier rgb;
-
-					rgb.speed=-1;
-					rgb.phase=0;
-					rgb.rgb = Color::FromABGR(tint);
-					rgb.type=RGBModifier::TINT;
-
-					sca->AlterPalette(rgb);
+					SetRGBMod(tint, sca);
 				}
 				sca->SetBlend();
 				if (playonce) {
@@ -755,7 +940,6 @@ int fx_overlay (Scriptable* Owner, Actor* target, Effect* fx)
 				} else {
 					sca->SetDefaultDuration(fx->Duration-core->GetGame()->Ticks);
 				}
-				sca->SetBlend();
 				sca->SetEffectOwned(true);
 				ScriptedAnimation *twin = sca->DetachTwin();
 				if (twin) {
@@ -888,11 +1072,8 @@ int fx_prayer (Scriptable* Owner, Actor* target, Effect* fx)
 	else if (ea<EA_GOODCUTOFF) type = 0;
 	else return FX_NOT_APPLIED;  //what happens if the target goes neutral during the effect? if the effect remains, make this FX_APPLIED
 
-	Map *map = target->GetCurrentArea();
+	const Map *map = target->GetCurrentArea();
 	int i = map->GetActorCount(true);
-	Effect *newfx = EffectQueue::CreateEffect(type?fx_curse_ref:fx_bless_ref, fx->Parameter1, fx->Parameter2, FX_DURATION_INSTANT_LIMITED);
-	memcpy(newfx, fx->Source,sizeof(ieResRef));
-	newfx->Duration = 60;
 	while(i--) {
 		Actor *tar=map->GetActor(i,true);
 		ea = tar->GetStat(IE_EA);
@@ -903,9 +1084,11 @@ int fx_prayer (Scriptable* Owner, Actor* target, Effect* fx)
 		//lets assume it is never resisted
 		//the effect will be destructed by ApplyEffect (not anymore)
 		//the effect is copied to a new memory area
+		Effect *newfx = EffectQueue::CreateEffect(type?fx_curse_ref:fx_bless_ref, fx->Parameter1, fx->Parameter2, FX_DURATION_INSTANT_LIMITED);
+		newfx->Source = fx->Source;
+		newfx->Duration = 60;
 		core->ApplyEffect(newfx, tar, Owner);
 	}
-	delete newfx;
 	return FX_APPLIED;
 }
 
@@ -913,9 +1096,9 @@ int fx_prayer (Scriptable* Owner, Actor* target, Effect* fx)
 int fx_move_view (Scriptable* /*Owner*/, Actor* /*target*/, Effect* fx)
 {
 	// print("fx_move_view(%2d): Speed: %d", fx->Opcode, fx->Parameter1);
-	Map *map = core->GetGame()->GetCurrentArea();
+	const Map *map = core->GetGame()->GetCurrentArea();
 	if (map) {
-		core->timer.SetMoveViewPort( Point(fx->PosX, fx->PosY), fx->Parameter1, true);
+		core->timer.SetMoveViewPort(fx->Pos, fx->Parameter1, true);
 	}
 	return FX_NOT_APPLIED;
 }
@@ -943,14 +1126,15 @@ int fx_embalm (Scriptable* /*Owner*/, Actor* target, Effect* fx)
 	}
 	return FX_APPLIED;
 }
+
 //0xcf fx_stop_all_action
+// FIXME: supposedly was (closer to) cutscene mode; recheck with spwi308 or any other users
 int fx_stop_all_action (Scriptable* /*Owner*/, Actor* /*target*/, Effect* fx)
 {
-	// print("fx_stop_all_action(%2d): Par2: %d", fx->Opcode, fx->Parameter2);
 	if (fx->Parameter2) {
-		core->GetGame()->TimeStop(NULL, 0xffffffff);
+		core->GetGame()->TimeStop(nullptr, 0);
 	} else {
-		core->GetGame()->TimeStop(NULL, 0);
+		core->GetGame()->TimeStop(nullptr, 0xffffffff);
 	}
 	return FX_NOT_APPLIED;
 }
@@ -959,32 +1143,31 @@ int fx_stop_all_action (Scriptable* /*Owner*/, Actor* /*target*/, Effect* fx)
 //GemRB extension: lets you specify not hardcoded values
 int fx_iron_fist (Scriptable* /*Owner*/, Actor* target, Effect* fx)
 {
-	ieDword p1,p2;
-
-	// print("fx_iron_fist(%2d): Par1: %d Par2: %d", fx->Opcode, fx->Parameter1, fx->Parameter2);
-	switch (fx->Parameter2)
-	{
-	case 0: p1 = 3; p2 = 6; break;
-	default:
-		p1 = ieWord (fx->Parameter1&0xffff);
-		p2 = ieWord (fx->Parameter1>>16);
+	ieDword toHitBonus = 3;
+	ieDword damageBonus = 6;
+	if (fx->Parameter2 != 0) {
+		toHitBonus = fx->Parameter1 & 0xffff;
+		damageBonus = fx->Parameter1 >> 16;
 	}
-	STAT_ADD(IE_FISTHIT, p1);
-	STAT_ADD(IE_FISTDAMAGE, p2);
+	STAT_ADD(IE_FISTHIT, toHitBonus);
+	STAT_ADD(IE_FISTDAMAGE, damageBonus);
 	return FX_APPLIED;
 }
 
 //0xd1 fx_hostile_image (Spell Effect: Soul Exodus)
-int fx_hostile_image (Scriptable* /*Owner*/, Actor* /*target*/, Effect* fx)
+int fx_hostile_image(Scriptable* /*Owner*/, Actor* /*target*/, Effect* fx)
 {
-	if (1) print("fx_hostile_image(%2d): Par1: %d Par2: %d TODO: not implemented!", fx->Opcode, fx->Parameter1, fx->Parameter2);
+	Log(ERROR, "PSTOpcodes","fx_hostile_image: not implemented! Source: {}", fx->Source);
 	return FX_NOT_APPLIED;
 }
 
 //0xd2 fx_detect_evil
 int fx_detect_evil (Scriptable* Owner, Actor* target, Effect* fx)
 {
-	// print("fx_detect_evil(%2d): Par1: %d Par2: %d", fx->Opcode, fx->Parameter1, fx->Parameter2);
+	if (fx->FirstApply) {
+		// fix duration, which was hardcoded in the original (50 + 25/level, max 5 min)
+		ConvertTiming(fx, std::min<int>(50 + 25 * fx->CasterLevel, 300));
+	}
 	ieDword type = fx->Parameter2;
 	//default is alignment/evil/speed 30/range 10
 	if (!type) type = 0x08031e0a;
@@ -996,14 +1179,13 @@ int fx_detect_evil (Scriptable* Owner, Actor* target, Effect* fx)
 		if (!color) color = 0xff00ff00;
 		Effect *newfx = EffectQueue::CreateEffect(fx_single_color_pulse_ref, color, speed<<16, FX_DURATION_INSTANT_PERMANENT_AFTER_BONUSES);
 		newfx->Target=FX_TARGET_PRESET;
-		EffectQueue *fxqueue = new EffectQueue();
-		fxqueue->SetOwner(Owner);
-		fxqueue->AddEffect(newfx);
-		delete newfx;
+		
+		EffectQueue fxqueue;
+		fxqueue.SetOwner(Owner);
+		fxqueue.AddEffect(newfx);
 
 		//don't detect self? if yes, then use NULL as last parameter
-		fxqueue->AffectAllInRange(target->GetCurrentArea(), target->Pos, (type&0xff000000)>>24, (type&0xff0000)>>16, (type&0xff)*10, target);
-		delete fxqueue;
+		fxqueue.AffectAllInRange(target->GetCurrentArea(), target->Pos, (type&0xff000000)>>24, (type&0xff0000)>>16, (type&0xff)*10, target);
 	}
 	return FX_APPLIED;
 }
@@ -1016,17 +1198,16 @@ int fx_jumble_curse (Scriptable* /*Owner*/, Actor* target, Effect* fx)
 	if (STATE_GET( STATE_DEAD) ) {
 		return FX_NOT_APPLIED;
 	}
-	Game *game = core->GetGame();
+	const Game *game = core->GetGame();
 	//do a hiccup every 75th refresh
 	if (fx->Parameter3/75!=fx->Parameter4/75) {
 		//hiccups
 		//PST has this hardcoded deep in the engine
 		//gemrb lets you specify the strref in P#1
-		ieStrRef tmp = fx->Parameter1;
-		if (!tmp) tmp = 46633;
-		String* tmpstr = core->GetString(tmp, IE_STR_SPEECH|IE_STR_SOUND);
-		target->SetOverheadText(*tmpstr);
-		delete tmpstr;
+		ieStrRef tmp = ieStrRef(fx->Parameter1);
+		if (!tmp) tmp = ieStrRef::PST_HICCUP;
+		String tmpstr = core->GetString(tmp, STRING_FLAGS::SPEECH | STRING_FLAGS::SOUND);
+		target->overHead.SetText(std::move(tmpstr));
 		target->GetHit();
 	}
 	fx->Parameter4=fx->Parameter3;
@@ -1041,14 +1222,18 @@ int fx_jumble_curse (Scriptable* /*Owner*/, Actor* target, Effect* fx)
 //0xd4 fx_speak_with_dead
 //This opcode is directly employed by the speak with dead projectile in the original engine
 //In GemRB it is used in a custom spell
-int fx_speak_with_dead (Scriptable* /*Owner*/, Actor* target, Effect* /*fx*/)
+int fx_speak_with_dead (Scriptable* /*Owner*/, Actor* target, Effect* fx)
 {
 	// print("fx_speak_with_dead(%2d)", fx->Opcode);
 	if (!STATE_GET( STATE_DEAD) ) {
 		return FX_NOT_APPLIED;
 	}
+	if (fx->FirstApply) fx->Parameter4 = fx->Duration - core->GetGame()->GameTime;
 
-	//TODO: reverse engineer the original opcode
+	if (fx->Parameter4 == 1) {
+		SetVariable(target, "Speak_with_Dead", 0, "GLOBAL");
+	}
+	fx->Parameter4--;
 	return FX_NOT_APPLIED;
 }
 
