@@ -30,13 +30,13 @@
 
 using namespace GemRB;
 
-SDL12VideoDriver::SDL12VideoDriver(void)
+SDL12VideoDriver::SDL12VideoDriver() noexcept
 {
 	disp = NULL;
 	inTextInput = false;
 }
 
-SDL12VideoDriver::~SDL12VideoDriver()
+SDL12VideoDriver::~SDL12VideoDriver() noexcept
 {
 	if (gameController != nullptr) {
  		SDL_JoystickClose(gameController);
@@ -56,7 +56,7 @@ int SDL12VideoDriver::Init(void)
 		setenv("SDL_HAS3BUTTONMOUSE", "SDL_HAS3BUTTONMOUSE", 1);
 #endif
 		if (SDL_InitSubSystem(SDL_INIT_JOYSTICK) == -1) {
-			Log(ERROR, "SDLJoystick", "InitSubSystem failed: %s", SDL_GetError());
+			Log(ERROR, "SDLJoystick", "InitSubSystem failed: {}", SDL_GetError());
 		} else {
 			if (SDL_NumJoysticks() > 0) {
 				gameController = SDL_JoystickOpen(0);
@@ -65,7 +65,7 @@ int SDL12VideoDriver::Init(void)
 	}
 
 	if (SDL_InitSubSystem(SDL_INIT_JOYSTICK) == -1) {
-		Log(ERROR, "SDLJoystick", "InitSubSystem failed: %s", SDL_GetError());
+		Log(ERROR, "SDLJoystick", "InitSubSystem failed: {}", SDL_GetError());
 	} else {
 		if (SDL_NumJoysticks() > 0) {
 			gameController = SDL_JoystickOpen(0);
@@ -75,16 +75,17 @@ int SDL12VideoDriver::Init(void)
 	return ret;
 }
 
-int SDL12VideoDriver::CreateSDLDisplay(const char* title)
+int SDL12VideoDriver::CreateSDLDisplay(const char* title, bool vsync)
 {
 	Log(MESSAGE, "SDL 1.2 Driver", "Creating display");
 	ieDword flags = SDL_SWSURFACE;
+	vsyncRequest = vsync;
 
 	Log(MESSAGE, "SDL 1.2 Driver", "SDL_SetVideoMode...");
 	disp = SDL_SetVideoMode(screenSize.w, screenSize.h, bpp, flags );
 	SDL_WM_SetCaption( title, 0 );
 	if (disp == NULL) {
-		Log(ERROR, "SDL 1.2 Driver", "%s", SDL_GetError());
+		Log(ERROR, "SDL 1.2 Driver", "{}", SDL_GetError());
 		return GEM_ERROR;
 	}
 	Log(MESSAGE, "SDL 1.2 Driver", "Checking for HardWare Acceleration...");
@@ -99,7 +100,7 @@ int SDL12VideoDriver::CreateSDLDisplay(const char* title)
 VideoBuffer* SDL12VideoDriver::NewVideoBuffer(const Region& r, BufferFormat fmt)
 {
 	if (fmt == BufferFormat::YV12) {
-		return new SDLOverlayVideoBuffer(r.Origin(), SDL_CreateYUVOverlay(r.w, r.h, SDL_YV12_OVERLAY, disp));
+		return new SDLOverlayVideoBuffer(r.origin, SDL_CreateYUVOverlay(r.w, r.h, SDL_YV12_OVERLAY, disp));
 	} else {
 		SDL_Surface* buf = NULL;
 		if (fmt == BufferFormat::RGB555) {
@@ -117,25 +118,25 @@ VideoBuffer* SDL12VideoDriver::NewVideoBuffer(const Region& r, BufferFormat fmt)
 		}
 		
 		if (buf == nullptr) {
-			Log(ERROR, "SDL 1.2", "%s", SDL_GetError());
+			Log(ERROR, "SDL 1.2", "{}", SDL_GetError());
 			return nullptr;
 		}
 
-		return new SDLSurfaceVideoBuffer(buf, r.Origin());
+		return new SDLSurfaceVideoBuffer(buf, r.origin);
 	}
 }
 
-IAlphaIterator* SDL12VideoDriver::StencilIterator(BlitFlags flags, SDL_Rect maskclip) const
+IAlphaIterator* SDL12VideoDriver::StencilIterator(BlitFlags flags, const Region& maskclip) const
 {
 	struct SurfaceAlphaIterator : RGBAChannelIterator {
-		SDLPixelIterator pixit;
+		SDLPixelIteratorWrapper wrap;
 		
-		SurfaceAlphaIterator(SDL_Surface* surface, const SDL_Rect& clip, Uint32 mask, Uint8 shift,
+		SurfaceAlphaIterator(SDL_Surface* surface, const Region& clip, Uint32 mask, Uint8 shift,
 							 IPixelIterator::Direction x, IPixelIterator::Direction y)
-		: RGBAChannelIterator(&pixit, mask, shift), pixit(surface, x, y, clip) {}
+		: RGBAChannelIterator(wrap.it, mask, shift), wrap(MakeSDLPixelIterator(surface, x, y, clip)) {}
 	} *maskit = nullptr;
 
-	if (flags&BLIT_STENCIL_MASK) {
+	if (flags & BlitFlags::STENCIL_MASK) {
 		SDL_Surface* maskSurf = CurrentStencilBuffer();
 		SDL_PixelFormat* fmt = maskSurf->format;
 
@@ -166,7 +167,7 @@ IAlphaIterator* SDL12VideoDriver::StencilIterator(BlitFlags flags, SDL_Rect mask
 	return maskit;
 }
 
-void SDL12VideoDriver::BlitSpriteBAMClipped(const Holder<Sprite2D> spr, const Region& src, const Region& dst,
+void SDL12VideoDriver::BlitSpriteRLEClipped(const Holder<Sprite2D>& spr, const Region& src, const Region& dst,
 											BlitFlags flags, const Color* t)
 {
 	Color tint(255,255,255,255);
@@ -196,11 +197,10 @@ void SDL12VideoDriver::BlitSpriteBAMClipped(const Holder<Sprite2D> spr, const Re
 	PaletteHolder palette = spr->GetPalette();
 	SDL_Surface* currentBuf = CurrentRenderBuffer();
 
-	SDL_Rect drect = RectFromRegion(dst);
-	IAlphaIterator* maskit = StencilIterator(flags, drect);
+	IAlphaIterator* maskit = StencilIterator(flags, dst);
 
 	// remove already handled flags and incompatible combinations
-	unsigned int remflags = flags & ~(BlitFlags::BLENDED | BlitFlags::MIRRORX | BlitFlags::MIRRORY | BLIT_STENCIL_MASK);
+	unsigned int remflags = flags & ~(BlitFlags::BLENDED | BlitFlags::MIRRORX | BlitFlags::MIRRORY | BlitFlags::STENCIL_MASK);
 	if (remflags & BlitFlags::GREY) remflags &= ~BlitFlags::SEPIA;
 
 	// TODO: we technically only need SRBlender_Alpha when there is a mask. Could boost performance noticably to account for that.
@@ -250,34 +250,29 @@ void SDL12VideoDriver::BlitSpriteBAMClipped(const Holder<Sprite2D> spr, const Re
 		}
 	}
 
-	spr->UnlockSprite();
 	delete maskit;
 }
 
-void SDL12VideoDriver::BlitSpriteNativeClipped(const sprite_t* spr, const SDL_Rect& srect, const SDL_Rect& drect, BlitFlags flags, const SDL_Color* tint)
+void SDL12VideoDriver::BlitSpriteNativeClipped(const sprite_t* spr, const Region& srect, const Region& drect, BlitFlags flags, const SDL_Color* tint)
 {
-	const SDLSurfaceSprite2D *sdlspr = static_cast<const SDLSurfaceSprite2D*>(spr);
-	SDL_Surface* surf = sdlspr->GetSurface();
-
 	Color c;
-	if (tint && (flags&BlitFlags::COLOR_MOD)){
+	if (tint && (flags & (BlitFlags::COLOR_MOD | BlitFlags::ALPHA_MOD))){
 		c = Color(tint->r, tint->g, tint->b, tint->unused);
 	}
 
-	if (surf->format->BytesPerPixel == 1) {
-		c.a = SDL_ALPHA_OPAQUE; // FIXME: this is probably actually contigent on something else...
-
-		if (flags&BlitFlags::COLOR_MOD) {
-			flags &= ~RenderSpriteVersion(sdlspr, flags, &c);
+	if (spr->Format().Bpp == 1) {
+		if (flags & (BlitFlags::COLOR_MOD | BlitFlags::ALPHA_MOD)) {
+			c.a = (flags & BlitFlags::ALPHA_MOD) ? c.a : SDL_ALPHA_OPAQUE;
+			flags &= ~spr->RenderWithFlags(flags, &c);
 		} else {
-			flags &= ~RenderSpriteVersion(sdlspr, flags);
+			flags &= ~spr->RenderWithFlags(flags);
 		}
 	}
 
-	BlitSpriteNativeClipped(surf, srect, drect, flags, c);
+	BlitSpriteNativeClipped(spr, srect, drect, flags, c);
 }
 
-void SDL12VideoDriver::BlitSpriteNativeClipped(SDL_Surface* surf, const SDL_Rect& srect, const SDL_Rect& drect, BlitFlags flags, Color tint)
+void SDL12VideoDriver::BlitSpriteNativeClipped(const sprite_t* spr, const Region& srect, const Region& drect, BlitFlags flags, Color tint)
 {
 	// non-BAM Blitting
 
@@ -297,114 +292,141 @@ void SDL12VideoDriver::BlitSpriteNativeClipped(SDL_Surface* surf, const SDL_Rect
 	// not handling the following effects at all:
 	// palettealpha
 
-	//		print("Unoptimized blit: %04X", flags);
+	//		Log(DEBUG, "SDL12Video", "Unoptimized blit: {:#x}", flags);
 
 	// remove already handled flags and incompatible combinations
 	if (flags & BlitFlags::GREY) flags &= ~BlitFlags::SEPIA;
 
-	SDL_Surface* currentBuf = CurrentRenderBuffer();
 	IAlphaIterator* maskIt = StencilIterator(flags, drect);
 	
+	SDL_Surface* surf = spr->GetSurface();
 	bool nativeBlit = (flags & ~(BlitFlags::HALFTRANS | BlitFlags::ALPHA_MOD | BlitFlags::BLENDED)) == 0
 						&& maskIt == nullptr && ((surf->flags & SDL_SRCCOLORKEY) != 0
 						|| (flags & BlitFlags::BLENDED) == 0);
 	if (nativeBlit) {
-		// must be checked afer palette versioning is done
-		
-		// the gamewin is an RGB surface (no alpha)
-		// RGBA->RGB with SDL_SRCALPHA
-		//     The source is alpha-blended with the destination, using the alpha channel. SDL_SRCCOLORKEY and the per-surface alpha are ignored.
-		// RGBA->RGB without SDL_SRCALPHA
-		//     The RGB data is copied from the source. The source alpha channel and the per-surface alpha value are ignored.
-		
-		Uint8 alpha = SDL_ALPHA_OPAQUE;
-		if (flags & BlitFlags::ALPHA_MOD) {
-			alpha = tint.a;
-		}
-		
-		if (flags & BlitFlags::HALFTRANS) {
-			alpha /= 2;
-		}
-		
-		if (flags & BlitFlags::BLENDED) {
-			SDL_SetAlpha(surf, SDL_SRCALPHA, alpha);
-		} else {
-			SDL_SetAlpha(surf, 0, alpha);
-		}
+		SDL_Rect s = RectFromRegion(srect);
+		SDL_Rect d = RectFromRegion(drect);
+		BlitSpriteNativeClipped(surf, &s, &d, flags, tint);
+	} else {		
+		SDLPixelIterator::Direction xdir = (flags&BlitFlags::MIRRORX) ? SDLPixelIterator::Reverse : SDLPixelIterator::Forward;
+		SDLPixelIterator::Direction ydir = (flags&BlitFlags::MIRRORY) ? SDLPixelIterator::Reverse : SDLPixelIterator::Forward;
 
-		SDL_Rect s = srect;
-		SDL_Rect d = drect;
-		SDL_LowerBlit(surf, &s, currentBuf, &d);
-	} else {
-		bool halftrans = flags & BlitFlags::HALFTRANS;
-		if (halftrans && (flags ^ BlitFlags::HALFTRANS)) { // other flags are set too
-			// handle halftrans with 50% alpha tinting
-			// force use of RGBBlendingPipeline with tint parameter if we aren't already
-			if (!(flags & (BlitFlags::COLOR_MOD | BlitFlags::ALPHA_MOD))) {
-				tint = ColorWhite;
-				flags |= BlitFlags::COLOR_MOD;
-			}
-			tint.a /= 2;
-		}
-
-		// FIXME: this always assumes some kind of blending if any "shader" flags are set
-		// we don't currently have a need for non blended sprites (we do for primitives, which is handled elsewhere)
-		// however, it could make things faster if we handled it
+		auto src = MakeSDLPixelIterator(surf, xdir, ydir, srect);
+		auto dst = MakeSDLPixelIterator(CurrentRenderBuffer(), SDLPixelIterator::Forward, SDLPixelIterator::Forward, drect);
 		
-		void (*BlendFn)(const Color& src, Color& dst) = ShaderBlend<true>;
-		if (flags & BlitFlags::ADD) {
-			BlendFn = ShaderAdditive;
-		} else if (flags & BlitFlags::MULTIPLY) {
-			BlendFn = ShaderTint;
-		}
-
-		if (flags & (BlitFlags::COLOR_MOD | BlitFlags::ALPHA_MOD)) {
-			if (flags&BlitFlags::GREY) {
-				RGBBlendingPipeline<SHADER::GREYSCALE, true> blender(tint, BlendFn);
-				BlitBlendedRect(surf, currentBuf, srect, drect, blender, flags, maskIt);
-			} else if (flags&BlitFlags::SEPIA) {
-				RGBBlendingPipeline<SHADER::SEPIA, true> blender(tint, BlendFn);
-				BlitBlendedRect(surf, currentBuf, srect, drect, blender, flags, maskIt);
-			} else {
-				RGBBlendingPipeline<SHADER::TINT, true> blender(tint, BlendFn);
-				BlitBlendedRect(surf, currentBuf, srect, drect, blender, flags, maskIt);
-			}
-		} else if (flags&BlitFlags::GREY) {
-			RGBBlendingPipeline<SHADER::GREYSCALE, true> blender(BlendFn);
-			BlitBlendedRect(surf, currentBuf, srect, drect, blender, flags, maskIt);
-		} else if (flags&BlitFlags::SEPIA) {
-			RGBBlendingPipeline<SHADER::SEPIA, true> blender(BlendFn);
-			BlitBlendedRect(surf, currentBuf, srect, drect, blender, flags, maskIt);
-		} else {
-			RGBBlendingPipeline<SHADER::NONE, true> blender(BlendFn);
-			BlitBlendedRect(surf, currentBuf, srect, drect, blender, flags, maskIt);
-		}
+		BlitWithPipeline(src, dst, maskIt, flags, tint);
 	}
 	
 	delete maskIt;
 }
 
-void SDL12VideoDriver::BlitVideoBuffer(const VideoBufferPtr& buf, const Point& p, BlitFlags flags, const Color* tint)
+void SDL12VideoDriver::BlitSpriteNativeClipped(SDL_Surface* surf, SDL_Rect* src, SDL_Rect* dst, BlitFlags flags, Color tint)
+{
+	// must be checked afer palette versioning is done
+	
+	// the gamewin is an RGB surface (no alpha)
+	// RGBA->RGB with SDL_SRCALPHA
+	//     The source is alpha-blended with the destination, using the alpha channel. SDL_SRCCOLORKEY and the per-surface alpha are ignored.
+	// RGBA->RGB without SDL_SRCALPHA
+	//     The RGB data is copied from the source. The source alpha channel and the per-surface alpha value are ignored.
+	
+	Uint8 alpha = SDL_ALPHA_OPAQUE;
+	if (flags & BlitFlags::ALPHA_MOD) {
+		alpha = tint.a;
+	}
+	
+	if (flags & BlitFlags::HALFTRANS) {
+		alpha /= 2;
+	}
+	
+	if (flags & BlitFlags::BLENDED) {
+		SDL_SetAlpha(surf, SDL_SRCALPHA, alpha);
+	} else {
+		SDL_SetAlpha(surf, 0, alpha);
+	}
+
+	SDL_LowerBlit(surf, src, CurrentRenderBuffer(), dst);
+}
+
+void SDL12VideoDriver::BlitWithPipeline(SDLPixelIterator& src, SDLPixelIterator& dst, IAlphaIterator* maskIt, BlitFlags flags, Color tint)
+{
+	bool halftrans = flags & BlitFlags::HALFTRANS;
+	if (halftrans && (flags ^ BlitFlags::HALFTRANS)) { // other flags are set too
+		// handle halftrans with 50% alpha tinting
+		// force use of RGBBlendingPipeline with tint parameter if we aren't already
+		if (!(flags & (BlitFlags::COLOR_MOD | BlitFlags::ALPHA_MOD))) {
+			tint = ColorWhite;
+			flags |= BlitFlags::COLOR_MOD;
+		}
+		tint.a /= 2;
+	}
+
+	// FIXME: this always assumes some kind of blending if any "shader" flags are set
+	// we don't currently have a need for non blended sprites (we do for primitives, which is handled elsewhere)
+	// however, it could make things faster if we handled it
+	
+	void (*BlendFn)(const Color& src, Color& dst) = ShaderBlend<true>;
+	if (flags & BlitFlags::ADD) {
+		BlendFn = ShaderAdditive;
+	} else if (flags & BlitFlags::MOD) {
+		BlendFn = ShaderTint;
+	}
+	
+	if (flags & (BlitFlags::COLOR_MOD | BlitFlags::ALPHA_MOD)) {
+		if (flags&BlitFlags::GREY) {
+			RGBBlendingPipeline<SHADER::GREYSCALE, true> blender(tint, BlendFn);
+			BlitBlendedRect(src, dst, blender, maskIt);
+		} else if (flags&BlitFlags::SEPIA) {
+			RGBBlendingPipeline<SHADER::SEPIA, true> blender(tint, BlendFn);
+			BlitBlendedRect(src, dst, blender, maskIt);
+		} else {
+			RGBBlendingPipeline<SHADER::TINT, true> blender(tint, BlendFn);
+			BlitBlendedRect(src, dst, blender, maskIt);
+		}
+	} else if (flags&BlitFlags::GREY) {
+		RGBBlendingPipeline<SHADER::GREYSCALE, true> blender(BlendFn);
+		BlitBlendedRect(src, dst, blender, maskIt);
+	} else if (flags&BlitFlags::SEPIA) {
+		RGBBlendingPipeline<SHADER::SEPIA, true> blender(BlendFn);
+		BlitBlendedRect(src, dst, blender, maskIt);
+	} else {
+		RGBBlendingPipeline<SHADER::NONE, true> blender(BlendFn);
+		BlitBlendedRect(src, dst, blender, maskIt);
+	}
+}
+
+void SDL12VideoDriver::BlitVideoBuffer(const VideoBufferPtr& buf, const Point& p, BlitFlags flags, Color tint)
 {
 	auto surface = static_cast<SDLSurfaceVideoBuffer&>(*buf).Surface();
 	const Region& r = buf->Rect();
-	Point origin = r.Origin() + p;
+	Point origin = r.origin + p;
+	
+	bool nativeBlit = (flags & ~(BlitFlags::HALFTRANS | BlitFlags::ALPHA_MOD | BlitFlags::BLENDED)) == 0
+						&& ((surface->flags & SDL_SRCCOLORKEY) != 0 || (flags & BlitFlags::BLENDED) == 0);
 
-	Color c;
-	if (tint) {
-		c = *tint;
+	if (nativeBlit) {
+		SDL_Rect srect = {0, 0, Uint16(r.w), Uint16(r.h)};
+		SDL_Rect drect = {Sint16(origin.x), Sint16(origin.y), Uint16(r.w), Uint16(r.h)};
+		BlitSpriteNativeClipped(surface, &srect, &drect, flags, tint);
+	} else {
+		const Region& srect = {Point(), r.size};
+		const Region& drect = {origin, r.size};
+
+		SDLPixelIterator::Direction xdir = (flags&BlitFlags::MIRRORX) ? SDLPixelIterator::Reverse : SDLPixelIterator::Forward;
+		SDLPixelIterator::Direction ydir = (flags&BlitFlags::MIRRORY) ? SDLPixelIterator::Reverse : SDLPixelIterator::Forward;
+
+		auto src = MakeSDLPixelIterator(surface, xdir, ydir, srect);
+		auto dst = MakeSDLPixelIterator(CurrentRenderBuffer(), SDLPixelIterator::Forward, SDLPixelIterator::Forward, drect);
+		
+		BlitWithPipeline(src, dst, nullptr, flags, tint);
 	}
-
-	SDL_Rect srect = {0, 0, Uint16(r.w), Uint16(r.h)};
-	SDL_Rect drect = {origin.x, origin.y, Uint16(r.w), Uint16(r.h)};
-	BlitSpriteNativeClipped(surface, srect, drect, flags, c);
 }
 
 void SDL12VideoDriver::DrawPointImp(const Point& p, const Color& color, BlitFlags flags)
 {
 	if (flags&BlitFlags::BLENDED && color.a < 0xff) {
 		DrawPointSurface<SHADER::BLEND>(CurrentRenderBuffer(), p, CurrentRenderClip(), color);
-	} else if (flags & BlitFlags::MULTIPLY) {
+	} else if (flags & BlitFlags::MOD) {
 		DrawPointSurface<SHADER::TINT>(CurrentRenderBuffer(), p, CurrentRenderClip(), color);
 	} else {
 		DrawPointSurface<SHADER::NONE>(CurrentRenderBuffer(), p, CurrentRenderClip(), color);
@@ -415,7 +437,7 @@ void SDL12VideoDriver::DrawPointsImp(const std::vector<Point>& points, const Col
 {
 	if (flags&BlitFlags::BLENDED && color.a < 0xff) {
 		DrawPointsSurface<SHADER::BLEND>(CurrentRenderBuffer(), points, CurrentRenderClip(), color);
-	} else if (flags & BlitFlags::MULTIPLY) {
+	} else if (flags & BlitFlags::MOD) {
 		DrawPointsSurface<SHADER::TINT>(CurrentRenderBuffer(), points, CurrentRenderClip(), color);
 	} else {
 		DrawPointsSurface<SHADER::NONE>(CurrentRenderBuffer(), points, CurrentRenderClip(), color);
@@ -426,7 +448,7 @@ void SDL12VideoDriver::DrawSDLPoints(const std::vector<SDL_Point>& points, const
 {
 	if (flags&BlitFlags::BLENDED && color.unused < 0xff) {
 		DrawPointsSurface<SHADER::BLEND>(CurrentRenderBuffer(), points, CurrentRenderClip(), reinterpret_cast<const Color&>(color));
-	} else if (flags & BlitFlags::MULTIPLY) {
+	} else if (flags & BlitFlags::MOD) {
 		DrawPointsSurface<SHADER::TINT>(CurrentRenderBuffer(), points, CurrentRenderClip(), reinterpret_cast<const Color&>(color));
 	} else {
 		DrawPointsSurface<SHADER::NONE>(CurrentRenderBuffer(), points, CurrentRenderClip(), reinterpret_cast<const Color&>(color));
@@ -437,7 +459,7 @@ void SDL12VideoDriver::DrawPolygonImp(const Gem_Polygon* poly, const Point& orig
 {
 	if (flags&BlitFlags::BLENDED && color.a < 0xff) {
 		DrawPolygonSurface<SHADER::BLEND>(CurrentRenderBuffer(), poly, origin, CurrentRenderClip(), color, fill);
-	} else if (flags & BlitFlags::MULTIPLY) {
+	} else if (flags & BlitFlags::MOD) {
 		DrawPolygonSurface<SHADER::TINT>(CurrentRenderBuffer(), poly, origin, CurrentRenderClip(), color, fill);
 	} else {
 		DrawPolygonSurface<SHADER::NONE>(CurrentRenderBuffer(), poly, origin, CurrentRenderClip(), color, fill);
@@ -448,7 +470,7 @@ void SDL12VideoDriver::DrawLineImp(const Point& start, const Point& end, const C
 {
 	if (flags&BlitFlags::BLENDED && color.a < 0xff) {
 		DrawLineSurface<SHADER::BLEND>(CurrentRenderBuffer(), start, end, CurrentRenderClip(), color);
-	} else if (flags & BlitFlags::MULTIPLY) {
+	} else if (flags & BlitFlags::MOD) {
 		DrawLineSurface<SHADER::TINT>(CurrentRenderBuffer(), start, end, CurrentRenderClip(), color);
 	} else {
 		DrawLineSurface<SHADER::NONE>(CurrentRenderBuffer(), start, end, CurrentRenderClip(), color);
@@ -459,7 +481,7 @@ void SDL12VideoDriver::DrawLinesImp(const std::vector<Point>& points, const Colo
 {
 	if (flags&BlitFlags::BLENDED && color.a < 0xff) {
 		DrawLinesSurface<SHADER::BLEND>(CurrentRenderBuffer(), points, CurrentRenderClip(), color);
-	} else if (flags & BlitFlags::MULTIPLY) {
+	} else if (flags & BlitFlags::MOD) {
 		DrawLinesSurface<SHADER::TINT>(CurrentRenderBuffer(), points, CurrentRenderClip(), color);
 	} else {
 		DrawLinesSurface<SHADER::NONE>(CurrentRenderBuffer(), points, CurrentRenderClip(), color);
@@ -478,8 +500,8 @@ void SDL12VideoDriver::DrawRectImp(const Region& rgn, const Color& color, bool f
 			const static OneMinusSrcA<false, false> blender;
 			
 			Region clippedrgn = ClippedDrawingRect(rgn);
-			SDLPixelIterator dstit(currentBuf, RectFromRegion(clippedrgn));
-			SDLPixelIterator dstend = SDLPixelIterator::end(dstit);
+			auto dstit = MakeSDLPixelIterator(currentBuf, clippedrgn);
+			auto dstend = SDLPixelIterator::end(dstit);
 			ColorFill(color, dstit, dstend, blender);
 		} else {
 			Uint32 val = SDL_MapRGBA( currentBuf->format, color.r, color.g, color.b, color.a );
@@ -487,18 +509,18 @@ void SDL12VideoDriver::DrawRectImp(const Region& rgn, const Color& color, bool f
 			SDL_FillRect( currentBuf, &drect, val );
 		}
 	} else if (flags&BlitFlags::BLENDED && color.a < 0xff) {
-		DrawHLineSurface<SHADER::BLEND>(currentBuf, rgn.Origin(), rgn.x + rgn.w - 1, screenClip, color);
-		DrawVLineSurface<SHADER::BLEND>(currentBuf, rgn.Origin(), rgn.y + rgn.h - 1, screenClip, color);
+		DrawHLineSurface<SHADER::BLEND>(currentBuf, rgn.origin, rgn.x + rgn.w - 1, screenClip, color);
+		DrawVLineSurface<SHADER::BLEND>(currentBuf, rgn.origin, rgn.y + rgn.h - 1, screenClip, color);
 		DrawHLineSurface<SHADER::BLEND>(currentBuf, Point(rgn.x, rgn.y + rgn.h - 1), rgn.x + rgn.w - 1, screenClip, color);
 		DrawVLineSurface<SHADER::BLEND>(currentBuf, Point(rgn.x + rgn.w - 1, rgn.y), rgn.y + rgn.h - 1, screenClip, color);
-	} else if (flags & BlitFlags::MULTIPLY) {
-		DrawHLineSurface<SHADER::TINT>(currentBuf, rgn.Origin(), rgn.x + rgn.w - 1, screenClip, color);
-		DrawVLineSurface<SHADER::TINT>(currentBuf, rgn.Origin(), rgn.y + rgn.h - 1, screenClip, color);
+	} else if (flags & BlitFlags::MOD) {
+		DrawHLineSurface<SHADER::TINT>(currentBuf, rgn.origin, rgn.x + rgn.w - 1, screenClip, color);
+		DrawVLineSurface<SHADER::TINT>(currentBuf, rgn.origin, rgn.y + rgn.h - 1, screenClip, color);
 		DrawHLineSurface<SHADER::TINT>(currentBuf, Point(rgn.x, rgn.y + rgn.h - 1), rgn.x + rgn.w - 1, screenClip, color);
 		DrawVLineSurface<SHADER::TINT>(currentBuf, Point(rgn.x + rgn.w - 1, rgn.y), rgn.y + rgn.h - 1, screenClip, color);
 	} else {
-		DrawHLineSurface<SHADER::NONE>(currentBuf, rgn.Origin(), rgn.x + rgn.w - 1, screenClip, color);
-		DrawVLineSurface<SHADER::NONE>(currentBuf, rgn.Origin(), rgn.y + rgn.h - 1, screenClip, color);
+		DrawHLineSurface<SHADER::NONE>(currentBuf, rgn.origin, rgn.x + rgn.w - 1, screenClip, color);
+		DrawVLineSurface<SHADER::NONE>(currentBuf, rgn.origin, rgn.y + rgn.h - 1, screenClip, color);
 		DrawHLineSurface<SHADER::NONE>(currentBuf, Point(rgn.x, rgn.y + rgn.h - 1), rgn.x + rgn.w - 1, screenClip, color);
 		DrawVLineSurface<SHADER::NONE>(currentBuf, Point(rgn.x + rgn.w - 1, rgn.y), rgn.y + rgn.h - 1, screenClip, color);
 	}
@@ -537,6 +559,15 @@ void SDL12VideoDriver::SwapBuffers(VideoBuffers& buffers)
 	if (flip) SDL_Flip( disp );
 }
 
+int SDL12VideoDriver::GetDisplayRefreshRate() const {
+	return 30;
+}
+
+int SDL12VideoDriver::GetVirtualRefreshCap() const {
+	// We don't know the value in SDL1, so have it fixed or unlimited
+	return vsyncRequest ? GetDisplayRefreshRate() : 0;
+}
+
 SDL12VideoDriver::vid_buf_t* SDL12VideoDriver::ScratchBuffer() const
 {
 	assert(scratchBuffer);
@@ -560,8 +591,8 @@ Holder<Sprite2D> SDL12VideoDriver::GetScreenshot(Region r,  const VideoBufferPtr
 	unsigned int Width = r.w ? r.w : screenSize.w;
 	unsigned int Height = r.h ? r.h : screenSize.h;
 
-	SDLSurfaceSprite2D *screenshot = new SDLSurfaceSprite2D(Region(0,0, Width, Height), 24,
-															0x00ff0000, 0x0000ff00, 0x000000ff, 0);
+	static const PixelFormat fmt(3, 0x00ff0000, 0x0000ff00, 0x000000ff, 0);
+	SDLSurfaceSprite2D *screenshot = new SDLSurfaceSprite2D(Region(0,0, Width, Height), fmt);
 	SDL_Rect src = RectFromRegion(r);
 	if (buf) {
 		auto surface = static_cast<SDLSurfaceVideoBuffer&>(*buf).Surface();
@@ -570,7 +601,7 @@ Holder<Sprite2D> SDL12VideoDriver::GetScreenshot(Region r,  const VideoBufferPtr
 		SDL_BlitSurface( disp, (r.w && r.h) ? &src : NULL, screenshot->GetSurface(), NULL);
 	}
 
-	return screenshot;
+	return Holder<Sprite2D>(screenshot);
 }
 
 bool SDL12VideoDriver::ToggleGrabInput()
@@ -678,7 +709,7 @@ bool SDL12VideoDriver::InTextInput()
 
 bool SDL12VideoDriver::TouchInputEnabled()
 {
-    return false;
+	return false;
 }
 
 #include "plugindef.h"
