@@ -20,69 +20,12 @@
 
 #include "SDL20Video.h"
 
-#ifdef BAKE_ICON
-#include "gemrb-icon.h"
-#endif
-
 #include "Interface.h"
 
 using namespace GemRB;
 
-SDL20VideoDriver::SDL20VideoDriver(void)
-{
-	renderer = NULL;
-	window = NULL;
-
-	stencilAlphaBlender = SDL_ComposeCustomBlendMode(SDL_BLENDFACTOR_ZERO, SDL_BLENDFACTOR_ONE,
-													 SDL_BLENDOPERATION_ADD, SDL_BLENDFACTOR_ZERO,
-													 SDL_BLENDFACTOR_ONE_MINUS_SRC_ALPHA, SDL_BLENDOPERATION_ADD);
-
-	// WARNING: do _not_ call opengl here
-	// all function pointers will be NULL
-	// until after SDL_CreateRenderer is called
-
-	SDL_version ver;
-	SDL_GetVersion(&ver);
-	sdl2_runtime_version = SDL_VERSIONNUM(ver.major, ver.minor, ver.patch);
-}
-
-SDL20VideoDriver::~SDL20VideoDriver(void)
-{
-	if (SDL_GameControllerGetAttached(gameController)) {
-		SDL_GameControllerClose(gameController);
-	}
-	
-	// we must release all buffers before SDL_DestroyRenderer
-	// we cant rely on the base destructor here
-	scratchBuffer = nullptr;
-	DestroyBuffers();
-	
-	SDL_DestroyRenderer(renderer);
-	SDL_DestroyWindow(window);
-
-	delete stencilShader;
-	delete spriteShader;
-}
-
-int SDL20VideoDriver::Init()
-{
-	int ret = SDLVideoDriver::Init();
-
-	if (SDL_InitSubSystem(SDL_INIT_GAMECONTROLLER) == -1) {
-		Log(ERROR, "SDL2", "InitSubSystem failed: %s", SDL_GetError());
-	} else {
-		for (int i = 0; i < SDL_NumJoysticks(); ++i) {
-			if (SDL_IsGameController(i)) {
-				gameController = SDL_GameControllerOpen(i);
-				if (gameController != nullptr) {
-					break;
-				}
-			}
-		}
-	}
-
-	return ret;
-}
+#ifdef BAKE_ICON
+#include "gemrb-icon.h"
 
 static void SetWindowIcon(SDL_Window* window)
 {
@@ -100,12 +43,84 @@ static void SetWindowIcon(SDL_Window* window)
 	SDL_SetWindowIcon(window, windowIcon);
 	SDL_FreeSurface(windowIcon);
 }
+#endif
 
-int SDL20VideoDriver::CreateSDLDisplay(const char* title)
+SDL20VideoDriver::SDL20VideoDriver() noexcept
+{
+	renderer = NULL;
+	window = NULL;
+
+	stencilAlphaBlender = SDL_ComposeCustomBlendMode(SDL_BLENDFACTOR_ZERO, SDL_BLENDFACTOR_ONE,
+													 SDL_BLENDOPERATION_ADD, SDL_BLENDFACTOR_ZERO,
+													 SDL_BLENDFACTOR_ONE_MINUS_SRC_ALPHA, SDL_BLENDOPERATION_ADD);
+	
+	oneMinusDstBlender = SDL_ComposeCustomBlendMode(SDL_BLENDFACTOR_ONE_MINUS_DST_COLOR, SDL_BLENDFACTOR_ONE,
+													SDL_BLENDOPERATION_ADD, SDL_BLENDFACTOR_ONE_MINUS_DST_COLOR,
+													SDL_BLENDFACTOR_ONE, SDL_BLENDOPERATION_ADD);
+	
+	dstBlender = SDL_ComposeCustomBlendMode(SDL_BLENDFACTOR_DST_COLOR, SDL_BLENDFACTOR_ONE,
+											SDL_BLENDOPERATION_ADD, SDL_BLENDFACTOR_DST_COLOR,
+											SDL_BLENDFACTOR_ONE, SDL_BLENDOPERATION_ADD);
+	
+	srcBlender = SDL_ComposeCustomBlendMode(SDL_BLENDFACTOR_SRC_COLOR, SDL_BLENDFACTOR_ONE,
+											SDL_BLENDOPERATION_ADD, SDL_BLENDFACTOR_SRC_COLOR,
+											SDL_BLENDFACTOR_ONE, SDL_BLENDOPERATION_ADD);
+
+	// WARNING: do _not_ call opengl here
+	// all function pointers will be NULL
+	// until after SDL_CreateRenderer is called
+
+	SDL_version ver;
+	SDL_GetVersion(&ver);
+	sdl2_runtime_version = SDL_VERSIONNUM(ver.major, ver.minor, ver.patch);
+}
+
+SDL20VideoDriver::~SDL20VideoDriver() noexcept
+{
+	if (SDL_GameControllerGetAttached(gameController)) {
+		SDL_GameControllerClose(gameController);
+	}
+
+	// we must release all buffers before SDL_DestroyRenderer
+	// we cant rely on the base destructor here
+	scratchBuffer = nullptr;
+	DestroyBuffers();
+
+	SDL_DestroyRenderer(renderer);
+	SDL_DestroyWindow(window);
+
+#if USE_OPENGL_BACKEND
+	delete blitRGBAShader;
+#endif
+}
+
+int SDL20VideoDriver::Init()
+{
+	int ret = SDLVideoDriver::Init();
+
+#ifdef USE_SDL_CONTROLLER_API
+	if (SDL_InitSubSystem(SDL_INIT_GAMECONTROLLER) == -1) {
+		Log(ERROR, "SDL2", "InitSubSystem failed: {}", SDL_GetError());
+		return ret;
+	}
+
+	for (int i = 0; i < SDL_NumJoysticks(); ++i) {
+		if (SDL_IsGameController(i)) {
+			gameController = SDL_GameControllerOpen(i);
+			if (gameController != nullptr) {
+				break;
+			}
+		}
+	}
+#endif
+
+	return ret;
+}
+
+int SDL20VideoDriver::CreateSDLDisplay(const char* title, bool vsync)
 {
 	Log(MESSAGE, "SDL 2 Driver", "Creating display");
-	// TODO: scale methods can be nearest or linear, and should be settable in config
-	SDL_SetHint(SDL_HINT_RENDER_SCALE_QUALITY, "nearest");
+	SDL_SetHint(SDL_HINT_RENDER_SCALE_QUALITY, "best");
 
 #if USE_OPENGL_BACKEND
 #if USE_OPENGL_API
@@ -128,9 +143,27 @@ int SDL20VideoDriver::CreateSDLDisplay(const char* title)
 	SDL_GL_SetAttribute(SDL_GL_CONTEXT_MAJOR_VERSION, 2);
 #endif
 
+
+#ifdef SDL_RESOLUTION_INDEPENDANCE
+	SDL_DisplayMode current;
+
+	int should_be_zero = SDL_GetCurrentDisplayMode(0, &current);
+
+	if (should_be_zero != 0) {
+		Log(ERROR, "SDL 2 Driver", "couldnt get resolution: {}", SDL_GetError());
+		return GEM_ERROR;
+	}
+
+	Log(DEBUG, "SDL 2 Driver", "Game Resolution: {}x{}, Screen Resolution: {}x{}",
+		screenSize.w, screenSize.h, current.w, current.h);
+	window = SDL_CreateWindow(title, SDL_WINDOWPOS_CENTERED, SDL_WINDOWPOS_CENTERED, current.w, current.h, winFlags);
+#else
+
 	window = SDL_CreateWindow(title, SDL_WINDOWPOS_CENTERED, SDL_WINDOWPOS_CENTERED, screenSize.w, screenSize.h, winFlags);
+#endif
+
 	if (window == NULL) {
-		Log(ERROR, "SDL 2 Driver", "couldnt create window:%s", SDL_GetError());
+		Log(ERROR, "SDL 2 Driver", "couldnt create window: {}", SDL_GetError());
 		return GEM_ERROR;
 	}
 
@@ -138,22 +171,43 @@ int SDL20VideoDriver::CreateSDLDisplay(const char* title)
 	SetWindowIcon(window);
 #endif
 
-	renderer = SDL_CreateRenderer(window, -1, SDL_RENDERER_TARGETTEXTURE | SDL_RENDERER_ACCELERATED);
+	int rendererFlags = SDL_RENDERER_TARGETTEXTURE | SDL_RENDERER_ACCELERATED;
+	if (vsync) {
+		rendererFlags |= SDL_RENDERER_PRESENTVSYNC;
+	}
+
+	renderer = SDL_CreateRenderer(window, -1, rendererFlags);
+
+	// Moved this here to fix a weird rendering issue
+
+	// we set logical size so that platforms where the window can be a diffrent size then requested
+	// function properly. eg iPhone and Android the requested size may be 640x480,
+	// but the window will always be the size of the screen
+	SDL_RenderSetLogicalSize(renderer, screenSize.w, screenSize.h);
+	//SDL_GetRendererOutputSize(renderer, &screenSize.w, &screenSize.h);
+
 	SDL_RendererInfo info;
 	SDL_GetRendererInfo(renderer, &info);
-	Log(DEBUG, "SDL20Video", "Renderer: %s", info.name);
+	Log(DEBUG, "SDL20Video", "Renderer: {}", info.name);
 
 	if (renderer == NULL) {
-		Log(ERROR, "SDL 2 Driver", "couldnt create renderer:%s", SDL_GetError());
+		Log(ERROR, "SDL 2 Driver", "couldnt create renderer: {}", SDL_GetError());
 		return GEM_ERROR;
 	}
 
 #if USE_OPENGL_BACKEND
-	Log(MESSAGE, "SDL 2 GL Driver", "OpenGL version: %s, renderer: %s, vendor: %s", glGetString(GL_VERSION), glGetString(GL_RENDERER), glGetString(GL_VENDOR));
-	Log(MESSAGE, "SDL 2 GL Driver", "  GLSL version: %s", glGetString(GL_SHADING_LANGUAGE_VERSION));
+	// glGetString can return null, fmt doesn't support const unsigned char* and std::string can handle neither
+	std::string tmp[4] = { "/" };
+	const int strings[4] = { GL_VERSION, GL_RENDERER, GL_VENDOR, GL_SHADING_LANGUAGE_VERSION };
+	for (int i = 0; i < 4; i++) {
+		auto glString = glGetString(strings[i]);
+		if (glString) tmp[i] = reinterpret_cast<const char*>(glString);
+	}
+	Log(MESSAGE, "SDL 2 GL Driver", "OpenGL version: {}, renderer: {}, vendor: {}", tmp[0], tmp[1], tmp[2]);
+	Log(MESSAGE, "SDL 2 GL Driver", "  GLSL version: {}", tmp[3]);
 
 	if (strcmp(info.name, driverName) != 0) {
-		Log(FATAL, "SDL 2 GL Driver", "OpenGL backend must be used instead of %s", info.name);
+		Log(FATAL, "SDL 2 GL Driver", "OpenGL backend must be used instead of {}", info.name);
 		return GEM_ERROR;
 	}
 
@@ -161,29 +215,41 @@ int SDL20VideoDriver::CreateSDLDisplay(const char* title)
 	glewInit();
 #endif
 
-	// must follow SDL_CreateRenderer
-	stencilShader = GLSLProgram::CreateFromFiles("Shaders/SDLTextureV.glsl", "Shaders/StencilF.glsl");
-	if (!stencilShader)
-	{
-		std::string msg = GLSLProgram::GetLastError();
-		Log(FATAL, "SDL 2 GL Driver", "Can't build shader program: %s", msg.c_str());
-		return GEM_ERROR;
-	}
+	scratchBuffer = CreateBuffer(Region(Point(), screenSize), BufferFormat::DISPLAY_ALPHA);
+	scratchBuffer->Clear();
 
-	spriteShader = GLSLProgram::CreateFromFiles("Shaders/SDLTextureV.glsl", "Shaders/GameSpriteF.glsl");
-	if (!spriteShader)
-	{
-		std::string msg = GLSLProgram::GetLastError();
-		Log(FATAL, "SDL 2 GL Driver", "Can't build shader program: %s", msg.c_str());
+	// Try to grab the texture shader program of SDL
+	static const SDL_Rect r = {0, 0, 1, 1};
+	SDL_RenderCopy(renderer, ScratchBuffer(), &r, &r);
+#if SDL_VERSION_ATLEAST(2, 0, 10)
+	SDL_RenderFlush(renderer);
+#endif
+
+	GLuint rgbaProgramID = 0;
+	glGetIntegerv(GL_CURRENT_PROGRAM, reinterpret_cast<GLint*>(&rgbaProgramID));
+	assert(rgbaProgramID > 0);
+
+	// Now, replace this program's shader units
+	this->blitRGBAShader =
+		GLSLProgram::CreateFromFiles("Shaders/SDLTextureV.glsl", "Shaders/BlitRGBA.glsl", rgbaProgramID);
+	if (!blitRGBAShader) {
+		Log(ERROR, "SDL 2 GL Driver", "RGBA shader setup failed: {}", GLSLProgram::GetLastError());
 		return GEM_ERROR;
 	}
 #endif
 
-	// we set logical size so that platforms where the window can be a diffrent size then requested
-	// function properly. eg iPhone and Android the requested size may be 640x480,
-	// but the window will always be the size of the screen
-	SDL_RenderSetLogicalSize(renderer, screenSize.w, screenSize.h);
-	//SDL_GetRendererOutputSize(renderer, &screenSize.w, &screenSize.h);
+	if (SDL_GetNumVideoDisplays() > 1) {
+		Log(WARNING, "SDL 2", "More than one display reported, possibly reporting wrong refresh rate.");
+	}
+
+	SDL_DisplayMode dm = {};
+	int displayModeResult = SDL_GetCurrentDisplayMode(0, &dm);
+	if (displayModeResult != 0 || dm.refresh_rate == 0) {
+		Log(WARNING, "SDL 2", "Unable to fetch refresh rate.");
+		return 0;
+	}
+
+	refreshRate = dm.refresh_rate;
 
 	SDL_StopTextInput(); // for some reason this is enabled from start
 
@@ -198,14 +264,22 @@ VideoBuffer* SDL20VideoDriver::NewVideoBuffer(const Region& r, BufferFormat fmt)
 	
 	SDL_Texture* tex = SDL_CreateTexture(renderer, format, SDL_TEXTUREACCESS_TARGET, r.w, r.h);
 	if (tex == nullptr) {
-		Log(ERROR, "SDL 2", "%s", SDL_GetError());
+		Log(ERROR, "SDL 2", "{}", SDL_GetError());
 		return nullptr;
 	}
-	return new SDLTextureVideoBuffer(r.Origin(), tex, fmt, renderer);
+	return new SDLTextureVideoBuffer(r.origin, tex, fmt, renderer);
 }
 
 void SDL20VideoDriver::SwapBuffers(VideoBuffers& buffers)
 {
+#if USE_OPENGL_BACKEND
+	// we have coopted SDLs shader, so we need to reset uniforms to values appropriate for the render targets
+	blitRGBAShader->SetUniformValue("u_greyMode", 1, 0);
+	blitRGBAShader->SetUniformValue("u_stencil", 1, 0);
+	blitRGBAShader->SetUniformValue("u_dither", 1, 0);
+	blitRGBAShader->SetUniformValue("u_rgba", 1, 1);
+#endif
+
 	SDL_SetRenderTarget(renderer, NULL);
 	SDL_SetRenderDrawColor(renderer, 0, 0, 0, SDL_ALPHA_OPAQUE);
 	SDL_RenderClear(renderer);
@@ -237,32 +311,6 @@ SDLVideoDriver::vid_buf_t* SDL20VideoDriver::CurrentStencilBuffer() const
 	return std::static_pointer_cast<SDLTextureVideoBuffer>(stencilBuffer)->GetTexture();
 }
 
-// WARNING: these are necissary hacks to combine SDL_Renderer with OpenGL (or other) native calls
-void SDL20VideoDriver::BeginCustomRendering()
-{
-	// To start we need to make sure SDL thinks it is using its texture shader
-	// so that we can change the program without it knowing
-	static const SDL_Rect r = {0, 0, 1, 1};
-	SDL_RenderCopy(renderer, ScratchBuffer(), &r, &r);
-	// if we ever call more than glUseProgram (and associates) then we will need to do more here
-	// we may want to add a 'flags' parameter to conditionally clear the other states
-#if SDL_VERSION_ATLEAST(2, 0, 10)
-	SDL_RenderFlush(renderer);
-#endif
-}
-
-void SDL20VideoDriver::EndCustomRendering()
-{
-	// to end we need SDL to "reset" the render state to restore the textrue shader
-	// we do this by changing the state to the primitive "solid" shader
-	SDL_RenderDrawPoint(renderer, -1, -1);
-	// if we ever call more than glUseProgram (and associates) then we will need to do more here
-	// we may want to add a 'flags' parameter to conditionally clear the other states
-#if SDL_VERSION_ATLEAST(2, 0, 10)
-	SDL_RenderFlush(renderer);
-#endif
-}
-
 int SDL20VideoDriver::UpdateRenderTarget(const Color* color, BlitFlags flags)
 {
 	// TODO: add support for BlitFlags::HALFTRANS, BlitFlags::COLOR_MOD, and others (no use for them ATM)
@@ -272,11 +320,11 @@ int SDL20VideoDriver::UpdateRenderTarget(const Color* color, BlitFlags flags)
 	assert(target);
 	int ret = SDL_SetRenderTarget(renderer, target);
 	if (ret != 0) {
-		Log(ERROR, "SDLVideo", "%s", SDL_GetError());
+		Log(ERROR, "SDLVideo", "{}", SDL_GetError());
 		return ret;
 	}
 
-	if (screenClip.Dimensions() == screenSize)
+	if (screenClip.size == screenSize)
 	{
 		// Some SDL backends complain on having a clip rect of the entire renderer size
 		// I'm not sure if it is an SDL bug; possibly its just 0 based so it is out of bounds?
@@ -288,7 +336,7 @@ int SDL20VideoDriver::UpdateRenderTarget(const Color* color, BlitFlags flags)
 	if (color) {
 		if (flags & BlitFlags::BLENDED) {
 			SDL_SetRenderDrawBlendMode(renderer, SDL_BLENDMODE_BLEND);
-		} else if (flags & BlitFlags::MULTIPLY) {
+		} else if (flags & BlitFlags::MOD) {
 			SDL_SetRenderDrawBlendMode(renderer, SDL_BLENDMODE_MOD);
 		} else {
 			SDL_SetRenderDrawBlendMode(renderer, SDL_BLENDMODE_NONE);
@@ -300,7 +348,7 @@ int SDL20VideoDriver::UpdateRenderTarget(const Color* color, BlitFlags flags)
 	return 0;
 }
 
-void SDL20VideoDriver::BlitSpriteNativeClipped(const SDLTextureSprite2D* spr, const SDL_Rect& src, const SDL_Rect& dst, BlitFlags flags, const SDL_Color* tint)
+void SDL20VideoDriver::BlitSpriteNativeClipped(const SDLTextureSprite2D* spr, const Region& src, const Region& dst, BlitFlags flags, const SDL_Color* tint)
 {
 	BlitFlags version = BlitFlags::NONE;
 #if !USE_OPENGL_BACKEND
@@ -308,64 +356,47 @@ void SDL20VideoDriver::BlitSpriteNativeClipped(const SDLTextureSprite2D* spr, co
 	version = (BlitFlags::GREY | BlitFlags::SEPIA) & flags;
 #endif
 	// WARNING: software fallback == slow
-	if (spr->Bpp == 8 && (flags & BlitFlags::ALPHA_MOD)) {
+	if (spr->Format().Bpp == 1 && (flags & BlitFlags::ALPHA_MOD)) {
 		version |= BlitFlags::ALPHA_MOD;
-		flags &= ~RenderSpriteVersion(spr, version, reinterpret_cast<const Color*>(tint));
+		flags &= ~spr->RenderWithFlags(version, reinterpret_cast<const Color*>(tint));
 	} else {
-		flags &= ~RenderSpriteVersion(spr, version);
+		flags &= ~spr->RenderWithFlags(version);
 	}
 
 	SDL_Texture* tex = spr->GetTexture(renderer);
 	BlitSpriteNativeClipped(tex, src, dst, flags, tint);
 }
 
-void SDL20VideoDriver::BlitSpriteNativeClipped(SDL_Texture* texSprite, const SDL_Rect& srect, const SDL_Rect& drect, BlitFlags flags, const SDL_Color* tint)
+void SDL20VideoDriver::BlitSpriteNativeClipped(SDL_Texture* texSprite, const Region& srgn, const Region& drgn, BlitFlags flags, const SDL_Color* tint)
 {
+	SDL_Rect srect = RectFromRegion(srgn);
+	SDL_Rect drect = RectFromRegion(drgn);
+	
 	int ret = 0;
-	if (flags&BLIT_STENCIL_MASK) {
+#if USE_OPENGL_BACKEND
+	UpdateRenderTarget();
+	ret = RenderCopyShaded(texSprite, &srect, &drect, flags, tint);
+#if SDL_VERSION_ATLEAST(2, 0, 10)
+	SDL_RenderFlush(renderer);
+#endif
+#else
+	if (flags & BlitFlags::STENCIL_MASK) {
 		// 1. clear scratchpad segment
 		// 2. blend stencil segment to scratchpad
 		// 3. blend texture to scratchpad
 		// 4. copy scratchpad segment to screen
 
-		SDL_Texture* stencilTex = CurrentStencilBuffer();
-		SDL_SetTextureBlendMode(stencilTex, stencilAlphaBlender);
-
 		std::static_pointer_cast<SDLTextureVideoBuffer>(scratchBuffer)->Clear(drect); // sets the render target to the scratch buffer
 
+		SDL_Texture* stencilTex = CurrentStencilBuffer();
+		SDL_SetTextureBlendMode(stencilTex, stencilAlphaBlender);
 		RenderCopyShaded(texSprite, &srect, &drect, flags & ~(BlitFlags::ALPHA_MOD|BlitFlags::HALFTRANS), tint);
-		
+		// alpha masking only
 		SDL_Rect stencilRect = drect;
 		stencilRect.x -= stencilBuffer->Origin().x;
 		stencilRect.y -= stencilBuffer->Origin().y;
-
-#if USE_OPENGL_BACKEND
-		BeginCustomRendering();
-		GLint channel = 3;
-		if (flags&BlitFlags::STENCIL_RED) {
-			channel = 0;
-		} else if (flags&BlitFlags::STENCIL_GREEN) {
-			channel = 1;
-		} else if (flags&BlitFlags::STENCIL_BLUE) {
-			channel = 2;
-		}
-
-		stencilShader->Use();
-		stencilShader->SetUniformValue("u_channel", 1, channel);
-		if (flags & BlitFlags::STENCIL_DITHER) {
-			stencilShader->SetUniformValue("u_dither", 1, 1);
-		} else {
-			stencilShader->SetUniformValue("u_dither", 1, 0);
-		}
-		
-		stencilShader->SetUniformValue("s_stencil", 1, 0);
-
 		SDL_RenderCopy(renderer, stencilTex, &stencilRect, &drect);
-		EndCustomRendering();
-#else
-		// alpha masking only
-		SDL_RenderCopy(renderer, stencilTex, &stencilRect, &drect);
-#endif
+
 		if (flags & (BlitFlags::ALPHA_MOD | BlitFlags::HALFTRANS)) {
 			Uint8 alpha = SDL_ALPHA_OPAQUE;
 			if (flags & BlitFlags::ALPHA_MOD) {
@@ -384,42 +415,122 @@ void SDL20VideoDriver::BlitSpriteNativeClipped(SDL_Texture* texSprite, const SDL
 		UpdateRenderTarget();
 		ret = RenderCopyShaded(texSprite, &srect, &drect, flags, tint);
 	}
+#endif
 
 	if (ret != 0) {
-		Log(ERROR, "SDLVideo", "%s", SDL_GetError());
+		Log(ERROR, "SDLVideo", "{}", SDL_GetError());
 	}
 }
 
-void SDL20VideoDriver::BlitVideoBuffer(const VideoBufferPtr& buf, const Point& p, BlitFlags flags, const Color* tint)
+void SDL20VideoDriver::BlitVideoBuffer(const VideoBufferPtr& buf, const Point& p, BlitFlags flags, Color tint)
 {
 	auto tex = static_cast<SDLTextureVideoBuffer&>(*buf).GetTexture();
 	const Region& r = buf->Rect();
-	Point origin = r.Origin() + p;
+	Point origin = r.origin + p;
 
-	SDL_Rect srect = {0, 0, r.w, r.h};
-	SDL_Rect drect = {origin.x, origin.y, r.w, r.h};
-	BlitSpriteNativeClipped(tex, srect, drect, flags, reinterpret_cast<const SDL_Color*>(tint));
+	const Region& srect = {0, 0, r.w, r.h};
+	const Region& drect = {origin, r.size};
+	BlitSpriteNativeClipped(tex, srect, drect, flags, reinterpret_cast<const SDL_Color*>(&tint));
 }
 
 int SDL20VideoDriver::RenderCopyShaded(SDL_Texture* texture, const SDL_Rect* srcrect,
 									   const SDL_Rect* dstrect, BlitFlags flags, const SDL_Color* tint)
 {
 #if USE_OPENGL_BACKEND
-	BeginCustomRendering();
-	spriteShader->Use();
-	if (flags&BlitFlags::GREY) {
-		spriteShader->SetUniformValue("u_greyMode", 1, 1);
-	} else if (flags&BlitFlags::SEPIA) {
-		spriteShader->SetUniformValue("u_greyMode", 1, 2);
-	} else {
-		spriteShader->SetUniformValue("u_greyMode", 1, 0);
+#if SDL_VERSION_ATLEAST(2, 0, 10)
+	SDL_RenderFlush(renderer);
+#endif
+
+	uint32_t format = 0;
+	SDL_QueryTexture(texture, &format, nullptr, nullptr, nullptr);
+	blitRGBAShader->Use();
+	
+	blitRGBAShader->SetUniformValue("s_sprite", 1, 0);
+	blitRGBAShader->SetUniformValue("s_stencil", 1, 1);
+	
+	bool isRGBA = SDL_ISPIXELFORMAT_ALPHA(format);
+	blitRGBAShader->SetUniformValue("u_rgba", 1, isRGBA ? 1 : 0);
+
+	GLint greyMode = 0;
+	if (flags & BlitFlags::GREY) {
+		greyMode = 1;
+	} else if (flags & BlitFlags::SEPIA) {
+		greyMode = 2;
 	}
 
-	spriteShader->SetUniformValue("s_sprite", 1, 0);
-#else
-	// "shaders" were already applied via software (RenderSpriteVersion)
-	// they had to be applied very first so we could create a texture from the software rendering
+	blitRGBAShader->SetUniformValue("u_greyMode", 1, greyMode);
+
+	GLint channel = 3;
+	if (flags & BlitFlags::STENCIL_RED) {
+		channel = 0;
+	} else if (flags & BlitFlags::STENCIL_GREEN) {
+		channel = 1;
+	} else if (flags & BlitFlags::STENCIL_BLUE) {
+		channel = 2;
+	}
+
+	blitRGBAShader->SetUniformValue("u_channel", 1, channel);
+
+	bool doStencil = flags & BlitFlags::STENCIL_MASK;
+	blitRGBAShader->SetUniformValue("u_stencil", 1, doStencil ? 1 : 0);
+
+	if (doStencil) {
+		assert(stencilBuffer && dstrect);
+
+		bool doDither = flags & BlitFlags::STENCIL_DITHER;
+		blitRGBAShader->SetUniformValue("u_dither", 1, doDither ? 1 : 0);
+
+		int texW = 0, texH = 0;
+		SDL_QueryTexture(CurrentStencilBuffer(), nullptr, nullptr, &texW, &texH);
+
+		GLfloat stencilTexW = 1.0f, stencilTexH = 1.0f,
+			stencilTexX = 0.0f, stencilTexY = 0.0f;
+
+		float scaleX = 0.0f, scaleY = 0.0f;
+		SDL_RenderGetScale(renderer, &scaleX, &scaleY);
+		
+		SDL_Rect stencilRect = *dstrect;
+		stencilRect.x -= stencilBuffer->Origin().x;
+		stencilRect.y -= stencilBuffer->Origin().y;
+
+		if (stencilRect.x < dstrect->x && stencilRect.y < dstrect->y) {
+			stencilTexX = -dstrect->x;
+			stencilTexY = -dstrect->y;
+		}
+
+#if !SDL_VERSION_ATLEAST(2, 0, 18)
+		// In versions earler, SDL uses a different vertex setup in case
+		// of flipping: (-w/2, -h/2) to (w/2, h/2) that are transformed by
+		// the OpenGL backend via matrices.
+		if (flags & BlitFlags::MIRRORX) {
+			stencilTexX = dstrect->w / 2.0f + stencilRect.x;
+			stencilTexY = dstrect->h / 2.0f + stencilRect.y;
+		}
 #endif
+
+		stencilTexW = 1.0f / (texW * scaleX);
+		stencilTexH = 1.0f / (texH * scaleY);
+
+		GLfloat mat[3][3] = {
+			{ stencilTexW,        0.0f, 0.0f },
+			{        0.0f, stencilTexH, 0.0f },
+			{ stencilTexX * stencilTexW, stencilTexY * stencilTexH, 1.0f }
+		};
+
+		blitRGBAShader->SetUniformMatrixValue("u_stencilMat", 3, 1, reinterpret_cast<GLfloat*>(&mat));
+		
+		// Ask OpenGL about the texture handle (that lies hidden in SDL_Texture otherwise)
+		auto texture = std::static_pointer_cast<SDLTextureVideoBuffer>(stencilBuffer)->GetTexture();
+		SDL_GL_BindTexture(texture, nullptr, nullptr);
+		GLuint stencilTextureID;
+		glGetIntegerv(GL_TEXTURE_BINDING_2D, reinterpret_cast<GLint*>(&stencilTextureID));
+		SDL_GL_UnbindTexture(texture);
+
+		glActiveTexture(GL_TEXTURE1);
+		glBindTexture(GL_TEXTURE_2D, stencilTextureID);
+	}
+#endif
+	
 	Uint8 alpha = SDL_ALPHA_OPAQUE;
 	if (flags & BlitFlags::ALPHA_MOD) {
 		alpha = tint->a;
@@ -439,37 +550,64 @@ int SDL20VideoDriver::RenderCopyShaded(SDL_Texture* texture, const SDL_Rect* src
 	
 	if (flags & BlitFlags::ADD) {
 		SDL_SetTextureBlendMode(texture, SDL_BLENDMODE_ADD);
-	} else if (flags & BlitFlags::MULTIPLY) {
+	} else if (flags & BlitFlags::MOD) {
 		SDL_SetTextureBlendMode(texture, SDL_BLENDMODE_MOD);
+	} else if (flags & BlitFlags::MUL) {
+#if SDL_VERSION_ATLEAST(2, 0, 12)
+		SDL_SetTextureBlendMode(texture, SDL_BLENDMODE_MUL);
+#endif
+	} else if (flags & BlitFlags::SRC) {
+		SDL_SetTextureBlendMode(texture, srcBlender);
+	} else if (flags & BlitFlags::ONE_MINUS_DST) {
+		SDL_SetTextureBlendMode(texture, oneMinusDstBlender);
+	} else if (flags & BlitFlags::DST) {
+		SDL_SetTextureBlendMode(texture, dstBlender);
 	} else if (flags & (BlitFlags::BLENDED | BlitFlags::HALFTRANS)) {
 		SDL_SetTextureBlendMode(texture, SDL_BLENDMODE_BLEND);
 	} else {
 		SDL_SetTextureBlendMode(texture, SDL_BLENDMODE_NONE);
 	}
-	
-	SDL_RendererFlip flipflags = (flags&BlitFlags::MIRRORY) ? SDL_FLIP_VERTICAL : SDL_FLIP_NONE;
-	flipflags = static_cast<SDL_RendererFlip>(flipflags | ((flags&BlitFlags::MIRRORX) ? SDL_FLIP_HORIZONTAL : SDL_FLIP_NONE));
 
-	int ret = SDL_RenderCopyEx(renderer, texture, srcrect, dstrect, 0.0, NULL, flipflags);
-#if USE_OPENGL_BACKEND
-	EndCustomRendering();
+	SDL_RendererFlip flipflags = (flags & BlitFlags::MIRRORY) ? SDL_FLIP_VERTICAL : SDL_FLIP_NONE;
+	flipflags = static_cast<SDL_RendererFlip>(flipflags | ((flags & BlitFlags::MIRRORX) ? SDL_FLIP_HORIZONTAL : SDL_FLIP_NONE));
+
+	return SDL_RenderCopyEx(renderer, texture, srcrect, dstrect, 0.0, nullptr, flipflags);
+}
+
+void SDL20VideoDriver::DrawRawGeometry(
+	const std::vector<float>& vertices,
+	const std::vector<Color>& colors,
+	BlitFlags blitFlags
+) {
+#if SDL_VERSION_ATLEAST(2, 0, 18)
+	if (blitFlags & BlitFlags::BLENDED) {
+		SDL_SetRenderDrawBlendMode(renderer, SDL_BLENDMODE_BLEND);
+	}
+
+	SDL_RenderGeometryRaw(
+		renderer,
+		nullptr,
+		vertices.data(),
+		2 * sizeof(float),
+		reinterpret_cast<const SDL_Color*>(colors.data()),
+		sizeof(Color),
+		nullptr,
+		0,
+		vertices.size() / 2,
+		nullptr,
+		0,
+		0
+	);
+#else
+	(void)vertices;
+	(void)colors;
+	(void)blitFlags;
 #endif
-	return ret;
 }
 
 void SDL20VideoDriver::DrawPointsImp(const std::vector<Point>& points, const Color& color, BlitFlags flags)
 {
-	// TODO: refactor Point to use int so this is not needed
-	std::vector<SDL_Point> sdlpoints;
-	sdlpoints.reserve(points.size());
-
-	for (size_t i = 0; i < points.size(); ++i) {
-		const Point& point = points[i];
-		SDL_Point sdlpoint = {point.x, point.y};
-		sdlpoints.push_back(sdlpoint);
-	}
-
-	DrawSDLPoints(sdlpoints, reinterpret_cast<const SDL_Color&>(color), flags);
+	DrawSDLPoints(reinterpret_cast<const std::vector<SDL_Point>&>(points), reinterpret_cast<const SDL_Color&>(color), flags);
 }
 
 void SDL20VideoDriver::DrawSDLPoints(const std::vector<SDL_Point>& points, const SDL_Color& color, BlitFlags flags)
@@ -489,17 +627,7 @@ void SDL20VideoDriver::DrawPointImp(const Point& p, const Color& color, BlitFlag
 
 void SDL20VideoDriver::DrawLinesImp(const std::vector<Point>& points, const Color& color, BlitFlags flags)
 {
-	// TODO: refactor Point to use int so this is not needed
-	std::vector<SDL_Point> sdlpoints;
-	sdlpoints.reserve(points.size());
-
-	for (size_t i = 0; i < points.size(); ++i) {
-		const Point& point = points[i];
-		SDL_Point sdlpoint = {point.x, point.y};
-		sdlpoints.push_back(sdlpoint);
-	}
-
-	DrawSDLLines(sdlpoints, reinterpret_cast<const SDL_Color&>(color), flags);
+	DrawSDLLines(reinterpret_cast<const std::vector<SDL_Point>&>(points), reinterpret_cast<const SDL_Color&>(color), flags);
 }
 
 void SDL20VideoDriver::DrawSDLLines(const std::vector<SDL_Point>& points, const SDL_Color& color, BlitFlags flags)
@@ -544,7 +672,7 @@ void SDL20VideoDriver::DrawPolygonImp(const Gem_Polygon* poly, const Point& orig
 		std::vector<SDL_Point> points(poly->Count() + 1);
 		size_t i = 0;
 		for (; i < poly->Count(); ++i) {
-			const Point& p = poly->vertices[i] - poly->BBox.Origin() + origin;
+			const Point& p = poly->vertices[i] - poly->BBox.origin + origin;
 			points[i].x = p.x;
 			points[i].y = p.y;
 		}
@@ -563,12 +691,12 @@ Holder<Sprite2D> SDL20VideoDriver::GetScreenshot(Region r, const VideoBufferPtr&
 	unsigned int Width = r.w ? r.w : screenSize.w;
 	unsigned int Height = r.h ? r.h : screenSize.h;
 
-	SDLTextureSprite2D* screenshot = new SDLTextureSprite2D(Region(0,0, Width, Height), 24,
-															0x00ff0000, 0x0000ff00, 0x000000ff, 0);
+	static const PixelFormat fmt(3, 0x00ff0000, 0x0000ff00, 0x000000ff, 0);
+	SDLTextureSprite2D* screenshot = new SDLTextureSprite2D(Region(0,0, Width, Height), fmt);
 
 	SDL_Texture* target = SDL_GetRenderTarget(renderer);
 	if (buf) {
-		auto texture = static_cast<SDLTextureVideoBuffer*>(drawingBuffer)->GetTexture();
+		auto texture = static_cast<SDLTextureVideoBuffer*>(buf.get())->GetTexture();
 		SDL_SetRenderTarget(renderer, texture);
 	} else {
 		SDL_SetRenderTarget(renderer, nullptr);
@@ -579,7 +707,7 @@ Holder<Sprite2D> SDL20VideoDriver::GetScreenshot(Region r, const VideoBufferPtr&
 
 	SDL_SetRenderTarget(renderer, target);
 
-	return screenshot;
+	return Holder<Sprite2D>(screenshot);
 }
 
 int SDL20VideoDriver::GetTouchFingers(TouchEvent::Finger(&fingers)[FINGER_MAX], SDL_TouchID device) const
@@ -587,7 +715,7 @@ int SDL20VideoDriver::GetTouchFingers(TouchEvent::Finger(&fingers)[FINGER_MAX],
 	int numf = SDL_GetNumTouchFingers(device);
 
 	for (int i = 0; i < numf; ++i) {
-		SDL_Finger* finger = SDL_GetTouchFinger(device, i);
+		const SDL_Finger* finger = SDL_GetTouchFinger(device, i);
 		assert(finger);
 
 		fingers[i].id = finger->id;
@@ -610,9 +738,10 @@ int SDL20VideoDriver::ProcessEvent(const SDL_Event & event)
 	Event e;
 
 	switch (event.type) {
+#ifdef USE_SDL_CONTROLLER_API
 		case SDL_CONTROLLERDEVICEREMOVED:
 			if (gameController != nullptr) {
-				SDL_GameController *removedController = SDL_GameControllerFromInstanceID(event.jdevice.which);
+				const SDL_GameController *removedController = SDL_GameControllerFromInstanceID(event.jdevice.which);
 				if (removedController == gameController) {
 					SDL_GameControllerClose(gameController);
 					gameController = nullptr;
@@ -630,7 +759,7 @@ int SDL20VideoDriver::ProcessEvent(const SDL_Event & event)
 				float pct = event.caxis.value / float(sizeof(Sint16));
 				bool xaxis = event.caxis.axis % 2;
 				// FIXME: I'm sure this delta needs to be scaled
-				int delta = (xaxis) ? pct * screenSize.w : pct * screenSize.h;
+				int delta = xaxis ? pct * screenSize.w : pct * screenSize.h;
 				InputAxis axis = InputAxis(event.caxis.axis);
 				e = EvntManager->CreateControllerAxisEvent(axis, delta, pct);
 				EvntManager->DispatchEvent(std::move(e));
@@ -645,6 +774,7 @@ int SDL20VideoDriver::ProcessEvent(const SDL_Event & event)
 				EvntManager->DispatchEvent(std::move(e));
 			}
 			break;
+#endif
 		case SDL_FINGERDOWN: // fallthough
 		case SDL_FINGERUP:
 			{
@@ -706,7 +836,7 @@ int SDL20VideoDriver::ProcessEvent(const SDL_Event & event)
 					unitIsPixels = true;
 				}
 				
-				int speed = (unitIsPixels) ? 1 : core->GetMouseScrollSpeed();
+				int speed = unitIsPixels ? 1 : core->GetMouseScrollSpeed();
 				if (SDL_GetModState() & KMOD_SHIFT) {
 					e = EvntManager->CreateMouseWheelEvent(Point(event.wheel.y * speed, event.wheel.x * speed));
 				} else {
@@ -728,8 +858,10 @@ int SDL20VideoDriver::ProcessEvent(const SDL_Event & event)
 			// TODO: must destroy all SDLTextureSprite2D textures
 
 			// fallthough
+		case SDL_APP_DIDENTERFOREGROUND:
 		case SDL_RENDER_TARGETS_RESET:
-			// TODO: must destroy all SDLTextureVideoBuffer textures
+			e = EventMgr::CreateRedrawRequestEvent();
+			EvntManager->DispatchEvent(std::move(e));
 			break;
 		case SDL_WINDOWEVENT://SDL 1.2
 			switch (event.window.event) {
@@ -841,6 +973,14 @@ bool SDL20VideoDriver::TouchInputEnabled()
 	return SDL_GetNumTouchDevices() > 0;
 }
 
+bool SDL20VideoDriver::CanDrawRawGeometry() const {
+#if SDL_VERSION_ATLEAST(2, 0, 18)
+	return true;
+#else
+	return false;
+#endif
+}
+
 void SDL20VideoDriver::SetGamma(int brightness, int /*contrast*/)
 {
 	// FIXME: hardcoded hack. in in Interface our default brigtness value is 10
