@@ -32,17 +32,19 @@ namespace GemRB {
 
 class SPLImporter : public SpellMgr {
 private:
-	DataStream* str;
-	int version;
+	DataStream* str = nullptr;
+	int version = 0;
 
 public:
-	SPLImporter(void);
-	~SPLImporter(void) override;
+	SPLImporter() noexcept = default;
+	SPLImporter(const SPLImporter&) = delete;
+	~SPLImporter() override;
+	SPLImporter& operator=(const SPLImporter&) = delete;
 	bool Open(DataStream* stream) override;
 	Spell* GetSpell(Spell *spl, bool silent=false) override;
 private:
-	void GetExtHeader(Spell *s, SPLExtHeader* eh);
-	void GetFeature(Spell *s, Effect *f);
+	void GetExtHeader(const Spell *s, SPLExtHeader* eh);
+	Effect *GetFeature(const Spell *s);
 };
 
 
