@@ -28,32 +28,20 @@
 namespace GemRB {
 
 class TTFFontManager : public FontManager {
-/*
-Private ivars
-*/
 private:
-	FT_Stream ftStream;
-	FT_Face face;
+	FT_Stream ftStream = nullptr;
+	FT_Face face = nullptr;
 
 public:
-/*
-Public methods
-*/
-	~TTFFontManager(void) override;
-	TTFFontManager(void);
+	TTFFontManager() noexcept = default;
+	TTFFontManager(const TTFFontManager&) = delete;
+	~TTFFontManager() override;
+	TTFFontManager& operator=(const TTFFontManager&) = delete;
 
-	bool Open(DataStream* stream) override;
+	bool Import(DataStream* stream) override;
 	void Close();
 
 	Font* GetFont(unsigned short pxSize, FontStyle style, bool background) override;
-
-	// freetype "callbacks"
-	static unsigned long read( FT_Stream       stream,
-							  unsigned long   offset,
-							  unsigned char*  buffer,
-							  unsigned long   count );
-
-	static void close( FT_Stream stream );
 };
 
 }
