@@ -19,14 +19,13 @@
 #ifndef LOGGER_APPLE_H
 #define LOGGER_APPLE_H
 
-#include "System/Logger/Stdio.h"
+#include "Logging/Loggers/Stdio.h"
 
 namespace GemRB {
 
 class GEM_EXPORT AppleLogger : public Logger::LogWriter {
 public:
 	AppleLogger();
-	~AppleLogger() override;
 
 protected:
 	void WriteLogMessage(const Logger::LogMessage& msg) override;
